c-----------------------------------------------------------------------
      program postutil
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641            Level: 110225           MAIN
c
c ---           D. Strimaitis
c ---           J. Scire
c
c     Copyright (c) 1998-2011 by Exponent, Inc.
c
c     ISORROPIA module used with permission
c     Copyright 1996-2008 
c     UNIVERSITY OF MIAMI,
c     CARNEGIE MELLON UNIVERSITY
c     GEORGIA INSTITUTE OF TECHNOLOGY
c     
c
c
c  PURPOSE:     Main program for processing data produced by CALPUFF
c               prior to applying CALPOST.  Central features include:
c
c               1. Creating new species as weighted combination of
c                  modeled species
c               2. Merging species that had been modeled separately
c               3. Scaling and Summing results from different source
c                  groups that had been modeled separately
c               4. Recomputing HNO3/NO3 concentration partition
c               5. Process Ammonia Limiting Method (ALM) in 1 only step
c
c
c               Feature not yet implemented:
c               5. Combining a temporal sequence of files into one
c
c               Receptors must be identical among all files combined
c
c --- UPDATES:
c
c --- Modifications - V1.64, level 101025 to V1.641, level 110225
c     1) Updated CALUTILS.FOR to version v2.58 (110225)
c        - Add control file variable type 5 (character array) and
c          retain commas in string returned so that array of
c          variable values can be parsed
c        Modified: READIN, ALTONU, SETVAR
c
c --- Modifications - V1.63, level 100917 to V1.64, level 101025
c     1) Update ISORROPIA codes from V1.7 to V2.1 (ISORROPIA-II)
c        REPLACED: ISRPIA.INC
c                  ISORROPIA.FOR (include-file)
c        MODIFIED: ISODRIVER
c
c --- Modifications - V1.62, level 091122 to V1.63, level 100917
c     1) Add ISORROPIA equilibrium option for nitrate partition (ALM)
c        as prepared by AER for API Contract No. 2006-102376.
c        ISORROPIA is coded as an option in place of CHEMEQ and is
c        selected using option MEQPHASE.
c     2) Add local logical to use the minimum TNO3 in the call to the
c        ISORROPIA driver as a floor rather than as a breakpoint for
c        NO3=0.0
c        New:      ISRPIA.INC,
c                  ISORROPIA.FOR
c        Modified: CTRL.UTL
c                  BLOCK DATA, READCF, QAINP, PARTITION, ALM1STEP
c
c --- Modifications - V1.61, level 090511 to V1.62, level 091122
c     1) Add explicit treatment of layers other than layer 1 to
c        facilitate use of v2.2 files from CALGRID.  Implement a
c        single layer at this time (all species processed in an
c        application must be in one layer).
c        Modified: CTRL.UTL
c                  BLOCK DATA, READCF, QAINP, ADJHEAD
c     2) Apply requirement for mass units only if nitrate processing
c        is done (MNITRATE is not 0)
c        Modified: QAINP
c
c --- Modifications - V1.6, level 071015 to V1.61, level 090511
c     1) CALUTILS from v2.56 Level 080407 to v2.571 Level 090511
c        Increase control file line length to 200 characters
c        Activate CPU clock using F95 system routine
c        Add routine to reformat a date string
c        New     : FMT_DATE
c        Modified: PARAMS.CAL, READIN, DATETM
c     2) Add run-time reporting using CALUTILS routines.
c        New:      QA.UTL, FIN
c     3) Reset control-file line-length limit to 200
c        Modified: TFERCF
c     4) Revised list-file output formats for exponential notation
c        Modified: READCF
c     5) Updated CALUTILS from v2.55 Level 070327 to v2.56 Level 080407
c        Control file entries in exponential notation were not correct
c        if decimal point was missing (2e-02 was read as 0.2e-02).
c        Modified: ALTONU
c     6) Add a check for CALMET size against MXGX and MXGY and
c        for 2D-MET size against MXGX and MXGY.
c        Both possibly needed for NO3/HNO3 repartitioning
c        Modified: MET1, GETHDRH
c        (C. Escoffier-Czaja)
c
c --- Modifications - V1.56, level 070627 to V1.6, level 071015
c    (1) Fix declaration of 132 character comment string for 1-D met
c        files
c        Modified:  GETDOC1D
c    (2) Initial implementation of CALPUFF output dataset version 2.2
c        (screen for g/m3 or g/m2/s units for all species that are used)
c        - Add header record after species-level list for units of
c          each species-level
c        - Base time zone read as character*8 (e.g. UTC-0500)
c        Modified:  HEAD.UTL, HEAD2.UTL
c                   MAIN, GETHEAD, GETHDRH, QAINP, ADJHEAD, WRIHEAD
c    (3) Introduce parameter MXSPEC_IN to set the size of species
c        arrays read in.  MXSPEC is reserved for the number of species
c        processed and written out.  This allows an input file with
c        many unused species to be read.  Control file arrays are also
c        set in READCF as well as in the PARAMS file, and these must
c        match.  Code updated to use MXSPEC=40.
c        Modified:  HEAD.UTL
c                   READCF, GETHEAD, GETHDRH
c
c --- Modifications - V1.55, level 070327 to V1.56, level 070627
c        Fix declaration of temporary array CTEMP for reading filenames
c        from the control file (submitted by B. Brashers)
c        Modified:  READFN
c
c --- Modifications - V1.54, level 060825 to V1.55, level 070327
c        CALUTILS from v2.52 Level 060519 to v2.55 Level 070327
c        Move GLOBE1 to COORDLIB
c        Allow negative increments in INCRS
c        Fixed format bug in subroutine BASRUTC for the case of time
c        zone zero (output string was 'UTC+0  0' instead of 'UTC+0000'
c        Modified:  INCRS, UTCBASR, BASRUTC
c        Removed:   GLOBE1
c
c --- Modifications - V1.53, level 060519 to V1.54, level 060825
c        When checking against MXFIL, NMET must be multiplied by 3
c        only in the case METFM=2, use of 2D met files (the 3 files,
c        for RHO, TEMP and RH).
c        Modified: READFN
c        (C. Escoffier-Czaja)
c
c --- Modifications - V1.52, level 060412 to V1.53, level 060519
c        CALUTILS from v2.51 Level 051019 to v2.52 Level 060519
c        Variable names in control file are not processed correctly
c        if there are too many characters (including blanks) to the
c        left of the "=" sign (run stops in setup phase).
c        Modified:  READIN
c
c --- Modifications - V1.51, level 060406 to V1.52, level 060412
c        Add a check in READCF subroutine so that a computed species
c        can not be the same as any of the input species.
c        (C. Escoffier-Czaja)
c
c --- Modifications - V1.5, level 060329 to V1.51, level 060406
c        update to RDMET subroutine, when NEXTFIL subroutine was called
c        the first entry, which call the unit number for the next met.
c        file was missing. It has been added.
c        (C. Escoffier-Czaja)
c
c --- Modifications - V1.44, level 060310 to V1.5, level 060329
c     1) Updated to CALUTILS V2.51 (051019) from V2.2 (0030528)
c        Filnames changed from c*70 to c*132 (for CALUTILS V2.3
c        and later)
c        Modified:  FILNAM.UTL
c                   READCF, READFN, WRFILES
c     2) Replaced INCRS with subroutine in CALUTILS V2.51 (051019)
c        Add list-file unit to argument list of call
c        Modified:  TIMESET, TIMEUP
c     3) Merge MOD6 components from CALPUFF to update previous
c        implementation
c        New:       DATASET.UTL
c                   FILVERS
c        Replaced:  MET1, RDMET, OUT
c        Modified:  MAIN, METHD.UTL
c                   EXMET, NEXTFIL
c     4) Reset default value for NH3TYP to 3 (use monthly NH3)
c        Map FLGNH3 from previous to NH3TYP:
c            FLGNH3=F            --> NH3TYP=3
c            FLGNH3=T            --> NH3TYP=0
c        Modified:  BLOCK DATA, READCF
c     5) Modification of how the error messages are written (in list-file instead 
c        of in screen) for MNITRATE options. 
c
c --- Modifications - V1.43, level 060206 to V1.44, level 060310
c     1) Remove FLGNH3 introduce in the previous version and replace it
c        with the PARAMETER  for the user to choose where NH3 is coming from:
c        MOnthly background NH3; NH3 from concentration file;
c        NH3 from the background file.
c     3) Add a new parameter as input: total NH3 = TNH3 = NH3gas+NO3+2SO4 (in ppb)
c        to differencie it from NH3 gas.
c     2) Add the possibility of MNITTRATE=3 (ALM application in 1 step)
c        a background pollutants files (including SO4, NO3, HNO3 and TNH3)
c        need then to be read (BCKGALM)
c        Modified (1): READCF, PARTITION, BLOCKDATA, CTRL.UTL
c        Modified (2): FILNAM.UTL, PARAMS.UTL, READFN, WRFILES
c                      new subroutine ALM1STEP
c
c
c --- Modifications - V1.42, Level 050311 to V1.43, level 060206
c
c     1) Add a Flag to determine if NH3 is input as background monthly
c        average NH3 in ppb in the POSTUTIL control input file or
c        if NH3 is input as a species in any of the CALPUFF concentration
c        file - because if NH3 is present in any of the CALPUFF concentration
c        file, NH3 monthly average background will not be used.
c        With the FLAG the user can choose if there is any NH3 in the
c        CALPUFF concentration file or not. and if the FLAG is not in agreement
c        with what is found in the concentration file, the run stop.
c        Flag is called: FLGNH3 and can be False or True.
c                                     (C. Escoffier-Czaja)
c        Modified: READCF, PARTITION
c
c
c --- Modifications - V1.41, Level 050302 to V1.42, Level 050311
c
c     1) Allow different dataset versions of CALPUFF.DAT files to be
c        merged where possible.      (D. Strimaitis)
c        (Initial Version)
c        Modified: GETHEAD, SAVEHEAD, TESTHEAD, ADJHEAD,
c                  SKIPREC, GETRCRD
c
c --- Modifications - V1.4, Level 040818 to V1.41, Level 050302
c
c     1) Implement the possibility to read meteorological data for
c        NO3/HNO3 repartition from three 2D CALPUFF output files
c        or 1D meteorological data file instead of CALMET.DAT files.
c                                    (C. Escoffier-Czaja)
c        Modified: READFN, BLOCKDATA, WRFILES, EXMET,
c                  GETHDRH, PARTITION, MAIN
c        New:      RTRHD.UTL
c                  TESTHEAD3, GETMET_2D, GETMET_1D
c
c --- Modifications - V1.32, Level 030807 to V1.4, Level 040818
c                                    (D. Strimaitis)
c     1) Implement CALPUFF dataset version 2.1 output format. The
c        MSOURCE flag in this format indicates if individual source
c        contributions are in the file in addition to the total from
c        all sources:
c                  MSOURCE = 0 Total from all sources
c                  MSOURCE = 1 Individual source contribution and Total
c        The total from all sources is processed, and the output file
c        does not contain source contributions (MSOURCE set to 0).
c
c        Also ipver6,imver6 flags are replaced with iptime and imtime
c        to distinguish old end-hour structure from newer begin/end
c        time structure (not just "Version 6").
c        Changed METHR.PUF and METHD.PUF to METHR.UTL and METHD.UTL
c
c        Modified: HEAD.UTL, HEAD2.UTL, METHD.UTL
c                  GETHEAD, GETHDRH, SAVEHEAD, TESTHEAD,
c                  WRIHEAD, GETDOC, GETRCRD, SKIPREC, WRIRCRD
c                  MET1, RDMET, RDI1D, RDI2D, RDR1D, RDR2D
c
c     2) Move time period adjustments to ADJHEAD
c
c     3) Move the skipped records computation from QAINP to QASKIP, and
c        allow start-times to differ among data files
c
c     4) Expand single control-file input of background ammonia to
c        monthly values (if single value is specified, it is used for
c        all 12 months)
c
c --- Modifications - V1.31, Level 030528 to V1.32, Level 030807
c                                    (D. Strimaitis)
c     1) RDMET:  read 2-D precip fields with noobs when npsta=-1
c
c --- Modifications - V1.3, Level 030402 to V1.31, Level 030528
c                                    (J. Scire, D. Strimaitis)
c     1) CALUTILS library updated to Version: 2.2, Level: 030528
c
c --- Modifications - V1.1, L010730_1 to V1.3, L030402
c                                    (F. Robe, J. Scire, D. Strimaitis)
c
c     1) Modified to accommodate the changes in the CALMET-CALPUFF
c        output for the NOOBS mode:  2D RH files in VISB.DAT,
c        no header record of nearest station info in VISB.DAT
c                                                (FRR 9/01, 4/02)
c
c        Changes to GETHD, GETRCRD,GETMET,RDMET,EXMET,GETHDRD,WRIHEAD
c        Changes to GETHDRH
c        Changes to common block HEAD.UTL
c        Use modified METHR.PUF and METHD.PUF
c
c     2) Updated 021028:                                    (FRR)
c        - fix in readin 2D-RH array (subroutine GETRCRD)
c        - fix in GETHDRH (i2drhu instead of idrhu)
c
c     3) Fix to read flag type flag (i2drhu) on header record of
c        concentration files (GETHEAD), store the value from the
c        first file read in common to compare with other files 
c        (SAVEHEAD), and test that the file type flags match 
c        in each file processed (TESTHEAD).  Modify common /HEAD0/
c        to include the i2drhu0 variable to store i2drhu from first
c        file read.
c           i2drhu = 0 (original concentration format), 
c                  = 1 (NO-OBS concentration format).       (JSS)
c
c     4) Replace common utility subroutines with the CALUTILS group
c        of subroutines CALUTILS.FOR (Version 2.1, Level 030402)
c        Add list file unit number to INCR, GRDAY, JULDAY   (DGS)
c
c     5) Changes to read/write extended header records
c        All control file documentation from first file retained
c        in header of output file;  only title information from other
c        files retained in output file                      (DGS)
c
c
c --- Modifications - V1.1, L010730 to V1.1, L010730_1 (J. Scire)
c
c      1)  Allow files with different start times, grid cell characteristics
c          (NX, NY, NZ, DELX, domain origin, etc.) to be processed as long
c          as all receptor coordinates match.  Define match of receptor
c          coordinates to be within 1 m in X,Y & 0.1 m in elevation
c          (eliminates roundoff problems with check of real numbers).  
c          Introduce warning messages if differences found, and write
c          header records if warning is triggered.  All changes made in
c          Subr. TESTHEAD.
c
c --- Modifications - V1.1, 001117 to V1.1, L010730 (D. Strimaitis)
c
c      1)  Change flags ipver and metver to ipver6 and imver6,
c          explicitly denoting whether or not formats are 'version 6'.
c            0:  Pre-dates Version 6  [hour ending]
c            1:  Version 6 (or later) [begin & end times]
c      2)  Use full YYYY year for 'old' output file format
c      3)  Multiple 'CALMET.DAT' files may be specified
c
c --- Modifications - V1.1, L000712 to V1.1, L001117 (D. Strimaitis)
c
c      1)  Test for negative scale factors (by species) for each input
c          file, and turn off compression of output file if any such
c          factor is negative (either A or B in x'=Ax+B).
c      2)  Remove Y2K notice from screen
c
c --- Modifications - V1.1, L000707 to V1.1, L000712 (J. Scire)
c
c      1)  Modify TESTHEAD to allow processing of data from different
c          versions of CALPUFF
c      2)  Expand information provided in error report
c
c --- Modifications - V1.1, L000315 to V1.1, L000707 (D. Strimaitis)
c
c      1) Modify test for CALMET level at which new header record
c         was added (format changed)
c
c --- Modifications - V1.1, L000112 to V1.1, L000315 (D. Strimaitis)
c
c      1)  Allow scaling factors (by species) for each input file
c
c --- Modifications - V1.1, L000106 to V1.1, L000112 (D. Strimaitis)
c
c      1)  Extend HNO3/NO3 partition option to source groups;  this
c          is enabled by using MNITRATE=2
c      2)  Fix assignment of species names needed for nitrate processing
c          so that input and output species lists need not match
c
c --- Modifications - V1.0, L990806a to V1.1, L000106 (D. Strimaitis)
c
c      1)  Introduce HNO3/NO3 partition option
c      2)  Fix format of error report in TESTHEAD
c
c --- Modifications - V1.0, L990806 to V1.0, L990806a (D. Strimaitis)
c
c     a1)  Introduce TOTAL FLUX 'TF' (wet & dry) designation to species-
c          level name;  requires compatible version of CALPOST
c
c
c  ARGUMENTS:   none
c
c  CALLING ROUTINES:    none
c
c  EXTERNAL ROUTINES:   READCF, GETHEAD, SAVEHEAD, TESTHEAD,
c                       QAINP, QASKIP, GETRCRD, SKIPREC, COMLINE,
c                       ADJHEAD, WRIHEAD, WRIRCRD, WRFILES,
c                       TPERIOD, TIMESET, TIMEUP, MET1, GETMET,
c                       PARTITION, NO3RATIO, TFERCF, FILVERS,
c                       DATETM, FIN
c-----------------------------------------------------------------------
      INCLUDE 'params.utl'
      INCLUDE 'conc.utl'
      INCLUDE 'ctrl.utl'
      INCLUDE 'dataset.utl'
      INCLUDE 'datehr.utl'
      INCLUDE 'filnam.utl'
      INCLUDE 'head.utl'
      INCLUDE 'qa.utl'
      INCLUDE 'wrkspc.utl'
      character*12 version,level

      logical ldb,lprt

c  Arrays used in GETRCRD
      real tcg(mxgx,mxgy),tcd(mxdrec),tct(mxctrec)

c  Set debug output logical
      data ldb/.FALSE./
c      data ldb/.TRUE./
c  Set logical to write header from 2D MET field.
       data lprt/.TRUE./

c  Set initial values for 'last' date-times for met data file
      data ndhrmetb/-000100100/,ndhrmete/-000100100/
      data nsecmetb/0/,nsecmete/0/
c
c  Identify version and level of POSTUTIL:
      version='1.641'
      level='110225'
c
c  Check that the version and level number in the parameter
c  file matches those in the code itself
      if(version.ne.mver.or.level.ne.mlevel)then
         write(*,2) version,level,mver,mlevel
2        format(/1x,'ERROR  -- The version ',
     1   'and level numbers do not match those in the parameter file'/
     2   5x,' POSTUTIL Code - Version: ',a12,' Level: ',a12/
     3   5x,'Parameter File - Version: ',a12,' Level: ',a12)
         stop
      endif

c  Make intiial call for system time at the start
      call DATETM(rdate,rtime,rcpu)
c
c  Get the name of the control file from the command line
      call COMLINE(utlinp)
c
c  Open the control file
      open(in2,file=utlinp,status='old')
c
c  Obtain program control information for run (/ctrl/)
      call READCF(in2,io1,version,level)

c  Transfer control file image to scratch file
      call TFERCF
c
c --- If this is an ALM application in 1 only step - MNITRATE=3, a background
c     pollutant file need to be read.
      if (MNITRATE.EQ.3) then
      nff=nfiles+1
        OPEN(in8,FILE=bckgalm,STATUS='old',FORM='unformatted')
        call GETHEAD(in8,nff)
        call SAVEHEAD
        call QASKIP(nff)
      end if
c
c  Get specifications for modeled data (/head/)
      do nf=1,nfiles
         open(inunit(nf),file=moddatl(nf),status='old',
     &        form='unformatted')
         call GETHEAD(inunit(nf),nf)
         if(nf.EQ.1) then
c ---       Place information in /head0/ for testing
            if (MNITRATE.EQ.3) then
            WRITE(*,*)'The BCKGALM Background file is compared to'
            WRITE(*,*)'the first CALPUFF.DAT concentration file to',
     1                ' make sure same receptors are used'
            call TESTHEAD       ! compare to backgroud file
            endif
         call SAVEHEAD
         else
c ---       Compare with information in /head0/
            call TESTHEAD
         endif
c ---    Calculate periods to skip to get to first record for processing
c ---    (result stored in /CTRL/)
         call QASKIP(nf)
      enddo
c
c  If this is a visibility calculation, open the file with relative
c  humidity data and read the header (LVISIB set FALSE for now!)
      if (LVISIB) then
         open(in3,file=visdat,status='old',form='unformatted')
         call GETHDRH(in3,lprt)
c ---    Compare with information in /head0/
         call TESTHEAD3(in3)
      endif
c
c  Open unformatted output file
      open(io2,file=utldat,status='unknown',form='unformatted')
c
c  Check consistency of information & echo values
      call QAINP(io1)
c
c  Adjust header data for output data file (matches control file input)
      call ADJHEAD
c
c  Write the files used in this run to the list file
      call WRFILES
c
c  Set dimension of data arrays for declarations in SKIPREC, GETRCRD
      idimx=MAX(1,ngx)
      idimy=MAX(1,ngy)
      idimd=MAX(1,ndrec)
      idimt=MAX(1,nctrec)
c
c  Set the date-time for the first and last period processed
      call TIMESET
c
c  Initialize end of file flag for reading data for each period
      ieof=0
c
c  If this run involves computing the HNO3/NO3 concentration partition,
c  obtain the header records of the met data file
c --- First file in list
      k=1
c - (CEC)- 060306 - MET data need to be opened also if MNITRATE=3 (ALM in 1 step)
      if(mnitrate.EQ.1.or.mnitrate.eq.3) then
       if (metfm.eq.0) then
         imetdat=k
         call FILVERS(in4,'BIN',utlmetl(k),'CALMET          ',vercmet)
         open(in4,file=utlmetl(k),status='old',form='unformatted')
         call MET1(ielmet,jelmet,ldb,tmp1,nw1)
c
       elseif(metfm.eq.2) then
c
         imetrhu=k
         call FILVERS(in5,'BIN',m2drhul(k),'RELHUM2D        ',verrhum)
         open(in5,file=m2drhul(k),status='old',form='unformatted')
         REWIND(in5)
         call GETHDRH(in5,lprt)
c ---    Compare with information in /head0/
         call TESTHEAD3(in5)
c
         imettmp=k
         call FILVERS(in6,'BIN',m2dtmpl(k),'TEMPK2D         ',vertemp)
         open(in6,FILE=m2dtmpl(k),STATUS='old',FORM='unformatted')
         REWIND(in6)
         call GETHDRH(in6,lprt)
c ---    Compare with information in /head0/
         call TESTHEAD3(in6)
c
         imetrho=k
         call FILVERS(in7,'BIN',m2drhol(k),'RHOAIR2D        ',verdens)
         open(in7,FILE=m2drhol(k),STATUS='old',FORM='unformatted')
         REWIND(in7)
         call GETHDRH(in7,lprt)
c ---    Compare with information in /head0/
         call TESTHEAD3(in7)
c
       elseif(metfm.eq.1) then
         OPEN(in5,FILE=met1d,status='old',form='unformatted')
         REWIND(in5)
         call getdoc1d(in5)
c
       else
       WRITE(io1,*)'error - metfm can only be 0, 1 or 2'
       WRITE(io1,*)'0 - if calmet.dat (MET_3D) files are used'
       WRITE(io1,*)'1 - if 1 MET_1D of Rh, Temp and Rho is used'
       WRITE(io1,*)'2 - if 3 MET_2D output of Rh, T and Rho are used'       
       stop
       endif
      endif
c
c  Skip to starting hour for processing data (/conc/)
      write(io1,*)
      write(io1,*)'Skipping periods in data files'
      write(io1,*)' Start Time ',isyr,jsday,ishr,issec
      write(io1,*)' Data File  ---  Skipped Periods'
      do nf=1,nfiles
         write(io1,*)'     ',nf,'          ',nskip(nf)
         if(nskip(nf).GT.0) then
           do i=1,nskip(nf)
            call SKIPREC(inunit(nf),nf,tcg,tcd,tct,idimx,idimy,idimd,
     &                   idimt,ieof)
            if(ieof.EQ.1) then
               write(io1,*)'          --- <<!!!>> ---                 '
               write(io1,*)'------------------------------------------'
               write(io1,*)'End Of File found when skipping period ',i
               write(io1,*)'Data File  ',nf
               write(io1,*)'------------------------------------------'
               write(io6,*)'          --- <<!!!>> ---                 '
               write(io6,*)'------------------------------------------'
               write(io6,*)'End Of File found when skipping period ',i
               write(io6,*)'Data File  ',nf
               write(io6,*)'------------------------------------------'
               stop
            endif
           enddo
         endif
      enddo
      write(io1,*)
c
c --- (CEC) - 060306
c  Skip to starting hour for processing background pollutant file
      write(io1,*)
      write(io1,*)'Skipping periods in background pollutant files'
      write(io1,*)' Start Time ',isyr,jsday,ishr,issec
      write(io1,*)' Data File  ---  Skipped Periods'
         write(io1,*)'     ',nfiles+1,'          ',nskip(nfiles+1)
         if(nskip(nfiles+1).GT.0) then
           do i=1,nskip(nfiles+1)
            call SKIPREC(in8,nfiles+1,tcg,tcd,tct,idimx,idimy,idimd,
     &                   idimt,ieof)
            if(ieof.EQ.1) then
               write(io1,*)'          --- <<!!!>> ---                 '
               write(io1,*)'------------------------------------------'
               write(io1,*)'End Of file found when skipping period ',i
               write(io1,*)'Background file BCKGALM ',nfiles+1
               write(io1,*)'------------------------------------------'
               write(io6,*)'          --- <<!!!>> ---                 '
               write(io6,*)'------------------------------------------'
               write(io6,*)'End Of File found when skipping period ',i
               write(io6,*)'Background file BCKGALM  ',nfiles+1
               write(io6,*)'------------------------------------------'
               stop
            endif
           enddo
         endif
c
c  Write header for output data file
      call WRIHEAD(io2,version,level)
c
c  Initialize period counter (iper)
      iper=0
c
c  Set maximum number of periods to process
      mxper=nper
      write(io6,*) 'Processing ',mxper,' periods'
      write(io6,*)
c
c  Loop over periods --------------------------
10    iper=iper+1
c
      if(mnitrate.EQ.1.or.mnitrate.EQ.3) then
c ---    Test date-time for period and update met data if needed
         call TPERIOD(1,ndathrm,nsecm,ndhrmetb,nsecmetb,
     &                ndhrmete,nsecmete,ilocate)

c ---    GETMET is for CALMET.DAT format, GETMET_2D is for CALPUFF
c        output met. data.
         if(ilocate.EQ.1) then
           if (metfm.eq.0) then
           call GETMET(ndathrm,nsecm,ndhrmetb,nsecmetb,
     &                                ndhrmete,nsecmete)
           ELSEIF (metfm.eq.2) then
           call GETMET_2D(in5,in6,in7,ndathrm,nsecm,
     &      ndhrmetb,nsecmetb,ndhrmete,nsecmete)
           ELSEIF (metfm.eq.1) then
           call GETMET_1D(in5,ndathrm,nsecm,ndhrmetb,nsecmetb,
     &      ndhrmete,nsecmete)
           endif
         endif
      endif
c
c --- Read modeled data and construct 'new' arrays
      call GETRCRD(in3,tcg,tcd,tct,idimx,idimy,idimd,idimt,ieof)
      if(ieof.EQ.1) then
         mxper=iper-1
         goto 11
      endif
c
c --- Apply chemical equilibrium to partition Nitrates at each receptor
      if(mnitrate.EQ.1) call PARTITION(ldb)
c
c --- Use partition of Nitrates (all sources) to repartition Nitrates
c --- for a particular source group at each receptor
      if(mnitrate.EQ.2) call NO3RATIO(ldb)
c
c --- Performe ALM (Ammonia Limitimg Method) in 1 only step
      IF(mnitrate.EQ.3) call ALM1STEP(tcg,tcd,tct,idimx,idimy,idimd,
     &  idimt,ldb)
c
c --- Write modeled data
      call WRIRCRD
c
c --- Next period
      if(iper.LT.mxper) then
         call TIMEUP(1)
         goto 10
      endif
11    continue
      write(io6,*) '  Finished ',mxper,' of ',nper,' periods'
      write(io6,*)

      if(ieof.EQ.1) then
         write(io6,*)
         write(io6,*)'Reached end of CALPUFF.DAT file before expected'
         write(io6,*)'CALPUFF.DAT file number = ',nf
         write(io6,*)'  Last period requested = ',nper
         write(io6,*)'  Last period processed = ',mxper
         write(io6,*)
         write(io6,*)'Adjust period used in the CALPOST control file'

         write(io1,*)
         write(io1,*)'Reached end of CALPUFF.DAT file before expected'
         write(io1,*)'CALPUFF.DAT file number = ',nf
         write(io1,*)'  Last period requested = ',nper
         write(io1,*)'  Last period processed = ',mxper
         write(io1,*)
         write(io1,*)'Adjust period used in the CALPOST control file'
      endif

c --- Termination routine
      call FIN

      stop
      end
c-----------------------------------------------------------------------
      block data
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641        Level: 100917          BLOCK DATA
c ---           D. Strimaitis
c
c     PURPOSE:     Provides initial values or default values to members
c                  of common blocks used in POSTUTIL code.
c
c
c-----------------------------------------------------------------------
      INCLUDE 'params.utl'

      parameter(mxspec2=mxspec*mxspec)
      parameter(mxspfile=mxspec*mxfile)

      INCLUDE 'ctrl.utl'
      INCLUDE 'filnam.utl'
      include 'wrkspc.utl'
c  Bring in selected CALPUFF commons with no changes (used in MET1
c  and RDMET subroutines)
      include 'methd.utl'
      include 'methr.utl'
c
c --- Section for members of common /ctrl/:
      data issec/0/
      data nfiles/1/,mduplct/0/,nmet/0/,metfm/0/
c --- Initialize all weights to zero
      data specwt/mxspec2*0.0/
      data ispecwt/mxspec*0/
c --- Set default scaling factors
      data ascale/mxspfile*1.0/
      data bscale/mxspfile*0.0/
c --- Set visibility processing logical to FALSE (future implementation)
      data lvisib/.FALSE./
c --- Set Nitrate species pointers to zero
      data ispecno3/0,0,0,0,0,0,0/
c --- Set initial NH3 type (choice of the source of NH3)
      DATA NH3TYP/3/
c --- Set default ammonia array  (bcknh3 for NH3gas and bcktnh3 for total NH3)
      data bcknh3/-999., -999.,-999.,-999.,-999.,-999.,
     &            -999.,-999.,-999.,-999.,-999.,-999./
      data bcktnh3/-999., -999.,-999.,-999.,-999.,-999.,
     &            -999.,-999.,-999.,-999.,-999.,-999./
c --- Set HNO3/NO3 phase partition model
      data meqphase/1/
      data ilayer/0/
c
c
c --- FILNAM, FILLOG common blocks
      data utlinp/'postutil.inp'/,utllst/'postutil.lst'/,
     &     moddatl(1)/'calpuff.dat'/,utldat/'model.dat'/,
     &     visdat/'calpuff.vis'/,utlmetl(1)/'met.dat'/,
     &     m2drhul(1)/'visb.dat'/,m2dtmpl(1)/'tk2d.dat'/,
     &     m2drhol(1)/'rho2d.data'/,met1d/'met_1d.dat'/,
     &     bckgalm/'bckgalm.dat'/
      data lcfiles/.true./
c
c --- METHD common block
c --- Nulls for using "old" CALMET met data file
      data llconfm/.FALSE./
      data xlat0m/-999./,xlon0m/-999./
      data rlat0m/-999./,rlon0m/-999./
      data xlat1m/-999./,xlat2m/-999./
c --- Defaults for using single-point met data files
      data anemht/10./,isigmav/1/,imixctdm/0/
      data ilanduin/20/,z0in/.25/,xlaiin/3.0/
      data elevin/0.0/,xlatin/-999./,xlonin/-999./
      data nss/0/,xlatss/mxss*-999./,xlonss/mxss*-999./
c
c --- METHR common block
c --- ISC defaults for potential temperature gradient (KST=5,6)
      data ptg/0.020,0.035/
c --- Initialize observed inversion strength to missing (-999.)
      data dptinvo/-999./
c
c --- WRKSPC common block
      data nw1/mxnxy/
c
      end
c
c-----------------------------------------------------------------------
      subroutine caps(aspec)
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 990806           CAPS
c ---           J. Scire, SRC
c ---           D. Strimaitis, SRC
c
c  PURPOSE:     Converts lower-case characters within species name to
c               upper-case.
c
c  ARGUMENTS:   none
c
c  CALLING ROUTINES:    GETCTRL
c
c  EXTERNAL ROUTINES:   none
c-----------------------------------------------------------------------
      INCLUDE 'params.utl'
c
      character*12 aspec
      character*1 cchar,clc(29),cuc(29)
c
      data clc/'i','n','x','a','e','o','u','b','c','d','f','g','h',
     1 'j','k','l','m','p','q','r','s','t','v','w','y','z','-','.',
     2 '*'/
      data cuc/'I','N','X','A','E','O','U','B','C','D','F','G','H',
     1 'J','K','L','M','P','Q','R','S','T','V','W','Y','Z','-','.',
     2 '*'/
c
      do 100 i=1,12
      cchar=aspec(i:i)
c
      do 50 j=1,29
      if(cchar.eq.clc(j))then
         aspec(i:i)=cuc(j)
         go to 52
      endif
50    continue
52    continue
100   continue
c
      return
      end
c-----------------------------------------------------------------------
      subroutine gethead(io,nf)
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 071015           GETHEAD
c ---           D. Strimaitis
c
c  PURPOSE:     Reads header information from concentration file
c               and passes information to main program through
c               commons /head/ and /head2/.
c
c  UPDATES:
c     V1.42(050311) to V1.6(071015)
c               (DGS) Add dataset version 2.2 format
c               (DGS) Use MXSPLV_IN for input arrays
c     V1.4(040818) to V1.42(050311)
c               (DGS) Allow mix of Dataset 2.* formats
c     V1.3(030402) to V1.4(040818)
c               (DGS) Add dataset version 2.1 format
c     V1.2(030210) to V1.3(030402)
c               (DGS) Dataset version 2.0 format
c               (DGS) add list-file unit to INCR, YR4
c     V1.1(010730)  to V1.2(030210)
c               (JSS) read file type flag (i2drhu)
c                     i2drhu = 0 (original file type)
c                            = 1 (NO-OBS file type)
c     V1.1(990806)  to V1.1(010730)
c               (DGS) output format for year is always YYYY
c
c  ARGUMENTS:
c     PASSED:   io      Unit number for accessing input file         [i]
c               nf      Number of the input file                     [i]
c
c  CALLING ROUTINES:    POSTUTIL
c
c  EXTERNAL ROUTINES:   INCR, YR4, GETDOC, UTCBASR
c-----------------------------------------------------------------------
      INCLUDE 'params.utl'
      INCLUDE 'ctrl.utl'
      INCLUDE 'head.utl'
      INCLUDE 'head2.utl'
c -- Note: all declaration statements for data read in by GETHEAD
c          are contained in these commons

c --- Variables added for DATASET VERSION 2.1
c     NSTYPE(mxfile,8) holds the number of each type of source
c                     in each file
c               1 - point sources (Control File)
c               2 - point sources (Var. Emissions File)
c               3 - area sources (Control File)
c               4 - area sources (Var. Emissions File)
c               5 - line sources (Control File)
c               6 - line sources (Var. Emissions File)
c               7 - volume sources (Control File)
c               8 - volume sources (Var. Emissions File)
      integer nstype(mxfile,8)
      character*4 cstype(8)

      character*48 msg(8),msg0
      logical lfatal,ldbg
      logical lspunit

      data msg0  /'  **********************************************'/
      data msg(1)/'  *      FATAL Problem Found in GETHEAD!       *'/
      data msg(2)/'  * ------------------------------------------ *'/
      data msg(3)/'  *  Size of Array(s) in CALPUFF.DAT too Large *'/
      data msg(4)/'  *  Re-size Array Parameter(s) in PARAMS.UTL  *'/
      data msg(5)/'  *         and Recompile POSTUTIL             *'/
      data msg(6)/'  * ------------------------------------------ *'/
      data msg(7)/'  * Parameter       PARAMS.UTL     CALPUFF.DAT *'/
      data msg(8)/'  * ---------       ----------     ----------- *'/


c --- Set names of each source type for error report section
      data cstype/'pt1 ','pt2 ','ar1 ','ar2 ',
     &            'ln1 ','ln2 ','vl1 ','vl2 '/

      ldbg=.FALSE.
      lfatal=.FALSE.
      lspunit=.FALSE.

c --- Process first section of CALPUFF output file (including comment
c --- records), and determine dataset version
      call GETDOC(io,nf,idtype,cpverx(nf))
c
c  Read control variables (record NCOM+3)

      if(idtype.EQ.0) then
c ---   Dataset EARLIER than Version 2.0
        i2drhu=0
        read(io,iostat=ierr) amodel,aver,alev,msyr,mjsday,mshr,
     *         mssec,mnrun,mavgpd,nsecdt,
     *         ielmet,jelmet,delx,dely,nz,xorigkm,yorigkm,nstas,
     *         iastar,iastop,jastar,jastop,
     *         isastr,jsastr,isastp,jsastp,
     *         meshdn,kpts,kareas,klines,kvols,
     *         ndrec,nctrec,LSGRID,nszout(nf),lcomprs(nf)
        if(ierr.EQ.0) then
          iptimex(nf)=2
          isrcinfox(nf)=0
          isrcindv=0

c --------------------------------------------------
cc ---     Record read OK, but check model version
cc ---     Convert to a real
c          read(aver,'(f12.0)') puffver
c          if(puffver.GT.5.9999) then
cc ---       Set flag indicating CALPUFF data version is 6 or later
c            ipver6=1
c            call YR4(io1,msyr,ierry)
c            if(ierry.NE.0) stop 'Halted in GETHEAD'
c          else
c            write(*,*)'Inconsistent header of CALPUFF file'
c            write(*,*)'Too many variables for Version ',aver
c            stop
c          endif
c --------------------------------------------------

        else
c ---     May have hit end of header record; try again with old format
          rewind(io)
          read(io) amodel,aver,alev,msyr,mjsday,mshr,mnhrs,mavg,
     *         ielmet,jelmet,delx,dely,nz,xorigkm,yorigkm,nstas,
     *         iastar,iastop,jastar,jastop,
     *         isastr,jsastr,isastp,jsastp,
     *         meshdn,kpts,kareas,klines,kvols,
     *         ndrec,nctrec,LSGRID,nszout(nf),lcomprs(nf),i2drhu
c ****         ndrec,nctrec,LSGRID,nszout(nf),lcomprs(nf)
          iptimex(nf)=1
          isrcinfox(nf)=0
          isrcindv=0
        endif

c --------------------------------------------------
cc ---     Record read OK, but check model version
cc ---     Convert to a real
c          read(aver,'(f12.0)') puffver
c          if(puffver.LT.6.0) then
cc ---       Set flag indicating CALPUFF data version pre-dates 6
c            ipver6=0
c            call YR4(io1,msyr,ierry)
c            if(ierry.NE.0) stop 'Halted in GETHEAD'
c          else
c            write(*,*)'Inconsistent header of CALPUFF file'
c            write(*,*)'Too few variables for Version ',aver
c            stop
c          endif
c --------------------------------------------------

      else

c ---   Dataset version 2.0 or later format
        if(cpverx(nf).EQ.'2.0             ') then
          read(io) amodel,aver,alev,msyr,mjsday,mshr,mnhrs,mavg,
     *         ielmet,jelmet,delx,dely,nz,xorigkm,yorigkm,nstas,
     *         iastar,iastop,jastar,jastop,
     *         isastr,jsastr,isastp,jsastp,
     *         meshdn,kpts,kareas,klines,kvols,
     *         ndrec,nctrec,LSGRID,nszout(nf),lcomprs(nf),i2drhu,
     *         iutmzn,feast,fnorth,rnlat0,relon0,
     *         xlat1,xlat2,pmap,utmhem,datum,daten,
     *         clat0,clon0,clat1,clat2
          iptimex(nf)=1
          isrcinfox(nf)=0
          isrcindv=0

        elseif(cpverx(nf).EQ.'2.1             ') then
          read(io) amodel,aver,alev,msyr,mjsday,mshr,mssec,xbtz,
     *         mnrun,mavgpd,nsecdt,
     *         ielmet,jelmet,delx,dely,nz,xorigkm,yorigkm,nstas,
     *         iastar,iastop,jastar,jastop,
     *         isastr,jsastr,isastp,jsastp,
     *         meshdn,nstype(nf,1),nstype(nf,2),nstype(nf,3),
     *         nstype(nf,4),nstype(nf,5),nstype(nf,6),nstype(nf,7),
     *         nstype(nf,8),msource,
     *         ndrec,nctrec,LSGRID,nszout(nf),lcomprs(nf),i2drhu,
     *         iutmzn,feast,fnorth,rnlat0,relon0,xlat1,xlat2,
     *         pmap,utmhem,datum,daten,clat0,clon0,clat1,clat2
          iptimex(nf)=2
          isrcinfox(nf)=1
          isrcindv=msource
          kpts=nstype(nf,1)+nstype(nf,2)
          kareas=nstype(nf,3)+nstype(nf,4)
          klines=nstype(nf,5)+nstype(nf,6)
          kvols=nstype(nf,7)+nstype(nf,8)

        elseif(cpverx(nf).EQ.'2.2             ') then
          read(io) amodel,aver,alev,msyr,mjsday,mshr,mssec,abtz,
     *         mnrun,mavgpd,nsecdt,
     *         ielmet,jelmet,delx,dely,nz,xorigkm,yorigkm,nstas,
     *         iastar,iastop,jastar,jastop,
     *         isastr,jsastr,isastp,jsastp,
     *         meshdn,nstype(nf,1),nstype(nf,2),nstype(nf,3),
     *         nstype(nf,4),nstype(nf,5),nstype(nf,6),nstype(nf,7),
     *         nstype(nf,8),msource,
     *         ndrec,nctrec,LSGRID,nszout(nf),lcomprs(nf),i2drhu,
     *         iutmzn,feast,fnorth,rnlat0,relon0,xlat1,xlat2,
     *         pmap,utmhem,datum,daten,clat0,clon0,clat1,clat2
          iptimex(nf)=2
          isrcinfox(nf)=1
          isrcindv=msource
          kpts=nstype(nf,1)+nstype(nf,2)
          kareas=nstype(nf,3)+nstype(nf,4)
          klines=nstype(nf,5)+nstype(nf,6)
          kvols=nstype(nf,7)+nstype(nf,8)
c ---     Convert from UTC string to real base time zone
          call UTCBASR(abtz,xbtz)
c ---     Specie-level units are in header
          lspunit=.TRUE.

        else
          write(io1,*)'ERROR:  Unknown DATASET Version: ',cpverx(nf)
          stop 'Halted in GETHEAD'
        endif

      endif

c --- Update of number of sources for output header
      if(isrcinfox(nf).EQ.0) then
c ---    DO NOT UPDATE SOURCE NUMBERS
cc ---    No individual source information is available, so just keep
cc ---    the maximum number of each type (cannot test for duplicates)
c         npts=MAX(npts,kpts)
c         nareas=MAX(nareas,kareas)
c         nlines=MAX(nlines,klines)
c         nvols=MAX(nvols,kvols)
      elseif(isrcinfox(nf).EQ.1) then
c ---    Retain all source names even though some may be duplicates
c ---    (2 files may hold different species for the same sources)
         npts=npts+kpts
         nareas=nareas+kareas
         nlines=nlines+klines
         nvols=nvols+kvols
      endif

c --- Check year format
      call YR4(io1,msyr,ierry)
      if(ierry.NE.0) stop 'Halted in GETHEAD'

      if(iptimex(nf).EQ.1) then
c ---    Move starting time back 1 hour to identify the start of the
c ---    first hour
         call INCR(io1,msyr,mjsday,mshr,-1)
c ---    Set period step to 1 hour
         nsecdt=3600
         mssec=0
c ---    Rename averaging period variables
         mnrun=mnhrs
         mavgpd=mavg
      endif

c --- Slip selected information about current file into output
c --- variables for QA checks
      isrcinfo=isrcinfox(nf)
      cpver=cpverx(nf)

c
c  Check size of array dimensions (compute gridded rec. array dims.)
c  with limits contained in PARAMS.utl file
      ngx=(isastp-isastr)*meshdn+1
      ngy=(jsastp-jsastr)*meshdn+1
      ngrec=ngx*ngy
      if(ngx .GT. mxgx) lfatal=.TRUE.
      if(ngy .GT. mxgy) lfatal=.TRUE.
      if(ndrec .GT. mxdrec) lfatal=.TRUE.
      if(nctrec .GT. mxctrec) lfatal=.TRUE.
      if(nszout(nf) .GT. mxsplv_in) lfatal=.TRUE.
      nsrc=0
      if(isrcinfox(nf).EQ.1) then
         do k=1,8
            nsrc=MAX(nsrc,nstot(k)+nstype(nf,k))
         enddo
      endif
      if(nsrc .GT. mxsrc) lfatal=.TRUE.

c  Set indicator for full sampling grid
      ifull=0
      if(ngrec .EQ. mxgrec) ifull=1

      if(LFATAL) then
c ---    Report problem information and quit
c ---    Screen (io6) and List File (io1):
c ---    Set for screen first
         iu=io6
         do k=1,2
           write(iu,*)
           write(iu,100) msg0
           do i=1,8
              write(iu,100) msg(i)
           enddo
           write(iu,101)'  *     MXGX',mxgx,ngx
           write(iu,101)'  *     MXGY',mxgy,ngy
           write(iu,101)'  *   MXDREC',mxdrec,ndrec
           write(iu,101)'  *  MXCTREC',mxctrec,nctrec
           write(iu,101)'  *MXSPLV_IN',mxsplv_in,nszout(nf)
           write(iu,101)'  *    MXSRC',mxsrc,nsrc
           write(iu,100) msg0
c ---      Set for list file unit
           iu=io1
         enddo

         stop
      endif
c
c  Read title of run (record NCOM+4)
      read(io) atitle
c
c  Read specie/level list (record NCOM+5)
      read(io) (asplst(i,nf),i=1,nszout(nf))

c  Read units for each specie/level in list (record NCOM+5a)
c  (introduced in Dataset 2.2)
      if(LSPUNIT) then
         read(io) (aspunit(i,nf),i=1,nszout(nf))
      else
         do i=1,nszout(nf)
            if(asplst(i,nf)(15:15).EQ.'F') then
               aspunit(i,nf)='g/m2/s          '
            else
               aspunit(i,nf)='g/m3            '
            endif
         enddo
      endif
c
c  Read coordinates of non-gridded receptors if any are used
c  (record NCOM+6)
      if(ndrec .NE. 0) read(io) (xrec(i),i=1,ndrec),
     *                          (yrec(i),i=1,ndrec),
     *                          (zrec(i),i=1,ndrec)
c
c  Read coordinates of complex terrain receptors if any are used
c  (record NCOM+7)
      if(nctrec .NE. 0) read(io) (xctr(i),i=1,nctrec),
     *                           (yctr(i),i=1,nctrec),
     *                           (zctr(i),i=1,nctrec),
     *                           (ihill(i),i=1,nctrec)
c

c  Set logicals for receptor types in file
      ldisc=.FALSE.
      lctsg=.FALSE.
      if(ndrec .GT. 0) ldisc=.TRUE.
      if(nctrec .GT. 0) lctsg=.TRUE.

c --- Dataset version 2.1 introduces source names
c --- HEADER RECORDs #NCOM+8 to 15 -- Source names
      if(isrcinfox(nf).EQ.1) then
         lfatal=.FALSE.
c ---    Loop over source types (points,areas,lines,volumes)
         do k=1,8
            if(nstype(nf,k).GT.0 .AND. .NOT.LFATAL) then
               n1=nstot(k)+1
               nstot(k)=nstot(k)+nstype(nf,k)
               read(io) ktype,(csource(n,k),n=n1,nstot(k))
               if(ktype.NE.k) lfatal=.TRUE.
            endif
         enddo
         if(LFATAL) then
c ---       Report problem information and quit
            write(*,*)
            write(*,*)
            write(*,100)'  *  Unexpected source type found, or     *'
            write(*,100)'  *  Unexpected source number found.      *'
            write(*,100)'  *  Number of sources expected:          *'
            write(*,101)'  *     NPTS',nstot(1),nstot(2)
            write(*,101)'  *   NAREAS',nstot(3),nstot(4)
            write(*,101)'  *   NLINES',nstot(5),nstot(6)
            write(*,101)'  *    NVOLS',nstot(7),nstot(8)
            write(*,100)'  *  Source Names Read:                   *'
            do k=1,8
               do n=1,nstot(k)
                  write(*,*)cstype(k),csource(n,k)
               enddo
            enddo
            stop
         endif
      endif
c
c  Compute number of averaging periods in file
      mnper=mnrun/mavgpd
c
c  Determine length of averaging period found in model file
      if(MOD(nsecdt,3600).EQ.0) then
c ---    Hours
         mavg=mavgpd*(nsecdt/3600)
         avtime='  HOUR'
      elseif(MOD(nsecdt,60).EQ.0) then
c ---    Minutes
         mavg=mavgpd*(nsecdt/60)
         avtime='MINUTE'
      else
c ---    Seconds
         mavg=mavgpd*nsecdt
         avtime='SECOND'
      endif
c
c  Identify model output file used in application
      write(io1,*)
      write(io1,*)
      write(io1,*)
      write(io1,*) 'PROCESSED MODEL FILE ---------- Number ',nf
      write(io1,*)
      write(io1,*) amodel,aver,alev
      write(io1,*)
      write(io1,*) atitle(1)
      write(io1,*) atitle(2)
      write(io1,*) atitle(3)
      write(io1,*)
      write(io1,*) 'Averaging time for values reported from model:'
      write(io1,*) '     ',mavg,' ',avtime
      write(io1,*)
      write(io1,*) 'Number of averaging periods in file from model:'
      write(io1,*) '     ',mnper
      write(io1,*)
      write(io1,*) 'Chemical species for each layer in model:'
      do i=1,nszout(nf)
         write(io1,*) asplst(i,nf),'  ',aspunit(i,nf)
      enddo
      write(io1,*)
      write(io1,*) ' msyr,mjsday      =',msyr,mjsday
      write(io1,*) ' mshr,mssec       =',mshr,mssec
      write(io1,*) ' nsecdt (period)  =',nsecdt
      write(io1,*) ' mnper,nszout,mavgpd =',mnper,nszout(nf),mavgpd
      write(io1,*) ' xorigkm,yorigkm,nstas = ',xorigkm,yorigkm,nstas
      write(io1,*) ' ielmet,jelmet =',ielmet,jelmet
      write(io1,*) ' delx,dely,nz =',delx,dely,nz
      write(io1,*) ' iastar,iastop,jastar,jastop =',iastar,iastop,
     *                                             jastar,jastop
      write(io1,*) ' isastr,isastp,jsastr,jsastp =',isastr,isastp,
     *                                             jsastr,jsastp
      write(io1,*) ' (computed) ngx,ngy  =',ngx,ngy
      write(io1,*) ' meshdn,npts,nareas  =',meshdn,npts,nareas
      write(io1,*) ' nlines,nvols        =',nlines,nvols
      write(io1,*) ' ndrec,nctrec,LSGRID =',ndrec,nctrec,LSGRID
      if(ndrec.GT.0 .AND. LDBG) then
         write(io1,*) ' '
         write(io1,*) 'Discrete Receptors (n,x,y,z):'
         do i=1,ndrec
            write(io1,*) i,xrec(i),yrec(i),zrec(i)
         enddo
      endif
      if(nctrec.GT.0 .AND. LDBG) then
         write(io1,*) ' '
         write(io1,*) 'Complex Terrain Receptors (n,x,y,z,hill):'
         do i=1,nctrec
            write(io1,*) i,xctr(i),yctr(i),zctr(i),ihill(i)
         enddo
      endif
      if(LVISIB .AND. LDBG) then
         write(io1,*) ' '
         write(io1,*) 'Surface Met Station UTMs (n,x,y):'
         do i=1,nstas
            write(io1,*) i,xkmsta(i),ykmsta(i)
         enddo
      endif
      if(isrcinfo.EQ.1) then
         write(io1,*) ' '
         write(io1,*) 'Source names stored (all files):'
         do k=1,8
            do n=1,nstot(k)
               write(io1,*)'type: ',cstype(k),'-  ',csource(n,k)
            enddo
         enddo
      endif
c
c  Set off data section from rest of output
      write(io1,*)
      write(io1,*)
      write(io1,*)


100   format(1x,a48)
101   format(1x,a12,2(8x,i8),'   *')

      return
      end
c-----------------------------------------------------------------------
      subroutine savehead
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 050311          SAVEHEAD
c ---           D. Strimaitis, SRC
c
c  PURPOSE:     Saves header information from first input data file
c               to common /head0/ variables to check against data in
c               subsequent input data files.
c
c  UPDATES:
c     V1.4(040818) to V1.42(050311)
c               (DGS) Allow mix of Dataset 2.* formats
c     V1.2(030210) to V1.4(040818)
c               (DGS) add CALPUFF file Version 2.1 checks
c     V1.1(990806) to V1.2(030210)
c               (JSS) add the file type variable (i2drhu) to the
c                     list of saved variables
c
c  ARGUMENTS:
c               none
c
c  CALLING ROUTINES:    POSTUTIL
c
c  EXTERNAL ROUTINES:   none
c-----------------------------------------------------------------------
      INCLUDE 'params.utl'
      INCLUDE 'head.utl'
      INCLUDE 'head0.utl'
c
c  Pass control variables

c
      do i=1,3
         atitle0(i)=atitle(i)
      enddo

      amodel0=amodel
      aver0=aver
      alev0=alev

      msyr0=msyr
      mjsday0=mjsday
      mshr0=mshr
      mssec0=mssec
      mnrun0=mnrun
      mavgpd0=mavgpd
      nsecdt0=nsecdt

      ielmet0=ielmet
      jelmet0=jelmet
      delx0=delx
      dely0=dely
      nz0=nz
      xorigkm0=xorigkm
      yorigkm0=yorigkm
      nstas0=nstas
      iastar0=iastar
      iastop0=iastop
      jastar0=jastar
      jastop0=jastop
      isastr0=isastr
      isastp0=isastp
      jsastr0=jsastr
      jsastp0=jsastp
      meshdn0=meshdn

      ndrec0=ndrec
      nctrec0=nctrec
      lsgrid0=lsgrid
      i2drhu0=i2drhu

c
c  Coordinates of non-gridded receptors if any are used
      if(ndrec .NE. 0) then
         do i=1,ndrec
            xrec0(i)=xrec(i)
            yrec0(i)=yrec(i)
            zrec0(i)=zrec(i)
         enddo
      endif
c
c  Coordinates of complex terrain receptors if any are used
      if(nctrec .NE. 0) then
         do i=1,nctrec
            xctr0(i)=xctr(i)
            yctr0(i)=yctr(i)
            zctr0(i)=zctr(i)
            ihill0(i)=ihill(i)
         enddo
      endif

c --- CALPUFF file Version 2.1 checks
      cpver0=cpverx(1)
      isrcinfo0=isrcinfox(1)
      isrcindv0=isrcindv

      return
      end
c-----------------------------------------------------------------------
      subroutine testhead
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 050311           TESTHEAD
c ---           D. Strimaitis, SRC
c
c  PURPOSE:     Tests header information from first input data file
c               saved in common /head0/ against data in another
c               subsequent input data file.
c
c  UPDATES:
c     V1.4(040818) to V1.42(050311)
c               (DGS) Allow mix of Dataset 2.* formats
c     V1.2(030210) to V1.4(040818)
c               (DGS) add CALPUFF file Version 2.1 checks
c     V1.1(010730_1) to V1.2(030210)
c               (JSS) modify checks to include a test on the file type
c                     variable (i2drhu).  Fatal error if file types do
c                     not match [i2drhu=0 (original format), 1 (NO-OBS
c                     format)].  Add new error type (lftyp).
c     V1.1(000712) to V1.1 (010730_1)
c               (JSS) modify checks on header records to allow different
c                     start times, grid characteristics, etc. as long as
c                     receptor coordinates match.  Define match of 
c                     X coordinate to be within 1 m (and elev 0.1 m) to 
c                     eliminate problems with .eq. checks of real numbers.
c                     Write non-fatal warning messages of non-essential
c                     header record variables do not match.
c     V1.1(000106) to V1.1(000712)
c               (JSS) allow data from different versions to be processed
c                     expand data provided in error report
c     V1.0(990806a) to V1.1(000106)
c               (DGS) fix format for error report
c
c  ARGUMENTS:
c               none
c
c  CALLING ROUTINES:    POSTUTIL
c
c  EXTERNAL ROUTINES:   none
c-----------------------------------------------------------------------
      INCLUDE 'params.utl'
      INCLUDE 'head.utl'
      INCLUDE 'head0.utl'

      logical lfatal,lfatald,lfatalct,lfatals,lwarn,lftyp

c --- Initialize logicals
      lfatal=.FALSE.
      lfatald=.FALSE.
      lfatalct=.FALSE.
      lfatals=.false.
      lwarn=.false.
      lftyp=.false.

c *** Allow data from different models/versions to be processed
      if(amodel0.NE.amodel) lwarn=.TRUE.
      if(aver0.NE.aver) lwarn=.TRUE.
      if(alev0.NE.alev) lwarn=.TRUE.

c *** Allow execution to continue if non-essential header
c *** variables do not match
      if(msyr0.NE.msyr) lwarn=.TRUE.
      if(mjsday0.NE.mjsday) lwarn=.TRUE.
      if(mshr0.NE.mshr) lwarn=.TRUE.
      if(mssec0.NE.mssec) lwarn=.TRUE.
      if(mnrun0.NE.mnrun) lwarn=.TRUE.
      if(mavgpd0.NE.mavgpd) lwarn=.TRUE.
      if(nsecdt0.NE.nsecdt) lwarn=.TRUE.

c *** Grid data need not match as long as sampling grid not used
      if(ielmet0.NE.ielmet) lwarn=.TRUE.
      if(jelmet0.NE.jelmet) lwarn=.TRUE.
      if(delx0.NE.delx) lwarn=.TRUE.
      if(dely0.NE.dely) lwarn=.TRUE.
      if(nz0.NE.nz) lwarn=.TRUE.
      if(xorigkm0.NE.xorigkm) lwarn=.TRUE.
      if(yorigkm0.NE.yorigkm) lwarn=.TRUE.
      if(nstas0.NE.nstas) lwarn=.TRUE.
      if(iastar0.NE.iastar) lwarn=.TRUE.
      if(iastop0.NE.iastop) lwarn=.TRUE.
      if(jastar0.NE.jastar) lwarn=.TRUE.
      if(jastop0.NE.jastop) lwarn=.TRUE.
      if(isastr0.NE.isastr) lwarn=.TRUE.
      if(isastp0.NE.isastp) lwarn=.TRUE.
      if(jsastr0.NE.jsastr) lwarn=.TRUE.
      if(jsastp0.NE.jsastp) lwarn=.TRUE.
      if(meshdn0.NE.meshdn) lwarn=.TRUE.
c
c --- File types must match (original vs. NO-OBS, MSOURCE)
c --- if critical differences exist
      if(i2drhu0.NE.i2drhu) lftyp=.true.
      if(cpver0.NE.cpver) then
         if(cpver0.EQ.'0.0             ') lftyp=.true.
         if(cpver.EQ. '0.0             ') lftyp=.true.
      endif
c      if(isrcinfo0.NE.isrcinfo) lftyp=.true.
      if(isrcindv0.NE.isrcindv) lftyp=.true.

c *** Receptors must match
      if(ndrec0.NE.ndrec) lfatal=.TRUE.
      if(nctrec0.NE.nctrec) lfatal=.TRUE.

      if(lsgrid0 .AND. .NOT.lsgrid) lfatal=.TRUE.
      if(.NOT.lsgrid0 .AND. lsgrid) lfatal=.TRUE.
      if(lsgrid0)then
c ***    Grid must match if sampling grid is being used
         if(ielmet0.NE.ielmet) lfatals=.TRUE.
         if(jelmet0.NE.jelmet) lfatals=.TRUE.
         if(delx0.NE.delx) lfatals=.TRUE.
         if(dely0.NE.dely) lfatals=.TRUE.
         if(xorigkm0.NE.xorigkm) lfatals=.TRUE.
         if(yorigkm0.NE.yorigkm) lfatals=.TRUE.
         if(nstas0.NE.nstas) lfatals=.TRUE.
         if(iastar0.NE.iastar) lfatals=.TRUE.
         if(iastop0.NE.iastop) lfatals=.TRUE.
         if(jastar0.NE.jastar) lfatals=.TRUE.
         if(jastop0.NE.jastop) lfatals=.TRUE.
         if(isastr0.NE.isastr) lfatals=.TRUE.
         if(isastp0.NE.isastp) lfatals=.TRUE.
         if(jsastr0.NE.jsastr) lfatals=.TRUE.
         if(jsastp0.NE.jsastp) lfatals=.TRUE.
         if(meshdn0.NE.meshdn) lfatals=.TRUE.
      endif
c
c --- Coordinates of non-gridded receptors if any are used
c --- must match (within 0.1 m)
      nd=MIN(ndrec,ndrec0)
      if(nd .GT. 0) then
         do i=1,nd
            if(abs(xrec0(i)-xrec(i)).GT.0.001)lfatald=.TRUE.
            if(abs(yrec0(i)-yrec(i)).GT.0.001)lfatald=.TRUE.
            if(abs(zrec0(i)-zrec(i)).GT.0.1)lfatald=.TRUE.
         enddo
      endif
c
c --- Coordinates of complex terrain receptors if any are used
c --- must match (within 0.1 m)
      nct=MIN(nctrec,nctrec0)
      if(nct .GT. 0) then
         do i=1,nct
            if(abs(xctr0(i)-xctr(i)).GT.0.001)lfatalct=.TRUE.
            if(abs(yctr0(i)-yctr(i)).GT.0.001)lfatalct=.TRUE.
            if(abs(zctr0(i)-zctr(i)).GT.0.1)lfatalct=.TRUE.
            if(ihill0(i).NE.ihill(i)) lfatalct=.TRUE.
         enddo
      endif

      if(LFATAL.OR.LFATALD.OR.LFATALCT.or.LFATALS.or.LFTYP) then
c ---    Write fatal message to screen
         write(*,*)'TESTHEAD: Essential header data in ',
     1   'CALPUFF.DAT file does not match header of first ',
     2   'CALPUFF.DAT file'
         write(*,*) atitle0
         write(*,*)'Current CALPUFF.DAT Title:'
         write(*,*) atitle
         write(*,*)
         write(*,*)'Consult LIST File for Details'
         write(*,*)
         write(*,*)'Execution terminating'
c
c ---    Write details to list file
c ---    Write error message to screen
         write(io1,*)
         write(io1,*)'TESTHEAD: Essential header data in ',
     1   'CALPUFF.DAT file does not match header of first ',
     2   'CALPUFF.DAT file'
         write(io1,*)'Execution terminating'
      endif
c
      if(LWARN) then
c ---    Write warning message to screen
        write(*,*)'Warning: some header information in the CALPUFF.DAT'
        WRITE(*,*)'files or BCKGALM.DAT file does not match the',
     1   ' header of the first CALPUFF.DAT file'
        write(*,*)'Check list file to confirm intended files ',
     1   'are being used'
        write(*,*)' '
c --- write warning message to list file
       write(io1,*)'Warning: some header information in the CALPUFF.DAT'
       WRITE(io1,*)'files or BCKGALM.DAT file does not match the',
     1   ' header of the first CALPUFF.DAT file'
        write(io1,*)'Check list file to confirm intended files ',
     1   'are being used'
        write(*,*)' '
      endif
c
      if(LFATAL.OR.LFATALD.OR.LFATALCT.or.LFATALS.or.LWARN.or.LFTYP)then
         write(io1,*)'First CALPUFF.DAT or BCKGALM.DAT Title:'
         write(io1,*) atitle0
         write(io1,*)'Current CALPUFF.DAT Title:'
         write(io1,*) atitle
         write(io1,*)
         write(io1,*)'MSYR0   = ',msyr0,   ' MSYR   = ',msyr
         write(io1,*)'MJSDAY0 = ',mjsday0, ' MJSDAY = ',mjsday
         write(io1,*)'MSHR0   = ',mshr0,   ' MSHR   = ',mshr
         write(io1,*)'MSSEC0  = ',mssec0,  ' MSSEC  = ',mssec
         write(io1,*)'MNRUN0  = ',mnrun0,  ' MNRUN  = ',mnrun 
         write(io1,*)'MAVGPD0 = ',mavgpd0, ' MAVGPD = ',mavgpd
         write(io1,*)'NSECDT0 = ',nsecdt0, ' NSECDT = ',nsecdt
         write(io1,*)'IELMET0 = ',ielmet0, ' IELMET = ',ielmet
         write(io1,*)'JELMET0 = ',jelmet0, ' JELMET = ',jelmet
         write(io1,*)'DELX0   = ',delx0,   ' DELX   = ',delx
         write(io1,*)'DELY0   = ',dely0,   ' DELY   = ',dely
         write(io1,*)'NZ0     = ',nz0,     ' NZ     = ',nz
         write(io1,*)'XORIGKM0= ',xorigkm0,' XORIGKM= ',xorigkm
         write(io1,*)'YORIGKM0= ',yorigkm0,' YORIGKM= ',yorigkm
         write(io1,*)'NSTAS0  = ',nstas0,  ' NSTAS  = ',nstas
         write(io1,*)'IASTAR0 = ',iastar0, ' IASTAR = ',iastar
         write(io1,*)'IASTOP0 = ',iastop0, ' IASTOP = ',iastop
         write(io1,*)'JASTAR0 = ',jastar0, ' JASTAR = ',jastar
         write(io1,*)'JASTOP0 = ',jastop0, ' JASTOP = ',jastop
         write(io1,*)'ISASTR0 = ',isastr0, ' ISASTR = ',isastr
         write(io1,*)'ISASTP0 = ',isastp0, ' ISASTP = ',isastp
         write(io1,*)'JSASTR0 = ',jsastr0, ' JSASTR = ',jsastr
         write(io1,*)'JSASTP0 = ',jsastp0, ' JSASTP = ',jsastp
         write(io1,*)'MESHDN0 = ',meshdn0, ' MESHDN = ',meshdn
         write(io1,*)'NDREC0  = ',ndrec0,  ' NDREC  = ',ndrec
         write(io1,*)'NCTREC0 = ',nctrec0, ' NCTREC = ',nctrec
         write(io1,*)'LSGRID0 = ',lsgrid0, ' LSGRID = ',lsgrid
         write(io1,*)'I2DRHU0 = ',i2drhu0, ' I2DRHU = ',i2drhu
         write(io1,*)'cpver0  = ',cpver0,  ' cpver  = ',cpver
         write(io1,*)'isrcinfo0=',isrcinfo0,'isrcinfo=',isrcinfo
         write(io1,*)'isrcindv0=',isrcindv0,'isrcindv=',isrcindv

         write(io1,101) 'amodel:',amodel0,amodel
101      format(10x,a,2(20x,a))
      endif

      if(LFTYP) then
         write(io1,*)
         write(io1,*)'--- File types do not match'
         write(io1,*)'--- (original vs. NO-OBS, or dataset version)'
         write(io1,*)
      endif

      if(LFATALD) then
         write(io1,*)
         write(io1,*)'--- Discrete receptor coords. are different'
         write(io1,*)
         nd=MIN(ndrec,ndrec0)
         if(nd .GT. 0) then
            do I=1,nd
               if(abs(xrec0(i)-xrec(i)).gt.0.001 .or.
     1            abs(yrec0(i)-yrec(i)).gt.0.001 .or.
     1            abs(zrec0(i)-zrec(i)).gt.0.1)then
                     write(io1,*)'Discrete receptor = ',i  
                     write(io1,*)'   XREC0(i) = ',xrec0(i),
     1                             ' XREC(i)  = ',xrec(i)
                     write(io1,*)'   YREC0(i) = ',yrec0(i),
     1                             ' YREC(i)  = ',yrec(i)
                     write(io1,*)'   ZREC0(i) = ',zrec0(i),
     1                             ' ZREC(i)  = ',zrec(i)
               endif                 
            enddo
         endif
      endif

      if(LFATALCT) then
         write(io1,*)
         write(io1,*)'--- CTSG receptor coords. are different'
         write(io1,*)
         nct=MIN(nctrec,nctrec0)
         if(nct .GT. 0) then
            do I=1,nct
               if(abs(xctr0(i)-xctr(i)).gt.0.001 .or. 
     1            abs(yctr0(i)-yctr(i)).gt.0.001 .or. 
     1            abs(zctr0(i)-zctr(i)).gt.0.1 .or. 
     1            ihill0(i).NE.ihill(i))then 
                     write(io1,*)'CTSG receptor = ',i  
                     write(io1,*)'   XCTR0(i) = ',xctr0(i),
     1                             ' XCTR(i)  = ',xctr(i)
                     write(io1,*)'   YCTR0(i) = ',yctr0(i),
     1                             ' YCTR(i)  = ',yctr(i)
                     write(io1,*)'   ZCTR0(i) = ',zctr0(i),
     1                             ' ZCTR(i)  = ',zctr(i)
                     write(io1,*)'   IHILL0(i)= ',ihill0(i),
     1                             ' IHILL(i) = ',ihill(i)
               endif
            enddo
         endif
      endif

      if(LFATALS) then
         write(io1,*)
         write(io1,*)'--- Sampling grids are different'
         write(io1,*)
      endif

      if(LFATAL.OR.LFATALD.OR.LFATALCT.or.LFATALS.or.LFTYP)stop

      return
      end
c-----------------------------------------------------------------------
      subroutine testhead3(io)
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641           Level: 050302          TESTHEAD3
c --- POSTUTIL  Version: 1.4             Level: 040818          TESTHEAD
c ---           D. Strimaitis, SRC
c
c  PURPOSE:     Tests header information from 3 met. data files output
c               of CALPUFF.
c               saved in common /head3/ against data in first calpuff
c               output file input in POSTUTIL.
c               copied from testhead
c
c  UPDATES:
c     V1.4(040818) to V1.41(050302)
c               (CEC) copied from TESTHEAD and adapted to create
c                     checked between 2D files and CALPUFF files
c     V1.2(030210) to V1.4(040818)
c               (DGS) add CALPUFF file Version 2.1 checks
c     V1.1(010730_1) to V1.2(030210)
c               (JSS) modify checks to include a test on the file type
c                     variable (i2drhu).  Fatal error if file types do
c                     not match [i2drhu=0 (original format), 1 (NO-OBS
c                     format)].  Add new error type (lftyp).
c     V1.1(000712) to V1.1 (010730_1)
c               (JSS) modify checks on header records to allow different
c                     start times, grid characteristics, etc. as long as
c                     receptor coordinates match.  Define match of 
c                     X coordinate to be within 1 m (and elev 0.1 m) to 
c                     eliminate problems with .eq. checks of real numbers.
c                     Write non-fatal warning messages of non-essential
c                     header record variables do not match.
c     V1.1(000106) to V1.1(000712)
c               (JSS) allow data from different versions to be processed
c                     expand data provided in error report
c     V1.0(990806a) to V1.1(000106)
c               (DGS) fix format for error report
c
c  ARGUMENTS:
c               none
c
c  CALLING ROUTINES:    POSTUTIL
c
c  EXTERNAL ROUTINES:   none
c-----------------------------------------------------------------------
      INCLUDE 'params.utl'
      INCLUDE 'rtrhd.utl'
      INCLUDE 'head.utl'

      logical lwarn,lftyp
      LOGICAL lwarns,lwarnd,lwarnct,lwarng

c --- Initialize logicals
      lwarng=.FALSE.
      lwarn=.false.
      lwarns=.false.
      lwarnd=.false.
      lwarnct=.false.
      lftyp=.false.

c *** Allow data from different models/versions to be processed
      if(amodel3.NE.amodel) lwarn=.TRUE.
      if(aver3.NE.aver) lwarn=.TRUE.
      if(alev3.NE.alev) lwarn=.TRUE.

c *** Allow execution to continue if non-essential header
c *** variables do not match
      if(msyr3.NE.msyr) lwarn=.TRUE.
      if(mjsday3.NE.mjsday) lwarn=.TRUE.
      if(mshr3.NE.mshr) lwarn=.TRUE.
      if(mssec3.NE.mssec) lwarn=.TRUE.
      if(mnrun3.NE.mnrun) lwarn=.TRUE.
      if(mavgpd3.NE.mavgpd) lwarn=.TRUE.
      if(nsecdt3.NE.nsecdt) lwarn=.TRUE.

c *** Grid data need to match - these 3 files should come
c *** with the same format as CALPUFF
      if(ielmet3.NE.ielmet) lwarng=.TRUE.
      if(jelmet3.NE.jelmet) lwarng=.TRUE.
      if(delx3.NE.delx) lwarng=.TRUE.
      if(dely3.NE.dely) lwarng=.TRUE.
      if(nz3.NE.nz) lwarng=.TRUE.
      if(xorigkm3.NE.xorigkm) lwarng=.TRUE.
      if(yorigkm3.NE.yorigkm) lwarng=.TRUE.
      if(nstas3.NE.nstas) lwarng=.TRUE.
      if(iastar3.NE.iastar) lwarng=.TRUE.
      if(iastop3.NE.iastop) lwarng=.TRUE.
      if(jastar3.NE.jastar) lwarng=.TRUE.
      if(jastop3.NE.jastop) lwarng=.TRUE.
      if(isastr3.NE.isastr) lwarng=.TRUE.
      if(isastp3.NE.isastp) lwarng=.TRUE.
      if(jsastr3.NE.jsastr) lwarng=.TRUE.
      if(jsastp3.NE.jsastp) lwarng=.TRUE.
      if(meshdn3.NE.meshdn) lwarng=.TRUE.
c
      x1_3=xorigkm3
      x2_3=xorigkm3+(delx3/2.)+delx3*(ielmet3-1)
      y1_3=yorigkm3
      y2_3=yorigkm3+(dely3/2.)+dely3*(jelmet3-1)
      x1_1=xorigkm
      x2_1=xorigkm+(delx/2.)+delx*(ielmet-1)
      y1_1=yorigkm
      y2_1=yorigkm+(dely/2.)+dely*(jelmet-1)
c
      if (x1_1.lt.x1_3.or.x1_1.gt.x2_3.or.x2_1.lt.x1_3
     &     .or.x2_1.gt.x2_3) then
      WRITE(*,*)'WARNING - x-coords of CALPUFF domain are outside of'
      WRITE(*,*)'2D MET data domain - check them to make sure you want'
      WRITE(*,*)'to continue the run'
      WRITE(*,*)'CALPUFF GRID X-coordinates = ',x1_1, x2_1
      WRITE(*,*)'2D MET X-coordinates =',x1_3, x2_3
      WRITE(*,*)' '
      end if
      if (y1_1.lt.y1_3.or.y1_1.gt.y2_3.or.y2_1.lt.y1_3
     &     .or.y2_1.gt.y2_3) then
      WRITE(*,*)'WARNING - y-coords of CALPUFF domain are outside of'
      WRITE(*,*)'2D MET data domain - check them to make sure you want'
      WRITE(*,*)'to continue the run'
      WRITE(*,*)'CALPUFF GRID Y-coordinates = ',y1_1, y2_1
      WRITE(*,*)'2D MET Y-coordinates =',y1_3, y2_3
      WRITE(*,*)' '
      end if
c
c --- File types must match (original vs. NO-OBS, MSOURCE)
      IF(io.eq.in5.or.io.eq.in3) i2drhu3=i2drhurh
      IF(io.eq.in6) i2drhu3=i2drhutp
      IF(io.eq.in7) i2drhu3=i2drhurho
      if(i2drhu3.NE.i2drhu) lftyp=.true.

c *** Receptors must match
      if(ndrec.NE.ndrec3) lwarn=.TRUE.
      if(nctrec.NE.nctrec3) lwarn=.TRUE.

      if(lsgrid3 .AND. .NOT.lsgrid) lwarn=.TRUE.
      if(.NOT.lsgrid3 .AND. lsgrid) lwarn=.TRUE.
      if(lsgrid)then
c ***    Grid must match if sampling grid is being used
         if(ielmet.NE.ielmet3) lwarns=.TRUE.
         if(jelmet.NE.jelmet3) lwarns=.TRUE.
         if(delx.NE.delx3) lwarns=.TRUE.
         if(dely.NE.dely3) lwarns=.TRUE.
         if(xorigkm.NE.xorigkm3) lwarns=.TRUE.
         if(yorigkm.NE.yorigkm3) lwarns=.TRUE.
         if(nstas.NE.nstas3) lwarns=.TRUE.
         if(iastar.NE.iastar3) lwarns=.TRUE.
         if(iastop.NE.iastop3) lwarns=.TRUE.
         if(jastar.NE.jastar3) lwarns=.TRUE.
         if(jastop.NE.jastop3) lwarns=.TRUE.
         if(isastr.NE.isastr3) lwarns=.TRUE.
         if(isastp.NE.isastp3) lwarns=.TRUE.
         if(jsastr.NE.jsastr3) lwarns=.TRUE.
         if(jsastp.NE.jsastp3) lwarns=.TRUE.
         if(meshdn.NE.meshdn3) lwarns=.TRUE.
      endif
c
c --- Coordinates of non-gridded receptors if any are used
c --- can be different
      nd=MIN(ndrec3,ndrec)
      if(nd .GT. 0) then
         do i=1,nd
            if(abs(xrec(i)-xrec3(i)).GT.0.001)lwarnd=.TRUE.
            if(abs(yrec(i)-yrec3(i)).GT.0.001)lwarnd=.TRUE.
            if(abs(zrec(i)-zrec3(i)).GT.0.1)lwarnd=.TRUE.
         enddo
      endif
c
c --- Coordinates of complex terrain receptors if any are used
c --- can be different
      nct=MIN(nctrec3,nctrec)
      if(nct .GT. 0) then
         do i=1,nct
            if(abs(xctr(i)-xctr3(i)).GT.0.001)lwarnct=.TRUE.
            if(abs(yctr(i)-yctr3(i)).GT.0.001)lwarnct=.TRUE.
            if(abs(zctr(i)-zctr3(i)).GT.0.1)lwarnct=.TRUE.
            if(ihill(i).NE.ihill3(i)) lwarnct=.TRUE.
         enddo
      endif

      if(LWARN.or.LwarnD.OR.LwarnCT.or.LwarnS.or.LFTYP.or.LwarnG) then
c ---    Write fatal message to screen
         write(*,*)' '
         write(*,*)'TESTHEAD3: Essential header data in ',
     1   '2DMET.DAT file does not match header of last ',
     2   'CALPUFF.DAT file'
c        write(*,*) atitle
c        write(*,*)'Current CALPUFF.DAT Title:'
c        write(*,*) atitle3
c         write(*,*)
         write(*,*)'Consult LIST File for Details and'
         WRITE(*,*)'confirm intended files are being used'
c         write(*,*)
c         write(*,*)'Execution terminating'
c
c ---    Write details to list file
c ---    Write error message to screen
         write(io1,*)
         write(io1,*)'TESTHEAD: Essential header data in ',
     1   '2DMET.DAT file does not match header of last ',
     2   'CALPUFF.DAT file'
c         write(io1,*)'Execution terminating'
      endif
c
c      if(LWARN) then
c ---    Write warning message to screen
c         write(*,*)'Warning: some header data in the ',
c     1   '2DMET.DAT file does not match the header of the ',
c     2   'last CALPUFF.DAT file'
c         write(*,*)'Check list file to confirm intended files ',
c     1   'are being used'
c      endif
c
      if(LWARN.OR.LWARND.OR.LWARNCT.or.LWARNS.or.LFTYP
     &      .or.LWARNG)then
         write(io1,*)'Last CALPUFF.DAT Title:'
         write(io1,*) atitle
         write(io1,*)'Current 2DMET.DAT field Title:'
         write(io1,*) atitle3
         write(io1,*)
         write(io1,*)'MSYR   = ',msyr,   ' MSYR3   = ',msyr3
         write(io1,*)'MJSDAY = ',mjsday, ' MJSDAY3 = ',mjsday3
         write(io1,*)'MSHR   = ',mshr,   ' MSHR3   = ',mshr3
         write(io1,*)'MSSEC  = ',mssec,  ' MSSEC3  = ',mssec3
         write(io1,*)'MNRUN  = ',mnrun,  ' MNRUN3  = ',mnrun3
         write(io1,*)'MAVGPD = ',mavgpd, ' MAVGPD3 = ',mavgpd3
         write(io1,*)'NSECDT = ',nsecdt, ' NSECDT3 = ',nsecdt3
         write(io1,*)'IELMET = ',ielmet, ' IELMET3 = ',ielmet3
         write(io1,*)'JELMET = ',jelmet, ' JELMET3 = ',jelmet3
         write(io1,*)'DELX   = ',delx,   ' DELX3   = ',delx3
         write(io1,*)'DELY   = ',dely,   ' DELY3   = ',dely3
         write(io1,*)'NZ     = ',nz,     ' NZ3     = ',nz3
         write(io1,*)'XORIGKM= ',xorigkm,' XORIGKM3= ',xorigkm3
         write(io1,*)'YORIGKM= ',yorigkm,' YORIGKM3= ',yorigkm3
         write(io1,*)'NSTAS  = ',nstas,  ' NSTAS3  = ',nstas3
         write(io1,*)'IASTAR = ',iastar, ' IASTAR3 = ',iastar3
         write(io1,*)'IASTOP = ',iastop, ' IASTOP3 = ',iastop3
         write(io1,*)'JASTAR = ',jastar, ' JASTAR3 = ',jastar3
         write(io1,*)'JASTOP = ',jastop, ' JASTOP3 = ',jastop3
         write(io1,*)'ISASTR = ',isastr, ' ISASTR3 = ',isastr3
         write(io1,*)'ISASTP = ',isastp, ' ISASTP3 = ',isastp3
         write(io1,*)'JSASTR = ',jsastr, ' JSASTR3 = ',jsastr3
         write(io1,*)'JSASTP = ',jsastp, ' JSASTP3 = ',jsastp3
         write(io1,*)'MESHDN = ',meshdn, ' MESHDN3 = ',meshdn3
         write(io1,*)'NDREC  = ',ndrec,  ' NDREC3  = ',ndrec3
         write(io1,*)'NCTREC = ',nctrec, ' NCTREC3 = ',nctrec3
         write(io1,*)'LSGRID = ',lsgrid, ' LSGRID3 = ',lsgrid3
         write(io1,*)'I2DRHU = ',i2drhu, ' I2DRHU3 = ',i2drhu3

         write(io1,101) 'amodel:',amodel,amodel3
101      format(10x,a,2(20x,a))
      endif

      if(LFTYP) then
         write(io1,*)
         write(io1,*)'--- File types do not match'
         write(io1,*)'--- (original vs. NO-OBS, or dataset version)'
         write(io1,*)
      endif

      if(LWARND) then
         write(io1,*)
         write(io1,*)'--- Discrete receptor coords. are different'
         write(io1,*)
         nd=MIN(ndrec3,ndrec)
         if(nd .GT. 0) then
            do I=1,nd
               if(abs(xrec(i)-xrec3(i)).gt.0.001 .or.
     1            abs(yrec(i)-yrec3(i)).gt.0.001 .or.
     1            abs(zrec(i)-zrec3(i)).gt.0.1)then
                     write(io1,*)'Discrete receptor = ',i  
                     write(io1,*)'   XREC0(i) = ',xrec(i),
     1                             ' XREC3(i)  = ',xrec3(i)
                     write(io1,*)'   YREC0(i) = ',yrec(i),
     1                             ' YREC3(i)  = ',yrec3(i)
                     write(io1,*)'   ZREC0(i) = ',zrec(i),
     1                             ' ZREC3(i)  = ',zrec3(i)
               endif                 
            enddo
         endif
      endif

      if(LWARNCT) then
         write(io1,*)
         write(io1,*)'--- CTSG receptor coords. are different'
         write(io1,*)
         nct=MIN(nctrec3,nctrec)
         if(nct .GT. 0) then
            do I=1,nct
               if(abs(xctr(i)-xctr3(i)).gt.0.001 .or.
     1            abs(yctr(i)-yctr3(i)).gt.0.001 .or.
     1            abs(zctr(i)-zctr3(i)).gt.0.1 .or.
     1            ihill(i).NE.ihill3(i))then
                     write(io1,*)'CTSG receptor = ',i  
                     write(io1,*)'   XCTR0(i) = ',xctr(i),
     1                             ' XCTR3(i)  = ',xctr3(i)
                     write(io1,*)'   YCTR0(i) = ',yctr(i),
     1                             ' YCTR3(i)  = ',yctr3(i)
                     write(io1,*)'   ZCTR0(i) = ',zctr(i),
     1                             ' ZCTR3(i)  = ',zctr3(i)
                     write(io1,*)'   IHILL0(i)= ',ihill(i),
     1                             ' IHILL3(i) = ',ihill3(i)
               endif
            enddo
         endif
      endif

      if(LWARNS) then
         write(io1,*)
         write(io1,*)'--- Sampling grids are different'
         write(io1,*)
      endif
C
      return
      end
c-----------------------------------------------------------------------
      subroutine adjhead
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 091122           ADJHEAD
c ---           D. Strimaitis
c
c  PURPOSE:     Modifies /head2/ information for output data file
c               based on information in commons /ctrl/ and /head/.
c
c  UPDATES:
c     V1.6(071015) to V1.62(091122)
c               (DGS) Use layer requested in control file
c               (DGS) Use species units read from file
c     V1.42(050311) to V1.6(071015)
c               (DGS) Add Dataset 2.2
c     V1.4(040818) to V1.42(050311)
c               (DGS) Allow mix of Dataset 2.* formats
c     V1.1(001117) to V1.4(040818)
c               (DGS) Place source and time period adjustments here
c     V1.1(000106) to V1.1(001117)
c               (DGS) Do not compress output file if x'=Ax+B can be
c                     negative
c     V1.0(990806a) to V1.1(000106)
c               (DGS) 'level' portion of species-level names must be '1'
c                     to denote concentrations when Nitrate partition
c                     option is selected
c     V1.0(990806) to V1.0(990806a)
c               (DGS) match 'level' portion of species-level names read
c                     from input data files; the only difference allowed
c                     is mix of DF and WF, and output in this case
c                     is TF (Total Flux) because wet & dry fluxes may
c                     be added
c
c  ARGUMENTS:
c     PASSED:   none
c
c  CALLING ROUTINES:    POSTUTIL
c
c  EXTERNAL ROUTINES:   INCR
c-----------------------------------------------------------------------
      INCLUDE 'params.utl'
      INCLUDE 'ctrl.utl'
      INCLUDE 'head.utl'
      INCLUDE 'head2.utl'

c --- Local logicals
      logical lfatal,ltf

c --- Pass title from control file to output data file header
      do i=1,3
         atitle2(i)=ctitle(i)
      enddo

c --- Set compression for output file to that in CALPUFF data file(s)
c --- (use .FALSE. if any input files are not compressed)
      lcomprs2=.TRUE.
      do nf=1,nfiles
         if(.NOT.lcomprs(nf)) lcomprs2=.FALSE.
      enddo

c --- Output file is not compressed if any species weights are negative
      if(lcomprs2) then
c --- Loop over species weights
         do k=1,nspeccmp
            j=ispecwt(k)
            do i=1,nspecinp
               if(specwt(i,j).LT.0.00) lcomprs2=.FALSE.
            enddo
         enddo
      endif

c --- Output file is not compressed if any scale factor (either A or B
c --- in x'=Ax+B) is negative
      if(lcomprs2) then
         do j=1,nfiles
            do i=1,nspecinp
               if(ascale(i,j).LT.0.00) lcomprs2=.FALSE.
               if(bscale(i,j).LT.0.00) lcomprs2=.FALSE.
            enddo
         enddo
      endif

c --- Number of species written to output data file
      nszout2=nspecout

c --- Test/set level for species-level written to output file
      lfatal=.FALSE.
      ltf=.FALSE.
      if(a3layer.NE.'   ') then
         atype=a3layer
      else
         atype=asplst(1,1)(13:15)
         do nf=2,nfiles
            if(atype.NE.asplst(1,nf)(13:15)) then
               if(atype.EQ.' DF') then
                  if(asplst(1,nf)(13:15).EQ.' WF') then
                     ltf=.TRUE.
                  else
                     lfatal=.TRUE.
                  endif
               elseif(atype.EQ.' WF') then
                  if(asplst(1,nf)(13:15).EQ.' DF') then
                     ltf=.TRUE.
                  else
                     lfatal=.TRUE.
                  endif
               else
                  lfatal=.TRUE.
               endif
            endif
         enddo
      endif
      if(mnitrate.GT.0) then
         if(atype.NE.'  1') lfatal=.TRUE.
      endif
      if(LFATAL) then
         write(io6,*)
         write(io6,*)'ADJHEAD: Type of data in CALPUFF.DAT files are',
     &            ' not consistent -- data must be concentration',
     &            ' or combination of wet and dry deposition fluxes',
     &            ' and must be concentration for Nitrate partition'
         write(io6,*)
         write(io6,*)'Consult LIST File for Details'
         write(io6,*)

         write(io1,*)
         write(io1,*)'ADJHEAD: Type of data in CALPUFF.DAT files are',
     &            ' not consistent -- data must be concentration',
     &            ' or combination of wet and dry deposition fluxes',
     &            ' and must be concentration for Nitrate partition'
         write(io1,*)
         write(io1,*)'Species-level names for each CALPUFF.DAT file:'
         do n=1,nf
            write(io1,*) 'File #',n
            do is=1,nszout(n)
               write(io1,*) '      ',asplst(is,n)
            enddo
         enddo
         stop
      elseif(LTF) then
         atype=' TF'
      endif

c --- Assign characteristics of output file
      iptime=iptimex(1)
      isrcinfo=isrcinfox(1)
      do nf=2,nfiles
         iptime=MAX(iptime,iptimex(nf))
         isrcinfo=MAX(isrcinfo,isrcinfox(nf))
      enddo
c --- Output dataset version is assigned to the 'newest' input
      cpver=cpverx(1)
      do nf=2,nfiles
         if(cpverx(nf).EQ.'2.1             ') cpver=cpverx(nf)
      enddo
      do nf=2,nfiles
         if(cpverx(nf).EQ.'2.2             ') cpver=cpverx(nf)
      enddo

c --- Set MSOURCE flag to indicate output of TOTAL from all sources
      msource2=0

c --- Place date-time data in /head2/ variables for output header
c --- Note:  this is the period processed (from control file)
      msyr2=isyr
      mjsday2=jsday
      mshr2=ishr
      mssec2=issec
      mnrun2=nper
      if(iptime.EQ.1) then
c ---    Shift start-time to end of first hour
         call INCR(io1,msyr2,mjsday2,mshr2,1)
      endif

c --- Construct list of output species-levels by appending the 'level'
c --- to the output species list.
      write(io1,*)
      write(io1,*)
      write(io1,*)'Chemical species names written to new file:'
      do i=1,nspecout
         asplst2(i)(13:15)=atype
         asplst2(i)(1:12)=aspeco(i)
c ---     Use units already assigned
c         if(asplst2(i)(15:15).EQ.'F') then
c            aspunit2(i)='g/m2/s          '
c         else
c            aspunit2(i)='g/m3            '
c         endif
         write(io1,*) asplst2(i), '  ', aspunit2(i)
      enddo
      write(io1,*)


      return
      end
c-----------------------------------------------------------------------
      subroutine wrihead(io,cver,clev)
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 091122           WRIHEAD
c ---           D. Strimaitis
c
c  PURPOSE:     Writes header information to output data file
c               based on information in commons /head/ and /head2/.
c
c  UPDATES:
c     V1.6(071015) to V1.62(091122)
c               (DGS) Allow for levels other than layer 1
c     V1.4(040818) to V1.6(071015)
c               (DGS) Add Dataset version 2.2 format
c     V1.3(030402) to V1.4(040818)
c               (DGS) Add Dataset version 2.1 format
c               (DGS) Replace IPVER6(0,1) with IPTIME(1,2)
c                                 1: end-time (no seconds)
c                                 2: begin-time / end-time
c     V1.1(990806) to V1.3(030402)
c               (DGS) Dataset version 2.0 format
c               (DGS) ATYPE placed in /HEAD2/ determines dataset name
c
c  ARGUMENTS:
c     PASSED:   io      Unit number for accessing output file        [i]
c
c  CALLING ROUTINES:    POSTUTIL
c
c  EXTERNAL ROUTINES:   none
c-----------------------------------------------------------------------
      INCLUDE 'params.utl'
      INCLUDE 'ctrl.utl'
      INCLUDE 'head.utl'
      INCLUDE 'head2.utl'
c -- Note: all declaration statements for data written by WRIHEAD
c          are contained in these commons

c --- Local Variables for header text
      character*8 cver,clev
      character*16 dataset,dataver
      character*33 blank33
      character*64 datamod
      character*132 comment1,blank
      logical lunits

      data blank33/'                                 '/

c --- Configure output documentation
      data datamod/'Header structure with coordinate parameters'/
      data comment1/'Produced by POSTUTIL Version: '/

c --- Assign dataset version
      dataver=cpver

c --- Initialize units flag to FALSE (dataset version before 2.2)
      lunits=.FALSE.

c --- Assign dataset name
      if(atype.EQ.' DF') then
         dataset='DFLX.DAT'
      elseif(atype.EQ.' WF') then
         dataset='WFLX.DAT'
      elseif(atype.EQ.' TF') then
         dataset='DFLX.DAT'
      elseif(atype.NE.'   ') then
         dataset='CONC.DAT'
      else
         write(*,*)'WRIHEAD:  Invalid file type = ',atype
         stop
      endif

c --- Set blank (132 characters)
      blank(1:33)=blank33
      blank(34:66)=blank33
      blank(67:99)=blank33
      blank(100:132)=blank33

c --- Construct the version-level comment string
      j=31
      do i=1,8
         if(cver(i:i).NE.' ') then
            comment1(j:j)=cver(i:i)
            j=j+1
         endif
      enddo
      j=j+1
      comment1(j:j+7)=' Level: '
      j=j+8
      do i=1,8
         if(clev(i:i).NE.' ') then
            comment1(j:j)=clev(i:i)
            j=j+1
         endif
      enddo

      if(dataver.EQ.'0.0             ') then
c ---    CALPUFF data file versions older than 2.0 are tagged as 0.0
c ---    Two types are supported:
c ---       CALPUFF MOD-5 with hourly steps    (IPTIME=1)
c ---       CALPUFF MOD-6 with begin/end times (IPTIME=2)
         if(iptime.EQ.1) then
            write(io) amodel,aver,alev,msyr2,mjsday2,mshr2,
     *         mnrun2,mavgpd,
     *         ielmet,jelmet,delx,dely,nz,xorigkm,yorigkm,nstas,
     *         iastar,iastop,jastar,jastop,
     *         isastr,jsastr,isastp,jsastp,
     *         meshdn,npts,nareas,nlines,nvols,
     *         ndrec,nctrec,LSGRID,nszout2,lcomprs2,i2drhu
         elseif(iptime.EQ.2) then
            write(io) amodel,aver,alev,msyr2,mjsday2,mshr2,
     *         mssec2,mnrun2,mavgpd,nsecdt,
     *         ielmet,jelmet,delx,dely,nz,xorigkm,yorigkm,nstas,
     *         iastar,iastop,jastar,jastop,
     *         isastr,jsastr,isastp,jsastp,
     *         meshdn,npts,nareas,nlines,nvols,
     *         ndrec,nctrec,LSGRID,nszout2,lcomprs2
         else
            write(io6,*)'WRIHEAD: fatal error'
            write(io6,*)'Unknown CALPUFF file structure'
            write(io6,*)'iptime (1 or 2) =  ',iptime
            stop
         endif

      else
c ---    CALPUFF data file version 2.0 or newer

c ---    Record #1 - File Declaration -- 24 words
         write(io) dataset,dataver,datamod
c ---    Record #2 - Number of comment lines -- 1 word
         ncom=ncommout+1
         write(io) ncom
c ---    Record #3 to NCOM+2 (Comment record section) -- 33 words each
         write(io) comment1
c ---    Go to beginning of the scratch file with control file images
         REWIND(iox)
c ---    Loop over records
         do i=1,ncommout
            comment1=blank
            read(iox,'(a132)') comment1
            write(io) comment1
         enddo
         CLOSE(iox)
c
c ---    HEADER RECORD #NCOM+3 -- General run parameters
         if(dataver.EQ.'2.0             ') then
            write(io) amodel,aver,alev,msyr2,mjsday2,mshr2,
     *         mnrun2,mavgpd,
     *         ielmet,jelmet,delx,dely,nz,xorigkm,yorigkm,nstas,
     *         iastar,iastop,jastar,jastop,
     *         isastr,jsastr,isastp,jsastp,
     *         meshdn,npts,nareas,nlines,nvols,
     *         ndrec,nctrec,LSGRID,nszout2,lcomprs2,i2drhu,
     *         iutmzn,feast,fnorth,rnlat0,relon0,
     *         xlat1,xlat2,pmap,utmhem,datum,daten,
     *         clat0,clon0,clat1,clat2

         elseif(dataver.EQ.'2.1             ') then
            write(io) amodel,aver,alev,msyr2,mjsday2,mshr2,
     *         mssec2,xbtz,mnrun2,mavgpd,nsecdt,
     *         ielmet,jelmet,delx,dely,nz,xorigkm,yorigkm,nstas,
     *         iastar,iastop,jastar,jastop,
     *         isastr,jsastr,isastp,jsastp,
     *         meshdn,nstot,msource2,
     *         ndrec,nctrec,LSGRID,nszout2,lcomprs2,i2drhu,
     *         iutmzn,feast,fnorth,rnlat0,relon0,
     *         xlat1,xlat2,pmap,utmhem,datum,daten,
     *         clat0,clon0,clat1,clat2

         elseif(dataver.EQ.'2.2             ') then
            lunits=.TRUE.
            write(io) amodel,aver,alev,msyr2,mjsday2,mshr2,
     *         mssec2,abtz,mnrun2,mavgpd,nsecdt,
     *         ielmet,jelmet,delx,dely,nz,xorigkm,yorigkm,nstas,
     *         iastar,iastop,jastar,jastop,
     *         isastr,jsastr,isastp,jsastp,
     *         meshdn,nstot,msource2,
     *         ndrec,nctrec,LSGRID,nszout2,lcomprs2,i2drhu,
     *         iutmzn,feast,fnorth,rnlat0,relon0,
     *         xlat1,xlat2,pmap,utmhem,datum,daten,
     *         clat0,clon0,clat1,clat2

         else
            write(io6,*)'WRIHEAD: fatal error'
            write(io6,*)'Unknown CALPUFF file structure'
            write(io6,*)'Data Version =  ',dataver
            stop
         endif

      endif
c
c  Write title of run
      write(io) atitle2
c
c  Write specie/level list
      write(io) (asplst2(i),i=1,nszout2)
c
c  Write specie/level units
      if(LUNITS) write(io) (aspunit2(i),i=1,nszout2)
c
c  Write coordinates of non-gridded receptors if any are used
      if(ndrec .NE. 0) write(io) (xrec(i),i=1,ndrec),
     *                           (yrec(i),i=1,ndrec),
     *                           (zrec(i),i=1,ndrec)
c
c  Write coordinates of complex terrain receptors if any are used
      if(nctrec .NE. 0) write(io) (xctr(i),i=1,nctrec),
     *                            (yctr(i),i=1,nctrec),
     *                            (zctr(i),i=1,nctrec),
     *                            (ihill(i),i=1,nctrec)
c
c  Write source names
      if(isrcinfo.EQ.1) then
c ---    Loop over source types (points,areas,lines,volumes)
         do k=1,8
            if(nstot(k).GT.0) then
               write(io) k,(csource(n,k),n=1,nstot(k))
            endif
         enddo
      endif

      return
      end
c-----------------------------------------------------------------------
      subroutine gethdrh(io,lecho)
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641           Level: 090511           GETHDRH
c ---           D. Strimaitis
c
c  PURPOSE:     Reads header information from relative humidity file
c               and passes information to main program through
c               common /rtrhd/.
c
c  UPDATES:
c     V1.6(071015) to V1.61(090511)
c               (CEC) Add a check to make sure ielmet3 and jelmet3 are
c                     smaller than MXGX and MXGY
c     V1.41(050302) to V1.6(071015)
c               (DGS) Add dataset version 2.2 format, which includes
c                     time zone as a string, and units record
c               (DGS) Use MXSPLV_IN for input arrays instead of MXSPLV
c     V1.4(040818) to V1.41(050302)
c               (CEC) change to accomodate TEMP, RH and RHO files
c                     the three files have different io, itime, i2drhu
c                     all gridded information need to be the same in
c                     all three files as it is in CALPUFF.
c                     A test is done with TESTHEAD3
c     V1.3(030402) to V1.4(040818)
c               (DGS) Add dataset version 2.1 format
c     V1.0(990709) to V1.3(030402)
c               (FRR) flag for 2D vs 1D RH array (noobs version)i2drhu
c                     Nearest station info only if 1-D RH (not if 2D RH)
c               (DGS) Dataset version 2.0 format
c               (DGS) add list-file unit to INCR, YR4
c
c     V5.0(980918) to V6.0(990709)
c               (DGS) allow general run-steps, with begin-time
c                     end-time format resolved to seconds
c                     (retain compatibility witholder CALPUFF versions)
c                     and enforce YYYY year format
c     V5.0(950106) to V5.0(980515)
c               (DGS) Expand error reports
c
c  ARGUMENTS:
c     PASSED:   io      Unit number for accessing input file         [i]
c
c  CALLING ROUTINES:    MAIN
c
c  EXTERNAL ROUTINES:   INCR, YR4, GETDOC, UTCBASR
c-----------------------------------------------------------------------
      INCLUDE 'params.utl'
      INCLUDE 'rtrhd.utl'
c -- Note: all declaration statements for data read in by GETHEAD
c          are contained in this common
c
      character*15 cname3,asprh3
      character*16 arhunit3
      character*8 abtz3

      character*48 msg(8),msg0
      logical lfatal,lcomprsrh,lecho,lrhunit

      data msg0  /'  **********************************************'/
      data msg(1)/'  *      FATAL Problem Found in GETHDRH!       *'/
      data msg(2)/'  * ------------------------------------------ *'/
      data msg(3)/'  *   Size of Array(s) in VISB.DAT too Large   *'/
      data msg(4)/'  *  Re-size Array Parameter(s) in PARAMS.UTL  *'/
      data msg(5)/'  *         and Recompile POSTUTIL             *'/
      data msg(6)/'  * ------------------------------------------ *'/
      data msg(7)/'  * Parameter       PARAMS.UTL        VISB.DAT *'/
      data msg(8)/'  * ---------       ----------        -------- *'/

      lfatal=.FALSE.
      lrhunit=.FALSE.

c --- Process first section of CALPUFF output file (including comment
c --- records), and determine dataset version
      ifil=0
      if (lecho) then
      ifil=1
      if (io.eq.in6) ifil=2
      if (io.eq.in7) ifil=3
      endif
c
      call getdoc(io,ifil,idtype3,crhver3)
c
c  Read control variables
      if(idtype3.GT.0) then

c ---   Dataset version 2.0 or later format
        if(crhver3.EQ.'2.0             ') then
          read(io) amodel3,aver3,alev3,msyr3,mjsday3,mshr3,mnhrs3,mavg3,
     *         ielmet3,jelmet3,delx3,dely3,nz3,xorigkm3,yorigkm3,nstas3,
     *         iastar3,iastop3,jastar3,jastop3,
     *         isastr3,jsastr3,isastp3,jsastp3,
     *         meshdn3,npts3,nareas3,nlines3,nvols3,
     *         ndrec3,nctrec3,LSGRID3,nszoutrh3,lcomprsrh3,i2drhu3,
     *         iutmzn3,feast3,fnorth3,rnlat03,relon03,
     *         xlat13,xlat23,pmap3,utmhem3,datum3,daten3,
     *         clat03,clon03,clat13,clat23
          if(io.eq.in3.or.io.eq.in5) then
             irhtime3=1
             i2drhurh=i2drhu3
          elseif(io.eq.in6) then
             itptime3=1
             i2drhutp=i2drhu3
          elseif(io.eq.in7) then
             irhotime3=1
             i2drhurho=i2drhu3
          endif

        elseif(crhver3.EQ.'2.1             ') then
          read(io) amodel3,aver3,alev3,msyr3,mjsday3,mshr3,mssec3,xbtz3,
     *         mnrun3,mavgpd3,nsecdt3,
     *         ielmet3,jelmet3,delx3,dely3,nz3,xorigkm3,yorigkm3,nstas3,
     *         iastar3,iastop3,jastar3,jastop3,
     *         isastr3,jsastr3,isastp3,jsastp3,
     *         meshdn3,npt13,npt23,nar13,nar23,nln13,nln23,nvl13,nvl23,
     *         msource3,ndrec3,nctrec3,LSGRID3,nszoutrh3,lcomprsrh3,
     *         i2drhu3,iutmzn3,feast3,fnorth3,rnlat03,relon03,xlat13,
     *         xlat23,pmap3,utmhem3,datum3,daten3,clat03,clon03,clat13,
     *         clat23
          if(io.eq.in3.or.io.eq.in5) then
             irhtime3=2
             i2drhurh=i2drhu3
          elseif(io.eq.in6) then
             itptime3=2
             i2drhutp=i2drhu3
          elseif(io.eq.in7) then
             irhotime3=2
             i2drhurho=i2drhu3
          endif

        elseif(crhver3.EQ.'2.2             ') then
          read(io) amodel3,aver3,alev3,msyr3,mjsday3,mshr3,mssec3,abtz3,
     *         mnrun3,mavgpd3,nsecdt3,
     *         ielmet3,jelmet3,delx3,dely3,nz3,xorigkm3,yorigkm3,nstas3,
     *         iastar3,iastop3,jastar3,jastop3,
     *         isastr3,jsastr3,isastp3,jsastp3,
     *         meshdn3,npt13,npt23,nar13,nar23,nln13,nln23,nvl13,nvl23,
     *         msource3,ndrec3,nctrec3,LSGRID3,nszoutrh3,lcomprsrh3,
     *         i2drhu3,iutmzn3,feast3,fnorth3,rnlat03,relon03,xlat13,
     *         xlat23,pmap3,utmhem3,datum3,daten3,clat03,clon03,clat13,
     *         clat23

c ---     Units provided for variable
          lrhunit=.TRUE.
c ---     Pull real time zone from string
          call UTCBASR(abtz3,xbtz3)

          if(io.eq.in3.or.io.eq.in5) then
             irhtime3=2
             i2drhurh=i2drhu3
          elseif(io.eq.in6) then
             itptime3=2
             i2drhutp=i2drhu3
          elseif(io.eq.in7) then
             irhotime3=2
             i2drhurho=i2drhu3
          endif

        else
          write(io1,*)'ERROR:  Unknown DATASET Version: ',crhver3
          stop 'Halted in GETHDRH'
        endif

      else
c ---   Dataset EARLIER than Version 2.0
        read(io,iostat=ierr) amodel3,aver3,alev3,msyr3,mjsday3,mshr3,
     *         mssec3,mnrun3,mavgpd3,nsecdt3,
     *         ielmet3,jelmet3,delx3,dely3,nz3,xorigkm3,yorigkm3,nstas3,
     *         iastar3,iastop3,jastar3,jastop3,
     *         isastr3,jsastr3,isastp3,jsastp3,
     *         meshdn3,npts3,nareas3,nlines3,nvols3,
     *         ndrec3,nctrec3,LSGRID3,nszoutrh3,lcomprsrh3
        if(ierr.EQ.0) then
          if(io.eq.in3.or.io.eq.in5) then
             irhtime3=2
          elseif(io.eq.in6) then
             itptime3=2
          elseif(io.eq.in7) then
             irhotime3=2
          endif
        else
c ---     May have hit end of header record; try again with old format
          rewind(io)
          read(io) amodel3,aver3,alev3,msyr3,mjsday3,mshr3,mnhrs3,mavg3,
     *         ielmet3,jelmet3,delx3,dely3,nz3,xorigkm3,yorigkm3,nstas3,
     *         iastar3,iastop3,jastar3,jastop3,
     *         isastr3,jsastr3,isastp3,jsastp3,
     *         meshdn3,npts3,nareas3,nlines3,nvols3,
     *         ndrec3,nctrec3,LSGRID3,nszoutrh3,lcomprsrh3,i2drhu3
          if(io.eq.in3.or.io.eq.in5) then
             irhtime3=1
             i2drhurh=i2drhu3
          elseif(io.eq.in6) then
             itptime3=1
             i2drhutp=i2drhu3
          elseif(io.eq.in7) then
             irhotime3=1
             i2drhurho=i2drhu3
          endif
        endif
      endif

c --- Check year format
      call YR4(io1,msyr3,ierry)
      if(ierry.NE.0) stop 'Halted in GETHDRH'

      if (io.eq.in3.or.io.eq.in5) then
      irhtime=irhtime3
      ELSEIF(io.eq.in6) then
      irhtime=itptime3
      ELSEIF(io.eq.in7) then
      irhtime=irhotime3
      end if

      if(irhtime.EQ.1) then
c ---    Move starting time back 1 hour to identify the start of the
c ---    first hour
         call INCR(io1,msyr3,mjsday3,mshr3,-1)
c ---    Set period step to 1 hour
         nsecdt3=3600
         mssec3=0
c ---    Rename averaging period variables
         mnrun3=mnhrs3
         mavgpd3=mavg3
      endif
c
c
c --- (CEC - Check that ielmet3 and jelmet3 are smaller than MXGX, MXGY in params.utl)
      if(ielmet3.gt.mxgx.or.jelmet3.gt.mxgy) then
      write(io1,*)'ERROR in subr. GETHDRH - size of 2D_MET grid smaller'
      write(io1,*)' than maximum array provided in PARAMS.UTL:'
      write(io1,*)' MXGX,MXGY= ',mxgx,mxgy,
     1   ' - 2D_MET grid = ',ielmet3,jelmet3
         write(*,*)
         stop 'Halted in GETHDRH -- see list file.'
      endif
c
c  Check size of array dimensions (compute gridded rec. array dims.)
c  with limits contained in PARAMS.utl file
      ngx3=(isastp3-isastr3)*meshdn3+1
      ngy3=(jsastp3-jsastr3)*meshdn3+1
      ngrec3=ngx3*ngy3
      if(ngx3 .GT. mxgx) lfatal=.TRUE.
      if(ngy3 .GT. mxgy) lfatal=.TRUE.
      if(ndrec3 .GT. mxdrec) lfatal=.TRUE.
      if(nctrec3 .GT. mxctrec) lfatal=.TRUE.
      if(nszoutrh3 .GT. mxsplv_in) lfatal=.TRUE.

      if(LFATAL) then
c ---    Report problem information and quit
c ---    Screen:
         write(io6,*)
         write(io6,100) msg0
         do i=1,8
            write(io6,100) msg(i)
         enddo
         write(io6,101)'  *     MXGX',mxgx,ngx3
         write(io6,101)'  *     MXGY',mxgy,ngy3
         write(io6,101)'  *   MXDREC',mxdrec,ndrec3
         write(io6,101)'  *  MXCTREC',mxctrec,nctrec3
         write(io6,101)'  *MXSPLV_IN',mxsplv_in,nszoutrh3
         write(io6,100) msg0
c ---    List File:
         write(io1,*)
         write(io1,100) msg0
         do i=1,8
            write(io1,100) msg(i)
         enddo
         write(io1,101)'  *     MXGX',mxgx,ngx3
         write(io1,101)'  *     MXGY',mxgy,ngy3
         write(io1,101)'  *   MXDREC',mxdrec,ndrec3
         write(io1,101)'  *  MXCTREC',mxctrec,nctrec3
         write(io1,101)'  *MXSPLV_IN',mxsplv_in,nszoutrh3
         write(io1,100) msg0

         stop
      endif
c
c  Read title of run
      read(io) atitle3
c
c  Read specie/level list
      read(io) asprh3

c  Read units for specie/level list (Dataset 2.2 on)
c  These units are not used!
      if(LRHUNIT) read(io) arhunit3
c
c  Read coordinates of non-gridded receptors if any are used
      if(ndrec3 .NE. 0) read(io) (xrec3(i),i=1,ndrec3),
     *                          (yrec3(i),i=1,ndrec3),
     *                          (zrec3(i),i=1,ndrec3)
c
c  Read coordinates of complex terrain receptors if any are used
      if(nctrec3 .NE. 0) read(io) (xctr3(i),i=1,nctrec3),
     *                           (yctr3(i),i=1,nctrec3),
     *                           (zctr3(i),i=1,nctrec3),
     *                           (ihill3(i),i=1,nctrec3)
c
c frr (4/02) no nearest station records if 2D RH
c frr (021028) bug fix: i2drhu not idrhu
      if(i2drhu3.eq.0) then
c      if(idrhu.eq.0) then 
c       Read the grid of nearest met station indices (NEARS)
         call rdint(io,cname3,nearss3,mxgx3,mxgy3,ielmet3,jelmet3)
c
c       Read x,y-coordinate of met stations (UTM-km)
         read(io) cname3,(xkmsta3(i),i=1,nstas3)
         read(io) cname3,(ykmsta3(i),i=1,nstas3)
      endif

100   format(1x,a48)
101   format(1x,a12,2(8x,i8),'   *')

c  Check for blank specie-level
      if(asprh3 .EQ. '               ') then
         stop 'GETHDRH: blank species listed in VISB.DAT'
      endif

      return
      end
c----------------------------------------------------------------------
      subroutine rdint(iounit,cname,ioutarr,mx,my,nx,ny)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level:  940430                 RDINT
c                J. Scire, SRC
c
c --- PURPOSE:  Read a gridded field of integers to binary file
c               (one 15-character identifier and a 2-D data array)
c
c --- INPUTS:
c          IOUNIT - integer      - Fortran unit no. of output file
c           CNAME - character*15 - Data identifier/name
c  IOUTARI(nx,ny) - int. array   - Data array
c              NX - integer      - Number of grid points in the
c                                  X direction
c              NY - integer      - Number of grid points in the
c                                  Y direction
c
c --- OUTPUT:  none
c
c --- RDINT called by:
c --- RDINT calls:      none
c
c----------------------------------------------------------------------
c
c      !integer ioutarr(nx,ny)
      integer ioutarr(mx,my)
      character*15 cname
c
      read(iounit)cname,((ioutarr(i,j),i=1,nx),j=1,ny)
      do i=1,nx
        do j=1,ny
          if(ioutarr(i,j).eq.0)then
            write(*,'("nearss(",i2,",",i2,") = 0")')i,j
            stop 'Error in relative humidity file visb.dat'
          endif
        enddo
      enddo
c
      return
      end
c-----------------------------------------------------------------------
      subroutine qainp(io)
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 100917              QAINP
c ---           D. Strimaitis
c
c  PURPOSE:     Compares information provided from CALPUFF data
c               file and program control file to check for
c               inconsistencies.  All information is written to
c               output file.  Mapping arrays for species names are
c               filled.
c
c  UPDATES:
c     V1.62(091122) to V1.63(100917)
c               (DGS) add MEQPHASE
c               (DGS) remove NAPER argument (not used from v1.4 on)
c     V1.6(071015) to V1.62(091122)
c               (DGS) add layer to species screening
c               (DGS) require mass concentration units only if
c                     MNITRATE is NOT 0
c               (DGS) Pass units to output species list in /head2/
c     V1.4(040818) to V1.6(071015)
c               (DGS) add check for units (dataset version 2.2)
c     V1.1(000315) to V1.4(040818)
c               (DGS) move number of averaging periods to skip to new
c                     subroutine (QASKIP)
c     V1.1(000112) to V1.1(000315)
c               (DGS) new checks for scaling option (NSCALED)
c     V1.1(000106) to V1.1(000112)
c               (DGS) new checks for MNITRATE=2 option
c               (DGS) assign ISPECNO3() using output species list rather
c                     than input species list
c     V1.0(990806) to V1.1(000106)
c               (DGS) new checks for Nitrate partition species
c
c
c  ARGUMENTS:
c     PASSED:   io      Unit number for accessing output file        [i]
c
c  CALLING ROUTINES:    POSTUTIL
c
c  EXTERNAL ROUTINES:   none
c-----------------------------------------------------------------------
      INCLUDE 'params.utl'
      INCLUDE 'head.utl'
      INCLUDE 'head2.utl'
      INCLUDE 'ctrl.utl'

      integer incheck(mxspec)
      character*15 aspeclv
      character*16 aspunit_in
      logical lok,lcmp
      logical lso4,lno3,lhno3,lnh3,lno3a,lhno3a,ltnh3
      data lso4/.FALSE./,lno3/.FALSE./,lhno3/.FALSE./,lnh3/.FALSE./
      data lno3a/.FALSE./,lhno3a/.FALSE./,ltnh3/.FALSE./
c
c  Initialize fatal error flag
      ifatal=0
c
c  Initialize checking array for input species names
      do i=1,nspecinp
         incheck(i)=0
      enddo

c --- Initialize layer (or level) part of species-level name
      aspeclv(13:15)=a3layer
c --- Set index for checking the layer
      if(a3layer.EQ.'   ') then
         nchk=12
      else
         nchk=15
      endif

c --- Map control file input species names to species found in file
c --- and check to see that all are there
      do nf=1,nfiles
         do j=1,nszout(nf)
            ispeci(j,nf)=0
         enddo
      enddo
      do i=1,nspecinp
c ---    Place species name into target species-level name
         aspeclv(1:12)=aspeci(i)
         lok=.FALSE.
         do nf=1,nfiles
            do j=1,nszout(nf)
               if(asplst(j,nf)(1:nchk).EQ.aspeclv(1:nchk)) then
                  incheck(i)=incheck(i)+1
                  ispeci(j,nf)=i
                  lok=.TRUE.
c ---             Screen units
                  if(aspunit(j,nf)(1:3).NE.'g/m' .AND.
     &                         mnitrate.NE.0) then
            write(io,*)
            write(io,*) '**************    FATAL    ***************'
            write(io,*) 'Input species must be in units of g/m3 or'
            write(io,*) 'g/m2/s for Nitrate processing in POSTUTIL'
            write(io,*) 'Data File Number: ',nf
            write(io,*) 'Species Name    : ',asplst(j,nf)
            write(io,*) 'Species Units   : ',aspunit(j,nf)
                     ifatal=1
                  endif
c ---             Save units name for use as computed species units
c ---             This is an initial implementation and must be refined
c ---             because it assumes that all input species are used and
c ---             therefore have the same units as computed species
                  aspunit_in=aspunit(j,nf)
               endif
            enddo
         enddo
         if(.NOT.lok) then
            write(io,*)
            write(io,*) '**************    FATAL    ***************'
            write(io,*) 'Requested species is not in data files'
            write(io,*) 'Looking for ',aspeclv,'  in the list:'
            do nf=1,nfiles
               write(io,*) 'Data File Number: ',nf
               write(io,*) (asplst(k,nf),k=1,nszout(nf))
            enddo
            ifatal=1
         endif
      enddo

c -- Map output species names to species found in file
c -- and check for species used in Nitrate partitioning
c -- Note: the Nitrate partitioning subroutine operates on the
c --  concentrations in the output array, so all species needed
c --  must be in the output list
      do nf=1,nfiles
         do j=1,nszout(nf)
            ispeco(j,nf)=0
         enddo
      enddo
      do i=1,nspecout
         if(aspeco(i).EQ.'SO4') then
            lso4=.TRUE.
            ispecno3(1)=i
         elseif(aspeco(i).EQ.'HNO3') then
            lhno3=.TRUE.
            ispecno3(2)=i
         elseif(aspeco(i).EQ.'NO3') then
            lno3=.TRUE.
            ispecno3(3)=i
         elseif(aspeco(i).EQ.'NH3') then
            lnh3=.TRUE.
            ispecno3(4)=i
         elseif(aspeco(i).EQ.'HNO3ALL') then
            lhno3a=.TRUE.
            ispecno3(5)=i
         elseif(aspeco(i).EQ.'NO3ALL') then
            lno3a=.TRUE.
            ispecno3(6)=i
c --- (CEC - 060309) - possibility of TNH3 added to species needed for ALM
c ---                  using MNITRATE=1 or MNITRATE=3
         elseif(aspeco(i).EQ.'TNH3') then
            ltnh3=.TRUE.
            ispecno3(7)=i
         endif
c ---    Place species name into target species-level name
         aspeclv(1:12)=aspeco(i)
         lok=.FALSE.
         do nf=1,nfiles
            do j=1,nszout(nf)
               if(asplst(j,nf)(1:nchk).EQ.aspeclv(1:nchk)) then
                  ispeco(j,nf)=i
c ---             Pass units to output
                  aspunit2(i)=aspunit(j,nf)
                  lok=.TRUE.
               endif
            enddo
         enddo
         lcmp=.FALSE.
         do k=1,nspeccmp
            if(ispecwt(k).EQ.i) then
c ---          Output species is computed:  pass units of last input
c ---          species to output
               aspunit2(i)=aspunit_in
               lcmp=.TRUE.
            endif
         enddo
         if(.NOT.lok .AND. .NOT.lcmp) then
            write(io,*)
            write(io,*) '**************    FATAL    ***************'
            write(io,*) 'Requested output species is not in data file'
            write(io,*) 'Looking for ',aspeclv,'  in the list:'
            do nf=1,nfiles
               write(io,*) 'Data File Number: ',nf
               write(io,*) (asplst(k,nf),k=1,nszout(nf))
            enddo
            ifatal=1
         endif
      enddo

c -- Output species names are either taken from the input list or are
c -- from the computed list
      do nf=1,nfiles
         do j=1,nszout(nf)
           if(ispeci(j,nf).EQ.0 .AND. ispeco(j,nf).GT.0) then
            write(io,*)
            write(io,*) '**************    FATAL    ***************'
            write(io,*) 'Requested output species name must be on the'
            write(io,*) 'input list if it matches a name found in the'
            write(io,*) 'data file.  Species Name: ',asplst(j,nf)
            ifatal=1
           endif
         enddo
      enddo

c -- Check for duplicate species names in input files
      do i=1,nspecinp
         if(incheck(i).GT.1) then
           if(mduplct.EQ.1) then
            write(io,*)
            write(io,*) '**************    FATAL    ***************'
            write(io,*) 'Requested input species name exists in more'
            write(io,*) 'than one data file. Species: ',aspeci(i)
            ifatal=1
           else
            write(io,*)
            write(io,*) '**************   WARNING   ***************'
            write(io,*) 'Requested input species name exists in more'
            write(io,*) 'than one data file. Species will be summed'
            write(io,*) '  Species Name: ',aspeci(i)
           endif
         endif
      enddo
c
c -- Check information provided for scaling option (done in READCF?)
      if(nscaled.GT.0) then
c --     Number of files must not be less than number of files scaled
         if(nfiles.LT.nscaled) then
            write(io,*)
            write(io,*) '**************    FATAL    ***************'
            write(io,*) 'Number of files scaled exceeds the number '
            write(io,*) 'of CALPUFF data files.'
            write(io,*) '  NFILES, NSCALED: ',nfiles,nscaled
            ifatal=1
         endif
      endif

c -- Check for HNO3/NO3 phase equilibrium method
         if(mnitrate.GT.0) then
           if(meqphase.LT.1 .OR. meqphase.GT.2) then
            write(io,*)
            write(io,*) '**************    FATAL    ***************'
            write(io,*) 'HNO3/NO3 phase equilibrium method invalid'
            write(io,*) 'Expected MEQPHASE = 1,2'
            write(io,*) 'Found    MEQPHASE = ',meqphase
            ifatal=1
           endif
         endif
c
c  Process species for Nitrate partitioning options
      if(mnitrate.EQ.1) then
         write(io,*)
         write(io,*)'**********    HNO3/NO3 Partitioning  **********'
         if(LSO4) then
            write(io,*)'    SO4 is available'
         else
            write(io,*)'    SO4 is NOT available (FATAL Error)'
            ifatal=1
         endif
         if(LNO3) then
            write(io,*)'    NO3 is available'
         else
            write(io,*)'    NO3 is NOT available (FATAL Error)'
            ifatal=1
         endif
         if(LHNO3) then
            write(io,*)'    HNO3 is available'
         else
            write(io,*)'    HNO3 is NOT available (FATAL Error)'
            ifatal=1
         endif
         if(LNH3) then
            write(io,*)'    NH3 gas is available'
         else
            write(io,*)'    NH3 gas is NOT available'
         endif
         if(LTNH3) then
            write(io,*)'    total NH3 is available'
         else
            write(io,*)'    total NH3 is NOT available'
         endif
         IF(.not.LNH3.and..not.LTNH3) then
           IF(nh3typ.eq.0) then
            WRITE(io,*)'    NH3 gas is NOT available and'
            WRITE(io,*)'    total NH3 is not available (FATAL Error)'
            ifatal=1
           ELSE
            WRITE(io,*)'   since no NH3gas and no totalNH3 are present'
            WRITE(io,*)'   in any CALPUFF concentration files'
            WRITE(io,*)'   NH3 from BCKNH3 or BCKTNH3 will be used'
           ENDIF
         endif
         write(io,*)
      elseif(mnitrate.EQ.2) then
         write(io,*)
         write(io,*)'**********    HNO3/NO3 Partitioning  **********'
         if(LNO3) then
            write(io,*)'    NO3 is available'
         else
            write(io,*)'    NO3 is NOT available (FATAL Error)'
            ifatal=1
         endif
         if(LHNO3) then
            write(io,*)'    HNO3 is available'
         else
            write(io,*)'    HNO3 is NOT available (FATAL Error)'
            ifatal=1
         endif
         if(LNO3A) then
            write(io,*)'    NO3ALL is available'
         else
            write(io,*)'    NO3ALL is NOT available (FATAL Error)'
            ifatal=1
         endif
         if(LHNO3A) then
            write(io,*)'    HNO3ALL is available'
         else
            write(io,*)'    HNO3ALL is NOT available (FATAL Error)'
            ifatal=1
         endif
      elseif(mnitrate.EQ.3) then
         write(io,*)
         write(io,*)'***** HNO3/NO3 Partitioning - ALM in 1 step  *****'
         if(LSO4) then
            write(io,*)'    SO4 is available'
         else
            write(io,*)'    SO4 is NOT available (FATAL Error)'
            ifatal=1
         endif
         if(LNO3) then
            write(io,*)'    NO3 is available'
         else
            write(io,*)'    NO3 is NOT available (FATAL Error)'
            ifatal=1
         endif
         if(LHNO3) then
            write(io,*)'    HNO3 is available'
         else
            write(io,*)'    HNO3 is NOT available (FATAL Error)'
            ifatal=1
         endif
         if(LNH3) then
            write(io,*)'    NH3 gas is available'
         else
            write(io,*)'    NH3 gas is NOT available'
         endif
         if(LTNH3) then
            write(io,*)'    total NH3 is available'
         else
            write(io,*)'    total NH3 is NOT available'
         endif
         IF(.not.LNH3.and..not.LTNH3) then
           IF(nh3typ.eq.0) then
            WRITE(io,*)'    NH3 gas is NOT available and'
            WRITE(io,*)'    total NH3 is not available (FATAL Error)'
            ifatal=1
           ELSE
            WRITE(io,*)'   since no NH3gas and no totalNH3 are present'
            WRITE(io,*)'   in any CALPUFF concentration files'
            WRITE(io,*)'   NH3 from BCKTNH3 or external file BCKGALM'
            WRITE(io,*)'   will be used'
           ENDIF
         endif
         write(io,*)
      endif
c
c  Stop processing here if a fatal error has been found
      if(ifatal .EQ. 1) then
         write(*,*) 'FATAL ERRORS were found in POSTUTIL.INP!'
         write(*,*) 'Review the messages written to the list-file.'
         stop
      endif
c
      return
101   format(3(10i2,4x))
102   format(6i2)
      end
c-----------------------------------------------------------------------
      subroutine qaskip(ifile)
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 040818             QASKIP
c ---           D. Strimaitis, SRC
c
c  PURPOSE:     Computes number of averaging periods to skip (naper)
c               to get to first period to be processed.
c
c
c  ARGUMENTS:
c     PASSED:   ifile   index for current data file                  [i]
c
c  CALLING ROUTINES:    POSTUTIL
c
c  EXTERNAL ROUTINES:   DELTT
c-----------------------------------------------------------------------
      INCLUDE 'params.utl'
      INCLUDE 'head.utl'
      INCLUDE 'ctrl.utl'


c --- Initialize fatal error flag
      ifatal=0

c --- Check starting hour and duration information in control file
c --- Find number of hours between start of data and processing start
      call DELTT(msyr,mjsday,mshr,isyr,jsday,ishr,idelhr)
      delhr=FLOAT(idelhr)+FLOAT(issec-mssec)/3600.
c --- Length of averaging period in hours
      avghr=FLOAT(nsecdt*mavgpd)/3600.
c --- Find number of averaging periods in this interval
      aper=delhr/avghr
      naper=NINT(aper)
c --- Length of interval (delhr) should be product of avg. time * # per,
c --- to the nearest second (.0002778 hour)
      if(ABS(naper*avghr-delhr).GT.0.0003) then
         write(io1,*)
         write(io1,*) '**************    FATAL    ***************'
         write(io1,*) 'Start time is fractional number of averaging',
     &              ' periods from start of model data!'
         write(io1,*) 'File = ',ifile
         write(io1,*) 'NPER= ',naper,' AVGHR= ',avghr,' DELHR= ',delhr
         ifatal=1
      endif
      if(naper .LT. 0) then
         write(io1,*)
         write(io1,*) '**************    FATAL    ***************'
         write(io1,*) 'Requested start time preceeds beginning of data'
         write(io1,*) 'File = ',ifile
         write(io1,*) 'naper = ',naper
         ifatal=1
      endif
c --- Check for number of periods to process
      if(nper .GT. mnper) then
         write(io1,*)
         write(io1,*) '**************    FATAL    ***************'
         write(io1,*) 'Number processing periods exceeds data periods!'
         write(io1,*) 'File = ',ifile
         write(io1,*) 'NPER = ',nper,' MNPER = ',mnper
         ifatal=1
      endif
c --- Check ending hour in control file
      if(nper+naper .GT. mnper) then
         write(io1,*)
         write(io1,*) '**************    FATAL    ***************'
         write(io1,*) 'Requested end time extends beyond data!'
         write(io1,*) 'File = ',ifile
         write(io1,*) 'NPER = ',nper,' nskip = ',naper,' MNPER = ',mnper
         ifatal=1
      endif

      if(ifatal .EQ. 1) then
         write(io1,*)
         write(io1,*) 'Requested in Control File'
         write(io1,*) 'Start YYYY,JJJ,HH: ',isyr,jsday,ishr
         write(io1,*) 'Number of Periods: ',nper
         write(io1,*)
         write(io1,*) 'Found in Data File ',ifile
         write(io1,*) 'Start YYYY,JJJ,HH: ',msyr,mjsday,mshr
         write(io1,*) 'Number of Periods: ',mnper
         write(io1,*)
      endif


c --- Stop processing here if a fatal error has been found
      if(ifatal .EQ. 1) then
         write(*,*) 'FATAL ERRORS were found in POSTUTIL.INP!'
         write(*,*) 'Review the messages written to the list-file.'
         stop
      endif

c --- Assign skip-periods to /CTRL/ variable
      nskip(ifile)=naper

      return
101   format(3(10i2,4x))
102   format(6i2)
      end
c-----------------------------------------------------------------------
      subroutine skiprec(io,nf,tcg,tcd,tct,ix,iy,idrec,itrec,ieof)
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 050311           SKIPREC
c ---           D. Strimaitis, SRC
c
c  PURPOSE:     Reads hourly data records from CALPUFF data file
c               and passes information to main program through
c               common /conc/, but does not process data.
c               (From GETRCRD)
c
c  UPDATES:
c     V1.4(040818) to V1.42(050311)
c               (DGS) Allow mix of Dataset 2.* formats
c     V1.1(990806) to V1.4(040818)
c               (DGS) Replace IPVER6(0,1) with IPTIME(1,2)
c                                 1: end-time (no seconds)
c                                 2: begin-time / end-time
c               (DGS) Implement CALPUFF dataset version 2.1 format with
c                     source information and individual source
c                     contributions
c
c  ARGUMENTS:
c     PASSED:   io        Unit number for accessing input file       [i]
c               nf        Number of data file                        [i]
c               tcg(i,j)  Temporary gridded receptor array          [ra]
c               tcd(i)    Temporary discrete receptor array         [ra]
c               tct(i)    Temporary complex terrain receptor array  [ra]
c               ix,iy     Dimensions for tcg array                   [i]
c               idrec     Dimension for tcd array                    [i]
c               itrec     Dimension for tct array                    [i]
c               ieof      End of file flag (1=EOF encountered)       [i]
c
c  CALLING ROUTINES:    POSTUTIL
c
c  EXTERNAL ROUTINES:   none
c-----------------------------------------------------------------------
c
      INCLUDE 'params.utl'
      INCLUDE 'head.utl'
      INCLUDE 'ctrl.utl'
      INCLUDE 'conc.utl'

c --- Declare temporary variables using actual array dimensions
      character*15 tsplv
      real tcg(ix,iy),tcd(idrec),tct(itrec)
      real xwork1(mxgx,mxgy),xwork2(mxdrec),xwork3(mxctrec)

      logical lfatal

      nwords=ix*iy

c --- Top of loop over sources
5     continue

c --- Read time record: note that time uses the 00-23 hour convention so
c --- that the 24th hour of day 12 has a start time of 23 on day 12 and
c --- an end time of 00 on day 13
      if(iptimex(nf).EQ.2) then
         read(io,end=999) myrb,mjdayb,mhrb,msecb,myre,mjdaye,
     &                    mhre,msece
      else
         read(io,end=999) myre,mjdaye,mhre
      endif

c --- Source information record
      if(isrcinfox(nf).EQ.1) then
         read(io,end=999) mstype,msnum,sname,sxkm,sykm
c ---    If just the TOTAL of all sources is in file, check info now
         if(isrcindv.EQ.0) then
            lfatal=.FALSE.
            if(sname.NE.'TOTAL           ') lfatal=.TRUE.
            if(mstype.NE.0) lfatal=.TRUE.
            if(msnum.NE.1) lfatal=.TRUE.
            if(LFATAL) then
               write(*,*)
               write(*,*)
               write(*,*)'ERROR in SKIPREC'
               write(*,*)'Unexpected source information'
               write(*,*)'Expected: 0 1 TOTAL'
               write(*,*)'Found   : ',mstype,msnum,sname
               stop
            endif
         endif
      endif
c
c --- Read modeled data 
c --- Loop over chemical species/levels
         do 10 isl=1,nszout(nf)
c
            if(lsgrid) then
               if(lcomprs(nf))then
                  read(io)ii
                  call uncomprs(xwork1,ii,io,nwords,tsplv,tcg)
               else
                  read(io) tsplv,tcg
               endif
            endif
c
            if(ldisc) then
               if(lcomprs(nf))then
                  read(io)ii
                  call uncomprs(xwork2,ii,io,idrec,tsplv,tcd)
               else
                  read(io) tsplv,tcd
               endif
            endif
c
            if(lctsg) then
               if(lcomprs(nf))then
                  read(io)ii
                  call uncomprs(xwork3,ii,io,itrec,tsplv,tct)
               else
                  read(io) tsplv,tct
               endif
            endif
10       continue

c --- Get another source result if individual source contributions
c --- are in the file, and if the TOTAL has not been read
      if(isrcindv.GT.0 .AND. sname.NE.'TOTAL           ') goto 5

      return

c --- EOF encountered in date-time record
999   ieof=1
      return
c
      end
c-----------------------------------------------------------------------
      subroutine getrcrd(iorh,tcg,tcd,tct,ix,iy,idrec,itrec,ieof)
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641           Level: 050311           GETRCRD
c ---           D. Strimaitis, SRC
c
c  PURPOSE:     Reads hourly data records from CALPUFF data file
c               and passes information to main program through
c               common /conc/ 
c
c  UPDATES:
c
c --- V1.41(050302)-V1.42(050311)
c               (DGS) Allow mix of Dataset 2.* formats
c
c --- V1.4(040818)-V1.41(050302)
c               (CEC) Change in the reading of VISB.DAT if LVISIB=TRUE
c                     to accomodate the changes in GETHDRH.
c                     rtrhd.utl has been added
c
c --- V1.3(030402)-V1.4(040818)
c               (DGS) Replace IPVER6(0,1) with IPTIME(1,2)
c                                 1: end-time (no seconds)
c                                 2: begin-time / end-time
c               (DGS) Implement CALPUFF dataset version 2.1 format with
c                     source information and individual source
c                     contributions
c               (DGS) Add time consistency checks
c     V1.1(000315) to V1.3(030402)
c               (DGS) add list-file unit to INCR, YR4
c     V1.1(990806) to V1.1(000315)
c               (DGS) add scaling option
c
c  ARGUMENTS:
c     PASSED:   iorh      Unit number for accessing input RH file    [i]
c               tcg(i,j)  Temporary gridded receptor array          [ra]
c               tcd(i)    Temporary discrete receptor array         [ra]
c               tct(i)    Temporary complex terrain receptor array  [ra]
c               ix,iy     Dimensions for tcg array                   [i]
c               idrec     Dimension for tcd array                    [i]
c               itrec     Dimension for tct array                    [i]
c               ieof      End of file flag (1=EOF encountered)       [i]
c
c  CALLING ROUTINES:    POSTUTIL
c
c  EXTERNAL ROUTINES:   INCR, YR4 
c-----------------------------------------------------------------------
c
      INCLUDE 'params.utl'
      INCLUDE 'head.utl'
      INCLUDE 'ctrl.utl'
      INCLUDE 'conc.utl'
      INCLUDE 'rtrhd.utl'
c
c --- Declare temporary variables using actual array dimensions
      character*15 tsplv
      real tcg(ix,iy),tcd(idrec),tct(itrec)
      integer irhss(mxss)
      real xwork1(mxgx,mxgy),xwork2(mxdrec),xwork3(mxctrec)
c frr(09/01) 2D array of relative humidity (dimensioned as actual met grid)
      integer irh2d(mxgx,mxgy)

      logical lproc,lout, ltotal,lfatal

      ieof=0
      nwords=ix*iy

c --- Initialize output arrays to zero,
      if(lsgrid) then
         do k=1,nspecout
         do j=1,iy
         do i=1,ix
            concg(i,j,k)=0.0
         enddo
         enddo
         enddo
      endif
      if(ldisc) then
         do k=1,nspecout
         do i=1,idrec
            concd(i,k)=0.0
         enddo
         enddo
      endif
      if(lctsg) then
         do k=1,nspecout
         do i=1,itrec
            conct(i,k)=0.0
         enddo
         enddo
      endif


c --- Loop over input data files
c ------------------------------
      do nf=1,nfiles   


c ---    Top of loop over sources within file (when present)
5        continue


c ---    Read time record from CALPUFF data file(s)
c ---    Note that time uses the 00-23 hour convention so that the 24th 
c ---    hour of day 12 has a start time of 23 on day 12 and an end
c ---    time of 00 on day 13
         if(iptimex(nf).EQ.2) then
            read(inunit(nf),end=999) myrb,mjdayb,mhrb,msecb,
     &                               myre,mjdaye,mhre,msece
c ---       Enforce YYYY format for year
            call YR4(io1,myrb,ierrb)
            call YR4(io1,myre,ierre)
            if(ierrb.NE.0 .OR. ierre.NE.0) stop 'Halted in GETRCRD'
c ---       Set target date-time
            mdathrb=myrb*100000+mjdayb*100+mhrb
            mdathre=myre*100000+mjdaye*100+mhre
            if(nf.EQ.1) then
               mdathrb1=mdathrb
               mdathre1=mdathre
            else
c ---          QA time periods across files
               if(mdathrb.NE.mdathrb1 .OR. mdathre.NE.mdathre1) then
                  write(io1,*)'ERROR in GETRCRD for file ',nf
                  write(io1,*)'Period in file 1: ',mdathrb1,mdathre1
                  write(io1,*)'Period in file  : ',mdathrb,mdathre
                  stop
               endif
            endif
         else
            read(inunit(nf),end=999) myre,mjdaye,mhre
            msece=0
            msecb=0
            myrb=myre
            mjdayb=mjdaye
            mhrb=mhre
            call INCR(io1,myrb,mjdayb,mhrb,-1)
c ---       Enforce YYYY format for year
            call YR4(io1,myrb,ierrb)
            call YR4(io1,myre,ierre)
            if(ierrb.NE.0 .OR. ierre.NE.0) stop 'Halted in GETRCRD'
c ---       Set target date-time
            mdathrb=myrb*100000+mjdayb*100+mhrb
            mdathre=myre*100000+mjdaye*100+mhre
            if(nf.EQ.1) then
               mdathrb1=mdathrb
               mdathre1=mdathre
            else
c ---          QA time periods across files
               if(mdathrb.NE.mdathrb1 .OR. mdathre.NE.mdathre1) then
                  write(io1,*)'ERROR in GETRCRD for file ',nf
                  write(io1,*)'Period in file 1: ',mdathrb1,mdathre1
                  write(io1,*)'Period in file  : ',mdathrb,mdathre
                  stop
               endif
            endif
         endif

c ---    Source information record
         if(isrcinfox(nf).EQ.1) then
            read(inunit(nf),end=999) mstype,msnum,sname,sxkm,sykm
            if(isrcindv.EQ.0) then
c ---          Total of all sources should be in file
               ltotal=.TRUE.
               lfatal=.FALSE.
               if(sname.NE.'TOTAL           ') lfatal=.TRUE.
               if(mstype.NE.0) lfatal=.TRUE.
               if(msnum.NE.1) lfatal=.TRUE.
               if(LFATAL) then
                  write(*,*)
                  write(*,*)
                  write(*,*)'ERROR in GETRCRD for file ',nf
                  write(*,*)'Unexpected source information'
                  write(*,*)'Expected: 0 1 TOTAL'
                  write(*,*)'Found   : ',mstype,msnum,sname
                  stop
               endif
            elseif(isrcindv.EQ.1) then
c ---          Check for Total of all sources
               if(sname.EQ.'TOTAL           ') then
                  ltotal=.TRUE.
               else
                  ltotal=.FALSE.
               endif
            endif
         else
c ---       No source information in file
            ltotal=.TRUE.
         endif

c ---    Read data and store species/level for output
c ---    Loop over chemical species/levels
         do isl=1,nszout(nf)
c
c ---       Set logical to process this species/level (if on the list)
            lproc=.FALSE.
            if(ispeci(isl,nf).GT.0 .AND. LTOTAL) lproc=.TRUE.
c ---       Set logical to pass this species/level to output array
            lout=.FALSE.
            if(ispeco(isl,nf).GT.0 .AND. LTOTAL) lout=.TRUE.
c ---       Select scaling factors for this species from this file
            if(lproc) then
               afac=ascale(ispeci(isl,nf),nf)
               bfac=bscale(ispeci(isl,nf),nf)
            endif

            if(lsgrid) then
               if(lcomprs(nf))then
                  read(inunit(nf))ii
                  call uncomprs(xwork1,ii,inunit(nf),nwords,tsplv,tcg)
               else
                  read(inunit(nf)) tsplv,tcg
               endif
c ---          Process data if index is on the input list
               if(lproc) then
                  do j=1,iy
                  do i=1,ix
c ---                Apply scaling
                     tcg(i,j)=tcg(i,j)*afac+bfac
c ---                Update computed species
                     do k=1,nspeccmp
                        kk=ispecwt(k)
                        concg(i,j,kk)=concg(i,j,kk)
     &                              +tcg(i,j)*specwt(ispeci(isl,nf),kk)
                     enddo
c ---                Pass to output array (conditional)
                     if(lout) concg(i,j,ispeco(isl,nf))=tcg(i,j)+
     &                                  concg(i,j,ispeco(isl,nf))
                  enddo
                  enddo
               endif
            endif
c
            if(ldisc) then
               if(lcomprs(nf))then
                  read(inunit(nf))ii
                  call uncomprs(xwork2,ii,inunit(nf),idrec,tsplv,tcd)
               else
                  read(inunit(nf)) tsplv,tcd
               endif
c ---          Process data if index is on the input list
               if(lproc) then
                  do i=1,idrec
c ---                Apply scaling
                     tcd(i)=tcd(i)*afac+bfac
c ---                Update computed species
                     do k=1,nspeccmp
                        kk=ispecwt(k)
                        concd(i,kk)=concd(i,kk)
     &                              +tcd(i)*specwt(ispeci(isl,nf),kk)
                     enddo
c ---                Pass to output array (conditional)
                     if(lout) concd(i,ispeco(isl,nf))=tcd(i)+
     &                                concd(i,ispeco(isl,nf))
                  enddo
               endif
            endif
c
            if(lctsg) then
               if(lcomprs(nf))then
                  read(inunit(nf))ii
                  call uncomprs(xwork3,ii,inunit(nf),itrec,tsplv,tct)
               else
                  read(inunit(nf)) tsplv,tct
               endif
c ---          Process data if index is on the input list
               if(lproc) then
                  do i=1,itrec
c ---                Apply scaling
                     tct(i)=tct(i)*afac+bfac
c ---                Update computed species
                     do k=1,nspeccmp
                        kk=ispecwt(k)
                        conct(i,kk)=conct(i,kk)
     &                              +tct(i)*specwt(ispeci(isl,nf),kk)
                     enddo
c ---                Pass to output array (conditional)
                     if(lout) conct(i,ispeco(isl,nf))=tct(i)+
     &                                conct(i,ispeco(isl,nf))
                  enddo
               endif
            endif
         enddo

c ---    Read the next set if results for all sources were not found
         if(.not.LTOTAL) goto 5

      enddo
c --- End of loop over files
c ------------------------------

c --- Read RH data from VISB.DAT file and match date-time to the hour
      if(LVISIB) then
11       if(irhtime3.EQ.2) then
            read(iorh,end=999) iyrb,ijdayb,ihrb,isecb,iyre,ijdaye,
     &                         ihre,isece
         else
            read(iorh,end=999) iyre,ijdaye,ihre
            isece=0
            isecb=0
            iyrb=iyre
            ijdayb=ijdaye
            ihrb=ihre
            call INCR(io1,iyrb,ijdayb,ihrb,-1)
         endif
c ---    Enforce YYYY format for year
         call YR4(io1,iyrb,ierrb)
         call YR4(io1,iyre,ierre)
         if(ierrb.NE.0 .OR. ierre.NE.0) stop 'Halted in GETRCRD'
c frr (09/01) read in 2D array of relative humidity if available (ielmetxjelmet)
         if (i2drhurh.eq.1)then
c frr bug fix (Oct 28, 2002);; inverted i,j
c            read(iorh) tsplv,((irh2d(i,j),j=1,jelmet),i=1,ielmet)
            read(iorh) tsplv,((irh2d(i,j),i=1,ielmet),j=1,jelmet)
         else
            read(iorh) tsplv,(irhss(is),is=1,nstas)
         endif
         idathrb=iyrb*100000+ijdayb*100+ihrb
         idathre=iyre*100000+ijdaye*100+ihre
         if(idathrb.LT.mdathrb) then
            goto 11
         elseif(idathrb.EQ.mdathrb .AND. isecb.LT.msecb) then
            goto 11
         elseif(idathrb.NE.mdathrb .OR. isecb.NE.msecb) then
            write(io6,*)'GETRCRD: date-time not found in VISB.DAT file'
            write(io1,*)'GETRCRD: date-time not found in VISB.DAT file'
            write(io1,*)'  Beginning date-Time needed: ',mdathrb,msecb
            write(io1,*)'     Ending date-Time needed: ',mdathre,msece
            write(io1,*)'   Beginning date-Time found: ',idathrb,isecb
            write(io1,*)'      Ending date-Time found: ',idathre,isece
            stop
         endif
      endif

      return

c --- EOF encountered
999   ieof=1
      return

      end
c-----------------------------------------------------------------------
      subroutine wrircrd
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 040818           WRIRCRD
c ---           D. Strimaitis, SRC
c
c  PURPOSE:     Writes hourly data records to data file
c
c  UPDATES:
c
c --- V1.1(010730)-V1.4(040818)
c               (DGS) Replace IPVER6(0,1) with IPTIME(1,2)
c                                 1: end-time (no seconds)
c                                 2: begin-time / end-time
c               (DGS) Implement CALPUFF dataset version 2.1 format with
c                     source information
c
c --- V1.1(990806)  to V1.1(010730)
c               (DGS) output format for year is always YYYY
c
c  ARGUMENTS:
c     PASSED:   none
c
c  CALLING ROUTINES:    POSTUTIL
c
c  EXTERNAL ROUTINES:   COMPRS, WRDAT
c-----------------------------------------------------------------------
c
      INCLUDE 'params.utl'
      INCLUDE 'conc.utl'
      INCLUDE 'ctrl.utl'
      INCLUDE 'head.utl'
      INCLUDE 'head2.utl'

c --- Declare temporary variables
      real tmp3(mxdrec),tmp5(mxctrec),tmp7(mxgx,mxgy),tmp8(mxgx,mxgy)

c --- Write date/time data record
      if(iptime .EQ. 2) then
c ---    Include beginning and ending times, with seconds
         write(io2) myrb,mjdayb,mhrb,msecb,myre,mjdaye,mhre,msece
      else
c ---    Include ending times, with no seconds
         write(io2) myre,mjdaye,mhre
      endif

c --- Source information
      if(isrcinfo.EQ.1) then
         write(io2) mstype,msnum,sname,sxkm,sykm
      endif

c --- Loop over output species list
      do i=1,nszout2

c ---    Gridded receptor concentrations
         if(lsgrid) then
            if(ifull.eq.1)then
               if(lcomprs2)then
c ---             Write compressed data records
                  call COMPRS(concg(1,1,i),mxgrec,tmp8,mxgrec,
     &                        asplst2(i),io2)
               else
c ---             Write uncompressed data record
                  call WRDAT(io2,asplst2(i),concg(1,1,i),ngx,ngy)
               endif
            else
               call XTRACT(concg(1,1,i),mxgx,mxgy,ngx,ngy,tmp7)
               if(lcomprs2)then
c ---             Write compressed data records
                  nwords=ngx*ngy
                  call COMPRS(tmp7,nwords,tmp8,mxgrec,
     &                        asplst2(i),io2)
               else
c ---             Write uncompressed data record
                  call WRDAT(io2,asplst2(i),tmp7,ngx,ngy)
               endif
            endif
         endif
c
c ---    Discrete receptor concentrations
         if(ldisc) then
            if(lcomprs2)then
c ---          Write compressed data records
               call COMPRS(concd(1,i),ndrec,tmp3,mxdrec,
     &                     asplst2(i),io2)
            else
c ---          Write uncompressed data record
               call WRDAT(io2,asplst2(i),concd(1,i),ndrec,1)
            endif
         endif
c
c ---    Discrete CTSG receptor concentrations
         if(lctsg) then
            if(lcomprs2)then
c ---          Write compressed data records
               call COMPRS(conct(1,i),nctrec,tmp5,mxctrec,
     &                     asplst2(i),io2)
            else
c ---          Write uncompressed data record
               call WRDAT(io2,asplst2(i),conct(1,i),nctrec,1)
            endif
         endif

      enddo

      return
      end
c-----------------------------------------------------------------------
      subroutine wrivl(ichan,vers,lev,ipage)
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 980515           WRIVL
c ---           D. Strimaitis, SRC
c
c  PURPOSE:     Write identification label that specifies version
c               number and level of CALPOST.
c
c  UPDATES:
c     V5.0(890515) to V5.0(980515)
c               (DGS) Change version and level to character*12
c
c  ARGUMENTS:
c     PASSED:   ichan   channel number for output file               [i]
c               vers    version number                               [c]
c               lev     level (modification date yymmdd)             [c]
c               ipage   flag for page break (0 = no page)            [i]
c   RETURNED:   none
c
c  CALLING ROUTINES:    CALPOST, READCF
c
c  EXTERNAL ROUTINES:   none
c-----------------------------------------------------------------------
      character*12 vers,lev
c  Define page break character pb:
      character*1 pb
      pb=' '
c
      if(ipage .NE. 0) write(ichan,*) pb
      write(ichan,100)
      write(ichan,110) vers,lev
      write(ichan,100)
      write(ichan,*) ' '
      write(ichan,*) ' '
100   format(13('**********'))
110   format(t47,'POSTUTIL Version ',a12,'  Level ',a12)
c
      return
      end
c----------------------------------------------------------------------
      subroutine readcf(ioin,ioout,version,level)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641   Level:  100917                 READCF
c ---            J. Scire
c ---            D. Strimaitis
c
c --- PURPOSE:  Read the POSTUTIL control file inputs
c
c --- UPDATES:
c     V1.62(091122) to V1.63(100917)
c               (DGS) Add MEQPHASE option (CHEMEQ, ISORROPIA, ...)
c     V1.61(090511) to V1.62(091122)
c               (DGS) Add layer to process (ilayer)
c     V1.6(071015) to V1.61(090511)
c               (DGS) Revised list-file output formats for exponential
c                     notation
c     V1.52(060412) to V1.6(071015)
c               (DGS) MXSPEC arrays updated from 20 to 40
c     V1.5(060329) to V1.52(060412)
c               (CEC) Add check on computed species (CSPECCMP) to be
c                     different from any input species.
c                     If any CSPECCMP is same as 1 ASPECI, the run
c                     will stop
c     V1.44(060310) to V1.5(060329)
c               (DGS) Filnames changed from c*70 to c*132
c                     (for CALUTILS V2.3 and later)
c               (DGS) Recognize flag FLGNH3 and translate to NH3TYP
c                     FLGNH3=F            --> NH3TYP=3
c                     FLGNH3=T            --> NH3TYP=0
c     V1.43(060206) to V1.44(060310)
c               (CEC) Remove Flag (FLGNH3) and replace it with
c                     parameter NH3TYP for the user to chose
c                     from where the NH3 come from
c     V1.4(040818) to V1.43(060206)
c               (CEC) add Flag (FLGNH3) for NH3 input form.
c     V1.3(030402) to V1.4(040818)
c               (DGS) update BCKNH3 to 12 (monthly) values
c     V1.0(010730) to V1.3(030402)
c               (DGS) add list-file unit to JULDAY, GRDAY, INCR
c               (DGS) leave control file open for TFERCF processing
c     V1.0(000315) to V1.1(010730)
c               (DGS) reset NMET=1 if zero and Nitrate partition is
c                     active (multiple met files feature) 
c     V1.1(000106) to V1.1(000315)
c               (DGS) add scaling factor option for CALPUFF data files
c     V1.0(990709) to V1.1(000106)
c               (DGS) add control file variables for Nitrate partition
c                     option (MNITRATE,BCKNH3)
c
c --- INPUTS:
c
c           IOIN - integer       - Fortran unit number of the POSTUTIL
c                                  control file (POSTUTIL.INP)
c          IOOUT - integer       - Fortran unit number of the POSTUTIL
c                                  output list file (POSTUTIL.LST)
c        VERSION - character*12  - Version of POSTUTIL
c          LEVEL - character*12  - Level of POSTUTIL
c       Parameters:
c           MXSG, MXVAR, MXSSG, MXSPEC
c
c --- OUTPUT:
c
c       Common block /CTRL/
c           isyr,ismo,isdy,jsday,ishr,issec,nper,mduplct,
c           nspecinp,nspecout,nspeccmp,specwt(mxspec,mxspec),
c           mnitrate,bcknh3,nscaled,ascale(mxspec,mxspec),bscale(mxspec,mxspec),
c           aspeci(mxspec),aspeco(mxspec),nh3typ,bcktnh3,ilayer,a3layer
c           meqphase
c
c --- READCF called by:  MAIN
c --- READCF calls:      READFN, QAYR4, WRIVL, READIN,
c                        INCR, JULDAY, GRDAY, CAPS
c----------------------------------------------------------------------
c
c --- Include parameters
      include 'params.utl'
      include 'params.cal'
c
      real rtemp(mxspec)
      real ab(2,mxspec)

      character*12 cvdic(mxvar,mxsg),cvdic3(mxvar,2)
      character*12 version,level
      character*12 cspeccmp
      character*132 cfilnam
      character*4 ctemp(12),cspeccmp4(12),cfilnam4(132)

      integer ivleng(mxvar,mxsg),ivtype(mxvar,mxsg)
      integer ivleng3(mxvar,2),ivtype3(mxvar,2)
      integer ifilechk(mxfile)

      logical lecho, lproblem, l3files
c
c --- Include common blocks
      include 'ctrl.utl'
      include 'filnam.utl'

      data lecho/.TRUE./
      data ifilechk/mxfile*0/

c --- Initialize local integer IFLGNH3 to -9 and then test for 
c --- presence in control file (as logical)
      iflgnh3=-9

c --- Identify input groups with letters as continuation character;
c --- and input subgroups with numbers as continuation character.

      data cvdic/
     a 'ISYR','ISMO','ISDY','ISHR','ISSEC','NPER',
     a 'NSPECINP','NSPECOUT','NSPECCMP','MDUPLCT','NSCALED',
     a 'MNITRATE','FLGNH3','NH3TYP','BCKNH3','BCKTNH3',
     a 'MEQPHASE','ILAYER', 42*' ',
     b 'CSPECCMP', 59*' ',
c ---  (Note: the 59*' ' above includes mxspec entries)
     2 'MODDAT', 59*' '/
c ---  (Note: the 59*' ' above includes mxspec entries)

      data cvdic3/
     b  'ASPECI', 59*' ',
     2  'ASPECO', 59*' '/
c
      data ivleng/
     a  14*1,2*12,2*1, 42*0,
     b  12,  59*0,
c ---   (Note: the 59*0 above includes mxspec entries)
     2  132,  59*0/
c ---   (Note: the 59*0 above includes mxspec entries)

      data ivleng3/
     b             12, 59*0,
     2             12, 59*0/

      data ivtype/
     a  12*2,3,2,2*1,2*2, 42*0,
     b  4,   59*0,
c ---   (Note:  the 59*0 includes mxspec entries of type 1)
     2  4,   59*0/
c ---   (Note:  the 59*0 includes mxspec entries of type 1)

      data ivtype3/
     b             4, 59*0,
     2             4, 59*0/

c --- Note that variable types (ivtype) are coded as:
c          0 = null
c          1 = real
c          2 = integer
c          3 = logical
c          4 = character

c --- Read title (First 3 lines of the control file)
      read(ioin,102)ctitle
102   format(a80/a80/a80)
c
c ----------------------------------------------
c --- Read file names from control file - IG # 0
c ----------------------------------------------
c --- Set output unit number to the "screen" (io6) for errors
      call readfn(ioin,io6)
c
c --- Open the output LIST file
      open(ioout,file=utllst,status='unknown')
c
c --- Write model version and level to output file (no page break)
      call wrivl(io1,version,level,0)
c
      write(ioout,1410)ctitle
1410  format(//2x,'Run Title:'/
     1 3(5x,a80/))
c
c ----------------------------------------------
c --- Read control file variables -- IG # 1
c ----------------------------------------------
      call readin(cvdic(1,1),ivleng(1,1),ivtype(1,1),ioin,ioout,lecho,
     1 ISYR,ISMO,ISDY,ISHR,ISSEC,NPER,
     2 NSPECINP,NSPECOUT,NSPECCMP,MDUPLCT,NSCALED,MNITRATE,IFLGNH3,
     3 NH3TYP,BCKNH3,BCKTNH3,MEQPHASE,ILAYER,
     4 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     4 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     5 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     6 idum,idum,idum,idum,idum,idum)

c --- Translate IFLGNH3 (if provided) to NH3TYP
      if(iflgnh3.EQ.1) then
c ---    True
         nh3typ=0
      elseif(iflgnh3.EQ.0) then
c ---    False
         nh3typ=3
      endif

c --- Process layer into a c*3 string
      if(ilayer.LT.0 .OR. ilayer.GT.999) then
         write(*,*)
         write(*,*)'Invalid model layer found: ',ilayer
         stop
      endif
      a3layer='   '
      if(ilayer.GT.99) then
         write(a3layer(1:3),'(i3)')ilayer
      elseif(ilayer.GT.9) then
         write(a3layer(2:3),'(i2)')ilayer
      elseif(ilayer.GT.0) then
         write(a3layer(3:3),'(i1)')ilayer
      endif

c---  QA on number of species allowed (MXSPEC)
      kspec=MAX(nspecinp,nspecout)
      kspec=MAX(nspeccmp,kspec)
      if(kspec.GT.mxspec) then
         write(*,*)
         write(*,*)'Number of species exceeds array dimensions'
         write(*,*)'Number specified: ',kspec
         write(*,*)'Number allowed  : ',mxspec
         write(*,*)
         write(*,*)'Reset MXSPEC in PARAMS.UTL and recompile code'
         write(*,*)'or reduce the number of species processed'
         stop
      endif
c --- Make sure that PARAMS file and code are consistent on MXSPEC
      if(mxspec.NE.40) then
         write(*,*)
         write(*,*)'Arrays used for species in control file code READCF'
         write(*,*)'must match size allowed in PARAMS file, MXSPEC. '
         write(*,*)'Number coded in READCF  : 40'
         write(*,*)'Number found in PARAMS  : ',mxspec
         write(*,*)
         write(*,*)'Reset MXSPEC in PARAMS.UTL and recompile code.'
         stop
      endif
c
c --- QA on starting year of simulation
      call QAYR4(ioout,isyr,0,ierr)
      if(ierr.NE.0) stop 'CORRECT THE CONTROL FILE'

c --- Update number of monthly ammonia values entered (BCKNH3)
c --- Array was initialized with -999. in (1-12)
c --- Count the number of negatives that survived
      negnh3=0
      do i=1,12
         if(bcknh3(i).LT.0.) negnh3=negnh3+1
      enddo
      if(negnh3.EQ.11 .AND. bcknh3(1).GE.0.0) then
c ---    Old control file format or no values entered
c ---    Distribute single value to all months
         do i=2,12
            bcknh3(i)=bcknh3(1)
         enddo
      endif
c
c --- Update number of monthly total ammonia values entered (BCKTNH3)
c --- Array was initialized with the default in (1) and -999. in (2-12)
c --- Count the number of negatives that survived
      negtnh3=0
      do i=1,12
         if(bcktnh3(i).LT.0.) negtnh3=negtnh3+1
      enddo
      if(negtnh3.EQ.11 .AND. bcktnh3(1).GE.0.0) then
c ---    Old control file format or no values entered
c ---    Distribute single value to all months
         do i=2,12
            bcktnh3(i)=bcktnh3(1)
         enddo
      endif
c
c --- Compute starting Julian day
      call JULDAY(ioout,isyr,ismo,isdy,jsday)
c --- Convert starting hour (ISHR) from the end-time of the first hour
c --- to the time at the beginning of the first hour
      call INCR(ioout,isyr,jsday,ishr,-1)
      call GRDAY(ioout,isyr,jsday,ismo,isdy)
c
c --- Multiple met files:  Set NMET=1 here if no files had been listed
c --- in the control file, so that the default filename can be used
      if(mnitrate.GT.0 .AND. nmet.EQ.0) nmet=1
c
c ----------------------------------------------
c --- Read control file variables -- IG # 2
c ----------------------------------------------
c
c --- Subgroup 2(a) - Species names from input file(s)
c     -------------
      do i=1,nspecinp
         do j=1,12
            ctemp(j)(1:1)=' '
         enddo

         call readin(cvdic3(1,1),ivleng3(1,1),ivtype3(1,1),ioin,ioout,
     1   lecho,
     1   CTEMP,
     2   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     3   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     4   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     5   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     6   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum)
c
c ---    Transfer CHAR*4 species name into CHAR*12 input species array
         aspeci(i)='            '
         do j=1,12
            aspeci(i)(j:j)=ctemp(j)(1:1)
         enddo
         call CAPS(aspeci(i))
      enddo
      do i=nspecinp+1,mxspec
         aspeci(i)='            '
      enddo
c
c --- Transfer species names from input species list into variable
c --- dictionary for use in subgroups 2c and 2d
      do i=2,mxspec+1
c ---    Subgroup 2c
         cvdic(i,2)=aspeci(i-1)
         ivleng(i,2)=1
         ivtype(i,2)=1
c ---    Subgroup 2d
         cvdic(i,3)=aspeci(i-1)
         ivleng(i,3)=2
         ivtype(i,3)=1
      enddo

c
c --- Subgroup 2(b) - Species names for output file
c     -------------
      do i=1,nspecout
         do j=1,12
            ctemp(j)(1:1)=' '
         enddo

         call readin(cvdic3(1,2),ivleng3(1,2),ivtype3(1,2),ioin,ioout,
     1   lecho,
     1   CTEMP,
     2   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     3   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     4   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     5   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     6   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum)
c
c ---    Transfer CHAR*4 species name into CHAR*12 variable
         aspeco(i)='            '
         do j=1,12
            aspeco(i)(j:j)=ctemp(j)(1:1)
         enddo
         call CAPS(aspeco(i))
      enddo
      do i=nspecout+1,mxspec
         aspeco(i)='            '
      enddo

c
c --- Subgroup 2(c) - Names for computed species and weights
c     -------------

      do i=1,nspeccmp

c ---    Initialize species name characters to blanks
         do ich=1,12
            cspeccmp4(ich)=' '
         enddo
c ---    Initialize weights to zero
         do is=1,mxspec
            rtemp(is)=0.0
         enddo

cc ---    The following call is set for MXSPEC=20
c         call readin(cvdic(1,2),ivleng(1,2),ivtype(1,2),ioin,ioout,
c     1   lecho,
c     1   CSPECCMP4,RTEMP(1),RTEMP(2),RTEMP(3),RTEMP(4),RTEMP(5),
c     2   RTEMP(6),RTEMP(7),RTEMP(8),RTEMP(9),RTEMP(10),
c     3   RTEMP(11),RTEMP(12),RTEMP(13),RTEMP(14),RTEMP(15),
c     4   RTEMP(16),RTEMP(17),RTEMP(18),RTEMP(19),RTEMP(20),
c     5   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
c     6   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
c     7   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
c     8   idum,idum,idum)
c ---    The following call is set for MXSPEC=40
         call readin(cvdic(1,2),ivleng(1,2),ivtype(1,2),ioin,ioout,
     1   lecho,
     1   CSPECCMP4,RTEMP(1),RTEMP(2),RTEMP(3),RTEMP(4),RTEMP(5),
     2   RTEMP(6),RTEMP(7),RTEMP(8),RTEMP(9),RTEMP(10),
     3   RTEMP(11),RTEMP(12),RTEMP(13),RTEMP(14),RTEMP(15),
     4   RTEMP(16),RTEMP(17),RTEMP(18),RTEMP(19),RTEMP(20),
     5   RTEMP(21),RTEMP(22),RTEMP(23),RTEMP(24),RTEMP(25),
     6   RTEMP(26),RTEMP(27),RTEMP(28),RTEMP(29),RTEMP(30),
     7   RTEMP(31),RTEMP(32),RTEMP(33),RTEMP(34),RTEMP(35),
     8   RTEMP(36),RTEMP(37),RTEMP(38),RTEMP(39),RTEMP(40),
     1   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     2   idum,idum,idum,idum,idum,idum,idum)
c
c ---    Transfer CHAR*4 species name into CHAR*12 variable
         CSPECCMP='            '
         do j=1,12
            CSPECCMP(j:j)=CSPECCMP4(j)(1:1)
         enddo
         call CAPS(CSPECCMP)
c --- CHeck that the computed species are not the same name as any of the input species
c --- (cec) - Check added on 060412
c --- Match species name to input species list
         ispin=0
         do k=1,nspecinp
            IF(cspeccmp.EQ.aspeci(k)) ispin=k
         enddo
            IF(ispin.gt.0) then
            WRITE(*,*)'Fatal Error in READCF:'
            WRITE(*,*)'A Computed species CSPECCMP can not have'
            WRITE(*,*)'the same name as any of the input species'
            WRITE(*,*)'Computed species name: ',cspeccmp
            write(*,*)'  Input species list: '
            write(*,'(10x,a)') (aspeci(ii),ii=1,nspecinp)
            stop
            endif

c ---    Match species name to output species list
         ispout=0
         do k=1,nspecout
            if(cspeccmp.EQ.aspeco(k)) ispout=k
         enddo
c ---    Place weight profile into array
         if(ispout.GT.0) then
            ispecwt(i)=ispout
            do ispinp=1,nspecinp
               specwt(ispinp,ispout)=rtemp(ispinp)
            enddo
         else
c ---       Computed species is not in output species list
            write(*,*)'Fatal Error in READCF'
            write(*,*)'Computed species is not in Output species list'
            write(*,*)'Computed species name: ',cspeccmp
            write(*,*)'  Output species list: '
            write(*,'(10x,a)') (aspeco(ii),ii=1,nspecout)
            stop
         endif
      enddo

c
c --- Subgroup 2(d) - Scaling factors for input species
c     -------------

      do i=1,nscaled

c ---    Initialize file name characters to blanks
         do ich=1,132
            cfilnam4(ich)=' '
         enddo
c ---    Initialize temporary scaling factors to defaults
         do is=1,mxspec
            ab(1,is)=1.0
            ab(2,is)=0.0
         enddo

cc ---    The following call is set for MXSPEC=20
c         call readin(cvdic(1,3),ivleng(1,3),ivtype(1,3),ioin,ioout,
c     1   lecho,
c     1   CFILNAM4,AB(1,1),AB(1,2),AB(1,3),AB(1,4),AB(1,5),
c     2   AB(1,6),AB(1,7),AB(1,8),AB(1,9),AB(1,10),
c     3   AB(1,11),AB(1,12),AB(1,13),AB(1,14),AB(1,15),
c     4   AB(1,16),AB(1,17),AB(1,18),AB(1,19),AB(1,20),
c     5   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
c     6   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
c     7   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
c     8   idum,idum,idum)
c ---    The following call is set for MXSPEC=40
         call readin(cvdic(1,3),ivleng(1,3),ivtype(1,3),ioin,ioout,
     1   lecho,
     1   CFILNAM4,AB(1,1),AB(1,2),AB(1,3),AB(1,4),AB(1,5),
     2   AB(1,6),AB(1,7),AB(1,8),AB(1,9),AB(1,10),
     3   AB(1,11),AB(1,12),AB(1,13),AB(1,14),AB(1,15),
     4   AB(1,16),AB(1,17),AB(1,18),AB(1,19),AB(1,20),
     5   AB(1,21),AB(1,22),AB(1,23),AB(1,24),AB(1,25),
     6   AB(1,26),AB(1,27),AB(1,28),AB(1,29),AB(1,30),
     7   AB(1,31),AB(1,32),AB(1,33),AB(1,34),AB(1,35),
     8   AB(1,36),AB(1,37),AB(1,38),AB(1,39),AB(1,40),
     1   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     2   idum,idum,idum,idum,idum,idum,idum)
c
c ---    Transfer CHAR*4 species name into CHAR*132 variable
         do j=1,132
            CFILNAM(j:j)=CFILNAM4(j)(1:1)
         enddo
c ---    Convert the file names to the proper case
         call FILCASE(lcfiles,cfilnam)

c ---    Match filename to CALPUFF data file list index
         ifile=0
         do k=1,nfiles
            if(cfilnam.EQ.moddatl(k)) ifile=k
         enddo
         if(ifile.EQ.0) then
            write(*,*)'Fatal Error in READCF (Subgroup 2d)'
            write(*,*)'CALPUFF data filename is not on file list'
            write(*,*)'     MODDAT: ',cfilnam
            write(*,*)'  File list: '
            do k=1,nfiles
               write(*,*) moddatl(k)
            enddo
            stop
         else
            ifilechk(ifile)=1+ifilechk(ifile)
         endif
c ---    Place factors into array (ordered by input spcies list)
         do ispinp=1,nspecinp
            ascale(ispinp,ifile)=ab(1,ispinp)
            bscale(ispinp,ifile)=ab(2,ispinp)
         enddo
      enddo

c --- Test ifilechk here while local array is available
      lproblem=.FALSE.
      if(nscaled.GT.nfiles) lproblem=.TRUE.
      do i=1,nfiles
         if(ifilechk(i).GT.1) lproblem=.TRUE.
      enddo
      if(LPROBLEM) then
         write(*,*)'Fatal Error in READCF (Subgroup 2d)'
         write(*,*)'CALPUFF data filename appears more than once'
         write(*,*)'  File list: '
         do k=1,nfiles
            write(*,'(i6,2x,a132)') ifilechk(k),moddatl(k)
         enddo
         stop
      endif
c
c ----------------- Print input parameters ---------------
c
      call WRIVL(ioout,version,level,1)

c -- Note: day/hr is TIME BEGINNING label for the hour, on 0-23 clock
      write(ioout,101) isyr,ismo,isdy,jsday,ishr,issec,nper
101   format(//2x,
     &'POSTUTIL Control File Input Summary ------------------------'//
     1 5x,'                  Run starting date -- year: ',i4/
     2 5x,'                                      month: ',i4/
     3 5x,'                                        day: ',i4/
     4 5x,'                                 Julian day: ',i4/
     5 5x,'              time beginning  -  hour(0-23): ',i4/
     6 5x,'                              -      second: ',i4/
     7 5x,'                       Run length (periods): ',i4/)
      write(ioout,*)
      write(ioout,*) 'Note: the length of a period is controlled by'
      write(ioout,*) '      the averaging time selected in the model'

c --- Report FLGNH3 translation
      if(mnitrate.GT.0 .AND. iflgnh3.NE.-9) then
        WRITE(ioout,*) ' '
        write(ioout,*) 'FLGNH3 translated to NH3TYP'
        if(iflgnh3.EQ.0) write(ioout,*) 'FLGNH3 = FALSE: NH3TYP = ',
     &                   nh3typ
        if(iflgnh3.EQ.1) write(ioout,*) 'FLGNH3 = TRUE: NH3TYP = ',
     &                   nh3typ
      endif

      write(ioout,*)
      if(mnitrate.EQ.1) then
        write(ioout,*) 'Partition between HNO3 and NO3 is computed'
        if(meqphase.EQ.2) then
          write(ioout,*) '   using ISORROPIA'
        else
          write(ioout,*) '   using CHEMEQ'
        endif
        WRITE(ioout,*) ' '
c --- CEC (060309) - Add information on from where NH3 come from (NH3TYP)
          IF(NH3TYP.EQ.0) then
          WRITE(ioout,*)'NH3 species (in g/m3) in the CALPUFF'
          WRITE(ioout,*)'concentration files is used'
          WRITE(ioout,*)' Background Ammonia is not used.'
          ELSEIF(NH3TYP.EQ.1) then
          WRITE(ioout,*)'NH3 concentration (in g/m3) listed as species'
          WRITE(ioout,*)'in concentration files will be use with'
          WRITE(ioout,*)'NH3 background listed in BCKNH3'
          ELSEIF(NH3TYP.EQ.2) then
          WRITE(*,*)'FATAL ERROR - see list file'
          WRITE(ioout,*)'ERROR - the external background file'
          WRITE(ioout,*)'can not be used with MNITRATE=1'
          STOP
          ELSEIF(NH3TYP.EQ.3) then
          WRITE(ioout,*)'NH3 background (in ppb) listed below will be'
          WRITE(ioout,*)'used alone for the NO3/HNO3 repartitioning'
          ELSEIF(NH3TYP.EQ.4) then
          WRITE(*,*)'FATAL ERROR - see list file'
          WRITE(ioout,*)'ERROR - the external background file'
          WRITE(ioout,*)'can not be used with MNITRATE=1'
          STOP
          endif
c
        IF(nh3typ.eq.1.or.nh3typ.eq.3) then
        write(ioout,*) 'Default background Ammonia (ppb) - NH3  TNH3 '
        write(ioout,*) '     January   =  ',bcknh3(1), bcktnh3(1)
        write(ioout,*) '     February  =  ',bcknh3(2), bcktnh3(2)
        write(ioout,*) '     March     =  ',bcknh3(3), bcktnh3(3)
        write(ioout,*) '     April     =  ',bcknh3(4), bcktnh3(4)
        write(ioout,*) '     May       =  ',bcknh3(5), bcktnh3(5)
        write(ioout,*) '     June      =  ',bcknh3(6), bcktnh3(6)
        write(ioout,*) '     July      =  ',bcknh3(7), bcktnh3(7)
        write(ioout,*) '     August    =  ',bcknh3(8), bcktnh3(8)
        write(ioout,*) '     September =  ',bcknh3(9), bcktnh3(9)
        write(ioout,*) '     October   =  ',bcknh3(10), bcktnh3(10)
        write(ioout,*) '     November  =  ',bcknh3(11), bcktnh3(11)
        write(ioout,*) '     December  =  ',bcknh3(12), bcktnh3(12)
        endif
c
      elseif(mnitrate.EQ.3) then
        write(ioout,*) 'Ammonia Limiting Method is performed in 1 step'
        if(meqphase.EQ.2) then
          write(ioout,*) '   using ISORROPIA'
        else
          write(ioout,*) '   using CHEMEQ'
        endif
        WRITE(ioout,*) ' '
c --- CEC (060309) - Add information on from where NH3 come from (NH3TYP)
          IF(NH3TYP.EQ.0) then
          WRITE(*,*)'FATAL ERROR - see list file'
          WRITE(ioout,*)'TNH3 used need to come either from'
          WRITE(ioout,*)'the external background file or'
          WRITE(ioout,*)'from the monthly Background Ammonia (BCKTNH3)'
          STOP
          ELSEIF(NH3TYP.EQ.1) then
          WRITE(ioout,*)'NH3 concentration (in g/m3) listed as species'
          WRITE(ioout,*)'in concentration files will be used with'
          WRITE(ioout,*)'TNH3 background listed in BCKTNH3'
          ELSEIF(NH3TYP.EQ.2) then
          WRITE(ioout,*)'NH3 concentration (in g/m3) listed as species'
          WRITE(ioout,*)'in concentration files will be used with'
          WRITE(ioout,*)'TNH3 background from the external background'
          WRITE(ioout,*)'file BCKGALM'
          ELSEIF(NH3TYP.EQ.3) then
          WRITE(ioout,*)'TNH3 background (in ppb) listed below will be'
          WRITE(ioout,*)'used alone for the NO3/HNO3 repartitioning'
          ELSEIF(NH3TYP.EQ.4) then
          WRITE(ioout,*)'TNH3 background from the external background'
          WRITE(ioout,*)'file BCKGALM will be used alone for NO3/HNO3'
          WRITE(ioout,*)'repartitioning'
          endif
c
        IF(nh3typ.eq.1.or.nh3typ.eq.3) then
        write(ioout,*) 'Default background Ammonia (ppb) - NH3  TNH3 '
        write(ioout,*) '     January   =  ',bcknh3(1), bcktnh3(1)
        write(ioout,*) '     February  =  ',bcknh3(2), bcktnh3(2)
        write(ioout,*) '     March     =  ',bcknh3(3), bcktnh3(3)
        write(ioout,*) '     April     =  ',bcknh3(4), bcktnh3(4)
        write(ioout,*) '     May       =  ',bcknh3(5), bcktnh3(5)
        write(ioout,*) '     June      =  ',bcknh3(6), bcktnh3(6)
        write(ioout,*) '     July      =  ',bcknh3(7), bcktnh3(7)
        write(ioout,*) '     August    =  ',bcknh3(8), bcktnh3(8)
        write(ioout,*) '     September =  ',bcknh3(9), bcktnh3(9)
        write(ioout,*) '     October   =  ',bcknh3(10), bcktnh3(10)
        write(ioout,*) '     November  =  ',bcknh3(11), bcktnh3(11)
        write(ioout,*) '     December  =  ',bcknh3(12), bcktnh3(12)
        endif
c
      ELSEIF(mnitrate.ne.1.and.mnitrate.ne.3) then
      write(ioout,*) 'Partition between HNO3 and NO3 is NOT computed'
      write(ioout,*) 'and 1-step Ammonia Limiting Method is not used'
      endif

      write(ioout,*)
      write(ioout,*)'      Species processed from Layer -- ',ilayer
      write(ioout,*)
      write(ioout,*)'      Species needed from input file --'
      write(ioout,'(30x,a)') (aspeci(ii),ii=1,nspecinp)
      if(nscaled.GT.0) then
         do is=1,nfiles
            write(ioout,*)
            write(ioout,*)'      Scaling factors (A,B where y=Ax+b)'
            write(ioout,'(30x,a132)') moddatl(is)
            write(ioout,'(30x,a12,1p2e20.6)')
     &          (aspeci(ii),ascale(ii,is),bscale(ii,is),ii=1,nspecinp)
         enddo
      endif
      write(ioout,*)
      write(ioout,*)'      Species written to output file --'
      write(ioout,'(30x,a)') (aspeco(ii),ii=1,nspecout)
      write(ioout,*)
      write(ioout,*)' Species computed from input species --'
      do ic=1,nspeccmp
         k=ispecwt(ic)
         write(ioout,*)
         write(ioout,'(30x,a,a)') aspeco(k),' = '
         do is=1,nspecinp
            write(ioout,'(30x,1pe12.6,a,a)') specwt(is,k),' * ',
     &                                       aspeci(is)
         enddo
      enddo

c --- Special restriction when performing Nitrate partition:  do not
c --- allow any computed species (this may be relaxed in the future)
      if(mnitrate.GT.0 .AND. nspeccmp.GT.0) then
         write(ioout,*)
         write(ioout,*)
         write(ioout,*)'*********************************************'
         write(ioout,*)'READCF:  Program terminating ---'
         write(ioout,*)'         No species may be computed when the'
         write(ioout,*)'         HNO3/NO3 partition option is active'
         write(ioout,*)'*********************************************'
         write(*,*)
         write(*,*)
         write(*,*)'*********************************************'
         write(*,*)'READCF:  Program terminating ---'
         write(*,*)'         No species may be computed when the'
         write(*,*)'         HNO3/NO3 partition option is active'
         write(*,*)'*********************************************'
         stop
      endif

c --- Stop run if any ammonia value is negative
      if(mnitrate.GT.0) then
        negnh3=0
        do i=1,12
          if(bcknh3(i).LT.0.) negnh3=negnh3+1
        enddo
        negtnh3=0
        do i=1,12
          if(bcktnh3(i).LT.0.) negtnh3=negtnh3+1
        enddo
        if(negnh3.GT.0.and.negtnh3.GT.0) then
         write(ioout,*)
         write(ioout,*)
         write(ioout,*)'*********************************************'
         write(ioout,*)'READCF:  Program terminating ---'
         write(ioout,*)'         12 non-negative background ammonia'
         write(ioout,*)'         concentrations must be provided'
         WRITE(ioout,*)'         either in BCKTNH3 or BCKNH3'
         write(ioout,*)'*********************************************'
         write(*,*)
         write(*,*)
         write(*,*)'*********************************************'
         write(*,*)'READCF:  Program terminating ---'
         write(*,*)'         12 non-negative background ammonia'
         write(*,*)'         concentrations must be provided'
         WRITE(*,*)'         either in BCKTNH3 or BCKNH3'
         write(*,*)'*********************************************'
         stop
        ELSEIF(negnh3.eq.0.and.negtnh3.eq.0) then
         write(ioout,*)
         write(ioout,*)
         write(ioout,*)'*********************************************'
         write(ioout,*)'READCF:  Program terminating ---'
         write(ioout,*)'         only one of the background ammonia'
         write(ioout,*)'         concentrations must be provided'
         WRITE(ioout,*)'         either BCKTNH3 or BCKNH3, not both'
         write(ioout,*)'*********************************************'
         write(*,*)
         write(*,*)
         write(*,*)'*********************************************'
         write(*,*)'READCF:  Program terminating ---'
         write(*,*)'         only one of the background ammonia'
         write(*,*)'         concentrations must be provided'
         WRITE(*,*)'         either BCKTNH3 or BCKNH3, not both'
         write(*,*)'*********************************************'
         stop
        ELSEIF(negnh3.eq.0.and.negtnh3.ne.12) then
         write(ioout,*)
         write(ioout,*)
         write(ioout,*)'*********************************************'
         write(ioout,*)'READCF:  Program terminating ---'
         write(ioout,*)'         only one of the background ammonia'
         write(ioout,*)'         concentrations must be provided'
         WRITE(ioout,*)'         either BCKTNH3 or BCKNH3, not both'
         write(ioout,*)'*********************************************'
         write(*,*)
         write(*,*)
         write(*,*)'*********************************************'
         write(*,*)'READCF:  Program terminating ---'
         write(*,*)'         only one of the background ammonia'
         write(*,*)'         concentrations must be provided'
         WRITE(*,*)'         either BCKTNH3 or BCKNH3, not both'
         write(*,*)'*********************************************'
         stop
        ELSEIF(negtnh3.eq.0.and.negnh3.ne.12) then
         write(ioout,*)
         write(ioout,*)
         write(ioout,*)'*********************************************'
         write(ioout,*)'READCF:  Program terminating ---'
         write(ioout,*)'         only one of the background ammonia'
         write(ioout,*)'         concentrations must be provided'
         WRITE(ioout,*)'         either BCKTNH3 or BCKNH3, not both'
         write(ioout,*)'*********************************************'
         write(*,*)
         write(*,*)
         write(*,*)'*********************************************'
         write(*,*)'READCF:  Program terminating ---'
         write(*,*)'         only one of the background ammonia'
         write(*,*)'         concentrations must be provided'
         WRITE(*,*)'         either BCKTNH3 or BCKNH3, not both'
         write(*,*)'*********************************************'
         stop
        ELSEIF(negtnh3.ne.0.and.negnh3.eq.12) then
         write(ioout,*)
         write(ioout,*)
         write(ioout,*)'*********************************************'
         write(ioout,*)'READCF:  Program terminating ---'
         write(ioout,*)'         At least one of the background ammonia'
         write(ioout,*)'         concentrations must be with 12 values,'
         WRITE(ioout,*)'         either BCKTNH3 or BCKNH3, not both'
         write(ioout,*)'*********************************************'
         write(*,*)
         write(*,*)
         write(*,*)'*********************************************'
         write(*,*)'READCF:  Program terminating ---'
         write(*,*)'         At least one of the background ammonia'
         write(*,*)'         concentrations must be with 12 values'
         WRITE(*,*)'         either BCKTNH3 or BCKNH3, not both'
         write(*,*)'*********************************************'
         stop
        ELSEIF(negnh3.ne.0.and.negtnh3.eq.12) then
         write(ioout,*)
         write(ioout,*)
         write(ioout,*)'*********************************************'
         write(ioout,*)'READCF:  Program terminating ---'
         write(ioout,*)'         At least one of the background ammonia'
         write(ioout,*)'         concentrations must be with 12 values,'
         WRITE(ioout,*)'         either BCKTNH3 or BCKNH3, not both'
         write(ioout,*)'*********************************************'
         write(*,*)
         write(*,*)
         write(*,*)'*********************************************'
         write(*,*)'READCF:  Program terminating ---'
         write(*,*)'         At least one of the background ammonia'
         write(*,*)'         concentrations must be with 12 values'
         WRITE(*,*)'         either BCKTNH3 or BCKNH3, not both'
         write(*,*)'*********************************************'
         stop
        endif
      endif
c
      return
      end
c----------------------------------------------------------------------
      subroutine uncomprs(xwork,ii,io,nwords,clabel,xdat)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level:  960422              UNCOMPRS
c                J. Scire, EARTH TECH
c
c --- PURPOSE:  Read a compressed data records and uncompress
c               the data
c
c --- INPUTS:
c        XWORK(nwork) - real array - Work array to temporarily store
c                                    compressed array
c                  II - integer    - Number of words in compressed
c                                    data record
c                  IO - integer    - Unit number of input file
c              NWORDS - integer    - Number of values in data array
c                                    after uncompression
c      PARAMETERS: IO6
c
c --- OUTPUTS:
c              CLABEL - char*15    - Character record header
c        XDAT(nwords) - real array - Array of uncompressed data to be
c                                    output
c
c --- UNCOMPRS called by: MAIN
c --- UNCOMPRS calls:     none
c----------------------------------------------------------------------
c
c --- Include parameter statements
      include 'params.utl'
c
      real xwork(ii),xdat(nwords)
      character*15 clabel
c
c --- Read the compressed data record
      read(io)clabel,xwork
c
c --- Uncompress the data
      jj=0
      do 100 i=1,ii
      if(xwork(i).gt.0.0)then
         jj=jj+1
         xdat(jj)=xwork(i)
      else
         nzero=-xwork(i)
         do j=1,nzero
            jj=jj+1
            xdat(jj)=0.0
         enddo
      endif
100   continue
c
c --- QA check that expanded array is correct size
      if(jj.ne.nwords)then
         write(io6,*)'ERROR in Subr. UNCOMPRS -- Expanded array ',
     1   'is not the correct size -- JJ = ',jj,' NWORDS = ',nwords
         stop
      endif
c
      return
      end
c----------------------------------------------------------------------
      subroutine readfn(ioin,ioout)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641     Level:  070627               READFN
c                J. Scire, D. Strimaitis
c
c --- PURPOSE:  Read the file containing the file names of the
c               input and output files of the run
c
c  UPDATES:
c     V1.54(060825) to V1.56(070627)
c               CTEMP array for filenames needs to be changed from
c               (132,8) to (132,9) (submitted by B. Brashers)
c     V1.5(060329) to V1.54(060825)
c               (CEC) In the checking against MXFIL, NMET need to be 
c               multiplied by 3 only for METFM=2 (when 2D met. files
c               are used, a total of 3 files, VIS, RHO and TEMP)
c     V1.44(060310) to V1.5(060329)
c               (DGS) Filnames changed from c*70 to c*132
c                     (for CALUTILS V2.3 and later)
c     V1.41(050302) to V1.44(060310)
c               (CEC) add a background pollutant files for ALM in 1 step
c               - MNITRATE=3
c     V1.1(010730) to V1.41(050302)
c               (CEC) add the possibility to read met data in other
c               files than CALMET.DAT
c     V1.1(000106) to V1.1(010730)
c               (DGS) add multiple met data file feature
c     V1.0(990806) to V1.1(000106)
c               (DGS) add met data file name for Nitrate partition
c
c --- INPUTS:
c         IOIN - integer - Unit number of input control file
c        IOOUT - integer - Unit number of output list file
c
c --- OUTPUT:
c
c ---    Common block /CTRL/ variables:
c           nfiles,nmet,inunit(mxfile),metfm
c ---    Common block /FILNAM/ variables:
c           utllst,moddatl(mxfile),utldat,utlmetl(mxfile),m2drhul(mxfile),
c            m2dtmpl(mxfile),m2drhol(mxfile),met1d
c ---    Common block /FILLOG/ variables:
c           lcfiles
c
c --- READFN called by:  READCF
c --- READFN calls:      READIN, FILCASE
c----------------------------------------------------------------------
      include 'params.utl'
c
      character*4 ctemp(132,9)
      character*12 cvdic(13)


      integer ivleng(13),ivtype(13)
      logical lecho
c
c --- Include common blocks
      include 'ctrl.utl'
      include 'filnam.utl'
c
      data lecho/.false./
      data cvdic/'UTLLST','MODDAT','UTLDAT','UTLMET',
     &           'M2DRHU','M2DTMP','M2DRHO','MET1D',
     &           'BCKGALM','METFM','NFILES','NMET',
     &           'LCFILES'/
c
      data ivleng/9*132,4*1/
      data ivtype/9*4,2,2,2,3/
c --- Note that variable types (ivtype) are coded as:
c          0 = null
c          1 = real
c          2 = integer
c          3 = logical
c          4 = character

c --------------------
c --- Process group 0a
c --------------------

c --- Read the output file names from the data file
c
c ---    Initialize the temporary arrays for the names
         do i=1,9
            do j=1,132
               ctemp(j,i)(1:1)=' '
            enddo
         enddo
c
c ---    Read the group 0a file name variables for the model run
         call readin(cvdic(1),ivleng(1),ivtype(1),ioin,ioout,lecho,
     1   ctemp(1,1),ctemp(1,2),ctemp(1,3),ctemp(1,4),ctemp(1,5),
     2   ctemp(1,6),ctemp(1,7),ctemp(1,8),ctemp(1,9),metfm,nfiles,nmet,
     3   lcfiles,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     4   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     5   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     6   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum)
c
c ---    Prepare any filenames included in the I/O file by erasing
c ---    the default filename set above
         if(ctemp(1,1)(1:1).ne.' ')utllst=' '
         if(ctemp(1,3)(1:1).ne.' ')utldat=' '
         if(ctemp(1,4)(1:1).ne.' ')utlmetl(1)=' '
         if(ctemp(1,5)(1:1).ne.' ')m2drhul(1)=' '
         if(ctemp(1,6)(1:1).ne.' ')m2dtmpl(1)=' '
         if(ctemp(1,7)(1:1).ne.' ')m2drhol(1)=' '
         IF(ctemp(1,9)(1:1).ne.' ')bckgalm=' '
c
c ---    Transfer the char*4 data into the char*132 variables
         do j=1,132
            if(ctemp(j,1)(1:1).ne.' ')utllst(j:j)=ctemp(j,1)(1:1)
            if(ctemp(j,3)(1:1).ne.' ')utldat(j:j)=ctemp(j,3)(1:1)
            if(ctemp(j,4)(1:1).ne.' ')utlmetl(1)(j:j)=ctemp(j,4)(1:1)
            if(ctemp(j,5)(1:1).ne.' ')m2drhul(1)(j:j)=ctemp(j,5)(1:1)
            if(ctemp(j,6)(1:1).ne.' ')m2dtmpl(1)(j:j)=ctemp(j,6)(1:1)
            if(ctemp(j,7)(1:1).ne.' ')m2drhol(1)(j:j)=ctemp(j,7)(1:1)
            IF(ctemp(j,9)(1:1).ne.' ')bckgalm(j:j)=ctemp(j,9)(1:1)
         enddo

c ---    Convert the file names to the proper case
         call filcase(lcfiles,utllst)
         call filcase(lcfiles,utldat)
         call filcase(lcfiles,bckgalm)
         call filcase(lcfiles,utlmetl(1))
         call filcase(lcfiles,m2drhul(1))
         call filcase(lcfiles,m2dtmpl(1))
         call filcase(lcfiles,m2drhol(1))

c --------------------
c --- Process group 0b
c --------------------
c
      if(metfm.eq.1.and.nmet.ne.1) then
      write(*,*)'error - IF METFM=1, NMET must be equal to 1'
      stop
      endif
c
      if(metfm.eq.0.or.metfm.eq.1) then
      nfilmax=nfiles+nmet
      elseif(metfm.eq.2) then
      nfilmax=nfiles+(nmet*3)
      else
      write(*,*)'error METFM must be 0, 1 or 2'
      stop
      endif
c
      nf=0
      nm=0
      nm5=0
      nm6=0
      nm7=0
      do n=1,nfilmax
c
c ---    Initialize the temporary arrays for the input file names
         do j=1,132
            ctemp(j,2)(1:1)=' '
            ctemp(j,4)(1:1)=' '
            ctemp(j,5)(1:1)=' '
            ctemp(j,6)(1:1)=' '
            ctemp(j,7)(1:1)=' '
            ctemp(j,8)(1:1)=' '
         enddo
c
c ---    Read a group 0b file name
         call readin(cvdic(1),ivleng(1),ivtype(1),ioin,ioout,lecho,
     1   ctemp(1,1),ctemp(1,2),ctemp(1,3),ctemp(1,4),ctemp(1,5),
     2   ctemp(1,6),ctemp(1,7),ctemp(1,8),ctemp(1,9),metfm,nfiles,nmet,
     3   lcfiles,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     4   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     5   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     6   idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum)

c ---    Process name by file type
         if(ctemp(1,2)(1:1).ne.' ') then
c ---       CALPUFF.DAT type
            nf=nf+1
            nf=MIN(nf,mxfile)
c ---       Erase the default filename
            moddatl(nf)=' '
c ---       Transfer the char*4 data into the char*132 variables
            do j=1,132
               if(ctemp(j,2)(1:1).ne.' ')moddatl(nf)(j:j)=
     &                                   ctemp(j,2)(1:1)
            enddo
c ---       Convert the file name to the proper case
            call filcase(lcfiles,moddatl(nf))
c ---       Set unit number for this file
            inunit(nf)=in1-1+nf
c
         elseif(ctemp(1,4)(1:1).ne.' ') then
c ---       CALMET.DAT type
            nm=nm+1
            nm=MIN(nm,mxfile)
c ---       Erase the default filename
            utlmetl(nm)=' '
c ---       Transfer the char*4 data into the char*132 variables
            do j=1,132
               if(ctemp(j,4)(1:1).ne.' ')utlmetl(nm)(j:j)=
     &                                   ctemp(j,4)(1:1)
            enddo
c ---       Convert the file name to the proper case
            call filcase(lcfiles,utlmetl(nm))
c
         elseif(ctemp(1,5)(1:1).ne.' ') then
c ---       RELATIVEHUM.DAT type
            nm5=nm5+1
            nm5=MIN(nm5,mxfile)
c ---       Erase the default filename
            m2drhul(nm5)=' '
c ---       Transfer the char*4 data into the char*132 variables
            do j=1,132
               if(ctemp(j,5)(1:1).ne.' ')m2drhul(nm5)(j:j)=
     &                                   ctemp(j,5)(1:1)
            enddo
c ---       Convert the file name to the proper case
            call filcase(lcfiles,m2drhul(nm5))
c
         elseif(ctemp(1,6)(1:1).ne.' ') then
c ---       TEMPERATURE.DAT type
            nm6=nm6+1
            nm6=MIN(nm6,mxfile)
c ---       Erase the default filename
            m2dtmpl(nm6)=' '
c ---       Transfer the char*4 data into the char*132 variables
            do j=1,132
               if(ctemp(j,6)(1:1).ne.' ')m2dtmpl(nm6)(j:j)=
     &                                   ctemp(j,6)(1:1)
            enddo
c ---       Convert the file name to the proper case
            call filcase(lcfiles,m2dtmpl(nm6))
c
         elseif(ctemp(1,7)(1:1).ne.' ') then
c ---       RHOAIR.DAT type
            nm7=nm7+1
            nm7=MIN(nm7,mxfile)
c ---       Erase the default filename
            m2drhol(nm7)=' '
c ---       Transfer the char*4 data into the char*132 variables
            do j=1,132
               if(ctemp(j,7)(1:1).ne.' ')m2drhol(nm7)(j:j)=
     &                                   ctemp(j,7)(1:1)
            enddo
c ---       Convert the file name to the proper case
            call filcase(lcfiles,m2drhol(nm7))
c
         elseif(ctemp(1,8)(1:1).ne.' ') then
c ---       MET_1D.DAT type
c ---       Erase the default filename
            met1d=' '
c ---       Transfer the char*4 data into the char*132 variables
            do j=1,132
               if(ctemp(j,8)(1:1).ne.' ')met1d(j:j)=
     &                                   ctemp(j,8)(1:1)
            enddo
c ---       Convert the file name to the proper case
            call filcase(lcfiles,met1d)
         else
            if (metfm.eq.0) then
            write(ioout,*)'READFN: Too few file names in Subgroup 0b'
            write(ioout,*)'Expected NMET + NFILES =',nfilmax
            write(ioout,*)'Found NMET, NFILES  = ',nm,nf
            write(*,*)'READFN: Too few file names in Subgroup 0b'
            write(*,*)'Expected NMET + NFILES = ',nfilmax
            write(*,*)'Found NMET, NFILES = ',nm,nf
            stop
            elseif(metfm.eq.2) then
            write(ioout,*)'READFN: Too few file names in Subgroup 0b'
            write(ioout,*)'Expected NMET + NFILES =',nfilmax
            write(ioout,*)'Found NMET, NFILES  = ',nm5+nm6+nm7,nf
            write(*,*)'READFN: Too few file names in Subgroup 0b'
            write(*,*)'Expected NMET + NFILES = ',nfilmax
            write(*,*)'Found NMET, NFILES = ',nm5+nm6+nm7,nf
            stop
            endif
         endif
c
      enddo

c --- Fatal error if number of files exceeds MXFILE
      if((nfiles+1).GT.mxfile) then
         write(ioout,*)'READFN: Too many CALPUFF.DAT files included'
         write(ioout,*)'nfiles,mxfile = ',nfiles+1,mxfile
         write(ioout,*)'Increase the MXFILE parameter in PARAMS.UTL'
         write(ioout,*)'and recompile the program'
         write(*,*)'READFN: Too many CALPUFF.DAT files included'
         write(*,*)'nfiles,mxfile = ',nfiles+1,mxfile
         write(*,*)'Increase the MXFILE parameter in PARAMS.UTL'
         write(*,*)'and recompile the program'
         stop
      elseif(nmet.GT.mxfile) then
         write(ioout,*)'READFN: Too many CALMET.DAT files included'
         write(ioout,*)'nmet,mxfile = ',nmet,mxfile
         write(ioout,*)'Increase the MXFILE parameter in PARAMS.UTL'
         write(ioout,*)'and recompile the program'
         write(*,*)'READFN: Too many CALMET.DAT files included'
         write(*,*)'nmet,mxfile = ',nmet,mxfile
         write(*,*)'Increase the MXFILE parameter in PARAMS.UTL'
         write(*,*)'and recompile the program'
         stop
c     elseif((nmet*3).GT.mxfile) then
c --- (cec)
      elseif((nmet*3).GT.mxfile.and.metfm.eq.2) then
         write(ioout,*)'READFN: Too many MET data files included'
         write(ioout,*)'nmet*3,mxfile = ',nmet*3,mxfile
         write(ioout,*)'Increase the MXFILE parameter in PARAMS.UTL'
         write(ioout,*)'and recompile the program'
         write(*,*)'READFN: Too many CALMET.DAT files included'
         write(*,*)'nmet*3,mxfile = ',nmet*3,mxfile
         write(*,*)'Increase the MXFILE parameter in PARAMS.UTL'
         write(*,*)'and recompile the program'
         stop
      endif

      return
      end
c----------------------------------------------------------------------
      subroutine wrfiles
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641     Level:  060329             WRFILES
c                J. Scire, SRC
c
c --- PURPOSE:  Write a table of the input and output file names
c               for the current run
c
c  UPDATES:
c     V1.44(060310) to V1.5(060329)
c               (DGS) Filnames changed from c*70 to c*132
c                     (for CALUTILS V2.3 and later)
c     V1.41(050302) to V1.44(060310)
c               (CEC) add names for background pollutant for MNITRATE=3
c     V1.1(010730) to V1.41(050302)
c               (CEC) add names of met. data files if metfm=2
c     V1.0(000106) to V1.1(010730)
c               (DGS) add multiple met data file feature
c     V1.0(990806) to V1.1(000106)
c               (DGS) add met data file name for Nitrate partition
c
c --- INPUTS:
c
c     Common block /CTRL/ variables:
c         NFILES, INUNIT(mxfile), MNITRATE, NMET
c     Common block /FILNAM/ variables:
c         UTLINP,MODDATL,UTLLST,UTLDAT,UTLMETL,M2DRHUL,M2DTMPL,M2DRHOL
c     Parameters: IN2, IN4, IO1, IO2, MXFILE
c
c --- OUTPUT:  none
c
c --- WRFILES called by:  MAIN
c --- WRFILES calls:      none
c----------------------------------------------------------------------
c
c --- Include parameter statements
      include 'params.utl'
c
c --- Include common blocks
      include 'ctrl.utl'
      include 'filnam.utl'
c
c --------------------------------------------------
c --- Write the list of INPUT files used in this run
c --------------------------------------------------
      write(io1,10)
10    format(//1x,13('----------')/10x,'INPUT FILES'//
     1 1x,'Default Name',5x,'Unit No.',5x,'File Name and Path'/
     2 1x,'------------',5x,'--------',5x,'------------------')
c
c --- POSTUTIL.INP
      write(io1,12)'POSTUTIL.INP',in2,utlinp
12    format(1x,a12,7x,i3,8x,a132)
c
c --- Concentration/wet flux/dry flux file (CALPUFF.DAT)
      write(io1,12)'CALPUFF.DAT',in1,moddatl(1)
      do nf=2,nfiles
         write(io1,12)' (none) ',inunit(nf),moddatl(nf)
      enddo
c
c --- Met data files for Nitrate partition (MET.DAT)
      if(mnitrate.EQ.1.and.metfm.eq.0) then
         write(io1,12)'MET.DAT',in4,utlmetl(1)
         do nf=2,nmet
            write(io1,12)' (none) ',in4,utlmetl(nf)
         enddo
      endif
c
c --- Met data files for Nitrate partition (MET.DAT)
      if(mnitrate.EQ.1.and.metfm.eq.2) then
         write(io1,12)'M2D.RH',in5,m2drhul(1)
         do nf=2,nmet
          write(io1,12)'M2D.RH',in5,m2drhul(nf)
         enddo
         WRITE(io1,12)'M2D.TEMP',in6,m2dtmpl(1)
         do nf=2,nmet
          write(io1,12)'M2D.TEMP',in6,m2dtmpl(nf)
         enddo
         WRITE(io1,12)'M2D.RHO',in7,m2drhol(1)
         do nf=2,nmet
          write(io1,12)'M2D.RHO',in7,m2drhol(nf)
         enddo
      endif
c
c --- Met data files for Nitrate partition (MET.DAT)
      if(mnitrate.EQ.1.and.metfm.eq.1) then
         write(io1,12)'MET_1D',in5,met1d
      endif
c
c --- Background pollutant for ALM application in 1 step (BCKGALM) - MNITRATE=3
      IF(mnitrate.EQ.3) then
        WRITE(io1,12)'BCKGALM.DAT',in8,bckgalm
      endif
c
c
c ---------------------------------------------------
c --- Write the list of OUTPUT files used in this run
c ---------------------------------------------------
      write(io1,30)
30    format(//1x,13('----------')/10x,'OUTPUT FILES'//
     1 1x,'Default Name',5x,'Unit No.',5x,'File Name and Path'/
     2 1x,'------------',5x,'--------',5x,'------------------')
c
c --- POSTUTIL.LST
      write(io1,12)'POSTUTIL.LST',io1,utllst
c
c --- POSTUTIL.LST
      write(io1,12)'MODEL.DAT',io2,utldat
c
      return
      end
c----------------------------------------------------------------------
      subroutine xtract(datarr,nxmax,nymax,nxact,nyact,outarr)
c----------------------------------------------------------------------
c
c --- CALPUFF    Version: 1.641    Level: 971107                 XTRACT
c                J. Scire, SRC
c
c --- PURPOSE:  Extract the active portion of a 2-D array
c
c --- UPDATE
c --- V4.0-V5.0     971107  (DGS): NYACT compared to NYMAX (not NXMAX)
c
c --- INPUTS:
c   DATARR(nxmax,nymax) - real    - Full data array
c                 NXMAX - integer - First dimension of data array
c                 NYMAX - integer - Second dimension of data array
c                 NXACT - integer - Number of active elements of the
c                                   array (first dimension)
c                 NYACT - integer - Number of active elements of the
c                                   array (second dimension)
c        Parameters:
c           IO6
c
c --- OUTPUT:
c   OUTARR(nxact,nyact) - real    - Output array consisting only
c                                   of the active elements of the
c                                   full input array
c
c --- XTRACT called by:  OUTPUT
c --- XTRACT calls:      none
c
c----------------------------------------------------------------------
c
      include 'params.utl'
c
      real datarr(nxmax,nymax),outarr(nxact,nyact)
c
c --- Check that values of array dimensions are reasonable
      if(nxact.le.0.or.nxact.gt.nxmax.or.
     1   nyact.le.0.or.nyact.gt.nymax)then
            write(io6,*)'ERROR in subr. XTRACT -- Invalid values ',
     1      'of array dimensions input -- NXACT = ',nxact,' NYACT = ',
     2      nyact,' NXMAX = ',nxmax,' NYMAX = ',nymax
            stop
      endif
c
c --- Extract the active portion of the input data array
      do 100 i=1,nxact
      do 100 j=1,nyact
      outarr(i,j)=datarr(i,j)
100   continue
c
      return
      end
c----------------------------------------------------------------------
      subroutine wrdat(iounit,cname,outarr,nx,ny)
c----------------------------------------------------------------------
c
c --- CALPUFF    Version: 1.641    Level: 940430                  WRDAT
c                J. Scire, SRC
c
c --- PURPOSE:  Write a gridded concentration or dry/wet
c               flux data record
c               (one 15-character identifier and a 2-D data array)
c
c --- INPUTS:
c          IOUNIT - integer      - Fortran unit no. of output file
c           CNAME - character*15 - Species identifier
c   OUTARR(nx,ny) - real array   - Array of concentration data (g/m**3)
c                                  or dry/wet flux data (g/m**2/s)
c              NX - integer      - Number of sampling grid points in the
c                                  X direction
c              NY - integer      - Number of sampling grid points in the
c                                  Y direction
c
c --- OUTPUT:  none
c
c --- WRDAT called by:  OUTPUT
c --- WRDAT calls:      none
c
c----------------------------------------------------------------------
c
      real outarr(nx,ny)
      character*15 cname
c
      write(iounit)cname,outarr
c
      return
      end
c----------------------------------------------------------------------
      subroutine comprs(xdat,nwords,xwork,nwork,clabel,io)
c----------------------------------------------------------------------
c
c --- CALPUFF    Version: 1.641    Level: 960521                 COMPRS
c                J. Scire, EARTH TECH
c
c --- PURPOSE:  Compress an array of concentrations, dry fluxes,
c               or wet fluxes by replacing strings of zero values
c               with a negative code indicating the number of zero
c               values
c
c --- INPUTS:
c        XDAT(nwords) - real array - Array of uncompressed data to be
c                                    output
c              NWORDS - integer    - Number of values in data array
c        XWORK(nwork) - real array - Work array to temporarily store
c                                    compressed array
c               NWORK - integer    - Dimension of work array - NWORK
c                                    must be >= NWORDS
c              CLABEL - char*15    - Character record header
c                  IO - integer    - Unit number of output file
c
c     Parameters: IO6
c
c --- OUTPUT:  none
c
c --- COMPRS called by: OUTPUT
c --- COMPRS calls:     WRDAT
c----------------------------------------------------------------------
c
c --- Include parameter statements
      include 'params.utl'
c
      real xdat(nwords),xwork(nwork)
      character*15 clabel
c
c --- Check that work array is sized large enough
      if(nwork.lt.nwords)then
         write(io6,*)'ERROR in Subr. COMPRS -- Work array ',
     1   'dimension is too small -- NWORK = ',nwork,' NWORDS = ',
     2   nwords
         stop
      endif
c
c --- Replace all zeroes with negative coded integer
      nzero=0
      ii=0
      do 100 i=1,nwords
c
         if(xdat(i).eq.0.0)then
            nzero=nzero+1
            go to 100
         else if(xdat(i).lt.0.0)then
            write(io6,*)'ERROR in Subr. COMPRS -- Negative value ',
     1      'encountered with COMPRESS option on -- I = ',i,
     2      ' XDAT(i) = ',xdat(i)
            write(io6,*)'COMPRESS option cannot be used when data ',
     1      'values are negative'
            stop
         endif
c
         if(nzero.eq.0)then
            ii=ii+1
            xwork(ii)=xdat(i)
         else
            ii=ii+1
            xwork(ii)=-(float(nzero)+0.0001)
            nzero=0
            ii=ii+1
            xwork(ii)=xdat(i)
         endif
100   continue
c
      if(nzero.gt.0)then
         ii=ii+1
         xwork(ii)=-(float(nzero)+0.0001)
      endif
c
c --- Write the data records (header, compressed data record)
      write(io)ii
      call wrdat(io,clabel,xwork,ii,1)
c
      return
      end
c----------------------------------------------------------------------
      subroutine timeset
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 060329                TIMESET
c                D. Strimaitis, Earth Tech
c
c --- PURPOSE:  Set date/time variables at the start of the run
c
c  UPDATES:
c     V1.3(030402) to V1.5(060329)
c               (DGS) add list-file unit to INCRS
c     V1.0(000106) to V1.3(030402)
c               (DGS) add list-file unit to GRDAY, INCR
c
c --- INPUT:
c
c     Common block /CTRL/
c        ISYR, ISMO, ISDY, ISHR, ISSEC, NPER
c     Common block /HEAD/
c        NSECDT, MAVGPD
c
c --- OUTPUT:
c
c     Common block /CTRL/
c        ISDATHR, IEDATHR, IESEC
c     Common block /DATEHR/
c        nyrb,nmob,ndayb,njulb,nhrb,nsecb,ndathrb,
c        nyre,nmoe,ndaye,njule,nhre,nsece,ndathre,
c        nyrm,njulm,nhrm,nsecm,ndathrm,
c        nhrind,nsecdta
c        
c
c --- TIMESET called by: MAIN
c --- TIMESET calls:     GRDAY,  INCRS
c----------------------------------------------------------------------
      include 'params.utl'
c
c --- Include common blocks
      include 'ctrl.utl'
      include 'datehr.utl'
      include 'head.utl'

c --- Mark periods with both the beginning and ending time
c --- Length of avg period is NSECDTA=NSECDT*MAVGPD in seconds
c --- Half of period is NSECDTA2 seconds
      nsecdta=nsecdt*mavgpd
      nsecdta2=nsecdta/2
c
c --- Set beginning time of first period
      nyrb=isyr
      nmob=ismo
      ndayb=isdy
      nhrb=ishr
      nsecb=issec
      njulb=jsday
      ndathrb=nyrb*100000+njulb*100+nhrb
c
c --- Set time at the middle of first period
      nyrm=nyrb
      njulm=njulb
      nhrm=nhrb
      nsecm=nsecb
      call INCRS(io1,nyrm,njulm,nhrm,nsecm,nsecdta2)
      ndathrm=nyrm*100000+njulm*100+nhrm
c
c --- Set ending time of first period
      nyre=nyrb
      njule=njulb
      nhre=nhrb
      nsece=nsecb
      call INCRS(io1,nyre,njule,nhre,nsece,nsecdta)
      call GRDAY(io1,nyre,njule,nmoe,ndaye)
      ndathre=nyre*100000+njule*100+nhre
c
c --- Use midpoint of period to assign an hour index (1-24)
      nhrind=nhrm+1
c
c --- Set date-time variables for start and end of processed period
      isdathr=ndathrb
      iedathr=isdathr
      ieyr=isyr
      iemo=ismo
      iedy=isdy
      iehr=ishr
      iesec=issec
      jeday=jsday
c --- Number of hours in period (as a real number)
      hours=FLOAT(nper*nsecdta)/3600.
c --- Number of whole hours
      ihours=hours
c --- Difference in seconds
      sec=3600.*(hours-ihours)
      isec=sec
c --- Compute date and time at end of period
      if(ihours.GT.0) call INCR(io1,ieyr,jeday,iehr,ihours)
      if(isec.GT.0) call INCRS(io1,ieyr,jeday,iehr,iesec,isec)
      iedathr=ieyr*100000+jeday*100+iehr

      return
      end
c----------------------------------------------------------------------
      subroutine timeup(i)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 060329                 TIMEUP
c                D. Strimaitis, Earth Tech
c
c --- PURPOSE:  Update date/time variables by 1 timestep
c
c  UPDATES:
c     V1.3(030402) to V1.5(060329)
c               (DGS) add list-file unit to INCRS
c     V1.0(000106) to V1.3(030402)
c               (DGS) add list-file unit to GRDAY
c
c --- INPUT:
c               I - integer      - Direction of update
c                                  i=+1  Forward 1 step
c                                  i=-1  Backward 1 step
c
c     Common block /DATEHR/
c        nyrb,nmob,ndayb,njulb,nhrb,nsecb,ndathrb,
c        nyre,nmoe,ndaye,njule,nhre,nsece,ndathre,
c        nyrm,njulm,nhrm,nsecm,ndathrm,nsecdta
c
c --- OUTPUT:
c
c     Common block /DATEHR/
c        nyrb,nmob,ndayb,njulb,nhrb,nsecb,ndathrb,
c        nyre,nmoe,ndaye,njule,nhre,nsece,ndathre,
c        nyrm,njulm,nhrm,nsecm,ndathrm,
c        nhrind
c        
c
c --- TIMEUP called by: MAIN
c --- TIMEUP calls:     GRDAY,  INCRS
c----------------------------------------------------------------------
      include 'params.utl'
c --- Include common blocks
      include 'datehr.utl'

c --- Set the update step (seconds)
      nstep=nsecdta*i

      if(i.EQ.1) then
c ---    Forward 1 step
c ---    Beginning
         nyrb=nyre
         nmob=nmoe
         ndayb=ndaye
         njulb=njule
         nhrb=nhre
         nsecb=nsece
         ndathrb=ndathre
c ---    Mid-point
         call INCRS(io1,nyrm,njulm,nhrm,nsecm,nstep)
         ndathrm=nyrm*100000+njulm*100+nhrm
c ---    Ending
         call INCRS(io1,nyre,njule,nhre,nsece,nstep)
         call GRDAY(io1,nyre,njule,nmoe,ndaye)
         ndathre=nyre*100000+njule*100+nhre

      elseif(i.EQ.-1) then
c ---    Backward 1 step
c ---    Ending
         nyre=nyrb
         nmoe=nmob
         ndaye=ndayb
         njule=njulb
         nhre=nhrb
         nsece=nsecb
         ndathre=ndathrb
c ---    Mid-point
         call INCRS(io1,nyrm,njulm,nhrm,nsecm,nstep)
         ndathrm=nyrm*100000+njulm*100+nhrm
c ---    Beginning
         call INCRS(io1,nyrb,njulb,nhrb,nsecb,nstep)
         call GRDAY(io1,nyrb,njulb,nmob,ndayb)
         ndathre=nyre*100000+njule*100+nhre

      else
c ---    Argument must be either -1 or 1
         write(*,*)
         write(*,*)'FATAL ERROR in TIMEUP---'
         write(*,*)'The update step is not -1 or 1: ',i
         stop
      endif

c --- Use midpoint of period to assign an hour index (1-24)
      nhrind=nhrm+1

      return
      end
c----------------------------------------------------------------------
      subroutine tperiod(ibe,ndhr,nsec,ndhrb,nsecb,ndhre,nsece,ilocate)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641     Level: 990709               TPERIOD
c
c     Taken from CALPUFF:
c
c --- CALPUFF    Version: 6.0       Level: 990709               TPERIOD
c                D. Strimaitis,  SRC
c
c --- PURPOSE:  Tests year/day/hour and seconds variables at beginning
c               and end of period against target date-time to see if
c               target lies within period
c
c --- INPUTS:
c           IBE - integer - Treatment of Beginning and End of period
c                           -1: Beginning time within period (not end)
c                            0: Beginning & End are within period
c                            1: Ending time within period (not begin)
c          NDHR - integer - YYYYJJJHH of target (year-day-hour)
c          NSEC - integer - SSSS of target (seconds)
c         NDHRB - integer - YYYYJJJHH at beginning of period
c         NSECB - integer - SSSS at beginning of period
c         NDHRE - integer - YYYYJJJHH at end of period
c         NSECE - integer - SSSS at end of period
c
c
c --- OUTPUT:
c       ILOCATE - integer - Location of target relative to period
c                           -1: target preceeds period
c                            0: target within period
c                            1: target lies beyond period
c
c --- TPERIOD called by:  COMP, METQA, EMQA, SIGSET
c                         RDMET, RDSIG, RDISC,RDPLM, RDMET4
c --- TPERIOD calls:      none
c----------------------------------------------------------------------

      ilocate=0
c
      if(ndhr.LT.ndhrb) then
c ---    Target date-hr before start of period
         ilocate=-1
      elseif(ndhr.GT.ndhre) then
c ---    Target date-hr after end of period
         ilocate=1
      elseif(ndhr.EQ.ndhrb) then
         if(nsec.LT.nsecb) then
c ---       Target time before start of period
            ilocate=-1
         elseif(ibe.EQ.1 .AND. nsec.EQ.nsecb) then
c ---       Target time before start of period
            ilocate=-1
         elseif(ndhrb.EQ.ndhre) then
            if(nsec.GT.nsece) then
c ---          Target time after end of period
               ilocate=1
            elseif(ibe.EQ.-1 .AND. nsec.EQ.nsece) then
c ---          Target time after end of period
               ilocate=1
            endif
         endif
      elseif(ndhr.EQ.ndhre) then
         if(nsec.GT.nsece) then
c ---       Target time after end of period
            ilocate=1
         elseif(ibe.EQ.-1 .AND. nsec.EQ.nsece) then
c ---       Target time after end of period
            ilocate=1
         endif
      endif

      return
      end
c----------------------------------------------------------------------
      subroutine partition(ldb)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 100917              PARTITION
c                D. Strimaitis
c
c --- PURPOSE:  Compute the partition of total nitrate between HNO3
c               and NO3 concentrations at each receptor
c
c  UPDATES:
c     V1.44(060310) to V1.63(100917)
c               (DGS) Add equilibrium method choice (MEQPHASE)
c               (DGS) Fix initial NH3 assignment typo (cnh3ppb=-999.)
c     V1.43(060206) to V1.44(060310)
c               (CEC) FLGNH3 has been removed and replace by NH3TYP
c                     for the choice of NH3 source for NO3/HNO3
c                     repartitioning
c     V1.41(050302) to V1.43(060206)
c               (CEC) Add a check if lnh3 and FLGNH3 agree to make sure
c                     that the NH3 used is the one the user wants.
c     V1.4(040818) to V1.41(050302)
c               (CEC) Add possibility to read met. data from 3 files
c                     output of CALPUFF instead of CALMET.DAT
c     V1.1(000106) to V1.4(040818)
c               (DGS) Add monthly background ammonia values
c
c --- INPUTS:
c             LDB - logical - Flag for debug output
c
c     Common block /CONC/ variables:
c           CONCG(mxgx,mxgy,mxspec),CONCD(mxdrec,mxspec),
c           CONCT(mxctrec,mxspec)
c     Common block /CTRL/ variables:
c           ISPECNO3(6), BCKNH3(12), MEQPHASE
c     Common block /DATEHR/ variables:
c           NMOB
c     Common block /HEAD/ variables:
c           IELMET, JELMET, DELX, DELY,
c           ISASTR, JSASTR, NGX, NGY, MESHDN,
c           NDREC, XREC(mxdrec), YREC(mxdrec),
c           NCTREC, XCTR(mxctrec), YCTR(mxctrec),
c           LSGRID, LDISC, LCTSG
c     Parameters:
c           MXGX, MXGY, MXDREC, MXCTREC, MXSPEC, IO1
c
c --- OUTPUT:
c     Common block /CONC/ variables:
c           CONCG(mxgx,mxgy,mxspec),CONCD(mxdrec,mxspec),
c           CONCT(mxctrec,mxspec)
c
c --- PARTITION called by:  MAIN
c --- PARTITION calls:      EXMET, CHEMEQ, ISODRIVER
c----------------------------------------------------------------------
c --- Include parameters
      include 'params.utl'

c --- Include common blocks
      include 'conc.utl'
      include 'ctrl.utl'
      include 'datehr.utl'
      include 'head.utl'
      include 'methr.utl'

      logical ldb,lnh3

      real tno3floor
      logical l_tno3floor

c --- Set local control for minimum TNO3 for ISODRIVER
c ----------------------------------------------------
c ---     Selecting l_tno3floor=.TRUE. uses the cut-off TNO3
c ---     concentration as a floor, and computes NO3 corresponding
c ---     to this floor for all TNO3 less than this floor.  The
c ---     resulting ratio of NO3/TNO3(floor) is then multiplied by
c ---     the actual TNO3 to obtain the final NO3 concentration.
c ---     Selecting l_tno3floor=.FALSE. retains the original API
c ---     method and NO3=0.0 for all TNO3<TNO3(floor).
c ---     Note: TNO3(floor) = 1.0E-10 mol/m3 = .0062 ug/m3 (approx)
      data l_tno3floor/.TRUE./
      data tno3floor/1.0E-10/

c --- Set grid cell length in meters
      dgrid=1000.*delx

c --- Use Monthly background ammonia BCKNH3 or BCKTNH3
      cnh3ppb=-999.
      IF(bcknh3(nmob).NE.-999.) then
      cnh3ppb=bcknh3(nmob)
      ELSEIF(bcktnh3(nmob).NE.-999.) then
      cnh3ppb=bcktnh3(nmob)
      else
       IF(nh3typ.eq.1.or.nh3typ.eq.3) then
       WRITE(*,*)' '
       WRITE(*,*)'FATAL ERROR - Review message written'
       WRITE(*,*)'in the list-file'
       WRITE(io1,*)' '
       WRITE(io1,*)'FATAL ERROR - monthly background ammonia = -999'
       stop
       endif
      endif
c -- Choice to use NH3 background or not.
      IF(nh3typ.eq.0) then
      cnh3ppb1=0
      ELSEIF(nh3typ.eq.1.or.nh3typ.eq.3) then
      cnh3ppb1=cnh3ppb
      else
      WRITE(*,*)' '
      WRITE(*,*)'FATAL ERROR - Review message written'
      WRITE(*,*)'in the list-file'
      WRITE(io1,*)' '
      WRITE(io1,*)'ERROR- nh3typ equal to 2 or 4 can not be used'
      WRITE(io1,*)'with MNITRATE=1'
      stop
      endif
c --- Set local flag indicating presence of modeled ammonia
      lnh3=.TRUE.
      if(ispecno3(4).EQ.0.and.ispecno3(7).EQ.0) lnh3=.FALSE.
      IF(.not.lnh3) then
        IF(nh3typ.eq.0) then
      WRITE(*,*)' '
      WRITE(*,*)'FATAL ERROR - Review message written'
      WRITE(*,*)'in the list-file'
      WRITE(io1,*)' '
      WRITE(io1,*)'ERROR - no NH3 or TNH3 in concentration file'
      WRITE(io1,*)' NH3 from background need to be used - nh3typ=1 or 3'
        stop
        endif
      endif
c
c --- CEC (060206) - compare FLAG entered by the user about
c                    where NH3 come from with what is found here.
      IF(lnh3) then
       if (nh3typ.eq.0.or.nh3typ.eq.1.or.nh3typ.eq.2) then
c       WRITE(*,*)'NH3 species in CALPUFF files is used for NO3/HNO3'
c       WRITE(*,*)'repartioning'
       else
       WRITE(*,*)' '
       WRITE(*,*)'FATAL ERROR - Review message written'
       WRITE(*,*)'in the list-file'
       WRITE(io1,*)' '
       WRITE(io1,*)'FATAL ERROR - NH3 species is listed in'
       WRITE(io1,*)'subgroup (2a&2b) as a processed species.'
       WRITE(io1,*)'DO NOT match options selected in control file'
c       WRITE(*,*)'If the user wants to use it to repartition NO3/HNO3,'
c       WRITE(*,*)'NH3TYP needs to be equal to 0 or 1'
c       WRITE(*,*)'If the user wants to use BCKNH3 or BCKTNH3 alone -'
c       WRITE(*,*)'remove NH3 and TNH3 species from the processed'
c       WRITE(*,*)'species in Subgroup (2a) and (2b). IF the user wants'
c       WRITE(*,*)'to use BCKNH3 in addition make NH3TYP to be 1'
       stop
       endif
      ELSE
       if (nh3typ.eq.0.or.nh3typ.eq.1.or.nh3typ.eq.2) then
       WRITE(*,*)' '
       WRITE(*,*)'FATAL ERROR - Review message written'
       WRITE(*,*)'in the list-file'
       WRITE(io1,*)' '
       WRITE(io1,*)'FATAL ERROR - If the user wants to use NH3 species'
       WRITE(io1,*)'from one of the CALPUFF file, it needs to be listed'
       WRITE(io1,*)'in subgroup (2a&2b) as a processed species'
       WRITE(io1,*)'DO NOT match options selected in control file'
c       WRITE(*,*)'Otherwise, choose to use NH3 monthly averaged'
c       WRITE(*,*)'background BCKNH3 or BCKTNH3 and select NH3TYP=3'
       stop
c      else
c      WRITE(*,*)'NH3 in ppb from BCKNH3 is used for NO3/HNO3'
c      WRITE(*,*)'repartitioning'
       endif
      endif
c
c --- Section for gridded receptors
c ---------------------------------
      if(LSGRID) then
         rx0=0.5+FLOAT(isastr)
         ry0=0.5+FLOAT(jsastr)
         do j=1,ngy
         do i=1,ngx
c ---       Compute met grid cell index for this receptor
            rx=rx0+FLOAT(i-1)/FLOAT(meshdn)
            ix=rx
            ix=MAX(ix,1)
            ix=MIN(ix,ielmet)
            ry=ry0+FLOAT(j-1)/FLOAT(meshdn)
            iy=ry
            iy=MAX(iy,1)
            iy=MIN(iy,jelmet)
c ---       Extract met. variables (flag=2)
            if (metfm.eq.2) then
            call EXMET(3,ix,iy,dgrid,tempk,qsw,irh,rhoair,
     &              cldcov,coszen)
            ELSEIF (metfm.eq.0) then
            call EXMET(2,ix,iy,dgrid,tempk,qsw,irh,rhoair,
     &              cldcov,coszen)
            elseif (metfm.eq.1) then
            tempk=temp1d
            irh=irh1d
            rhoair=rho1d
            endif
            rhum=irh
c ---       Set conversion factor from g/m**3 to ppb
c ---       Note: constant 28.97e6 is based on rho in kg/m**3
            f=28.97e6/rhoair
c ---       Convert concentrations from g/m**3 to ppb
            cso4ppb=concg(i,j,ispecno3(1))*f/96.
            chno3ppb=concg(i,j,ispecno3(2))*f/63.
            cno3ppb=concg(i,j,ispecno3(3))*f/62.
            ctno3ppb=cno3ppb+chno3ppb
c ---       Convert concentrations from g/m**3 to moles/m3
            tso4 = concg(i,j,ispecno3(1))/96.
            tso4 = MAX(tso4,1.E-12)
            tno3 = ctno3ppb/f
c --- CEC (060303) - add option to have NH3 from concentration
c ---                and BCKNH3 to be used together.
c --- CEC (060309) - ispecno3(4) is NH3gas, ispecno3(7) is total NH3
c                    If both in CALPUFF concentration files can be added
            if(LNH3) then
              IF(nh3typ.eq.1) then
              cnh3ppb=cnh3ppb1+concg(i,j,ispecno3(4))*f/17.+
     &         concg(i,j,ispecno3(7))*f/17.
              ELSEIF(nh3typ.eq.0) then
              cnh3ppb=concg(i,j,ispecno3(4))*f/17.+
     &         concg(i,j,ispecno3(7))*f/17.
              else
              WRITE(*,*)' '
              WRITE(*,*)'FATAL ERROR - Review message written'
              WRITE(*,*)'in the list-file'
              WRITE(io1,*)' '
              WRITE(io1,*)'Fatal Error - NH3TYP can only be 0 or 1'
              stop
              endif
            else
              IF(nh3typ.eq.3) then
              cnh3ppb=cnh3ppb1
              else
              WRITE(*,*)' '
              WRITE(*,*)'FATAL ERROR - Review message written'
              WRITE(*,*)'in the list-file'
              WRITE(io1,*)' '
              WRITE(io1,*)'Fatal Error - NH3TYP can only be 3'
              stop
             endif
            endif
          IF(cnh3ppb.LT.0.) then
        WRITE(*,*)' '
        WRITE(*,*)'FATAL ERROR - Review message written'
        WRITE(*,*)'in the list-file'
        WRITE(io1,*)' '
        WRITE(io1,*)'FATAL ERROR - There is no NH3 and/or TNH3 provided'
          stop
          endif
c           if(LNH3) cnh3ppb=concg(i,j,ispecno3(4))*f/17.
            if(meqphase.EQ.2) then
c ---          ISORROPIA
               tnh3 = cnh3ppb/f
               rhfrac = rhum * 0.01    ! fractional relative humidity
               if(tno3.GT.tno3floor .AND. tnh3.GT.0.0)then
                  call ISODRIVER(tso4,tno3,tnh3,rhfrac,tempk,
     &                           pno3,gnh3c)
               elseif(l_tno3floor .AND. tnh3.gt.0.0) then
                  call ISODRIVER(tso4,tno3floor,tnh3,rhfrac,tempk,
     &                           pno3,gnh3c)
                  pno3=pno3*(tno3/tno3floor)
               else
                  pno3 = 0.
                  gnh3c = 0.
               endif
c ---          Update stored nitrate concentrations in g/m**3
               concg(i,j,ispecno3(2))=(tno3 - pno3) *63.
               concg(i,j,ispecno3(3))=pno3*62.
            else
c ---          CHEMEQ
c ---          Adjust total ammonia for that absorbed by SO4
               canh3=cnh3ppb-2.*cso4ppb
               if(canh3.GT.0.0)then
c ---             Compute NH4NO3 concentration from chemical equilibrium
                  call CHEMEQ(canh3,ctno3ppb,rhum,tempk,cno3ppb,
     &                        chno3ppb,enh3,io6)
               else
                  cno3ppb=0.0
                  chno3ppb=ctno3ppb
                  enh3=0.0
               endif
c ---          Update stored nitrate concentrations in g/m**3
               concg(i,j,ispecno3(2))=chno3ppb*63./f
               concg(i,j,ispecno3(3))=cno3ppb*62./f
            endif
         enddo
         enddo

c ---    Write information for last gridded receptor in DEBUG mode
         if(ldb)then
            if(meqphase.EQ.2) then
               chno3ppb=concg(i,j,ispecno3(2))*f/63.
               cno3ppb=concg(i,j,ispecno3(3))*f/62.
               enh3 = gnh3c * f
            endif
            write(io1,2028)cso4ppb,ctno3ppb,chno3ppb,cno3ppb,enh3,
     &                         ix,iy,rhum,tempk
         endif

      endif
c
c --- Section for discrete receptors
c ----------------------------------
      if(LDISC) then
         do i=1,ndrec
c ---       Compute met grid cell index for this receptor
            rx=(xrec(i)-xorigkm)/delx
            ix=rx+1
            ix=MAX(ix,1)
            ix=MIN(ix,ielmet)
            ry=(yrec(i)-yorigkm)/dely
            iy=ry+1
            iy=MAX(iy,1)
            iy=MIN(iy,jelmet)
c ---       Extract met. variables (flag=2)
            if (metfm.eq.2) then
            call EXMET(3,ix,iy,dgrid,tempk,qsw,irh,rhoair,
     &              cldcov,coszen)
            ELSEIF (metfm.eq.0) then
            call EXMET(2,ix,iy,dgrid,tempk,qsw,irh,rhoair,
     &              cldcov,coszen)
            elseif (metfm.eq.1) then
            tempk=temp1d
            irh=irh1d
            rhoair=rho1d
            endif
            rhum=irh
c
c ---       Set conversion factor from g/m**3 to ppb
c ---       Note: constant 28.97e6 is based on rho in kg/m**3
            f=28.97e6/rhoair
c ---       Convert concentrations from g/m**3 to ppb
            cso4ppb=concd(i,ispecno3(1))*f/96.
            chno3ppb=concd(i,ispecno3(2))*f/63.
            cno3ppb=concd(i,ispecno3(3))*f/62.
            ctno3ppb=cno3ppb+chno3ppb
c ---       Convert concentrations from g/m**3 to moles/m3
            tso4 = concd(i,ispecno3(1))/96.
            tso4 = MAX(tso4,1.E-12)
            tno3 = ctno3ppb/f
c --- CEC (060303) - add option to have NH3 from concentration
c ---                and BCKNH3 to be used together.
c --- CEC (060309) - ispecno3(4) is NH3gas, ispecno3(7) is total NH3
c                    If both in CALPUFF concentration files can be added
            if(LNH3) then
              IF(nh3typ.eq.1) then
              cnh3ppb=cnh3ppb1+concd(i,ispecno3(4))*f/17.+
     &         concd(i,ispecno3(7))*f/17.
              ELSEIF(nh3typ.eq.0) then
              cnh3ppb=concd(i,ispecno3(4))*f/17.+
     &         concd(i,ispecno3(7))*f/17.
              else
              WRITE(*,*)' '
              WRITE(*,*)'FATAL ERROR - Review message written'
              WRITE(*,*)'in the list-file'
              WRITE(io1,*)' '
              WRITE(io1,*)'Fatal Error - NH3TYP can only be 0 or 1'
              stop
              endif
            else
              IF(nh3typ.eq.3) then
              cnh3ppb=cnh3ppb1
              else
              WRITE(*,*)' '
              WRITE(*,*)'FATAL ERROR - Review message written'
              WRITE(*,*)'in the list-file'
              WRITE(io1,*)' '
              WRITE(io1,*)'Fatal Error - NH3TYP can only be 3'
              stop
             endif
            endif
          IF(cnh3ppb.LT.0.) then
        WRITE(*,*)' '
        WRITE(*,*)'FATAL ERROR - Review message written'
        WRITE(*,*)'in the list-file'
        WRITE(io1,*)' '
        WRITE(io1,*)'FATAL ERROR - There is no NH3 and/or TNH3 provided'
          stop
          endif
c             cnh3ppb2=concd(i,ispecno3(4))*f/17.
c            WRITE(*,*)'NH3used bckgM conc ',cnh3ppb,cnh3ppb1,cnh3ppb2
c             if(LNH3) cnh3ppb=concd(i,ispecno3(4))*f/17.
            if(meqphase.EQ.2) then
c ---          ISORROPIA
               tnh3 = cnh3ppb/f
               rhfrac = rhum * 0.01    ! fractional relative humidity
               if(tno3.GT.tno3floor .AND. tnh3.GT.0.0)then
                  call ISODRIVER(tso4,tno3,tnh3,rhfrac,tempk,
     &                           pno3,gnh3c)
               elseif(l_tno3floor .AND. tnh3.gt.0.0) then
                  call ISODRIVER(tso4,tno3floor,tnh3,rhfrac,tempk,
     &                           pno3,gnh3c)
                  pno3=pno3*(tno3/tno3floor)
               else
                  pno3 = 0.
                  gnh3c = 0.
               endif
c ---          Update stored nitrate concentrations in g/m**3
               concd(i,ispecno3(2))=(tno3 - pno3) *63.
               concd(i,ispecno3(3))=pno3*62.
            else
c ---          CHEMEQ
c ---          Adjust total ammonia for that absorbed by SO4
               canh3=cnh3ppb-2.*cso4ppb
               if(canh3.GT.0.0)then
                  call CHEMEQ(canh3,ctno3ppb,rhum,tempk,cno3ppb,
     &                     chno3ppb,enh3,io6)
               else
                  cno3ppb=0.0
                  chno3ppb=ctno3ppb
                  enh3=0.0
               endif
c ---          Update stored nitrate concentrations in g/m**3
               concd(i,ispecno3(2))=chno3ppb*63./f
               concd(i,ispecno3(3))=cno3ppb*62./f
            endif
         enddo

c ---    Write information for last discrete receptor in DEBUG mode
         if(ldb)then
            if(meqphase.EQ.2) then
               chno3ppb=concd(i,ispecno3(2))*f/63.
               cno3ppb=concd(i,ispecno3(3))*f/62.
               enh3 = gnh3c * f
            endif
            write(io1,2028)cso4ppb,ctno3ppb,chno3ppb,cno3ppb,enh3,
     &                         ix,iy,rhum,tempk
         endif
      endif
c
c --- Section for complex terrain receptors
c -----------------------------------------
      if(LCTSG) then
         do i=1,nctrec
c ---       Compute met grid cell index for this receptor
            rx=(xctr(i)-xorigkm)/delx
            ix=rx+1
            ix=MAX(ix,1)
            ix=MIN(ix,ielmet)
            ry=(yctr(i)-yorigkm)/dely
            iy=ry+1
            iy=MAX(iy,1)
            iy=MIN(iy,jelmet)
c ---       Extract met. variables (flag=2)
            if (metfm.eq.2) then
            call EXMET(3,ix,iy,dgrid,tempk,qsw,irh,rhoair,
     &              cldcov,coszen)
            ELSEIF (metfm.eq.0) then
            call EXMET(2,ix,iy,dgrid,tempk,qsw,irh,rhoair,
     &              cldcov,coszen)
            elseif (metfm.eq.1) then
            tempk=temp1d
            irh=irh1d
            rhoair=rho1d
            endif
            rhum=irh
c ---       Set conversion factor from g/m**3 to ppb
c ---       Note: constant 28.97e6 is based on rho in kg/m**3
            f=28.97e6/rhoair
c ---       Convert concentrations from g/m**3 to ppb
            cso4ppb=conct(i,ispecno3(1))*f/96.
            chno3ppb=conct(i,ispecno3(2))*f/63.
            cno3ppb=conct(i,ispecno3(3))*f/62.
            ctno3ppb=cno3ppb+chno3ppb
c ---       Convert concentrations from g/m**3 to moles/m3
            tso4 = conct(i,ispecno3(1))/96.
            tso4 = MAX(tso4,1.E-12)
            tno3 = ctno3ppb/f
c --- CEC (060303) - add option to have NH3 from concentration
c ---                and BCKNH3 to be used together.
c --- CEC (060309) - ispecno3(4) is NH3gas, ispecno3(7) is total NH3
c                    If both in CALPUFF concentration files can be added
            if(LNH3) then
              IF(nh3typ.eq.1) then
              cnh3ppb=cnh3ppb1+conct(i,ispecno3(4))*f/17.+
     &         conct(i,ispecno3(7))*f/17.
              ELSEIF(nh3typ.eq.0) then
              cnh3ppb=conct(i,ispecno3(4))*f/17.+
     &         conct(i,ispecno3(7))*f/17.
              else
              WRITE(*,*)' '
              WRITE(*,*)'FATAL ERROR - Review message written'
              WRITE(*,*)'in the list-file'
              WRITE(io1,*)' '
              WRITE(io1,*)'Fatal Error - NH3TYP can only be 0 or 1'
              stop
              endif
            else
              IF(nh3typ.eq.3) then
              cnh3ppb=cnh3ppb1
              else
              WRITE(*,*)' '
              WRITE(*,*)'FATAL ERROR - Review message written'
              WRITE(*,*)'in the list-file'
              WRITE(io1,*)' '
              WRITE(io1,*)'Fatal Error - NH3TYP can only be 3'
              stop
             endif
            endif
          IF(cnh3ppb.LT.0.) then
        WRITE(*,*)' '
        WRITE(*,*)'FATAL ERROR - Review message written'
        WRITE(*,*)'in the list-file'
        WRITE(io1,*)' '
        WRITE(io1,*)'FATAL ERROR - There is no NH3 and/or TNH3 provided'
          stop
          endif
c            if(LNH3) cnh3ppb=conct(i,ispecno3(4))*f/17.
            if(meqphase.EQ.2) then
c ---          ISORROPIA
               tnh3 = cnh3ppb/f
               rhfrac = rhum * 0.01    ! fractional relative humidity
               if(tno3.GT.tno3floor .AND. tnh3.GT.0.0)then
                  call ISODRIVER(tso4,tno3,tnh3,rhfrac,tempk,
     &                           pno3,gnh3c)
               elseif(l_tno3floor .AND. tnh3.gt.0.0) then
                  call ISODRIVER(tso4,tno3floor,tnh3,rhfrac,tempk,
     &                           pno3,gnh3c)
                  pno3=pno3*(tno3/tno3floor)
               else
                  pno3 = 0.
                  gnh3c = 0.
               endif
c ---          Update stored nitrate concentrations in g/m**3
               conct(i,ispecno3(2))=(tno3 - pno3) *63.
               conct(i,ispecno3(3))=pno3*62.
            else
c ---          CHEMEQ
c ---          Adjust total ammonia for that absorbed by SO4
               canh3=cnh3ppb-2.*cso4ppb
               if(canh3.GT.0.0)then
                  call CHEMEQ(canh3,ctno3ppb,rhum,tempk,cno3ppb,
     &                     chno3ppb,enh3,io6)
               else
                  cno3ppb=0.0
                  chno3ppb=ctno3ppb
                  enh3=0.0
               endif
c ---          Update stored nitrate concentrations in g/m**3
               conct(i,ispecno3(2))=chno3ppb*63./f
               conct(i,ispecno3(3))=cno3ppb*62./f
            endif
         enddo

c ---    Write information for last CTSG receptor in DEBUG mode
         if(ldb)then
            if(meqphase.EQ.2) then
               chno3ppb=conct(i,ispecno3(2))*f/63.
               cno3ppb=conct(i,ispecno3(3))*f/62.
               enh3 = gnh3c * f
            endif
            write(io1,2028)cso4ppb,ctno3ppb,chno3ppb,cno3ppb,enh3,
     &                         ix,iy,rhum,tempk
         endif
      endif

      return

2028  format(1x, 'PARTITION Results for last receptor:',/5x,
     1      'SO4(ppb)=',f7.3,' TNO3(ppb)=',f7.3,' HNO3(ppb)=',f7.3,
     2      ' NO3(ppb)=',f7.3,' NH3(ppb)=',f7.3,/5x,'MET Cellx=',i4,
     3      ' MET Celly=',i4,' RHUM=',f4.0,' TEMPK=',f5.1)

      end
c----------------------------------------------------------------------
      subroutine no3ratio(ldb)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 000112               NO3RATIO
c                D. Strimaitis,  Earth Tech
c
c --- PURPOSE:  Recompute the partition of nitrate between HNO3 and
c               NO3 concentrations at each receptor for a particular
c               source group, using the partition found for the total
c               concentrations from all sources
c
c
c --- INPUTS:
c             LDB - logical - Flag for debug output
c
c     Common block /CONC/ variables:
c           CONCG(mxgx,mxgy,mxspec),CONCD(mxdrec,mxspec),
c           CONCT(mxctrec,mxspec)
c     Common block /CTRL/ variables:
c           ISPECNO3(6)
c     Common block /HEAD/ variables:
c           NGX, NGY, NDREC, NCTREC,
c           LSGRID, LDISC, LCTSG
c     Parameters:
c           MXGX, MXGY, MXDREC, MXCTREC, MXSPEC, IO1
c
c --- OUTPUT:
c     Common block /CONC/ variables:
c           CONCG(mxgx,mxgy,mxspec),CONCD(mxdrec,mxspec),
c           CONCT(mxctrec,mxspec)
c
c --- NO3RATIO called by:  MAIN
c --- NO3RATIO calls:      none
c----------------------------------------------------------------------
c --- Include parameters
      include 'params.utl'

c --- Include common blocks
      include 'conc.utl'
      include 'ctrl.utl'
      include 'head.utl'

      logical ldb

c --- Set the molecular wt ratio of NO3(62) to HNO3(63)
      data r62_63/0.984127/
c --- Set the molecular wt ratio of HNO3(63) to NO3(62)
      data r63_62/1.016129/

c --- Section for gridded receptors
c ---------------------------------
      if(LSGRID) then
         do j=1,ngy
         do i=1,ngx
            chno3=concg(i,j,ispecno3(2))
            cno3=concg(i,j,ispecno3(3))
            ctno3=cno3+chno3*r62_63
            chno3all=concg(i,j,ispecno3(5))
            cno3all=concg(i,j,ispecno3(6))
            ctno3all=cno3all+chno3all*r62_63
            if(ctno3all.GT.0.0) then
c ---          Repartition the nitrate
               cno3=ctno3*cno3all/ctno3all
               chno3=(ctno3-cno3)*r63_62
               chno3=AMAX1(chno3,0.0)
c ---          Update stored nitrate concentrations
               concg(i,j,ispecno3(2))=chno3
               concg(i,j,ispecno3(3))=cno3
            endif
         enddo
         enddo

c ---    Write information for last gridded receptor in DEBUG mode
         if(ldb)write(io1,2028)cno3,chno3,ctno3,
     &                         cno3all,chno3all,ctno3all

      endif
c
c --- Section for discrete receptors
c ----------------------------------
      if(LDISC) then
         do i=1,ndrec
            chno3=concd(i,ispecno3(2))
            cno3=concd(i,ispecno3(3))
            ctno3=cno3+chno3*r62_63
            chno3all=concd(i,ispecno3(5))
            cno3all=concd(i,ispecno3(6))
            ctno3all=cno3all+chno3all*r62_63
            if(ctno3all.GT.0.0) then
c ---          Repartition the nitrate
               cno3=ctno3*cno3all/ctno3all
               chno3=(ctno3-cno3)*r63_62
               chno3=AMAX1(chno3,0.0)
c ---          Update stored nitrate concentrations
               concd(i,ispecno3(2))=chno3
               concd(i,ispecno3(3))=cno3
            endif
         enddo

c ---    Write information for last discrete receptor in DEBUG mode
         if(ldb)write(io1,2028)cno3,chno3,ctno3,
     &                         cno3all,chno3all,ctno3all
      endif
c
c --- Section for complex terrain receptors
c -----------------------------------------
      if(LCTSG) then
         do i=1,nctrec
            chno3=conct(i,ispecno3(2))
            cno3=conct(i,ispecno3(3))
            ctno3=cno3+chno3*r62_63
            chno3all=conct(i,ispecno3(5))
            cno3all=conct(i,ispecno3(6))
            ctno3all=cno3all+chno3all*r62_63
            if(ctno3all.GT.0.0) then
c ---          Repartition the nitrate
               cno3=ctno3*cno3all/ctno3all
               chno3=(ctno3-cno3)*r63_62
               chno3=AMAX1(chno3,0.0)
c ---          Update stored nitrate concentrations
               conct(i,ispecno3(2))=chno3
               conct(i,ispecno3(3))=cno3
            endif
         enddo

c ---    Write information for last CTSG receptor in DEBUG mode
         if(ldb)write(io1,2028)cno3,chno3,ctno3,
     &                         cno3all,chno3all,ctno3all
      endif

      return

2028  format(1x, 'NO3RATIO Results for last receptor:',/5x,
     1      'NO3=',e10.4,' HNO3=',e10.4,' TNO3=',e10.4,
     2      ' NO3(all)=',e10.4,' HNO3(all)=',e10.4,' TNO3(all)=',e10.4)

      end
c----------------------------------------------------------------------
      subroutine exmet(iflag,ixs,iys,dgrid,tempk,qsw,irh,rhoair,
     &                 cc,czen)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 060329                  EXMET
c --- CALPUFF    Version: 6.0      Level: 980304                  EXMET
c                J. Scire, SRC
c
c --- PURPOSE:  Find the closest non-missing value of air temperature,
c               short-wave radiation, relative humidity, and air
c               density to a particular grid point.
c               NOTE: if IFLAG = 1, only temperature is returned
c                     if IFLAG = 2, values for all four variables
c                                   are returned
c
c --- UPDATE
c --- V1.41-V1.5    060329  (DGS): Use I2DMET in place of DVERSM
c --- V1.3-V1.41    050302  (CEC): add option if iflag=3, use 2D values
c                                  of RH,TEMP,RHO from CALPUFF output files
c --- V1.0-V1.3     030402  (DGS): use DATASET Vers. to read 2D fields
c                   010901  (FRR): use 2D values of RHO,QSW,RH,TEMP
c                                  if available in CALMET 
c --- V5.0-V5.0     980304  (DGS): add clouds & sun angle
c
c --- INPUTS:
c            IFLAG - integer - Flag determining which variables are
c                              computed ( 1 ==> TEMPK only,
c                              2 ==> TEMP, QSW, IRH, RHOAIR)
c              IXS - integer - X index of the closest met. grid point
c                              to the puff/slug center
c              IYS - integer - Y index of the closest met. grid point
c                              to the puff/slug center
c            DGRID - real    - Grid spacing (m)
c
c     Common block /METHD/ variables:
c           NXM, NYM, NEARS(mxnx,mxny), XSSTA(mxss), YSSTA(mxss),
c           NSSTA, LCALGRD, I2DMET
c     Common block /METHR/ variables:
c           TMET(mxnx,mxny,mxnz), TEMPSS(mxss), RHOSS(mxss),
c           QSWSS(mxss), IRHSS(mxss), CCSS(mxss),CZENSS(mxss)
c frr (09/01)
c           TEMP2D(mxnx,mxny), RHO2D(mxnx,mxny),
c           QSW2D(mxnx,mxny), IRH2D(mxnx,mxny), CC2D(mxnx,mxny),
c           CZEN2D(mxnx,mxny)
c     Parameters:
c           MXNX, MXNY, MXNZ, MXSS, MXUS, MXPS, IO6
c
c --- OUTPUT:
c            TEMPK - real    - Air temperature (deg. K)
c              QSW - real    - Short-wave solar radiation (W/m**2)
c              IRH - integer - Relative humidity (percent)
c           RHOAIR - real    - Air density (kg/m**3)
c               CC - real    - Cloud cover (tenths)
c             CZEN - real    - Cosine of solar zenith angle
c
c --- EXMET called by:  COMP
c --- EXMET calls:      FINDR, FINDI
c----------------------------------------------------------------------
c
      include 'params.utl'
c
      logical ldb2
c
      include 'methd.utl'
      include 'methr.utl'
      include 'rtrhd.utl'
c
c --- Missing value indicators for integer, real variables
      data imiss/9999/,xmiss/9999./
c
c --- Compute grid point coordinates (m) relative to met. grid origin
      xgrd=(float(ixs)-0.5)*dgrid
      ygrd=(float(iys)-0.5)*dgrid
c
c frr (09/01)- Use 2D arrays if available in CALMET
       if(i2dmet.EQ.1) then

          tempk=temp2d(ixs,iys)
          if(iflag.eq.2)then
            qsw=qsw2d(ixs,iys)
            irh=irh2d(ixs,iys)
            rhoair=rho2d(ixs,iys)
            cc=cc2d(ixs,iys)
            czen=czen2d(ixs,iys)
         else
c ---       Flag variables other than temperature as missing
            qsw=xmiss
            irh=imiss
            rhoair=xmiss
            cc=xmiss
            czen=xmiss
         endif
C --- Use 2D arrays of RH, TEMP, RHOAIR if 3 CALPUFF output files are used.
C
       elseif (iflag.eq.3.and.i2drhurh.eq.1) then
            tempk=temp2d(ixs,iys)
            irh=irh2d(ixs,iys)
            rhoair=rho2d(ixs,iys)
            qsw=xmiss
            cc=xmiss
            czen=xmiss
       ELSEIF (iflag.eq.3.and.i2drhurh.ne.1) then
            WRITE(*,*)'error - files output from CALPUFF must'
            WRITE(*,*)'be 2D files- version of file= ', crhver3
            stop
       else

c ---    ISTA is the surface met. stations closest to grid point (IXS,IYS)
         ista=nears(ixs,iys)
c
c ---    AIR TEMPERATURE (deg. K)
         if(lcalgrd)then
c ---    Use gridded temp. data, if available
            tempk=tmet(ixs,iys,1)
         else
c ---       Use air temp. at closest surface station
            tempk=tempss(ista)
c ---       Find temp. at nearest station with non-missing data
            if(tempk.ge.xmiss)then
               call findr(xssta,yssta,nssta,tempss,xgrd,ygrd,
     1          ista,tempk,ierr)
c
               if(ierr.eq.1)then
                  write(io6,*)'ERROR in subr. EXMET -- All surface ',
     1            'temp. data missing -- IERR = ',ierr
                  write(io6,*)'TEMPSS = ',(tempss(n),n=1,nssta)
                  write(*,*)
                  stop 'Halted in EXMET -- see list file.'
               endif
            endif
         endif
c
c
         if(iflag.eq.2)then
c
c ---       SHORT-WAVE RADIATION (W/m**2)
            qsw=qswss(ista)
            if(qsw.ge.xmiss)then
               call findr(xssta,yssta,nssta,qswss,xgrd,ygrd,
     1          ista,qsw,ierr)
c
               if(ierr.eq.1)then
                  write(io6,*)'ERROR in subr. EXMET -- All surface ',
     1            'short-wave radiation data missing -- IERR = ',ierr
                  write(io6,*)'QSWSS = ',(qswss(n),n=1,nssta)
                  stop
               endif
            endif
c
c ---       RELATIVE HUMIDITY (percent)
            irh=irhss(ista)
            if(irh.ge.imiss)then
               call findi(xssta,yssta,nssta,irhss,xgrd,ygrd,
     1          ista,irh,ierr)
c
               if(ierr.eq.1)then
                  write(io6,*)'ERROR in subr. EXMET -- All surface ',
     1            'relative humidity data missing -- IERR = ',ierr
                  write(io6,*)'IRHSS = ',(irhss(n),n=1,nssta)
               endif
            endif
c
c ---       AIR DENSITY (kg/m**3)
            rhoair=rhoss(ista)
            if(rhoair.ge.xmiss)then
               call findr(xssta,yssta,nssta,rhoss,xgrd,ygrd,
     1          ista,rhoair,ierr)
c
               if(ierr.eq.1)then
                  write(io6,*)'ERROR in subr. EXMET -- All surface ',
     1            'air density data missing -- IERR = ',ierr
                  write(io6,*)'RHOSS = ',(rhoss(n),n=1,nssta)
                  stop
               endif
            endif
c
c ---       CLOUD COVER (tenths)
            cc=ccss(ista)
            if(cc.ge.xmiss)then
               call findr(xssta,yssta,nssta,ccss,xgrd,ygrd,
     1          ista,cc,ierr)
c
               if(ierr.eq.1)then
                  write(io6,*)'ERROR in subr. EXMET -- All surface ',
     1            'cloud cover data missing -- IERR = ',ierr
                  write(io6,*)'CCSS = ',(ccss(n),n=1,nssta)
                  stop
               endif
            endif
c
c ---       Cosine of solar zenith angle
            czen=czenss(ista)
            if(czen.ge.xmiss)then
               call findr(xssta,yssta,nssta,czenss,xgrd,ygrd,
     1          ista,czen,ierr)
c
               if(ierr.eq.1)then
                  write(io6,*)'ERROR in subr. EXMET -- All surface ',
     1            'solar zenth data missing -- IERR = ',ierr
                  write(io6,*)'CZENSS = ',(czenss(n),n=1,nssta)
                  stop
               endif
            endif
c
         else
c
c ---       Flag variables other than temperature as missing
            qsw=xmiss
            irh=imiss
            rhoair=xmiss
            cc=xmiss
            czen=xmiss
         endif
c
c*****
         ldb2=.false.
         if(ldb2)then
            write(io6,*)
            write(io6,*)'SUBR. EXMET -- iflag = ',iflag,' ixs = ',ixs,
     1      ' iys = ',iys,' dgrid = ',dgrid,' ista = ',ista
            write(io6,*)'tempk = ',tempk,' qsw = ',qsw,' irh = ',irh,
     1      ' rhoair = ',rhoair
            write(io6,*)'cc = ',cc,' czen = ',czen
         endif
      endif
c*****
      return
      end
c----------------------------------------------------------------------
      subroutine findr(xsta,ysta,nsta,dat,x,y,ista,value,ierr)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 900228                  FINDR
c
c --- Taken from CALPUFF:
c
c --- CALPUFF    Version: 6.0      Level: 900228                  FINDR
c                J. Scire, SRC
c
c --- PURPOSE:  Find the closest station to a specified (X,Y)
c               coordinate that has non-missing data and pass
c               the real variable at that station back to the
c               calling routine
c
c --- INPUTS:
c       XSTA(nsta) - real    - Array of X coordinates (m) for each
c                              station relative to the met. grid origin
c                              at (0., 0.)
c       YSTA(nsta) - real    - Array of Y coordinates (m) for each
c                              station relative to the met. grid origin
c                              at (0., 0.)
c             NSTA - integer - Number of stations
c        DAT(nsta) - real    - Values of the variable at each station
c                              (NOTE: 9999. used as a missing value
c                              indicator)
c                X - real    - Reference X coordinate (m)
c                Y - real    - Reference Y coordinate (m)
c
c --- OUTPUT:
c             ISTA - integer - Station number of closest station
c            VALUE - real    - Value of from DAT array for closest
c                              station "ISTA"
c             IERR - integer - Error code (0=no error, 1=all data
c                              missing, so VALUE = missing value
c                              indicator
c
c --- FINDR called by:  EXMET, GETOZ
c --- FINDR calls:      none
c----------------------------------------------------------------------
c
      real xsta(nsta),ysta(nsta),dat(nsta)
      data xmax/1.e38/,xmiss/9999./
c
      ista=0
      value=xmiss
      dmin2=xmax
c
      do 10 i=1,nsta
c
c --- Compute the (distance)**2 to each station with non-missing data
      if(dat(i).lt.xmiss)then
         dist2=(xsta(i)-x)**2+(ysta(i)-y)**2
c
c ---    Keep track of the closest station to the reference point
         if(dist2.lt.dmin2)then
            dmin2=dist2
            ista=i
         endif
      endif
10    continue
c
      if(ista.eq.0)then
         ierr=1
         go to 999
      endif
c
c --- Extract the value at closest station from the data array
      value=dat(ista)
      ierr=0
c
999   continue
      return
      end
c----------------------------------------------------------------------
      subroutine findi(xsta,ysta,nsta,idat,x,y,ista,ivalue,ierr)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 900228                  FINDI
c
c --- Taken from CALPUFF:
c
c --- CALPUFF    Version: 6.0      Level: 900228                  FINDI
c                J. Scire, SRC
c
c --- PURPOSE:  Find the closest station to a specified (X,Y)
c               coordinate that has non-missing data and pass
c               the integer variable at that station back to the
c               calling routine
c
c --- INPUTS:
c       XSTA(nsta) - real    - Array of X coordinates (m) for each
c                              station relative to the met. grid origin
c                              at (0., 0.)
c       YSTA(nsta) - real    - Array of Y coordinates (m) for each
c                              station relative to the met. grid origin
c                              at (0., 0.)
c             NSTA - integer - Number of stations
c       IDAT(nsta) - integer - Values of the variable at each station
c                              (NOTE: 9999 used as a missing value
c                              indicator)
c                X - real    - Reference X coordinate (m)
c                Y - real    - Reference Y coordinate (m)
c
c --- OUTPUT:
c             ISTA - integer - Station number of closest station
c           IVALUE - real    - Value of from IDAT array for closest
c                              station "ISTA"
c             IERR - integer - Error code (0=no error, 1=all data
c                              missing, so VALUE = missing value
c                              indicator
c
c --- FINDI called by:  EXMET
c --- FINDI calls:      none
c----------------------------------------------------------------------
c
      real xsta(nsta),ysta(nsta)
      integer idat(nsta)
      data xmax/1.e38/,imiss/9999/
c
      ista=0
      ivalue=imiss
c
      dmin2=xmax
c
      do 10 i=1,nsta
c
c --- Compute the (distance)**2 to each station with non-missing data
      if(idat(i).lt.imiss)then
         dist2=(xsta(i)-x)**2+(ysta(i)-y)**2
c
c ---    Keep track of the closest station to the reference point
         if(dist2.lt.dmin2)then
            dmin2=dist2
            ista=i
         endif
      endif
10    continue
c
      if(ista.eq.0)then
         ierr=1
         go to 999
      endif
c
c --- Extract the value at closest station from the data array
      ivalue=idat(ista)
      ierr=1
c
999   continue
      return
      end
c----------------------------------------------------------------------
      SUBROUTINE CHEMEQ(CANH3,CTNO3,RHUM,TEMP,CPNO3,EHNO3,ENH3,io6)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 960612                 CHEMEQ
c
c --- Taken from CALPUFF:
c
c --- CALPUFF    Version: 6.0      Level: 960612                 CHEMEQ
c --- Adapted from MESOPUFF II Subr. CHEMEQ
c     MESOPUFF II   VERSION 4.41  LEVEL 821201   CHEMEQ     ERT
C
C      THIS ROUTINE DETERMINES AMMONIUM NITRATE CONCENTRATION
C      FROM TOTAL NITRATE CONCENTRATION.
C
C INPUTS:  CANH3 = TOTAL AMMONIA AVAILIABLE FOR NH4NO3 FORMATION (PPB)
C                  (IE. TOTAL AMMONIA - SULFATE)
C          CTNO3 = PUFF TOTAL NITRATE CONCENTRATION (PPB)
C           RHUM = RELATIVE HUMIDITY (PERCENT)
C           TEMP = TEMP (DEG K)
C           io6 = LOGICAL UNIT OF PRINTED OUTPUT FILE
C
C OUTPUTS: CPNO3 = EQUILIBRIUM NH4NO3 AEROSOL CONCENTRATION (PPB)
C          EHNO3 = EQUILIBRIUM GASEOUS HNO3 CONCENTRATION (PPB)
C          ENH3  = EQUILIBRIUM GASEOUS NH3 CONCENTRATION (PPB)
C
C METHOD:  WE ASSUME HNO3, NH3, AND NH4NO3 ARE IN EQUILIBRIUM.
C
C          HNO3 + NH3 <---> NH4NO3
C
C          K = (NH3)*(HNO3)/(NH4NO3)
C
C          K HAS BEEN SHOWN TO BE A NONLINEAR FUNCTION OF
C          RELATIVE HUMIDITY AND TEMPERATURE.
C
C          WE USE DOUBLE LINEAR INTERPOLATION ON TABULATED DATA
C          TO DETERMINE K FROM RHUM AND TEMP.
C
C          EQUILIBRIUM CONSTANT DATA OBTAINED FROM:
C
C          STELSON,A.W. AND J.H.SEINFELD 1982.  RELATIVE HUMIDITY
C          AND TEMPERATURE DEPENDENCE OF AMMONIUM NITRATE
C          DISSOCIATION CONSTANT. ATMOSPHERIC ENVIRONMENT.
C          VOL.16,NO.5,PP 983-992.
C
C          GIVEN CTNO3, CANH3, AND K, NH4NO3 IS DETERMINED
C          FROM THE FOLLOWING EQUATION:
C
C          NH4NO3 = 0.5 * (B - SQRT(B**2 -4*CTNO3*CANH3) )
C          WHERE B = K + CTNO3 + CANH3
C
      DIMENSION EQKD(10,5), RHUMD(10), TEMPD(5)
      double precision b,r
C
C------ RELATIVE HUMIDITY VECTOR    (DATA IN PERCENT)
      DATA RHUMD /0.,50.,60.,70.,75.,80.,85.,90.,95.,100./
      DATA NRHUMD /10/
C------ TEMPERATURE VECTOR     (DATA IN DEGREES KELVIN)
      DATA TEMPD /273.,283.,293.,303.,313./
      DATA NTEMPD /5/
C------ EQUILIBRIUM CONSTANT ARRAY   (DATA IN PPB)
      DATA EQKD /        5*.029,  .02,  .012,  .008,  .003,  .0001,
     1              4*.535,.400,  .30,   .24,   .15,   .04,   .001,
     2         3*8.0,   7.,  6.,   4.,    3.,   1.5,    .4,    .01,
     3   2*99.6, 98.,  75., 60.,  40.,   28.,   13.,    4.,    .02,
     4  2*1047.,1000.,700.,500., 400.,  250.,  120.,   20.,    .05/
C
C------- FIND THE INDICES IN TEMPD WHICH BRACKET TEMP
C
      TEMPX = TEMP
      TEMPX = AMAX1(TEMPX,TEMPD(1))
      TEMPX = AMIN1(TEMPX,TEMPD(NTEMPD))
      N = NTEMPD - 1
c *** NOTE: If tempx=tempd(1) i.e., temp < tempd(1) won't satisfy the
c ***       following if therefore changing .GT. to .GE. -- 5/30/93
      DO 100 K=1,N
c***  IF( (TEMPX.GT.TEMPD(K)) .AND. (TEMPX.LE.TEMPD(K+1)) )  GO TO 110
      IF( (TEMPX.ge.TEMPD(K)) .AND. (TEMPX.LE.TEMPD(K+1)) )  GO TO 110
100   CONTINUE
110   K1 = K
      K2 = K + 1
C------- INTERPOLATION ON RELATIVE HUMIDITY AT
C        THE UPPER AND LOWER TEMPERATURES.
      EQK1 = TRPF(RHUM,RHUMD,EQKD(1,K1),NRHUMD)
      EQK2 = TRPF(RHUM,RHUMD,EQKD(1,K2),NRHUMD)
C------- INTERPOLATE ON TEMPERATURE TO FIND K
      EQK = EQK1 + ((TEMPX-TEMPD(K1))/(TEMPD(K2)-TEMPD(K1)))*(EQK2-EQK1)
C------- SOLVE THE QUADRATIC EQN FOR NH4NO3 CONCENTRATION
      B = EQK + CANH3 + CTNO3
      R = B**2 - 4.*CANH3*CTNO3
      IF(R.LT.0.)  GO TO 120
c *** Change to double precision to avoid precision problems
c *** as recommended by J. Vimont (NPS) - 6/96
c *** CPNO3 = (B -SQRT(R))/2.
      CPNO3 = 0.5d0*(B-DSQRT(R))
      if((ctno3-cpno3).lt.0.0)then
        cpno3=ctno3
      endif
c --- Add checks to avoid problems when b**2 = r
      cpno3 = amax1(cpno3,0.0)
      EHNO3 = CTNO3 - CPNO3
      ehno3 = amax1(ehno3,0.0)
      ENH3 = CANH3 - CPNO3
      enh3 = amax1(enh3,0.0)
      RETURN
120   WRITE(io6,130) CANH3,CTNO3,TEMP,rhum,EQK
130   FORMAT(56H1JOB ABORTED IN CHEMEQ -- QUADRATIC EQN NOT SOLVABLE
     1,//, 30H NH3, TNO3, TEMP, RHUM, EQK =         ,5E12.4)
      STOP
      END
c----------------------------------------------------------------------
      FUNCTION TRPF(X,XS,FS,NXF)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 941215                   TRPF
c
c --- Taken from CALPUFF:
c
c --- CALPUFF    Version: 6.0      Level: 941215                   TRPF
c --- Adapted from MESOPUFF II Subr. CHEM
c     MESOPUFF II   VERSION 4.41  LEVEL 821201   TRPF     ERT
c
C--------- TRPF IS A LINEAR INTERPOLATOR
      DIMENSION XS(NXF),FS(NXF)
      IF(XS(NXF)-XS(1))  100,120,120
100   DO 110 K =1,NXF
      IF(X-XS(K))  110,110,140
110   CONTINUE
      K=NXF + 1
      GO TO 140
120   DO 130 K=1,NXF
      IF(XS(K)-X)  130,130,140
130   CONTINUE
      K=NXF + 1
140   KL=K-1
      IF(K-2) 160,170,150
150   IF(K-NXF) 170,170,180
160   TRPF = FS(1)
      RETURN
170   DX = XS(K) - XS(KL)
      IF(DX.NE.0.0000)  GO TO 175
      TRPF = FS(KL)
      RETURN
175   TRPF =FS(KL)+(X-XS(KL))*(FS(K)-FS(KL))/DX
      RETURN
180   TRPF = FS(NXF)
      RETURN
      END
c----------------------------------------------------------------------
      subroutine met1(nx,ny,lecho,wrk1,nwork)
c----------------------------------------------------------------------
c --- POSTUTIL   Version: 1.641    Level: 090511                   MET1
c
c --- CALPUFF    Version: 6.11     Level: 050915                   MET1
c                J. Scire, Earth Tech
c
c --- PURPOSE:  Read header records of the CALMET meteorological
c               data file
c
c --- UPDATE
c --- V1.56, level 060329 to V1.61, level 090511
c                           (CEC): Add a check on size of CALMET grid
c                                  against maximum grid size in params.utl
c --- V5.7-V6.1     050915  (DGS): Add 'dataset 2.1' file format, and
c                                  use c*16 variable VERCMET in new
c                                  /DATASET/ common to identify version.
c                                  (DVERSM not used).  Implement full
c                                  begin/end time processing.
c --- V5.5-V5.7     030402  (DGS): Add list file unit to YR4 call
c                   030402  (DGS): Change (f6) to (f6.0)
c                   030402  (DGS): Add echo of CALMET control file (new
c                                  header format)
c                   030402  (DGS): Add DVERSM (dataset version) to 
c                                  /METHD/ and use throughout code to
c                                  identify CALMET.DAT format, and
c                                  compute I2DMET here
c --- V5.4-V5.5     010730  (JSS): include IWAT2 on list file output
c --- V5.4-V5.4     000602_4(DGS): pass base time zone into /METHD/
c --- V5.2-V5.4     000602  (DGS): add message to "stop"
c                   000602  (DGS): add version check for record '2a'
c --- V5.0-V5.2     991104  (DGS): YYYY year format
c --- V5.0-V5.0     990228a (DGS): add IEOF to RDR2D arguments
c --- V5.0-V5.0     980304  (DGS): accept new/old CALMET headers
c --- V4.0-V5.0     971107  (DGS): pass start-time from met file
c                                  for use if METRUN=1
c
c --- INPUTS:
c            NX - integer       - Number of X cells in meteorological
c                                 grid
c            NY - integer       - Number of Y cells in meteorological
c                                 grid
c         LECHO - logical       - Control variable for output of header
c                                 record information
c   WRK1(nwork) - real array    - Work array of length "NWORK" words
c         NWORK - integer       - Dimension of work arrays -- NOTE:
c                                 NWORK must be at least as large as
c                                 nx*ny
c
c     Common block /DATASET/ variables:
c           VERCMET
c     Parameters:
c           MXNZP1, MXSS, MXUS, MXPS, IO6-->IO1
c
c --- OUTPUT:
c     Common block /METHD/ variables:
c           NXM, NYM, NZM, XGRIDM, XORIGM, YORIGM, XBTZM,
c           NSSTA, NPSTA, NLU, IWAT1, IWAT2, ZFACEM(mxnzp1),
c           XSSTA(mxss),YSSTA(mxss),XUSTA(mxus),YUSTA(mxus),
c           XPSTA(mxps),YPSTA(mxps),Z0(mxnx,mxny),ILANDU(mxnx,mxny),
c           ELEV(mxnx,mxny),XLAI(mxnx,mxny),NEARS(mxnx,mxny),
c           LCALGRD,
c           i2dmet, itimes
c           ibymet,ibmmet,ibdmet,ibjdmet,ibhmet,ibsmet
c           ieymet,iemmet,iedmet,iejdmet,iehmet,iesmet
c           levmet, llconfm, xlat1m, xlat2m, rlat0m, rlon0m
c           iutmznm,feastm,fnorthm,pmapm,datumm,datenm,utmhemm
c
c --- MET1 called by:  SETUP
c --- MET1 calls:      RDR1D, RDR2D, RDI2D, METQA, OUT, YR4
c                      INCR, GRDAY, JULDAY
c----------------------------------------------------------------------
c
c !!! Change io6 to io1 for POSTUTIL list file
c !!! Change itimes definition for POSTUTIL:
c        1 = End-time
c        2 = Begin and End times
c --------------------------------------------

c --- Include parameters
      include 'params.utl'
c
c --- Include common block /METHD/ -- CALMET header record data
      include 'methd.utl'
c --- Include common block /DATASET/  -- Input file dataset version
      include 'dataset.utl'
c
      real wrk1(nwork)
      character*80 title(3)
      character*70 messag
      character*8 vermet,clab1,clab2
      character*8 clabexp,clabex1,clabex2
      logical ldate,lecho

c --- Control for printing entire CALMET control file
      logical lmetcf

c --- Local Variables
      character*8 axbtzm
      character*16 dataset,dataver
      character*33 blank33
      character*64 datamod
      character*80 doc1
      character*132 comment1,blank

      data ldate/.false./
      data lmetcf/.false./
      data blank33/'                                 '/

c --- Set blank (132 characters)
      blank(1:33)=blank33
      blank(34:66)=blank33
      blank(67:99)=blank33
      blank(100:132)=blank33

      if(lecho) then
         write(io1,*)
         write(io1,*)
         write(io1,*)
         write(io1,*)'CALMET Control file information:'
         write(io1,*)'--------------------------------'
         write(io1,*)
      endif

c --- Set dataset version as a real  (not currently used)
      read(vercmet,'(f16.0)') dversm

c --- Read records
c ----------------
c
      if(vercmet.EQ.'2.1') then
c ---   Dataset 2.1 with comment records and begin/end times
c ----------------------------------------------------------
c !!!   itimes=1
        itimes=2
        i2dmet=1
c
c ---   Record #1 - File Declaration -- 24 words
        read(io7) dataset,dataver,datamod
        if(lecho) write(io1,'(2a16,a64)') dataset,dataver,datamod
c
c ---   Record #2 - Number of comment lines -- 1 word
        read(io7) ncom
c ---   Loop over comment records
        do i=1,ncom
          comment1=blank
          read(io7) comment1
          if(i.EQ.1) then
c ---       Save model version line
            doc1=comment1(1:80)
          elseif(i.LE.4) then
c ---       Save 3 title lines
            title(i-1)=comment1(1:80)
          endif
          if(lmetcf) write(io1,*) comment1
        enddo
        if(lecho .AND. .NOT.lmetcf) then
           write(io1,*)
           write(io1,*) doc1
           write(io1,*) title(1)
           write(io1,*) title(2)
           write(io1,*) title(3)
           write(io1,*)
        endif
c
c ---   record #NCOM+3 - run control parameters -- 39 words
        read(io7) ibymet,ibmmet,ibdmet,ibhmet,ibsmet,
     1            ieymet,iemmet,iedmet,iehmet,iesmet,
     2            axbtzm,irlgmet,irtype,
     3   nxm, nym, nzm, xgridm, xorigm, yorigm, iwfcod, nssta,
     4   nusta, npsta, nowsta, nlu, iwat1, iwat2, lcalgrd,
     5   pmapm,datumm,datenm,feastm,fnorthm,utmhemm,iutmznm,
     6   rnlat0m,relon0m,xlat1m,xlat2m

        call YR4(io1,ibymet,ierrb)
        call YR4(io1,ieymet,ierre)
        if(ierrb.NE.0 .OR. ierre.NE.0) stop 'Halted in MET1'

        call JULDAY(io1,ibymet,ibmmet,ibdmet,ibjdmet)
        call JULDAY(io1,ieymet,iemmet,iedmet,iejdmet)

c ---   Process character AXBTZM into real XBTZM
        call UTCBASR(axbtzm,xbtzm)

        rlat0m=rnlat0m
        rlon0m=-relon0m

      elseif(vercmet.EQ.'2.0') then
c ---   Dataset 2.0 with comment records
c --------------------------------------
c !!!   itimes=0
        itimes=1
        i2dmet=1
c
c ---   Record #1 - File Declaration -- 24 words
        read(io7) dataset,dataver,datamod
        if(lecho) write(io1,'(2a16,a64)') dataset,dataver,datamod
c
c ---   Record #2 - Number of comment lines -- 1 word
        read(io7) ncom
c ---   Loop over comment records
        do i=1,ncom
          comment1=blank
          read(io7) comment1
          if(i.EQ.1) then
c ---       Save model version line
            doc1=comment1(1:80)
          elseif(i.LE.4) then
c ---       Save 3 title lines
            title(i-1)=comment1(1:80)
          endif
          if(lmetcf) write(io1,*) comment1
        enddo
        if(lecho .AND. .NOT.lmetcf) then
           write(io1,*)
           write(io1,*) doc1
           write(io1,*) title(1)
           write(io1,*) title(2)
           write(io1,*) title(3)
           write(io1,*)
        endif
c
c ---   record #NCOM+3 - run control parameters -- 37 words
        read(io7) ibymet,ibmmet,ibdmet,ibhmet,ibtzm,irlgmet,irtype,
     1   nxm, nym, nzm, xgridm, xorigm, yorigm, iwfcod, nssta,
     2   nusta, npsta, nowsta, nlu, iwat1, iwat2, lcalgrd,
     3   pmapm,datumm,datenm,feastm,fnorthm,utmhemm,iutmznm,
     4   rnlat0m,relon0m,xlat1m,xlat2m

        call YR4(io1,ibymet,ierrb)
        if(ierrb.NE.0) stop 'Halted in MET1'

c ---   Place integer IBTZM into real XBTZM
        xbtzm=FLOAT(ibtzm)

        rlat0m=rnlat0m
        rlon0m=-relon0m

c ---   Convert start time from end to beginning of first hour
        call JULDAY(io1,ibymet,ibmmet,ibdmet,ibjdmet)
        call INCR(io1,ibymet,ibjdmet,ibhmet,-1)
        call GRDAY(io1,ibymet,ibjdmet,ibmmet,ibdmet)
        ibsmet=0
c ---   Find end of run (irlgmet is integer hours)
        ieymet=ibymet
        iemmet=ibmmet
        iedmet=ibdmet
        iejdmet=ibjdmet
        iehmet=ibhmet
        call INCR(io1,ieymet,iejdmet,iehmet,irlgmet)
        call GRDAY(io1,ieymet,iejdmet,iemmet,iedmet)
        iesmet=0

      else
c ---   Read older CALMET header records
c --------------------------------------
c !!!   itimes=0
        itimes=1
        i2dmet=0
c
c ---   record #1 - run title -- 60 words
        read(io7)title
c
c ---   record #2 - run control parameters -- 26 words
c ---   (vermet, levmet are both 8 bytes)
        read(io7)vermet,levmet,ibymet,ibmmet,ibdmet,ibhmet,ibtzm,
     1   irlgmet,irtype,nxm, nym, nzm, xgridm, xorigm, yorigm,
     2   iutmznm, iwfcod, nssta, nusta, npsta, nowsta, nlu, iwat1,
     3   iwat2, lcalgrd

        if(lecho) then
           write(io1,104)vermet,levmet
104        format(1x,'CALMET Version: ',a8,3x,'Level: ',a8)
        endif

c ---   Place integer IBTZM into real XBTZM
        xbtzm=FLOAT(ibtzm)

c ---   Enforce YYYY format
        call YR4(io1,ibymet,ierrb)
        if(ierrb.NE.0) then
           write(*,*)
           stop 'Halted in MET1'
        endif
c
c ---   New record -- #3 - additional run control data -- 8 words
c ---   Null values are assigned in block data
c ---   This record was introduced in CALMET Version 5.0 (980304)
        if(vercmet.EQ.'1.5') then
           read(io7)xlat0m,xlon0m,llconfm,conecm,xlat1m,xlat2m,
     &              rlat0m,rlon0m
        endif
c
c ---   Recast map projection information
        if(LLCONFM) then
           pmapm='LCC     '
        else
           pmapm='UTM     '
           UTMHEMM='N   '
           if(xlat0m.LT.0.) UTMHEMM='S   '
        endif
c ---   Set datum to unknown
        datumm='unknown'

c ---   Convert start time from end to beginning of first hour
        call JULDAY(io1,ibymet,ibmmet,ibdmet,ibjdmet)
        call INCR(io1,ibymet,ibjdmet,ibhmet,-1)
        call GRDAY(io1,ibymet,ibjdmet,ibmmet,ibdmet)
        ibsmet=0
c ---   Find end of run (irlgmet is integer hours)
        ieymet=ibymet
        iemmet=ibmmet
        iedmet=ibdmet
        iejdmet=ibjdmet
        iehmet=ibhmet
        call INCR(io1,ieymet,iejdmet,iehmet,irlgmet)
        call GRDAY(io1,ieymet,iejdmet,iemmet,iedmet)
        iesmet=0

      endif
c
      if(LECHO)then
         write(io1,*)
         write(io1,*)
         write(io1,*)'-- Data begin at:'
         write(io1,*)' IBYR    = ',ibymet
         write(io1,*)' IBMO    = ',ibmmet
         write(io1,*)' IBDY    = ',ibdmet
         write(io1,*)' IBHR    = ',ibhmet
         write(io1,*)' IBSEC   = ',ibsmet
         write(io1,*)'-- Data end at:'
         write(io1,*)' IEYR    = ',ieymet
         write(io1,*)' IEMO    = ',iemmet
         write(io1,*)' IEDY    = ',iedmet
         write(io1,*)' IEHR    = ',iehmet
         write(io1,*)' IESEC   = ',iesmet
         write(io1,*)
         write(io1,*)' XBTZM   = ',xbtzm
         write(io1,*)' IRLG    = ',irlgmet
         write(io1,*)' IRTYPE  = ',irtype
         write(io1,*)' LCALGRD = ',lcalgrd
         write(io1,*)' PMAP    = ',pmapm
         write(io1,*)' DATUM   = ',datumm
         write(io1,*)' NIMADATE= ',datenm
         write(io1,*)' FEASTM  = ',feastm
         write(io1,*)' FNORTHM = ',fnorthm
         if(PMAPM.EQ.'UTM     ') then
            write(io1,*)' IUTMZN  = ',iutmznm
            write(io1,*)' UTMHEM  = ',utmhemm
         else
            write(io1,*)' XLAT1M  = ',xlat1m
            write(io1,*)' XLAT2M  = ',xlat2m
            write(io1,*)' RLAT0M  = ',rlat0m
            write(io1,*)' RLON0M  = ',rlon0m
         endif
         write(io1,*)' NXM     = ',nxm
         write(io1,*)' NYM     = ',nym
         write(io1,*)' NZM     = ',nzm
         write(io1,*)' XGRIDM  = ',xgridm
         write(io1,*)' XORIGM  = ',xorigm
         write(io1,*)' YORIGM  = ',yorigm
         write(io1,*)' IWFCOD  = ',iwfcod
         write(io1,*)' NSSTA   = ',nssta
         write(io1,*)' NUSTA   = ',nusta
         write(io1,*)' NPSTA   = ',npsta
         write(io1,*)' NOWSTA  = ',nowsta
         write(io1,*)' NLU     = ',nlu
         write(io1,*)' IWAT1   = ',iwat1
         write(io1,*)' IWAT2   = ',iwat2

      endif
c
c --- (CEC - Check that NXM and NYM are smaller than MXGX, MXGY in params.utl)
      if(nxm.gt.mxgx.or.nym.gt.mxgy) then
      write(io1,*)'ERROR in subr. MET1 - size of MET grid smaller than',
     1 ' maximum array provided in PARAMS.UTL: MXGX,MXGY= ',mxgx,mxgy,
     2   '   CALMET grid = ',nxm,nym
         write(*,*)
         stop 'Halted in MET1 -- see list file.'
      endif
c
c --- Check that enough workspace has been allocated
      nec=nxm*nym
      if(nwork.lt.nec)then
         write(io1,*)'ERROR in subr. MET1 -- insufficient workspace ',
     1   'allocated to work arrays -- Allocated (NWORK) = ',nwork,
     2   '   Required (NEC) = ',nec
         write(*,*)
         stop 'Halted in MET1 -- see list file.'
      endif
c
      nzmp1=nzm+1
      if(mxnzp1.lt.nzmp1)then
         write(io1,*)'ERROR in subr. MET1 -- dimension of ZFACEM ',
     1   'array is too small for no. layers in MET file'
         write(io1,*)'MXNZP1 = ',mxnzp1,' NZMP1 = ',nzmp1
         write(*,*)
         stop 'Halted in MET1 -- see list file.'
      endif

c -------------------------------------
c
c --- record #NCOM+4 - cell face heights (NZ + 1 words)
      call rdr1d(io7,itimes,zfacem,nzmp1,clab1,idum,idum,idum,idum)
c
      if(LECHO)then
         write(io1,203)(zfacem(n),n=1,nzmp1)
203      format(1x,'ZFACEM  = ',10(f9.3,', ')/)
      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'ZFACE')then
         clabexp='ZFACE'
         go to 999
      endif
c -------------------------------------
c
c ************************************************
c --- Perform QA checks on header record variables
c ************************************************
      call metqa

      nums=nssta+nusta+npsta
      if(lecho .AND. nums.GT.0) then
         write(io1,*)'Met Station Locations (Met Grid meters)'
      endif
c
c --- records #NCOM+5 & 6 - x, y coordinates of surface stations
c --- (NSSTA words each record)
      if(nssta.gt.0)then
         call rdr1d(io7,itimes,xssta,nssta,clab1,idum,idum,idum,idum)
         call rdr1d(io7,itimes,yssta,nssta,clab2,idum,idum,idum,idum)
         if(LECHO)then
            write(io1,*)clab1,' = ',(xssta(n),n=1,nssta)
            write(io1,*)clab2,' = ',(yssta(n),n=1,nssta)
         endif
c
c ---    Check that record labels matches expected labels
         if(clab1.ne.'XSSTA'.or.clab2.ne.'YSSTA')then
            clabex1='XSSTA'
            clabex2='YSSTA'
            go to 1999
         endif
      endif
c -------------------------------------
c
c --- records #NCOM+7 & 8 - x, y coordinates of upper air stations
c --- (NUSTA words each record)
      if(nusta.gt.0)then
         call rdr1d(io7,itimes,xusta,nusta,clab1,idum,idum,idum,idum)
         call rdr1d(io7,itimes,yusta,nusta,clab2,idum,idum,idum,idum)
         if(LECHO)then
            write(io1,*)clab1,' = ',(xusta(n),n=1,nusta)
            write(io1,*)clab2,' = ',(yusta(n),n=1,nusta)
         endif
c
c ---    Check that record labels matches expected labels
         if(clab1.ne.'XUSTA'.or.clab2.ne.'YUSTA')then
            clabex1='XUSTA'
            clabex2='YUSTA'
            go to 1999
         endif
      endif
c -------------------------------------
c
c --- records #NCOM+9 & 10 - x, y coordinates of precipitation stations
c --- (NPSTA words each record)
      if(npsta.gt.0)then
         call rdr1d(io7,itimes,xpsta,npsta,clab1,idum,idum,idum,idum)
         call rdr1d(io7,itimes,ypsta,npsta,clab2,idum,idum,idum,idum)
         if(LECHO)then
            write(io1,*)clab1,' = ',(xpsta(n),n=1,npsta)
            write(io1,*)clab2,' = ',(ypsta(n),n=1,npsta)
         endif
c
c ---    Check that record labels matches expected labels
         if(clab1.ne.'XPSTA'.or.clab2.ne.'YPSTA')then
            clabex1='XPSTA'
            clabex2='YPSTA'
            go to 1999
         endif
      endif
c -------------------------------------
c
c --- record #NCOM+11 - surface roughness lengths (NX * NY words)
      call rdr2d(io7,itimes,z0,wrk1,mxnx,mxny,nx,ny,clab1,
     &           idum,idum,idum,idum,ieof)
      if(ieof.EQ.1) then
         write(*,*)
         stop 'MET1: Unexpected EOF in CALMET header'
      endif
      if(LECHO)then
         messag='Surface roughness lengths (m)'
         messag(63:70)=clab1
         call out(z0,idum,1,5,ldate,messag,nx,ny)
      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'Z0')then
         clabexp='Z0'
         go to 999
      endif
c -------------------------------------
c
c --- record #NCOM+12 - land use categories (NX * NY words)
      call rdi2d(io7,itimes,ilandu,wrk1,mxnx,mxny,nx,ny,clab1,
     &           idum,idum,idum,idum)
      if(LECHO)then
         messag='Land use categories'
         messag(63:70)=clab1
         call out(xdum,ilandu,2,5,ldate,messag,nx,ny)
      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'ILANDU')then
         clabexp='ILANDU'
         go to 999
      endif
c -------------------------------------
c
c --- record #NCOM+13 - elevations (NX * NY words)
      call rdr2d(io7,itimes,elev,wrk1,mxnx,mxny,nx,ny,clab1,
     &          idum,idum,idum,idum,ieof)
      if(ieof.EQ.1) then
         write(*,*)
         stop 'MET1: Unexpected EOF in CALMET header'
      endif
      if(LECHO)then
         messag='Terrain heights (m)'
         messag(63:70)=clab1
         call out(elev,idum,1,5,ldate,messag,nx,ny)
      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'ELEV')then
         clabexp='ELEV'
         go to 999
      endif
c -------------------------------------
c
c --- record #NCOM+14 - leaf area index (NX * NY words)
      call rdr2d(io7,itimes,xlai,wrk1,mxnx,mxny,nx,ny,clab1,
     &           idum,idum,idum,idum,ieof)
      if(ieof.EQ.1) then
         write(*,*)
         stop 'MET1: Unexpected EOF in CALMET header'
      endif
      if(LECHO)then
         messag='Leaf area index'
         messag(63:70)=clab1
         call out(xlai,idum,1,5,ldate,messag,nx,ny)
      endif
c
c --- Check that record label matches expected label
      if(clab1.ne.'XLAI')then
         clabexp='XLAI'
         go to 999
      endif
c -------------------------------------
c
c --- record #NCOM+15 - nearest surface station to each grid point
c                  (NX * NY words)
c frr (09/01)
      if (nssta.ge.1) then
         call rdi2d(io7,itimes,nears,wrk1,mxnx,mxny,nx,ny,clab1,
     &              idum,idum,idum,idum)
         if(LECHO)then
           messag='Nearest surface station no. to each grid point'
           messag(63:70)=clab1
           call out(xdum,nears,2,5,ldate,messag,nx,ny)
         endif
c
c ---    Check that record label matches expected label
         if(clab1.ne.'NEARS')then
            clabexp='NEARS'
            go to 999
         endif

      endif

c -------------------------------------
c
      return
c
c --- Write error messages -- incorrect record label read
999   continue
      write(io1,1001)clab1,clabexp
1001  format(/1x,'ERROR in subr. MET1 -- incorrect record label ',
     1 'read from MET data file'//1x,'Label read     = ',a8/
     2                               1x,'Label expected = ',a8)
      write(*,*)
      stop 'Halted in MET1 -- see list file.'
1999  continue
      write(io1,2001)clab1,clab2,clabex1,clabex2
2001  format(/1x,'ERROR in subr. MET1 -- incorrect record label ',
     1 'read from MET data file'//
     2 1x,'Labels read     = ',a8,2x,a8/
     3 1x,'Labels expected = ',a8,2x,a8)
      write(*,*)
      stop 'Halted in MET1 -- see list file.'
c
      end
c----------------------------------------------------------------------
      subroutine metqa
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 000106                  METQA
c                D. Strimaitis,  Earth Tech
c
c --- PURPOSE:  Perform consistency checks of data read from the
c               CALMET header records with CALPUFF header records
c
c
c --- INPUTS:
c     Common block /CTRL/ variables:
c           ISDATHR, ISSEC, IEDATHR, IESEC
c     Common block /HEAD/ variables:
c           IELMET, JELMET, NZ, DELX, DELY, XORIGKM, YORIGKM
c     Common block /METHD/ variables:
c           NXM, NYM, NZM, XGRIDM, XORIGM, YORIGM, IUTMZNM,
c           NSSTA, NUSTA, NPSTA, ZFACEM(mxnzp1)
c           ibymet,ibmmet,ibdmet,ibjdmet,ibhmet,ibsmet,
c           ieymet,iemmet,iedmet,iejdmet,iehmet,iesmet
c     Parameters:
c           MXNZP1, MXSS, IO1
c
c --- OUTPUT:
c         none
c
c --- METQA called by:  MET1
c --- METQA calls:      TPERIOD
c----------------------------------------------------------------------
c
c --- Include parameters
      include 'params.utl'
c
c --- Include common blocks
      include 'ctrl.utl'
      include 'head.utl'
      include 'methd.utl'
c
c --- Initialize error flag
      ierr=0
c
c --- No. X, Y grid cells
      if(ielmet.ne.nxm)then
         write(io1,102)
102      format(/1x,'ERROR in subr. METQA -- MET header record ',
     1   'data are inconsistent with the control file input')
         write(io1,*)'NX = ',ielmet,' NXM = ',nxm
         ierr=1
      endif
      if(jelmet.ne.nym)then
         write(io1,102)
         write(io1,*)'NY = ',jelmet,' NYM = ',nym
         ierr=1
      endif
c
c --- Grid size
      if(abs(xgridm-1000.*delx).gt.0.1)then
         write(io1,102)
         write(io1,*)'DELX(km) = ',delx,' XGRIDM(m) = ',xgridm
         ierr=1
      endif
c
c --- Grid reference coordinates
      if(abs(xorigm-1000.*xorigkm).gt.1.0)then
         write(io1,102)
         write(io1,*)'XORIG(km) = ',xorigkm,' XORIGM(m) = ',xorigm
         ierr=1
      endif
      if(abs(yorigm-1000.*yorigkm).gt.1.0)then
         write(io1,102)
         write(io1,*)'YORIG(km) = ',yorigkm,' YORIGM(m) = ',yorigm
         ierr=1
      endif
c
c --- MXSS in CALPUFF parameter file must be at least as large as
c     the number of surface stations used in the CALMET run
      if(nssta.gt.mxss)then
         write(io1,106)nssta,mxss
106      format(/1x,'ERROR in subr. METQA -- NSSTA read from ',
     1   'MET header record is greater than MXSS in the POSTUTIL ',
     2   'parameter file'/1x,'NSSTA = ',i6,3x,'MXSS = ',i6)
         ierr=1
      endif
c
c --- Check start of met data
c --- Met data must not start after beginning of simulation
      kdathr=ibymet*100000+ibjdmet*100+ibhmet
      call TPERIOD(1,kdathr,ibsmet,isdathr,issec,iedathr,iesec,iloc)
      if(iloc.NE.-1) then
         write(io1,*)
         write(io1,*)'WARNING in subr. METQA -- Met data begin late'
         write(io1,*)'     Start of simulation: ',isdathr,issec
         write(io1,*)'     Start of met data  : ',kdathr,ibsmet
      endif
c
c --- Check end of met data
      kdathr=ieymet*100000+iejdmet*100+iehmet
      call TPERIOD(-1,kdathr,iesmet,isdathr,issec,iedathr,iesec,iloc)
      if(iloc.NE.1) then
         write(io1,*)
         write(io1,*)'WARNING  subr. METQA -- Met data end early'
         write(io1,*)'     End of simulation: ',iedathr,iesec
         write(io1,*)'     End of met data  : ',kdathr,iesmet
         write(io1,*)
      endif
c
c --- Stop execution if any errors encountered
      if(ierr.eq.1)STOP
c
      return
      end
c----------------------------------------------------------------------
      subroutine getmet_2d(ior1,ior2,ior3,ndathrm,
     &                  nsecm,idathrb,isecb,idathre,isece)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641     Level: 050302             GETMET_2D
c                C. Escoffier-Czaja,  Earth Tech
c
c --- PURPOSE:  Read RH, TEMP and RHOAIR from 3 CALPUFF output files
c               of MET DATA
c              
c               
c --- INPUTS:
c       NDATHRM - integer - YYYYJJJHH of target (year-day-hour)
c         NSECM - integer - SSSS of target (seconds)
c
c --- OUTPUT:
c      NDHRMETB - integer - YYYYJJJHH at beginning of met period
c      NSECMETB - integer - SSSS at beginning of met period
c      NDHRMETE - integer - YYYYJJJHH at end of met period
c      NSECMETE - integer - SSSS at end of met period
c
c    Common block /METHR/ variables:
c         all
c
c --- GETMET called by:  MAIN
c --- GETMET calls:      RDMET
c----------------------------------------------------------------------
      INCLUDE 'params.utl'
      INCLUDE 'rtrhd.utl'
      INCLUDE 'ctrl.utl'
      INCLUDE 'conc.utl'
      INCLUDE 'methr.utl'
c
c --- Declare temporary variables using actual array dimensions
      character*15 tsplv

      ieof=0

c --- Read RH data from VISB.DAT file and match date-time to the hour
11       if(irhtime3.EQ.2) then
            read(ior1,end=999) iyrb,ijdayb,ihrb,isecb,iyre,ijdaye,
     &                         ihre,isece
         else
            read(ior1,end=999) iyre,ijdaye,ihre
            isece=0
            isecb=0
            iyrb=iyre
            ijdayb=ijdaye
            ihrb=ihre
            call INCR(io1,iyrb,ijdayb,ihrb,-1)
         endif
c
c ---    Enforce YYYY format for year
         call YR4(io1,iyrb,ierrb)
         call YR4(io1,iyre,ierre)
         if(ierrb.NE.0 .OR. ierre.NE.0) stop 'Halted in GETMET_2D'
c
         if (i2drhurh.eq.1)then
            read(ior1) tsplv,((irh2d(i,j),i=1,ielmet3),j=1,jelmet3)
         else
            read(ior1) tsplv,(irhss(is),is=1,nstas3)
         endif
         idathrb=iyrb*100000+ijdayb*100+ihrb
         idathre=iyre*100000+ijdaye*100+ihre
c
c --- Check to see if required date-time has been read
      kdathrb=idathrb
      kdathre=idathre
      ksecb=isecb
      ksece=isece
c
      call TPERIOD(0,ndathrm,nsecm,kdathrb,ksecb,kdathre,
     &             ksece,ilocate)
      if(ilocate.EQ.1) then
c ---    Obtain next time period in all 3 met. files
         go to 11
      elseif(ilocate.EQ.-1) then
         write(io1,*)'ERROR in subr. GETMET_2D--current time not found',
     1   ' in the 3 met. data files'
         write(io1,*)' -- Current date/hour,sec = ',ndathrm,nsecm
         write(io1,*)' -- Last date/hour,sec read = ',kdathrb,ksecb
         write(io1,*)'                           to ',kdathre,ksece
         stop
      endif
c
	ieof=0
c
c --- Read TEMP data from TEMP.DAT file and match date-time to the hour
12       if(itptime3.EQ.2) then
            read(ior2,end=998) iyrb,ijdayb,ihrb,isecb,iyre,ijdaye,
     &                         ihre,isece
         else
            read(ior2,end=998) iyre,ijdaye,ihre
            isece=0
            isecb=0
            iyrb=iyre
            ijdayb=ijdaye
            ihrb=ihre
            call INCR(io1,iyrb,ijdayb,ihrb,-1)
         endif
c
c ---    Enforce YYYY format for year
         call YR4(io1,iyrb,ierrb)
         call YR4(io1,iyre,ierre)
         if(ierrb.NE.0 .OR. ierre.NE.0) stop 'Halted in GETMET_2D'
c
         if (i2drhutp.eq.1)then
            read(ior2) tsplv,((temp2d(i,j),i=1,ielmet3),j=1,jelmet3)
         else
            read(ior2) tsplv,(tempss(is),is=1,nstas3)
         endif
         idathrb=iyrb*100000+ijdayb*100+ihrb
         idathre=iyre*100000+ijdaye*100+ihre
         if(idathrb.LT.kdathrb) then
             goto 12
          elseif(idathrb.EQ.kdathrb .AND. isecb.LT.ksecb) then
             goto 12
          elseif(idathrb.NE.kdathrb .OR. isecb.NE.ksecb) then
           write(io6,*)'METPUF_T: date-time not found in T2D.DAT file'
           write(io1,*)'METPUF_T: date-time not found in T2D.DAT file'
           write(io1,*)'  Beginning date-Time needed: ',kdathrb,ksecb
           write(io1,*)'     Ending date-Time needed: ',kdathre,ksece
           write(io1,*)'   Beginning date-Time found: ',idathrb,isecb
           write(io1,*)'      Ending date-Time found: ',idathre,isece
             stop
          endif
c
	ieaf=0
c
c --- Read RHOAIR data from RHOAIR.DAT file and match date-time to the hour
13       if(irhotime3.EQ.2) then
            read(ior3,end=997) iyrb,ijdayb,ihrb,isecb,iyre,ijdaye,
     &                         ihre,isece
         else
            read(ior3,end=997) iyre,ijdaye,ihre
            isece=0
            isecb=0
            iyrb=iyre
            ijdayb=ijdaye
            ihrb=ihre
            call INCR(io1,iyrb,ijdayb,ihrb,-1)
         endif
c
c ---    Enforce YYYY format for year
         call YR4(io1,iyrb,ierrb)
         call YR4(io1,iyre,ierre)
         if(ierrb.NE.0 .OR. ierre.NE.0) stop 'Halted in GETMET_2D'
c
         if (i2drhurho.eq.1)then
            read(ior3) tsplv,((rho2d(i,j),i=1,ielmet3),j=1,jelmet3)
         else
            read(ior3) tsplv,(rhoss(is),is=1,nstas3)
         endif
         idathrb=iyrb*100000+ijdayb*100+ihrb
         idathre=iyre*100000+ijdaye*100+ihre
         if(idathrb.LT.kdathrb) then
            goto 13
         elseif(idathrb.EQ.kdathrb .AND. isecb.LT.ksecb) then
            goto 13
         elseif(idathrb.NE.kdathrb .OR. isecb.NE.ksecb) then
         write(io6,*)'METPUF_rho: date-time not found in RHO2D.DAT file'
         write(io1,*)'METPUF_rho: date-time not found in RHO2D.DAT file'
         write(io1,*)'  Beginning date-Time needed: ',kdathrb,ksecb
         write(io1,*)'     Ending date-Time needed: ',kdathre,ksece
         write(io1,*)'   Beginning date-Time found: ',idathrb,isecb
         write(io1,*)'      Ending date-Time found: ',idathre,isece
            stop
          endif
c
      return

c --- EOF encountered
999   ieof=1
      if(ieof.EQ.1) then
c ---    Reached end of RHMET.DAT file; check for another file
         call NEXTFIL(ior1,nxm,nym,ifound)
         if(ifound.EQ.1) then
            goto 11
         else
            write(*,*)'RDMET: Unexpected EOF in RHMET.DAT records'
            write(*,*)'Target  Date-time: ',ndathrm,nsecm
            write(*,*)'Reading Date-time: ',iyrb,ijdayb,ihrb
            write(*,*)'               to: ',iyre,ijdaye,ihre
            stop 
         endif
      endif
c
998   ieof=1
      if(ieof.EQ.1) then
c ---    Reached end of TEMP.DAT file; check for another file
         call NEXTFIL(ior2,nxm,nym,ifound)
         if(ifound.EQ.1) then
            goto 12
         else
            write(*,*)'RDMET: Unexpected EOF in TEMP.DAT records'
            write(*,*)'Target  Date-time: ',ndathrm,nsecm
            write(*,*)'Reading Date-time: ',iyrb,ijdayb,ihrb
            write(*,*)'               to: ',iyre,ijdaye,ihre
            stop 
         endif
      endif
c
997   ieof=1
      if(ieof.EQ.1) then
c ---    Reached end of RHOAIR.DAT file; check for another file
         call NEXTFIL(ior3,nxm,nym,ifound)
         if(ifound.EQ.1) then
            goto 13
         else
            write(*,*)'RDMET: Unexpected EOF in RHOAIR.DAT records'
            write(*,*)'Target  Date-time: ',ndathrm,nsecm
            write(*,*)'Reading Date-time: ',iyrb,ijdayb,ihrb
            write(*,*)'               to: ',iyre,ijdaye,ihre
            stop 
         endif
      endif
c
      return
      end
c----------------------------------------------------------------------
      subroutine getmet_1d(ior1,ndathrm,
     &                  nsecm,idathrb,isecb,idathre,isece)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641     Level: 050302             GETMET_1D
c                C. Escoffier-Czaja,  Earth Tech
c
c --- PURPOSE:  Read RH, TEMP and RHOAIR from 1D file
c               of MET DATA
c              
c               
c --- INPUTS:
c       NDATHRM - integer - YYYYJJJHH of target (year-day-hour)
c         NSECM - integer - SSSS of target (seconds)
c
c --- OUTPUT:
c      NDHRMETB - integer - YYYYJJJHH at beginning of met period
c      NSECMETB - integer - SSSS at beginning of met period
c      NDHRMETE - integer - YYYYJJJHH at end of met period
c      NSECMETE - integer - SSSS at end of met period
c
c    Common block /METHR/ variables:
c         all
c
c --- GETMET called by:  MAIN
c --- GETMET calls:      RDMET
c----------------------------------------------------------------------
      INCLUDE 'params.utl'
      INCLUDE 'ctrl.utl'
      INCLUDE 'conc.utl'
      INCLUDE 'methr.utl'

      ieof=0

c --- Read RH data from VISB.DAT file and match date-time to the hour
11       READ(ior1,END=999)iyre,ijdaye,ihre,irh1d,temp1d,rho1d
c100     FORMAT(3i4,i6,f7.1,f7.2)
            isece=0
            isecb=0
            iyrb=iyre
            ijdayb=ijdaye
            ihrb=ihre
            call INCR(io1,iyrb,ijdayb,ihrb,-1)
c ---    Enforce YYYY format for year
         call YR4(io1,iyrb,ierrb)
         call YR4(io1,iyre,ierre)
         if(ierrb.NE.0 .OR. ierre.NE.0) stop 'Halted in GETMET_1D'
c
         idathrb=iyrb*100000+ijdayb*100+ihrb
         idathre=iyre*100000+ijdaye*100+ihre
c
c --- Check to see if required date-time has been read
      kdathrb=idathrb
      kdathre=idathre
      ksecb=isecb
      ksece=isece
c
      call TPERIOD(0,ndathrm,nsecm,kdathrb,ksecb,kdathre,
     &             ksece,ilocate)
      if(ilocate.EQ.1) then
c ---    Obtain next time period in all 3 met. files
         go to 11
      elseif(ilocate.EQ.-1) then
         write(io1,*)'ERROR in subr. GETMET_1D--current time not found',
     1   ' in the 1D data files'
         write(io1,*)' -- Current date/hour,sec = ',ndathrm,nsecm
         write(io1,*)' -- Last date/hour,sec read = ',kdathrb,ksecb
         write(io1,*)'                           to ',kdathre,ksece
         stop
      endif
c
      return

c --- EOF encountered
999   ieof=1
      return
      end
c----------------------------------------------------------------------
      subroutine getmet(ndathrm,nsecm,ndhrmetb,nsecmetb,
     &                  ndhrmete,nsecmete)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 000106                 GETMET
c                D. Strimaitis,  Earth Tech
c
c --- PURPOSE:  Isolate call to RDMET so that CALPUFF common that holds
c               the met data is not in MAIN
c
c --- Modified: 010901  (FRR)
c               calling list to subroutine RDMET changed to allow
c               new 2D arrays in NOOBS CALMET (v5)
c              
c               
c --- INPUTS:
c       NDATHRM - integer - YYYYJJJHH of target (year-day-hour)
c         NSECM - integer - SSSS of target (seconds)
c
c    Common block /WRKSPC/ variables:
c         TMP1, NW1
c
c --- OUTPUT:
c      NDHRMETB - integer - YYYYJJJHH at beginning of met period
c      NSECMETB - integer - SSSS at beginning of met period
c      NDHRMETE - integer - YYYYJJJHH at end of met period
c      NSECMETE - integer - SSSS at end of met period
c
c    Common block /METHR/ variables:
c         all
c
c --- GETMET called by:  MAIN
c --- GETMET calls:      RDMET
c----------------------------------------------------------------------
c --- Include parameters
      include 'params.utl'
c
c --- Include common blocks
      INCLUDE 'methr.utl'
      INCLUDE 'wrkspc.utl'

      logical ldbmet
      ldbmet=.FALSE.

c frr (09/01)-  allow for 2D arrays of TEMP, RHO, QS, RH, IPCODE (NOOBS calmet)
c
      call RDMET(ndathrm,nsecm,tmp1,nw1,ldbmet,umet,vmet,tmet,
     1           ipgt,htmix,ustar,xmonin,wstar,rmm,tempss,rhoss,
     2           qswss,irhss,ipcode,temp2d,rho2d,qsw2d,irh2d,ipcode2d,
     3           ndhrmete,nsecmete,ndhrmetb,nsecmetb,wdiv)

c         call RDMET(ndathrm,nsecm,tmp1,nw1,ldbmet,umet,vmet,tmet,
c     1           ipgt,htmix,ustar,xmonin,wstar,rmm,tempss,rhoss,
c     2           qswss,irhss,ipcode,ndhrmete,nsecmete,ndhrmetb,
c     3           nsecmetb,wdiv)

      return
      end
c----------------------------------------------------------------------
      subroutine rdmet(idathrm,isecm,wrk1,nwork,lecho,umet,vmet,tmet,
     1 ipgt,htmix,ustar,xmonin,wstar,rmm,tempss,rhoss,qswss,irhss,
     2 ipcode,temp2d,rho2d,qsw2d,irh2d,ipcode2d,kdathre,ksece,kdathrb,
     3 ksecb,wdiv)
c----------------------------------------------------------------------
c --- POSTUTIL   Version: 1.641    Level: 060406                  RDMET
c
c --- CALPUFF    Version: 6.11     Level: 050915                  RDMET
c                J. Scire, SRC
c
c --- PURPOSE:  Read CALMET meteorological data for one hour
c
c --- POSTUTIL Modifications -
c
c ---  Version 1.5, level 060329 to Version 1.51, level 060406
c          The number of entry in NEXTFIL subroutine has been changed.
c          TO accomodate POSTUTIL, an additional 4th entry is
c          needed: the unit number of the met. file.
c          (CEC)
c
c --- UPDATE
c --- V5.7-V6.1     050915  (DGS): allow for begin/end time format
c                                  resolved to seconds, and YYYY
c                                  format used for year (retain
c                                  compatibility with older CALMET)
c --- V5.5-V5.7     030402  (DGS): Add list file unit to YR4 call
c                   030119  (FRR): bug fix to make sure ipcode2d data
c                                  is read in when npsta=-1
c
c ---               090101  (FRR): read CALMET records V5.3, level 010901
c frr (09/01)
c ---  Different format after CALMET version V5.3 - Level 010901
c ---  NOOBS mode allowed (no upper and possibly no surface stations)
c ---  2D records of sf Temp, RH, IPCODE, RHO, QS (previously 1-D)
c ---  Precipitation data if NPSTA>0 or NPSTA=-1 (prognostic rain data)
c ---
c --- V5.2-V5.4     000602  (DGS): add message to "stop"
c --- V5.0-V5.2     991104  (DGS): YYYY format used for year
c --- V5.0-V5.0     990228a (DGS): add IEOF to RDR2D arguments and add
c                                  NEXTFIL to open next CALMET file
c --- V5.0-V5.0     990228  (DGS): correct dimension in call to RDR2D
c                                  for w-field
c --- V4.0-V5.0     971107  (DGS): store w for vertical divergence field
c                   971107  (DGS): add record-skip logic from COMP to
c                                  find current date-time
c
c --- INPUTS:
c       IDATHRM - integer       - Mid-pt date/hour of required
c                                 data (YYYYJJJHH)
c         ISECM - integer       - Mid-pt second of required data (SSSS)
c   WRK1(nwork) - real array    - Work array of length "NWORK" words
c         NWORK - integer       - Dimension of work array -- NOTE:
c                                 NWORK must be at least as large as
c                                 the max. of (NXM * NYM or NSSTA)
c         LECHO - logical       - Flag controlling printing of input
c                                 meteorological data fields
c    Common block /METHD/ variables:
c         NXM, NYM, NZM, NSSTA, NPSTA, LCALGRD, I2DMET, ITIMES
c    Parameters:
c         MXNZMP1, IO6-->IO1
c
c --- OUTPUT:
c
c   UMET(mxnx,mxny,mxnz) - real    - U component of the wind (m/s)
c                                    at each grid point
c   VMET(mxnx,mxny,mxnz) - real    - V-component of the wind (m/s)
c   TMET(mxnx,mxny,mxnz) - real    - 3-D temperature field (deg. K)
c        IPGT(mxnx,mxny) - integer - PGT stability class
c       HTMIX(mxnx,mxny) - real    - Mixing height (m)
c       USTAR(mxnx,mxny) - real    - Surface friction velocity (m/s)
c      XMONIN(mxnx,mxny) - real    - Monin-Obukhov length (m)
c       WSTAR(mxnx,mxny) - real    - Convective velocity scale (m/s)
c         RMM(mxnx,mxny) - real    - Precipitation rate (mm/hr)
c          TEMPSS(mxss) - real    - Air temperature (deg. K) at
c                                    each surface met. station
c           RHOSS(mxss) - real    - Air density (kg/m**3)
c           QSWSS(mxss) - real    - Short wave solar radiation
c                                    (W/m**2)
c           IRHSS(mxss) - integer - Relative humidity (percent)
c          IPCODE(mxss) - integer - Precipitation code at each surface
c                                    station
c      TEMP2D(mxnx,mxny) - real    - Air temperature (deg. K)
c       RHO2D(mxnx,mxny) - real    - Air density (kg/m**3)
c       QSW2D(mxnx,mxny) - real    - Short wave solar radiation
c                                    (W/m**2)
c       IRH2D(mxnx,mxny) - integer - Relative humidity (percent)
c    IPCODE2D(mxnx,mxny) - integer - Precipitation code 
c                 KDATHR - integer - Date of hour of data (YYYYJJJHH)
c                 KDATHR - integer - Date of hour of data (YYYYJJJHH)
c                KDATHRE - integer - End date/hour of data (YYYYJJJHH)
c                  KSECE - integer - End second of data (SSSS)
c                KDATHRB - integer - Begin date/hour of data (YYYYJJJHH)
c                  KSECB - integer - Begin second of data (SSSS)
c   WDIV(mxnx,mxny,mxnz) - real    - Vertical divergence dw/dz; this
c                                    array holds the w-data here
c
c --- RDMET called by:  COMP
c --- RDMET calls:      RDR1D, RDR2D, RDI1D, RDI2D, INCR, OUT,
c                       NEXTFIL, TPERIOD, YR4
c----------------------------------------------------------------------
c
c !!! Change io6 to io1 for POSTUTIL list file
c --------------------------------------------
c
c --- Include parameter statements
      include 'params.utl'
c
c --- Include common /METHD/ -- met. data parameters
      include 'methd.utl'
c
      real umet(mxnx,mxny,mxnz),vmet(mxnx,mxny,mxnz)
      real tmet(mxnx,mxny,mxnz)
      real htmix(mxnx,mxny),ustar(mxnx,mxny),xmonin(mxnx,mxny)
      real wstar(mxnx,mxny),rmm(mxnx,mxny)
      real wrk1(nwork)
c frr (09/01)in noobs mode, nssta can be = 0 => dimension with mxss
c     real tempss(nssta),rhoss(nssta),qswss(nssta)
      real tempss(mxss),rhoss(mxss),qswss(mxss)
      real wdiv(mxnx,mxny,mxnz)
c
c frr (09/01) - additional 2D arrays for temp,rh,rho,ipcode,qsw
      real    temp2d(mxnx,mxny),rho2d(mxnx,mxny),qsw2d(mxnx,mxny)
      integer irh2d (mxnx,mxny),ipcode2d(mxnx,mxny)
      real    rdum(mxnx,mxny)

c frr (09/01)in noobs mode, nssta can be = 0 => dimension with mxss
c     integer ipgt(mxnx,mxny),irhss(nssta),ipcode(nssta)
      integer ipgt(mxnx,mxny),irhss(mxss),ipcode(mxss)
c
      character*70 messag
      character*8 clabel
      character*8 clabexp
c
      logical lecho,ldate
c
      data ldate/.false./
c
c --- Check that enough workspace has been allocated
      nec=max0(nxm*nym,nssta)
      if(nwork.lt.nec)then
         write(io1,*)'ERROR in subr. RDMET -- insufficient workspace ',
     1   'allocated to work arrays -- Allocated (NWORK) = ',nwork,
     2   '   Required (NEC) = ',nec
         write(*,*)
         stop 'Halted in RDMET -- see list file.'
      endif

c --- Begin reading data records for one time period
1     continue

c
c --- Track date-hr of end of this period to check for problems in file
      kdathre=0
      ksece=0
c
c --- read the U, V wind components
      do 10 iz=1,nzm
      call rdr2d(io7,itimes,umet(1,1,iz),wrk1,mxnx,mxny,nxm,nym,clabel,
     &           ndathrb1,nsecb1,ndathre1,nsece1,ieof)
      if(ieof.EQ.1) then
c ---    Reached end of CALMET.DAT file; check for another file
c --- cec(060406) - unit number for met. file is added in NEXTFIL parameters entries.
         call NEXTFIL(io7,nxm,nym,ifound)
         if(ifound.EQ.1) then
            goto 1
         else
            write(*,*)
            write(*,*)'RDMET: Unexpected EOF in CALMET records'
            write(*,*)'Target  Date-time: ',idathrm,isecm
            write(*,*)'Reading Date-time: ',ndathrb1,nsecb1
            write(*,*)'               to: ',ndathre1,nsece1
            stop 
         endif
      endif
c
c --- Check that record label matches expected label
      clabexp='U-LEV'
      write(clabexp(6:8),'(i3)')iz
      if(clabel.ne.clabexp)go to 999
c

      call rdr2d(io7,itimes,vmet(1,1,iz),wrk1,mxnx,mxny,nxm,nym,clabel,
     &           ndathrb2,nsecb2,ndathre2,nsece2,ioef)
      if(ieof.EQ.1) then
         write(*,*)
         stop 'RDMET: Unexpected EOF in CALMET records'
      endif
      clabexp(1:1)='V'
      if(clabel.ne.clabexp)go to 999
c

c --- Use the W wind component (conditional) for w-divergence
      if(lcalgrd)then
         call rdr2d(io7,itimes,wdiv(1,1,iz),wrk1,mxnx,mxny,nxm,nym,
     &              clabel,ndathrb3,nsecb3,ndathre3,nsece3,ieof)
         if(ieof.EQ.1) then
            write(*,*)
            stop 'RDMET: Unexpected EOF in CALMET records'
         endif
         clabexp(1:5)='WFACE'
         if(clabel.ne.clabexp)go to 999
         if(ndathre2.ne.ndathre3 .OR. nsece2.ne.nsece3)go to 2999
      endif
c

c --- Check for consistency of date/time at end of period
      if(ndathre1.ne.ndathre2 .OR. nsece1.ne.nsece2)go to 2999
      if(kdathre.eq.0)then
c ---    first vertical level
         kdathre=ndathre1
         ksece=nsece1
      else if(ndathre1.ne.kdathre .OR. nsece1.ne.ksece)then
c ---    date/time does not match value for previous layer
         ndathre=ndathre1
         nsece=nsece1
         go to 3999
      endif
10    continue
c
c --- Read the 3-D temperature field
      if(lcalgrd)then
         clabexp='T-LEV'
         do 12 iz=1,nzm
         call rdr2d(io7,itimes,tmet(1,1,iz),wrk1,mxnx,mxny,nxm,nym,
     &              clabel,ndathrb,nsecb,ndathre,nsece,ieof)
         if(ieof.EQ.1) then
            write(*,*)
            stop 'RDMET: Unexpected EOF in CALMET records'
         endif
c
         write(clabexp(6:8),'(i3)')iz
         if(clabel.ne.clabexp)go to 999
         if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
12       continue
      endif
c

c --- read other 2-D meteorological fields
c
c --- PGT stability class
      call rdi2d(io7,itimes,ipgt,wrk1,mxnx,mxny,nxm,nym,clabel,
     &           ndathrb,nsecb,ndathre,nsece)
      clabexp='IPGT'
      if(clabel.ne.clabexp)go to 999
      if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
c
c --- FRICTION VELOCITY
      call rdr2d(io7,itimes,ustar,wrk1,mxnx,mxny,nxm,nym,clabel,
     &           ndathrb,nsecb,ndathre,nsece,ieof)
      if(ieof.EQ.1) then
         write(*,*)
         stop 'RDMET: Unexpected EOF in CALMET records'
      endif
      clabexp='USTAR'
      if(clabel.ne.clabexp)go to 999
      if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
c
c --- MIXING HEIGHT
      call rdr2d(io7,itimes,htmix,wrk1,mxnx,mxny,nxm,nym,clabel,
     &           ndathrb,nsecb,ndathre,nsece,ieof)
      if(ieof.EQ.1) then
         write(*,*)
         stop 'RDMET: Unexpected EOF in CALMET records'
      endif
      clabexp='ZI'
      if(clabel.ne.clabexp)go to 999
      if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
c
c --- MONIN-OBUKHOV LENGTH
      call rdr2d(io7,itimes,xmonin,wrk1,mxnx,mxny,nxm,nym,clabel,
     &           ndathrb,nsecb,ndathre,nsece,ieof)
      if(ieof.EQ.1) then
         write(*,*)
         stop 'RDMET: Unexpected EOF in CALMET records'
      endif
      clabexp='EL'
      if(clabel.ne.clabexp)go to 999
      if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
c
c --- CONVECTIVE VELOCITY SCALE
      call rdr2d(io7,itimes,wstar,wrk1,mxnx,mxny,nxm,nym,clabel,
     &           ndathrb,nsecb,ndathre,nsece,ieof)
      if(ieof.EQ.1) then
         write(*,*)
         stop 'RDMET: Unexpected EOF in CALMET records'
      endif
      clabexp='WSTAR'
      if(clabel.ne.clabexp)go to 999
      if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
c
c --- PRECIPITATION DATA
c frr (09/01) - Additional option: if NPSTA=-1, prognostic gridded rain data
c      if(npsta.gt.0)then
      if(npsta.ne.0)then
         call rdr2d(io7,itimes,rmm,wrk1,mxnx,mxny,nxm,nym,clabel,
     &              ndathrb,nsecb,ndathre,nsece,ieof)
         if(ieof.EQ.1) then
            write(*,*)
            stop 'RDMET: Unexpected EOF in CALMET records'
         endif
      endif

c
c frr (09/01)
c --- NOOBS CALMET: full 2-D fields of the following variables instead
c --- of 1-D fields (values at the surface stations only)
c           - Air temperature (deg. K),
c           - Air density (kg/m**3),
c           - Short-wave solar radiation (W/m**2),
c           - Relative humidity (percent),
c           - Precipitation code

      if(i2dmet.EQ.1) then

c ---   New CALMET output format - 2D arrays -
c       call rdr1d(io7,tempss,nssta,clabel,ndathr)
        call rdr2d(io7,itimes,temp2d,wrk1,mxnx,mxny,nxm,nym,clabel,
     &           ndathrb,nsecb,ndathre,nsece,ieof)
        if(ieof.EQ.1) then
           write(*,*)
           stop 'RDMET: Unexpected EOF in CALMET records'
        endif
        clabexp='TEMPK'
        if(clabel.ne.clabexp)go to 999
        if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
c
c       call rdr1d(io7,rhoss,nssta,clabel,ndathr)
        call rdr2d(io7,itimes,rho2d,wrk1,mxnx,mxny,nxm,nym,clabel,
     &           ndathrb,nsecb,ndathre,nsece,ieof)
        if(ieof.EQ.1) then
           write(*,*)
           stop 'RDMET: Unexpected EOF in CALMET records'
        endif
        clabexp='RHO'
        if(clabel.ne.clabexp)go to 999
        if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
c
c       call rdr1d(io7,qswss,nssta,clabel,ndathr)
        call rdr2d(io7,itimes,qsw2d,wrk1,mxnx,mxny,nxm,nym,clabel,
     &           ndathrb,nsecb,ndathre,nsece,ieof)
        if(ieof.EQ.1) then
           write(*,*)
           stop 'RDMET: Unexpected EOF in CALMET records'
        endif
        clabexp='QSW'
        if(clabel.ne.clabexp)go to 999
        if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
c
c       call rdi1d(io7,irhss,nssta,clabel,ndathr)
        call rdi2d(io7,itimes,irh2d,wrk1,mxnx,mxny,nxm,nym,clabel,
     &           ndathrb,nsecb,ndathre,nsece)
        clabexp='IRH'
        if(clabel.ne.clabexp)go to 999
        if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
c
c ---   Precipitation code at surface stations
c        if(npsta.gt.0)then
c frr 030119 - bug fix: also precipitation code if prognostic rain
c       (i.e npsta,eq.-1)
        if(npsta.ne.0)then
c         call rdi1d(io7,ipcode,nssta,clabel,ndathr)
          call rdi2d(io7,itimes,ipcode2d,wrk1,mxnx,mxny,nxm,nym,clabel,
     &           ndathrb,nsecb,ndathre,nsece)
          clabexp='IPCODE'
          if(clabel.ne.clabexp)go to 999
          if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
       endif


      elseif(i2dmet.EQ.0) then
c ---    read 1-D meteorological fields:
c           Air temp. (deg. K),
c           Air density (kg/m**3),
c           Short-wave solar radiation (W/m**2),
c           Relative humidity (percent),
c           Precipitation code
         call rdr1d(io7,itimes,tempss,nssta,clabel,
     &           ndathrb,nsecb,ndathre,nsece)
         clabexp='TEMPK'
         if(clabel.ne.clabexp)go to 999
         if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
c
         call rdr1d(io7,itimes,rhoss,nssta,clabel,
     &           ndathrb,nsecb,ndathre,nsece)
         clabexp='RHO'
         if(clabel.ne.clabexp)go to 999
         if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
c
         call rdr1d(io7,itimes,qswss,nssta,clabel,
     &           ndathrb,nsecb,ndathre,nsece)
         clabexp='QSW'
         if(clabel.ne.clabexp)go to 999
         if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
c
         call rdi1d(io7,itimes,irhss,nssta,clabel,
     &           ndathrb,nsecb,ndathre,nsece)
         clabexp='IRH'
         if(clabel.ne.clabexp)go to 999
         if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
c
c ---    Precipitation code at surface stations
         if(npsta.gt.0)then
            call rdi1d(io7,itimes,ipcode,nssta,clabel,
     &              ndathrb,nsecb,ndathre,nsece)
            clabexp='IPCODE'
            if(clabel.ne.clabexp)go to 999
            if(ndathre.ne.kdathre .OR. nsece.ne.ksece)go to 3999
         endif

      else
         write(*,*)'Subr. RDMET:  Invalid I2DMET = ',i2dmet
         stop
      endif
c
c --- Set date-time for period
c ---------------------------------
      kdathre=ndathre
      ksece=nsece
c !!! if(itimes.EQ.0) then
      if(itimes.EQ.1) then
c ---    Construct beginning time for older CALMET data
         iyy=kdathre/100000
         iyyddd=kdathre/100
         iddd=iyyddd-iyy*1000
         ihh=kdathre-iyyddd*100
         call INCR(io1,iyy,iddd,ihh,-1)
         kdathrb=iyy*100000+iddd*100+ihh
         ksecb=0
      else
         kdathrb=ndathrb
         ksecb=nsecb
      endif

c --- Enforce YYYY format for year
      ibyr=kdathrb/100000
      ibdyhr=kdathrb-ibyr*100000
      ieyr=kdathre/100000
      iedyhr=kdathre-ieyr*100000
      call YR4(io1,ibyr,ierrb)
      call YR4(io1,ieyr,ierre)
      if(ierrb.NE.0 .OR. ierre.NE.0) stop 'Halted in RDMET'
      kdathrb=ibyr*100000+ibdyhr
      kdathre=ieyr*100000+iedyhr

c --- Check to see if required date-time has been read
      call TPERIOD(0,idathrm,isecm,kdathrb,ksecb,kdathre,
     &             ksece,ilocate)
      if(ilocate.EQ.1) then
c ---    Obtain next time period in MET file
         go to 1
      elseif(ilocate.EQ.-1) then
         write(io1,*)'ERROR in subr. RDMET -- current time not found',
     1   ' in the MET data file'
         write(io1,*)' -- Current date/hour,sec = ',idathrm,isecm
         write(io1,*)' -- Last date/hour,sec read = ',kdathrb,ksecb
         write(io1,*)'                           to ',kdathre,ksece
         stop
      endif
c
c --- Write the input fields, if requested
      if(LECHO)then
c
c ---    Date and time
         write(io1,*)
         write(io1,*)'------ MET. FIELDS BEGINNING: ',kdathrb,ksecb
         write(io1,*)'------            AND ENDING: ',kdathre,ksece
         write(io1,*)
c
c ---    3-D fields
         do 101 iz=1,nzm
         messag='U-component (m/s) -- Level: '
         write(messag(29:31),'(i3)')iz
         call out(umet(1,1,iz),idum,1,5,ldate,messag,nxm,nym)
         messag(1:1)='V'
         call out(vmet(1,1,iz),idum,1,5,ldate,messag,nxm,nym)
         messag(1:1)='W'
         call out(wdiv(1,1,iz),idum,1,5,ldate,messag,nxm,nym)
101      continue
c
c ---    2-D fields
         messag='PGT stability class'
         call out(xdum,ipgt,2,1,ldate,messag,nxm,nym)
         messag='Friction velocity (m/s)'
         call out(ustar,idum,1,5,ldate,messag,nxm,nym)
         messag='Mixing height (m)'
         call out(htmix,idum,1,5,ldate,messag,nxm,nym)
         messag='Monin-Obukhov length (m)'
         call out(xmonin,idum,1,5,ldate,messag,nxm,nym)
         messag='Convective velocity scale (m/s)'
         call out(wstar,idum,1,5,ldate,messag,nxm,nym)
         if(npsta.gt.0)then
            messag='Precipitation rate (mm/hr)'
            call out(rmm,idum,1,5,ldate,messag,nxm,nym)
         endif
c
c ---    1-D fields
c frr (09/01) after level 010901, 2D CALMET fields-  Before, 1D
         if(i2dmet.EQ.1) then
            messag='Surface Air Temperature (deg. K)'
            call out(temp2d,idum,1,5,ldate,messag,nxm,nym)
            messag='Surface Air Density (Kg/m**3)'
            call out(rho2d,idum,1,5,ldate,messag,nxm,nym)
            messag='Short Wave Radiation (W/m**2)'
            call out(qsw2d,idum,1,5,ldate,messag,nxm,nym)
            messag='Surface Relative Humidity (%)'
            call out(rdum,irh2d,2,5,ldate,messag,nxm,nym)
            if(npsta.ne.0)then
               messag='Precipitation code'
               call out(rdum,ipcode2d,2,5,ldate,messag,nxm,nym)
            endif
         elseif(i2dmet.EQ.0) then
            write(io1,*)'TEMPSS (deg. K) = ',(tempss(n),n=1,nssta)
            write(io1,*)'RHOSS (kg/m**3)  = ',(rhoss(n),n=1,nssta)
            write(io1,*)'QSWSS (W/m**2)   = ',(qswss(n),n=1,nssta)
            write(io1,*)'IRHSS (percent)  = ',(irhss(n),n=1,nssta)
            if(npsta.gt.0)write(io1,*)'IPCODE           = ',
     1       (ipcode(n),n=1,nssta)
         else
            write(*,*)'Subr. RDMET:  Invalid I2DMET = ',i2dmet
            stop
         endif
      endif
c
      return
c
c --- Write error messages -- incorrect record label read
999   continue
      write(io1,1001)clabel,clabexp
1001  format(/1x,'ERROR in subr. RDMET -- incorrect record label ',
     1 'read from MET data file'//1x,'Label read     = ',a8/
     2                               1x,'Label expected = ',a8)
      write(*,*)
      stop 'Halted in RDMET -- see list file.'

c --- date/time variables do not match
2999  continue
      write(io1,*)'ERROR in subr. RDMET -- date/time variables for',
     & ' wind components do not match --  '
      write(io1,*)'   NDATHRE1,NSECE1= ',ndathre1,nsece1
      write(io1,*)'   NDATHRE2,NSECE2= ',ndathre2,nsece2
      write(io1,*)'   NDATHRE3,NSECE3= ',ndathre3,nsece3
      stop 'Halted in RDMET -- see list file.'
3999  continue
      write(io1,*)'ERROR in subr. RDMET -- date/time variables ',
     1 'do not match -- '
      write(io1,*)'   NDATHRE ,NSECE = ',ndathre,nsece
      write(io1,*)'   KDATHRE ,KSECE = ',kdathre,ksece
      stop 'Halted in RDMET -- see list file.'
      end
c----------------------------------------------------------------------
      subroutine out(rarray,iarray,ityp,nsigd,ldate,messag,nx,ny)
c----------------------------------------------------------------------
c --- POSTUTIL   Version: 1.641    Level: 060329                    OUT
c
c --- CALPUFF    Version: 6.11     Level: 050915                    OUT
c ---            J. Scire, SRC
c
c --- PURPOSE:  Write a gridded field of real or integer numbers
c
c --- UPDATE
c --- V5.4-V6.1     050915  (DGS): resolve times to the second and
c --- V5.2-V5.4     000602  (DGS): add message to "stop"
c                   000602  (DGS): increase format for cell IDs
c --- V5.0-V5.2     991104  (DGS): YYYY format for year
c
c --- INPUTS:
c     RARRAY(MXNX,MXNY) - Real array  - Array of real numbers to print
c                                       (used only if ITYP = 1)
c     IARRAY(MXNX,MXNY) - Int. array  - Array of integer numbers to
c                                       print (used only if ITYP = 2)
c                  ITYP - Integer     - Array type (1=real, 2=integer)
c                 NSIGD - Integer     - No. digits to print (valid range
c                                       for NSIGD is 1 to 5)
c                 LDATE - Logical     - Control variable for printing
c                                       of date (.true. = print date in
c                                       common /GEN/, .false. = do not
c                                       print date)
c                MESSAG - Char.*70    - Label of table
c                    NX - Integer     - No. X grid cells being used in
c                                       array
c                    NY - Integer     - No. Y grid cells being used in
c                                       array
c       Common block /DATEHR/ variables: - (Used only if LDATE=.true.)
c          NYRB, NMOB, NDAYB, NJULB, NHRB, NSECB,
c          NYRE, NMOE, NDAYE, NJULE, NHRE, NSECE
c       Parameters: MXNX, MXNY, IO6-->IO1
c
c --- OUTPUT:  none
c
c --- OUT    called by:  MET1
c --- OUT    calls:      WRT, WRT2
c----------------------------------------------------------------------
c
c !!! Change io6 to io1 for POSTUTIL list file
c --------------------------------------------
c
c --- Include parameters
      include 'params.utl'
c
c --- Include date/hour common block
      include 'datehr.utl'
c
      real rarray(mxnx,mxny)
c
      integer iarray(mxnx,mxny),icol(5)
      integer iout(mxnx)
c
      logical ldate
c
      character*70 messag
      character*1 sign(mxnx),plus,minus
      character*24 form1(5)
      character*21 form2(5)
      character*18 form3(5)
c
      data icol /40,40,30,25,20/
      data plus,minus /'+','-'/
      data form1 /'(1x,i3,1x,1hI,40(i3,1x))',
     1            '(1x,i3,1x,1hI,40(i3,1x))',
     2            '(1x,i3,1x,1hI,40(i3,1x))',
     3            '(1x,i3,1x,1hI,40(i4,1x))',
     4            '(1x,i3,1x,1hI,40(i5,1x))'/
      data form2 /'(5x,1hI,40(2x,a1,1x))',
     1            '(5x,1hI,40(2x,a1,1x))',
     2            '(5x,1hI,40(2x,a1,1x))',
     3            '(5x,1hI,40(3x,a1,1x))',
     4            '(5x,1hI,40(4x,a1,1x))'/
      data form3 /'(6x,40(i3,1x))',
     1            '(6x,40(i3,1x))',
     2            '(6x,40(i3,1x))',
     3            '(6x,40(i4,1x))',
     4            '(6x,40(i5,1x))'/
c
c --- check that valid values of array type (ityp) and print digits
c --- (nsigd) have been passed to routine
      if(ityp.ne.1.and.ityp.ne.2)then
         write(io1,*)'ERROR in SUBR. OUT -- invalid value of ITYP -- ',
     1   'ITYP = ',ityp
         write(*,*)
         stop 'Halted in OUT -- see list file.'
      endif
      if(nsigd.lt.1.or.nsigd.gt.5)then
         write(io1,*)'ERROR in SUBR. OUT -- invalid value of NSIGD -- ',
     1   'NSIGD = ',nsigd
         write(*,*)
         stop 'Halted in OUT -- see list file.'
      endif
c
      icr=2
      if(nsigd.eq.1)icr=1
      if(mod(nx,icol(nsigd)).eq.0)then
         npass=nx/icol(nsigd)
      else
         npass=nx/icol(nsigd)+1
      endif
c
c --- real array -- find min. & max. values
      if(ityp.ne.1)go to 50
      xmax=-1.e-25
      xmin=1.e25
      do 10 i=1,nx
      do 10 j=1,ny
      if(rarray(i,j).gt.xmax)xmax=rarray(i,j)
      if(rarray(i,j).lt.xmin)xmin=rarray(i,j)
10    continue
      if(xmin.ne.0.0.or.xmax.ne.0.0)go to 12
      if(ldate)write(io1,94)messag,nyrb,nmob,ndayb,njulb,nhrb,nsecb,
     &                             nyre,nmoe,ndaye,njule,nhre,nsece
      if(.not.ldate)write(io1,95)messag
      write(io1,11)
11    format(1x,'GRID NOT PRINTED -- all values zero')
      return
c
12    continue
      xexp=xmax
      if(abs(xmin).gt.xmax)xexp=abs(xmin)
      iexp=alog10(xexp)
      if(xexp.lt.1.0)iexp=iexp-1
      nexp=iexp-(nsigd-icr)
      xscale=10.**(-nexp)
c
      ic2=0
      do 30 ipass=1,npass
c
      if(ldate)write(io1,94)messag,nyrb,nmob,ndayb,njulb,nhrb,nsecb,
     &                             nyre,nmoe,ndaye,njule,nhre,nsece
      if(.not.ldate)write(io1,95)messag
      write(io1,109)nexp
c
      ic1=ic2+1
      ic2=ic2+icol(nsigd)
      if(ic2.gt.nx)ic2=nx
c
         do 20 jj=ny,1,-1
         icnt=0
c
            do 18 i=ic1,ic2
            icnt=icnt+1
            if(rarray(i,jj).lt.0)then
               iout(icnt)=-(rarray(i,jj)*xscale-0.5)
               sign(icnt)=minus
            else
               iout(icnt)=rarray(i,jj)*xscale+0.5
               sign(icnt)=plus
            endif
18          continue
         call wrt(form1(nsigd),form2(nsigd),jj,iout,sign,icnt,io1)
20       continue
      nund=(nsigd+1)*icnt-1
      if(nsigd.eq.1)nund=(nsigd+2)*icnt-1
      write(io1,101)(minus,n=1,nund)
101   format(5x,128a1)
      call wrt2(form3(nsigd),ic1,ic2,io1)
30    continue
      return
c
c --- integer array -- find min. & max. values
50    continue
      kmax=-9999999
      kmin=9999999
      do 110 i=1,nx
      do 110 j=1,ny
      if(iarray(i,j).gt.kmax)kmax=iarray(i,j)
      if(iarray(i,j).lt.kmin)kmin=iarray(i,j)
110   continue
      if(kmin.ne.0.or.kmax.ne.0)go to 102
      if(ldate)write(io1,94)messag,nyrb,nmob,ndayb,njulb,nhrb,nsecb,
     &                             nyre,nmoe,ndaye,njule,nhre,nsece
      if(.not.ldate)write(io1,95)messag
      write(io1,11)
      return
c
102   continue
      xexp=kmax
      if(iabs(kmin).gt.kmax)xexp=iabs(kmin)
      iexp=alog10(xexp)
      if(xexp.lt.1.0)iexp=iexp-1
      nexp=iexp-(nsigd-icr)
      xscale=10.**(-nexp)
c
      ic2=0
      do 130 ipass=1,npass
c
      if(ldate)write(io1,94)messag,nyrb,nmob,ndayb,njulb,nhrb,nsecb,
     &                             nyre,nmoe,ndaye,njule,nhre,nsece
      if(.not.ldate)write(io1,95)messag
      write(io1,109)nexp
c
      ic1=ic2+1
      ic2=ic2+icol(nsigd)
      if(ic2.gt.nx)ic2=nx
c
         do 120 jj=ny,1,-1
         icnt=0
c
            do 118 i=ic1,ic2
            icnt=icnt+1
            if(iarray(i,jj).lt.0)then
               iout(icnt)=-(iarray(i,jj)*xscale-0.5)
               sign(icnt)=minus
            else
               iout(icnt)=iarray(i,jj)*xscale+0.5
               sign(icnt)=plus
            endif
118         continue
         call wrt(form1(nsigd),form2(nsigd),jj,iout,sign,icnt,io1)
120      continue
      nund=(nsigd+1)*icnt-1
      if(nsigd.eq.1)nund=(nsigd+2)*icnt-1
      write(io1,101)(minus,n=1,nund)
      call wrt2(form3(nsigd),ic1,ic2,io1)
130   continue
c
      return

94    format(/1x,a70,2x,'year: ',i4,2x,'month: ',i2,2x,'day: ',i2,2x,
     1       'Julian day: ',i3,2x,'hr: ',i2,2x,'sec: ',i4/,t70,
     2       'to year: ',i4,2x,'month: ',i2,2x,'day: ',i2,2x,
     3       'Julian day: ',i3,2x,'hr: ',i2,2x,'sec: ',i4/)
95    format(/1x,a70/)
109   format(1x,'Multiply all values by 10 ** ',i3/)

      end
c----------------------------------------------------------------------
      subroutine wrt(form1,form2,jj,iout,sign,n,io6)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 940430                    WRT
c
c --- Taken from CALPUFF:
c
c --- CALPUFF    Version: 6.0      Level: 940430                    WRT
c ---            J. Scire, SRC
c
c --- PURPOSE:  Write one Y row of gridded data
c
c --- INPUTS:
c              FORM1 - Char.*24    - Format field for Y label and data
c                                    to be printed
c              FORM2 - Char.*21    - Format field for sign of data
c                 JJ - Integer     - Y grid cell number
c            IOUT(N) - Int. array  - Array of data to be printed
c                                    (one Y row)
c            SIGN(N) - Char.*1     - Array containing sign of data
c                                    ('+' or '-')
c                  N - Integer     - Number of cells in this row
c                IO6 - Integer     - Fortran unit no. of output
c
c --- OUTPUT:  none
c
c --- WRT called by:  OUT, OUTSAM
c --- WRT calls:      none
c----------------------------------------------------------------------
      integer iout(n)
c
      character*1 sign(n)
      character*24 form1
      character*21 form2
c
      write(io6,form1)jj,iout
      write(io6,form2)sign
c
      return
      end
c----------------------------------------------------------------------
      subroutine wrt2(form,n1,n2,io6)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 940430                   WRT2
c
c --- Taken from CALPUFF:
c
c --- CALPUFF    Version: 6.0      Level: 940430                   WRT2
c ---            J. Scire, SRC
c
c --- PURPOSE:  Write a line labeling grid cell numbers
c
c --- INPUTS:
c               FORM - Char.*18    - Format field of data to be printed
c                 N1 - Integer     - Starting grid cell number
c                 N2 - Integer     - Ending grid cell number
c                IO6 - Integer     - Fortran unit no. of output
c
c --- OUTPUT:  none
c
c --- WRT2 called by:  OUT, OUTSAM
c --- WRT2 calls:      none
c----------------------------------------------------------------------
      character*18 form
c
      write(io6,form)(i,i=n1,n2)
      return
      end
c----------------------------------------------------------------------
      subroutine rdi1d(iomet,mtver,idat,nwords,clabel,ndathrb,
     &                 nsecb,ndathre,nsece)
c----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 040818           RDI1D
c --- CALPUFF   Version: 6.0            Level: 990709           RDI1D
c                J. Scire, SRC
c
c --- PURPOSE:  Read "NWORDS" of a one-dimensional integer array
c
c --- UPDATE
c --- V1.1-V1.4     040818  (DGS): Update MTVER setting from (0,1) to
c                                  (1,2)
c --- V5.0-V6.0     990709  (DGS): resolve times to the second, and
c                                  include begin/end times
c                                  (remains compatible with
c                                  older end-time version)
c
c --- INPUTS:
c         IOMET - integer       - Fortran unit number of input file
c         MTVER - integer       - Time-mark flag
c                                 2: begin-time / end-time
c                                 1: end-time (no seconds)
c
c --- OUTPUT:
c  IDAT(nwords) - integer array - Array read from file
c        NWORDS - integer       - Number of words to read
c        CLABEL - character*8   - Variable name
c       NDATHRB - integer       - Beginning date and time (YYYYJJJHH)
c         NSECB - integer       - Beginning seconds (SSSS)
c       NDATHRE - integer       - Ending date and time (YYYYJJJHH)
c         NSECE - integer       - Ending seconds (SSSS)
c
c --- RDI1D called by:  RDMET
c --- RDI1D calls:      none
c----------------------------------------------------------------------
c
      integer idat(nwords)
      character*8 clabel
c
      if(mtver.EQ.2) then
         read(iomet)clabel,ndathrb,nsecb,ndathre,nsece,idat
      else
         read(iomet)clabel,ndathre,idat
         nsece=0
         ndathrb=0
         nsecb=0
      endif
      return
      end
c----------------------------------------------------------------------
      subroutine rdi2d(iomet,mtver,idat,ibuf,mxnx,mxny,nx,ny,clabel,
     &                 ndathrb,nsecb,ndathre,nsece)
c----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 040818           RDI2D
c --- CALPUFF   Version: 6.0            Level: 990509           RDI2D
c                J. Scire, SRC
c
c --- PURPOSE:  Read NX * NY words of a 2-D integer array
c
c --- UPDATE
c --- V1.1-V1.4     040818  (DGS): Update MTVER setting from (0,1) to
c                                  (1,2)
c --- V5.0-V6.0     990709  (DGS): resolve times to the second, and
c                                  include begin/end times for CALMET
c                                  Version 6 (remains compatible with
c                                  older version of CALMET)
c
c --- INPUTS:
c               IOMET - integer       - Fortran unit number of input
c                                       file
c               MTVER - integer       - Time-mark flag
c                                       2: begin-time / end-time
c                                       1: end-time (no seconds)
c         IBUF(nx,ny) - integer array - Buffer to hold input data
c           MXNX,MXNY - integers      - Dimensions of data array
c               NX,NY - integers      - Actual size of grid to read
c
c --- OUTPUT:
c     IDAT(mxnx,mxny) - integer array - Input data array (padded if
c                                       necessary)
c              CLABEL - character*8   - Variable name
c             NDATHRB - integer       - Beginning date and time (YYYYJJJHH)
c               NSECB - integer       - Beginning seconds (SSSS)
c             NDATHRE - integer       - Ending date and time (YYYYJJJHH)
c               NSECE - integer       - Ending seconds (SSSS)
c
c --- RDI2D called by:  MET1, RDMET
c --- RDI2D calls:      none
c----------------------------------------------------------------------
      integer idat(mxnx,mxny),ibuf(nx,ny)
      character*8 clabel
c
      if(nx.eq.mxnx.and.ny.eq.mxny)then
c
c ---    entire array is being used -- read full grid
         if(mtver.EQ.2) then
            read(iomet)clabel,ndathrb,nsecb,ndathre,nsece,idat
         else
            read(iomet)clabel,ndathre,idat
            nsece=0
            ndathrb=0
            nsecb=0
         endif
      else
c
c ---    only a portion of grid being used -- read and
c ---    transfer from buffer
c
         if(mtver.EQ.2) then
            read(iomet)clabel,ndathrb,nsecb,ndathre,nsece,ibuf
         else
            read(iomet)clabel,ndathre,ibuf
            nsece=0
            ndathrb=0
            nsecb=0
         endif
c
         do 10 i=1,nx
         do 10 j=1,ny
         idat(i,j)=ibuf(i,j)
10       continue
      endif
c
      return
      end
c----------------------------------------------------------------------
      subroutine rdr1d(iomet,mtver,x,nwords,clabel,
     &                 ndathrb,nsecb,ndathre,nsece)
c----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 040818           RDR1D
c --- CALPUFF   Version: 6.0            Level: 990709           RDR1D
c                J. Scire, SRC
c
c --- PURPOSE:  Read "NWORDS" of a one-dimensional real array
c
c --- UPDATE
c --- V1.1-V1.4     040818  (DGS): Update MTVER setting from (0,1) to
c                                  (1,2)
c --- V5.0-V6.0     990709  (DGS): resolve times to the second, and
c                                  include begin/end times for CALMET
c                                  Version 6 (remains compatible with
c                                  older version of CALMET)
c
c --- INPUTS:
c         IOMET - integer     - Fortran unit number of input file
c         MTVER - integer     - Time-mark flag
c                               2: begin-time / end-time
c                               1: end-time (no seconds)
c
c --- OUTPUT:
c     X(nwords) - real array  - Array read from file
c        NWORDS - integer     - Number of words to read
c        CLABEL - character*8 - Variable name
c       NDATHRB - integer     - Beginning date and time (YYYYJJJHH)
c         NSECB - integer     - Beginning seconds (SSSS)
c       NDATHRE - integer     - Ending date and time (YYYYJJJHH)
c         NSECE - integer     - Ending seconds (SSSS)
c
c --- RDR1D called by:  MET1, RDMET
c --- RDR1D calls:      none
c----------------------------------------------------------------------
      real x(nwords)
      character*8 clabel
c
      if(mtver.EQ.2) then
         read(iomet)clabel,ndathrb,nsecb,ndathre,nsece,x
      else
         read(iomet)clabel,ndathre,x
         nsece=0
         ndathrb=0
         nsecb=0
      endif
      return
      end
c----------------------------------------------------------------------
      subroutine rdr2d(iomet,mtver,x,xbuf,mxnx,mxny,nx,ny,clabel,
     &                 ndathrb,nsecb,ndathre,nsece,ieof)
c----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641          Level: 040818           RDR2D
c --- CALPUFF   Version: 6.0            Level: 990709           RDR2D
c                J. Scire, SRC
c
c --- PURPOSE:  Read NX * NY words of a 2-D real array
c
c --- UPDATE 
c --- V1.1-V1.4     040818  (DGS): Update MTVER setting from (0,1) to
c                                  (1,2)
c --- V5.0-V6.0     990709  (DGS): resolve times to the second, and
c                                  include begin/end times for CALMET
c                                  Version 6 (remains compatible with
c                                  older version of CALMET)
c --- V5.0-V5.0     990228a (DGS): add IEOF to recover from end-of-file
c
c --- INPUTS:
c            IOMET - integer     - Fortran unit number of input file
c            MTVER - integer     - Time-mark flag
c                                  2: begin-time / end-time
c                                  1: end-time (no seconds)
c      XBUF(nx,ny) - real array  - Buffer to hold input data
c      MXNX,MXNY   - integers    - Dimensions of data array
c            NX,NY - integers    - Actual size of grid to read
c
c --- OUTPUT:
c     X(mxnx,mxny) - real array  - Input data array (padded if nec.)
c           CLABEL - character*8 - Variable name
c          NDATHRB - integer     - Beginning date and time (YYYYJJJHH)
c            NSECB - integer     - Beginning seconds (SSSS)
c          NDATHRE - integer     - Ending date and time (YYYYJJJHH)
c            NSECE - integer     - Ending seconds (SSSS)
c             IEOF - integer     - End-of-File status
c                                  0 = pointer within file
c                                  1 = EOF reached on read
c
c --- RDR2D called by:  MET1, RDMET
c --- RDR2D calls:      none
c----------------------------------------------------------------------
      real x(mxnx,mxny),xbuf(nx,ny)
      character*8 clabel

c --- Set EOF
      ieof=0
c
      if(nx.eq.mxnx.and.ny.eq.mxny)then
c
c ---    entire array is being used -- read full grid
         if(mtver.EQ.2) then
            read(iomet,end=999)clabel,ndathrb,nsecb,ndathre,nsece,x
         else
            read(iomet,end=999)clabel,ndathre,x
            nsece=0
            ndathrb=0
            nsecb=0
         endif
      else
c
c ---    only a portion of grid being used -- read and
c ---    transfer from buffer
         if(mtver.EQ.2) then
            read(iomet,end=999)clabel,ndathrb,nsecb,ndathre,nsece,xbuf
         else
            read(iomet,end=999)clabel,ndathre,xbuf
            nsece=0
            ndathrb=0
            nsecb=0
         endif
c
         do 10 i=1,nx
         do 10 j=1,ny
         x(i,j)=xbuf(i,j)
10       continue
      endif
c
      return

999   ieof=1
      return

      end
c----------------------------------------------------------------------
      subroutine nextfil(io,nx,ny,ifound)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 060329                NEXTFIL
c                D. Strimaitis, Earth Tech
c
c --- PURPOSE:  Prepare the next CALMET.DAT file data records if
c               multiple files are used
c
c --- MODIFICATION: 
c
c     V1.41 level 050302 to V1.5 level 060329  (DGS)
c               Add FILVERS call for dataset version (active for
c               CALMET only at this time)
c
c     Version 1.4 to Version 1.41 level 050302
c     (CEC)
c     Add features to accomodate multiple files for 2D met fields output
c     of CALPUFF (RH, TEMP, RHO).
c
c --- INPUTS:
c                 IO - integer - File unit number for CALMET.DAT file
c              NX,NY - integer - Met grid cell number
c
c     Common block /CTRL/ variables:
c           NMET, IMETDAT,METFM 
c     Common block /FILNAM/ variables:
c           UTLMETL, M2DRHUL, M2DTMPL, M2DRHOL
c
c --- OUTPUT:
c             IFOUND - integer - Flag indicating if more data records
c                                are available (1=yes)
c     Common block /DATASET/ variables:
c           vercmet,verrhum,vertemp,verdens
c
c --- Parameters used:
c        MXFILE
c
c --- NEXTFIL called by: RDMET, GETMET_2D
c --- NEXTFIL calls:     MET1, GETHDRH, FILVERS
c----------------------------------------------------------------------
c
c --- Include parameters
      include 'params.utl'
c
c --- Include common blocks
      include 'ctrl.utl'
      include 'dataset.utl'
      include 'filnam.utl'
      include 'wrkspc.utl'

      logical lprt

      data lprt/.FALSE./

c --- Close the current CALMET.DAT or 2DMET file
      close(io)

      if(metfm.eq.0) then

c ---    Screen out obvious cases
         if(imetdat.GE.nmet) then
            ifound=0
            return
         endif

c ---    Open the next data file in the list
         imetdat=imetdat+1
         k=imetdat
         call FILVERS(io,'BIN',utlmetl(k),'CALMET          ',vercmet)
         open(io,file=utlmetl(k),status='old',form='unformatted')

c ---    Read and check the header records
         lprt=.false.
         call met1(nx,ny,lprt,tmp1,nw1)

c ---    Set flag to 'yes'
         ifound=1

      elseif(metfm.eq.2.and.io.eq.in5) then

c ---    Screen out obvious cases
         if(io.eq.in5.and.imetrhu.GE.nmet) then
            ifound=0
            return
         endif

c ---    Open the next data file in the list
         imetrhu=imetrhu+1
         k=imetrhu
         call FILVERS(io,'BIN',m2drhul(k),'RELHUM2D        ',verrhum)
         open(io,file=m2drhul(k),status='old',form='unformatted')

c ---    Read and check the header records
         lprt=.false.
         call GETHDRH(in5,lprt)

c ---    Set flag to 'yes'
         ifound=1

      elseif(metfm.eq.2.and.io.eq.in6) then

c ---    Screen out obvious cases
         if(io.eq.in6.and.imettmp.GE.nmet) then
            ifound=0
            return
         endif

c ---    Open the next data file in the list
         imettmp=imettmp+1
         k=imettmp
         call FILVERS(io,'BIN',m2dtmpl(k),'TEMPK2D         ',vertemp)
         open(io,file=m2dtmpl(k),status='old',form='unformatted')

c ---    Read and check the header records
         lprt=.false.
         call GETHDRH(in6,lprt)

c ---    Set flag to 'yes'
         ifound=1

      elseif(metfm.eq.2.and.io.eq.in7) then

c ---    Screen out obvious cases
         if(io.eq.in7.and.imetrho.GE.nmet) then
            ifound=0
            return
         endif

c ---    Open the next data file in the list
         imetrho=imetrho+1
         k=imetrho
         call FILVERS(io,'BIN',m2drhol(k),'RHOAIR2D        ',verdens)
         open(io,file=m2drhol(k),status='old',form='unformatted')

c ---   Read and check the header records
         lprt=.false.
         call GETHDRH(in7,lprt)

c ---    Set flag to 'yes'
         ifound=1
      
      else
         write(*,*)'error- metfm=0 or 2 to allow multiple files'
         stop
      endif

      return
      end
c----------------------------------------------------------------------
      subroutine tfercf
c----------------------------------------------------------------------
c
c --- POSTUTIL Version: 1.641     Level: 090511                  TFERCF
c ---          D. Strimaitis
c
c --- PURPOSE: Transfer image of control input file to scratch file
c
c --- UPDATES:
c --- V1.3 (030402) to V1.61 (090511) (DGS)
c     - Reset length of line in control file to 200
c       (store image as 132, splitting lines if needed)
c
c --- INPUTS:
c
c        Parameters: mxcol, iox, in2
c
c --- OUTPUT:
c        Common /CTRL/:
c          NCOMMOUT - integer       - Number of comment records
c
c --- TFERCF called by:  (main)
c --- TFERCF calls:      none
c----------------------------------------------------------------------
c
      include 'params.utl'
      include 'params.cal'

      include 'ctrl.utl'

      character*132 aline, blank
      character*200 aline200, blank200
      character*33 blank33

c --- Set blanks
      data blank33/'                                 '/
      blank(1:33)=blank33
      blank(34:66)=blank33
      blank(67:99)=blank33
      blank(100:132)=blank33
      blank200(1:100)=blank(1:100)
      blank200(101:200)=blank(1:100)

c --- Confirm max size of control file record
      if(mxcol.GT.200) then
         write(*,*)'ERROR in TFERCF!'
         write(*,*)'Control file records are assumed to be <= 200'
         write(*,*)'Actual limit is MXCOL = ',mxcol
         write(*,*)'POSTUTIL code needs to be modified'
         stop
      endif      

c --- Reposition input control file
      REWIND(in2)

c --- Open scratch file
      OPEN(iox,status='scratch')

c --- Initialize number of records written to scratch file
      ncommout=0

c --- Transfer control file records
10    aline=blank
      aline200=blank200
      read(in2,'(a200)',end=999) aline200
      write(iox,'(a132)') aline200(1:132)
      ncommout=ncommout+1
      klen=LEN_TRIM(aline200)
      if(klen.GT.132) then
c ---    Line is split
         aline(1:68)=aline200(133:200)
         write(iox,'(a132)') aline
         ncommout=ncommout+1
      endif
      goto 10

c --- Transfer completed
999   close(in2)

      return
      end
c----------------------------------------------------------------------
      subroutine getdoc(io,ifil,ifilver,dataver)
c----------------------------------------------------------------------
c
c --- POSTUTIL Version: 1.641     Level: 040818                  GETDOC
c ---          D. Strimaitis,    Earth Tech, Inc.
c
c --- PURPOSE:  Read header comment records of the CALPUFF output
c               data file, and set file type.
c
c --- UPDATES:
c     V1.3(030402) to V1.4(040818)
c               (DGS) Add dataver to output arg list
c
c --- INPUTS:
c            io - integer    - Unit number for CALPUFF file
c          ifil - integer    - File number
c
c     Parameters:
c           IO1, IOX
c
c --- OUTPUT:
c       ifilver - integer    - Dataset version flag
c                              0: Before 2.0
c                              1: 2.0 or later
c       dataver - C*16       - Dataset version string
c
c     Common /CTRL/:
c           ncommout
c
c --- GETDOC called by:  GETHEAD, GETHDRH
c --- GETDOC calls:      none
c----------------------------------------------------------------------
c
c --- Include parameters
      include 'params.utl'
      include 'ctrl.utl'

c --- Local Variables
      character*16 dataset,dataset0,dataver,temp16
      character*33 blank33,break33
      character*64 datamod
      character*132 comment1,blank,break

      data blank33/'                                 '/
      data break33/'.................................'/

c --- Set blank (132 characters)
      blank(1:33)=blank33
      blank(34:66)=blank33
      blank(67:99)=blank33
      blank(100:132)=blank33

c --- Set break (132 characters)
      break(1:33)=break33
      break(34:66)=break33
      break(67:99)=break33
      break(100:132)=break33

c --- Read and test first record to check header format
c --- Record #1 - File Declaration -- 24 words
      read(io) dataset,dataver,datamod
      ifilver=0
      if(dataset.EQ.'CONC.DAT') then
         ifilver=1
      elseif(dataset.EQ.'DFLX.DAT') then
         ifilver=1
      elseif(dataset.EQ.'WFLX.DAT') then
         ifilver=1
      elseif(dataset.EQ.'VISB.DAT') then
         ifilver=1
      ELSEIF(dataset.EQ.'TK2D.DAT') then
         ifilver=1
      ELSEIF(dataset.EQ.'RHO2D.DAT') then
         ifilver=1
      endif

      if(ifilver.EQ.0) then
c ---    Old file format with no comment records
         dataver='0.0             '
         REWIND(io)

      elseif(ifilver.EQ.1) then

c ---    Store dataset name
         dataset0=dataset
         if(ifil.EQ.1) then
c ---       Retain all comment records for output file
c ---       To scratch file
            write(iox,'(a132)') blank
            write(iox,'(a132)') break
            write(iox,'(a132)') blank
            write(iox,'(2a16,a64)') dataset,dataver,datamod
            ncommout=ncommout+4
c ---       Number of comment records
            read(io) ncom
            ncommout=ncommout+ncom
            do k=1,ncom
               comment1=blank
               read(io) comment1
               write(iox,'(a132)') comment1
            enddo
         else
c ---       Keep only CALPUFF control file title information
            keep=1
            kout=0
c ---       To scratch file
            if(ifil.ne.0) then
            write(iox,'(a132)') blank
            write(iox,'(a132)') break
            write(iox,'(a132)') blank
            write(iox,'(2a16,a64)') dataset,dataver,datamod
            ncommout=ncommout+4
            endif
c ---       Number of comment records
            read(io) ncom
            do k=1,ncom
               comment1=blank
               read(io) comment1
               dataset=comment1(1:16)
               if(keep.EQ.1) then
c ---             Pass records until another dataset is found
                  if(dataset.EQ.'CALMET.DAT') then
                     keep=0
                     kout=0
                  elseif(ifil.eq.0) then
                     keep=0
                     kout=0
                  elseif(dataset.EQ.dataset0) then
                     keep=1
                     kout=0
c ---                To scratch file
                     write(iox,'(a132)') comment1
                     ncommout=ncommout+1
                  else
                     kout=kout+1
                     if(kout.LE.4) then
c ---                   To scratch file
                        write(iox,'(a132)') comment1
                        ncommout=ncommout+1
                     endif
                  endif
               elseif(keep.EQ.0) then
c ---             Skip records until CALPUFF dataset is found
                  if(dataset.EQ.dataset0) then
                     keep=1
                     kout=0
c ---                To scratch file
                     write(iox,'(a132)') comment1
                     ncommout=ncommout+1
                  endif
               endif
            enddo
         endif

      endif

c --- Prepare dataver string for testing in calling program
      temp16='                '
      k=0
      do i=1,16
         if(dataver(i:i).NE.' ') then
            k=k+1
            temp16(k:k)=dataver(i:i)
         endif
      enddo
      dataver=temp16

      return
      end
c
c----------------------------------------------------------------------
      subroutine getdoc1d(io)
c----------------------------------------------------------------------
c
c --- POSTUTIL Version: 1.641     Level: 071015                GETDOC1D
c ---          copied from GETDOC
c
c --- PURPOSE:  Read header comment records of the MET 1D file
c               and set file type.
c
c --- UPDATES:
c     V1.6 (071015)
c               (DGS) Declare COMMENT1 as 132 characters
c
c --- INPUTS:
c            io - integer    - Unit number for CALPUFF file
c
c     Parameters:
c           IO1, IOX
c
c --- OUTPUT:
c       ifilver - integer    - Dataset version flag
c                              0: Before 2.0
c                              1: 2.0 or later
c       dataver - C*16       - Dataset version string
c
c     Common /CTRL/:
c           ncommout
c
c --- GETDOC called by:  GETHEAD, GETHDRH
c --- GETDOC calls:      none
c----------------------------------------------------------------------
c
c --- Include parameters
      include 'params.utl'
      include 'ctrl.utl'

c --- Local Variables
      character*16 dataset,dataset0,dataver
      character*33 blank33,break33
      character*64 datamod
      character*132 comment1,blank,break

      data blank33/'                                 '/
      data break33/'.................................'/

c --- Set blank (132 characters)
      blank(1:33)=blank33
      blank(34:66)=blank33
      blank(67:99)=blank33
      blank(100:132)=blank33

c --- Set break (132 characters)
      break(1:33)=break33
      break(34:66)=break33
      break(67:99)=break33
      break(100:132)=break33

c --- Read and test first record to check header format
c --- Record #1 - File Declaration -- 24 words
      read(io) dataset,dataver,datamod
c
      if(dataset.NE.'TR_1D.DAT') then
      WRITE(*,*)'error - 1D MET does not have the right name',dataset
      WRITE(*,*)'it should be TR_1D.DAT'
      stop
      endif

c ---       Retain all comment records for output file
c ---       To scratch file
            write(iox,'(a132)') blank
            write(iox,'(a132)') break
            write(iox,'(a132)') blank
            write(iox,'(2a16,a64)') dataset,dataver,datamod
            ncommout=ncommout+4
c ---       Number of comment records
            read(io) ncom
            ncommout=ncommout+ncom
            do k=1,ncom
               comment1=blank
               read(io) comment1
               write(iox,'(a132)') comment1
            enddo
c
      return
      end
c
c-----------------------------------------------------------------------
      subroutine ALM1STEP(tcg,tcd,tct,ix,iy,idrec,itrec,ldb)
c-----------------------------------------------------------------------
c
c --- POSTUTIL  Version: 1.641           Level: 100917         ALM1STEP
c ---           C. Escoffier-Czaja
c
c  PURPOSE:     Read background concentration file and performed
c               Ammonia Limiting Method in one step - All three steps
c               combined.
c               Combination of GETRCRD, PARTITION and NO3RATIO subroutines
c
c --- UPDATES:
c     V1.44(060310) to V1.63(100917)
c               (DGS) Add equilibrium method choice (MEQPHASE)
c
c  ARGUMENTS:
c
c  CALLING ROUTINES:    POSTUTIL
c
c  EXTERNAL ROUTINES:   INCR, YR4, EXMET, CHEMEQ, ISODRIVER 
c-----------------------------------------------------------------------
c --- Include parameters
      INCLUDE 'params.utl'
      INCLUDE 'head.utl'
      INCLUDE 'ctrl.utl'
      INCLUDE 'conc.utl'
      INCLUDE 'rtrhd.utl'
      include 'datehr.utl'
      include 'methr.utl'
c
c --- Declare temporary variables using actual array dimensions
      character*15 tsplv
      real tcg(ix,iy),tcd(idrec),tct(itrec)
      real xwork1(mxgx,mxgy),xwork2(mxdrec),xwork3(mxctrec)
c
      logical ldb,lproc,lout,ltotal,lfatal,lnh3,ltnh3
      LOGICAL lbnoso4,lbnohno3,lbnono3,lbnotnh3

      real tno3floor
      logical l_tno3floor

c --- Set local control for minimum TNO3 for ISODRIVER
c ----------------------------------------------------
c ---     Selecting l_tno3floor=.TRUE. uses the cut-off TNO3
c ---     concentration as a floor, and computes NO3 corresponding
c ---     to this floor for all TNO3 less than this floor.  The
c ---     resulting ratio of NO3/TNO3(floor) is then multiplied by
c ---     the actual TNO3 to obtain the final NO3 concentration.
c ---     Selecting l_tno3floor=.FALSE. retains the original API
c ---     method and NO3=0.0 for all TNO3<TNO3(floor).
c ---     Note: TNO3(floor) = 1.0E-10 mol/m3 = .0062 ug/m3 (approx)
      data l_tno3floor/.TRUE./
      data tno3floor/1.0E-10/
c
c --- Set grid cell length in meters
      dgrid=1000.*delx
c --- Set the molecular wt ratio of NO3(62) to HNO3(63)
      data r62_63/0.984127/
c --- Set the molecular wt ratio of HNO3(63) to NO3(62)
      data r63_62/1.016129/
c
      ieof=0
      nwords=ix*iy
      nff=nfiles+1
c
c ---    Top of loop over sources within file (when present)
5        continue


c ---    Read time record from BACKGROUND pollutant file(s)
c ---    Note that time uses the 00-23 hour convention so that the 24th 
c ---    hour of day 12 has a start time of 23 on day 12 and an end
c ---    time of 00 on day 13
         if(iptimex(nff).EQ.2) then
            read(in8,end=999) myrb,mjdayb,mhrb,msecb,
     &                               myre,mjdaye,mhre,msece
c ---       Enforce YYYY format for year
            call YR4(io1,myrb,ierrb)
            call YR4(io1,myre,ierre)
            if(ierrb.NE.0 .OR. ierre.NE.0) stop 'Halted in GETRCRD'
c ---       Set target date-time
            mdathrb=myrb*100000+mjdayb*100+mhrb
            mdathre=myre*100000+mjdaye*100+mhre
         else
            read(in8,end=999) myre,mjdaye,mhre
            msece=0
            msecb=0
            myrb=myre
            mjdayb=mjdaye
            mhrb=mhre
            call INCR(io1,myrb,mjdayb,mhrb,-1)
c ---       Enforce YYYY format for year
            call YR4(io1,myrb,ierrb)
            call YR4(io1,myre,ierre)
            if(ierrb.NE.0 .OR. ierre.NE.0) stop 'Halted in backgALM'
c ---       Set target date-time
            mdathrb=myrb*100000+mjdayb*100+mhrb
            mdathre=myre*100000+mjdaye*100+mhre
         endif

c ---    Source information record
         if(isrcinfox(nff).EQ.1) then
            read(in8,end=999) mstype,msnum,sname,sxkm,sykm
            if(isrcindv.EQ.0) then
c ---          Total of all sources should be in file
               ltotal=.TRUE.
               lfatal=.FALSE.
               if(sname.NE.'TOTAL           ') lfatal=.TRUE.
               if(mstype.NE.0) lfatal=.TRUE.
               if(msnum.NE.1) lfatal=.TRUE.
               if(LFATAL) then
                  write(*,*)
                  write(*,*)
                  write(*,*)'ERROR in reading BCKGALM '
                  write(*,*)'Unexpected source information'
                  write(*,*)'Expected: 0 1 TOTAL'
                  write(*,*)'Found   : ',mstype,msnum,sname
                  stop
               endif
            elseif(isrcindv.EQ.1) then
c ---          Check for Total of all sources
               if(sname.EQ.'TOTAL           ') then
                  ltotal=.TRUE.
               else
                  ltotal=.FALSE.
               endif
            endif
         else
c ---       No source information in file
            ltotal=.TRUE.
         endif
c
c --- Initialized FLAG for background species needed to run ALM
            lbnoso4=.true.
            lbnono3=.true.
            lbnohno3=.true.
            lbnotnh3=.true.
c ---    Read data and store species/level for output
c ---    Loop over chemical species/levels
         do isl=1,nszout(nff)
c
            if(lsgrid) then
               if(lcomprs(nff))then
                  read(in8)ii
                  call uncomprs(xwork1,ii,in8,nwords,tsplv,tcg)
               else
                  read(in8) tsplv,tcg
               endif
               do j=1,iy
                do i=1,ix
                IF(asplst(isl,nff)(1:12).EQ.'SO4') then
                   lbnoso4=.false.
                   concgb(i,j,1)=tcg(i,j)
                ELSEIF(asplst(isl,nff)(1:12).EQ.'HNO3') then
                   lbnohno3=.false.
                   concgb(i,j,2)=tcg(i,j)
                ELSEIF(asplst(isl,nff)(1:12).EQ.'NO3') then
                   lbnono3=.false.
                   concgb(i,j,3)=tcg(i,j)
                ELSEIF(asplst(isl,nff)(1:12).eq.'TNH3') then
                   lbnotnh3=.false.
                   concgb(i,j,4)=tcg(i,j)
                ELSEIF(asplst(isl,nff)(1:12).EQ.'NH3') then
                   WRITE(*,*)' '
                   WRITE(*,*)'FATAL ERROR - Review message written'
                   WRITE(*,*)'in the list-file'
                   WRITE(io1,*)' '
                   WRITE(io1,*)'FATAL ERROR - In the background'
                   WRITE(io1,*)'Total NH3 (TNH3) is needed to run ALM'
                   WRITE(io1,*)' not NH3 gas (NH3)'
                   stop
                endif
                end do
               end do
            endif
c
            if(ldisc) then
               if(lcomprs(nff))then
                  read(in8)ii
                  call uncomprs(xwork2,ii,in8,idrec,tsplv,tcd)
               else
                  read(in8) tsplv,tcd
               endif
               do i=1,idrec
               IF(asplst(isl,nff)(1:12).EQ.'SO4') then
                  lbnoso4=.false.
                  concdb(i,1)=tcd(i)
               elseIF(asplst(isl,nff)(1:12).EQ.'HNO3') then
                  lbnohno3=.false.
                  concdb(i,2)=tcd(i)
               elseIF(asplst(isl,nff)(1:12).EQ.'NO3') then
                  lbnono3=.false.
                  concdb(i,3)=tcd(i)
               elseIF(asplst(isl,nff)(1:12).EQ.'TNH3') then
                  lbnotnh3=.false.
                  concdb(i,4)=tcd(i)
               ELSEIF(asplst(isl,nff)(1:12).EQ.'NH3') then
                  WRITE(*,*)' '
                  WRITE(*,*)'FATAL ERROR - Review message written'
                  WRITE(*,*)'in the list-file'
                  WRITE(io1,*)' '
                  WRITE(io1,*)'FATAL ERROR - In the background'
                  WRITE(io1,*)'Total NH3 (TNH3) is needed to run ALM'
                  WRITE(io1,*)' not NH3 gas (NH3)'
               stop
               endif
               enddo
            endif
c
            if(lctsg) then
               if(lcomprs(nff))then
                  read(in8)ii
                  call uncomprs(xwork3,ii,in8,itrec,tsplv,tct)
               else
                  read(in8) tsplv,tct
               endif
               do i=1,itrec
               IF(asplst(isl,nff)(1:12).EQ.'SO4') then
                  lbnoso4=.false.
                  conctb(i,1)=tct(i)
               elseIF(asplst(isl,nff)(1:12).EQ.'HNO3') then
                  lbnohno3=.false.
                  conctb(i,2)=tct(i)
               elseIF(asplst(isl,nff)(1:12).EQ.'NO3') then
                  lbnono3=.false.
                  conctb(i,3)=tct(i)
               elseIF(asplst(isl,nff)(1:12).EQ.'TNH3') then
                  lbnotnh3=.false.
                  conctb(i,4)=tct(i)
               ELSEIF(asplst(isl,nff)(1:12).EQ.'NH3') then
                  WRITE(*,*)' '
                  WRITE(*,*)'FATAL ERROR - Review message written'
                  WRITE(*,*)'in the list-file'
                  WRITE(io1,*)' '
                  WRITE(io1,*)'FATAL ERROR - In the background'
                  WRITE(io1,*)'Total NH3 (TNH3) is needed to run ALM'
                  WRITE(io1,*)' not NH3 gas (NH3)'
               stop
               endif
               enddo
            endif
         enddo
         IF(lbnoso4) then
             WRITE(*,*)'ERROR - SO4 is missing in '
             WRITE(*,*)'the background file'
         endif
         IF(lbnohno3)then
              WRITE(*,*)'ERROR - HNO3 is missing in '
              WRITE(*,*)'the background file'
         endif
         IF(lbnono3) then
             WRITE(*,*)'ERROR - NO3 is missing in '
             WRITE(*,*)'the background file'
         endif
         IF(lbnoso4.OR.lbnohno3.or.lbnono3) stop
         IF(lbnotnh3) then
           IF(nh3typ.eq.0.or.nh3typ.eq.2.or.nh3typ.eq.4) then
           WRITE(*,*)' '
           WRITE(*,*)'FATAL ERROR - Review message written'
           WRITE(*,*)'in the list-file'
           WRITE(io1,*)' '
           WRITE(io1,*)'FATAL ERROR- total NH3 (TNH3) from monthly'
         WRITE(io1,*)'background (BCKNH3) need to be used since no TNH3'
           WRITE(io1,*)'is found in external background file'
           stop
           ELSEIF(nh3typ.gt.4) then
           WRITE(*,*)' '
           WRITE(*,*)'FATAL ERROR - Review message written'
           WRITE(*,*)'in the list-file'
           WRITE(io1,*)' '
           WRITE(io1,*)'FATAL ERROR - NH3TYP can not be larger than 4'
           stop
           endif
          else
           IF(nh3typ.eq.1.or.nh3typ.eq.3) then
           WRITE(*,*)' '
           WRITE(*,*)'FATAL ERROR - Review message written'
           WRITE(*,*)'in the list-file'
           WRITE(io1,*)' '
           WRITE(io1,*)'FATAL ERROR - TNH3 is present in external'
           WRITE(io1,*)'background file. It will be used to repartition'
           WRITE(io1,*)'NO3/HNO3 - NH3TYP must be equal to 2 or 4'
           stop
           ELSEIF(nh3typ.eq.0) then
           WRITE(*,*)' '
           WRITE(*,*)'FATAL ERROR - Review message written'
           WRITE(*,*)'in the list-file'
           WRITE(io1,*)' '
           WRITE(io1,*)'FATAL ERROR -TNH3 is present in external'
           WRITE(io1,*)'background file. It must be used for TNH3'
           WRITE(io1,*)'background. NH3 from concentration alone can'
           WRITE(io1,*)'not be used'
           stop
           ELSEIF(nh3typ.gt.4) then
           WRITE(*,*)' '
           WRITE(*,*)'FATAL ERROR - Review message written'
           WRITE(*,*)'in the list-file'
           WRITE(io1,*)' '
           WRITE(io1,*)'FATAL ERROR - NH3TYP can not be larger than 4'
           stop
           endif
          endif
c
c --- (CEC) - 060306
c --- PROCESS THE THREE STEPS FOR ALM PUT TOGETHER - PARTITION and NO3RATIO COMBINED
c------------------------------------------------------------------------------------
c
c --- NH3 background coming from monthly averaged ammonia (BCKTNH3 only possible)
      cnh3ppb=-999.
      IF(bcknh3(nmob).NE.-999.and.(nh3typ.eq.1.or.nh3typ.eq.3)) then
      WRITE(*,*)' '
      WRITE(*,*)'FATAL ERROR - Review message written'
      WRITE(*,*)'in the list-file'
      WRITE(io1,*)' '
      WRITE(io1,*)' FATAL ERROR - BCKTNH3 (total NH3) is needed when'
      WRITE(io1,*)'MNITRATE=3 not BCKNH3 (NH3 gas only)'
      stop
      ELSEIF(bcktnh3(nmob).NE.-999.) then
      cnh3ppb=bcktnh3(nmob)
      else
       IF(nh3typ.eq.1.or.nh3typ.eq.3) then
       WRITE(*,*)' '
       WRITE(*,*)'FATAL ERROR - Review message written'
       WRITE(*,*)'in the list-file'
       WRITE(io1,*)' '
       WRITE(io1,*)'FATAL ERROR - monthly background ammonia = -999'
       stop
       endif
      endif
      IF(nh3typ.eq.1.or.nh3typ.eq.3) then
      cnh3ppb1=cnh3ppb
      else
      cnh3ppb1=0
      endif
c
      lnh3=.TRUE.
      ltnh3=.TRUE.
      if(ispecno3(4).EQ.0) lnh3=.FALSE.
      IF(ispecno3(7).EQ.0) ltnh3=.FALSE.
c --- CEC (060310) - When MNITRATE=3, we don not want TNH3 in any concentration files
      IF(ltnh3) then
      WRITE(*,*)' '
      WRITE(*,*)'FATAL ERROR - Review message written'
      WRITE(*,*)'in the list-file'
      WRITE(io1,*)' '
      WRITE(io1,*)'FATAL ERROR- When MNITRATE=3, TNH3 cannot be present'
      WRITE(io1,*)'in any of the CALPUFF concentration files MODDAT'
      stop
      endif
c --- CEC (060206) - compare FLAG entered by the user about
c                    where NH3 come from with what is found here.
      IF(lnh3) then
       if (nh3typ.eq.0.or.nh3typ.eq.1.or.nh3typ.eq.2) then
c       WRITE(*,*)'NH3 species in CALPUFF files is used for NO3/HNO3'
c       WRITE(*,*)'repartioning'
       else
       WRITE(*,*)' '
       WRITE(*,*)'FATAL ERROR - Review message written'
       WRITE(*,*)'in the list-file'
       WRITE(io1,*)' '
       WRITE(io1,*)'FATAL ERROR - NH3 species is listed in'
       WRITE(io1,*)'subgroup (2a&2b) as a processed species.'
       WRITE(io1,*)'DO NOT match options selected in control file'
c     WRITE(*,*)'If the user wants to use it to repartition NO3/HNO3,'
c     WRITE(*,*)'NH3TYP needs to be equal to 0 or 1'
c     WRITE(*,*)'If the user wants to use BCKNH3 or BCKTNH3 alone -'
c       WRITE(*,*)'remove NH3 and TNH3 species from the processed'
c       WRITE(*,*)'species in Subgroup (2a) and (2b). IF the user wants'
c       WRITE(*,*)'to use BCKNH3 or BCKTNH3 in addition make NH3TYP'
c       WRITE(*,*)'to be 1'
       stop
       endif
      ELSE
       if (nh3typ.eq.0.or.nh3typ.eq.1.or.nh3typ.eq.2) then
       WRITE(*,*)' '
       WRITE(*,*)'FATAL ERROR - Review message written'
       WRITE(*,*)'in the list-file'
       WRITE(io1,*)' '
       WRITE(io1,*)'FATAL ERROR - If the user wants to use NH3 species'
       WRITE(io1,*)'from one of the CALPUFF file, it needs to be listed'
       WRITE(io1,*)'in subgroup (2a&2b) as a processed species.'
       WRITE(io1,*)'DO NOT match options selected in control file'
c       WRITE(*,*)'Otherwise, choose to use NH3 monthly averaged'
c       WRITE(*,*)'background BCKNH3 or BCKTNH3 and select NH3TYP=3'
       stop
c      else
c      WRITE(*,*)'NH3 in ppb from BCKGNH3 is used for NO3/HNO3'
c      WRITE(*,*)'repartitioning'
       endif
      endif
c
c --- Section for gridded receptors
c ---------------------------------
      if(LSGRID) then
         rx0=0.5+FLOAT(isastr)
         ry0=0.5+FLOAT(jsastr)
         do j=1,ngy
         do i=1,ngx
c ---       Compute met grid cell index for this receptor
            rx=rx0+FLOAT(i-1)/FLOAT(meshdn)
            ix=rx
            ix=MAX(ix,1)
            ix=MIN(ix,ielmet)
            ry=ry0+FLOAT(j-1)/FLOAT(meshdn)
            iy=ry
            iy=MAX(iy,1)
            iy=MIN(iy,jelmet)
c ---       Extract met. variables (flag=2)
            if (metfm.eq.2) then
            call EXMET(3,ix,iy,dgrid,tempk,qsw,irh,rhoair,
     &              cldcov,coszen)
            ELSEIF (metfm.eq.0) then
            call EXMET(2,ix,iy,dgrid,tempk,qsw,irh,rhoair,
     &              cldcov,coszen)
            elseif (metfm.eq.1) then
            tempk=temp1d
            irh=irh1d
            rhoair=rho1d
            endif
            rhum=irh
c ---       Set conversion factor from g/m**3 to ppb
c ---       Note: constant 28.97e6 is based on rho in kg/m**3
            f=28.97e6/rhoair
c ---       Convert concentrations from g/m**3 to ppb
            cso4ppb=concg(i,j,ispecno3(1))*f/96.
            chno3ppb=concg(i,j,ispecno3(2))*f/63.
            cno3ppb=concg(i,j,ispecno3(3))*f/62.
c --- keep in memory in cno3ppb1 and chno3ppb1 - NO3 and HNO3 coming from the source
            cno3ppb1=cno3ppb
            chno3ppb1=chno3ppb
c --- Add to the source - background NO3, HNO3, SO4 and TNH3
            cso4ppb=(concg(i,j,ispecno3(1))+concgb(i,j,1))*f/96.
            chno3ppb=(concg(i,j,ispecno3(2))+concgb(i,j,2))*f/63.
            cno3ppb=(concg(i,j,ispecno3(3))+concgb(i,j,3))*f/62.
c
            ctno3ppb=cno3ppb+chno3ppb
c ---       Convert concentrations from g/m**3 to moles/m3
            tso4 = cso4ppb/f
            tso4 = MAX(tso4,1.E-12)
            tno3 = ctno3ppb/f
c
c --- Get TNH3 from different sources.
            if(LNH3) then
             IF(nh3typ.eq.1) then
              IF(lbnotnh3) then
              cnh3ppb=cnh3ppb1+concg(i,j,ispecno3(4))*f/17.+
     &         concg(i,j,ispecno3(7))*f/17.
              ELSE
              WRITE(*,*)' '
              WRITE(*,*)'FATAL ERROR - Review message written'
              WRITE(*,*)'in the list-file'
              WRITE(io1,*)' '
              WRITE(io1,*)'FATAL ERROR - can not use BCKTNH3 because'
              WRITE(io1,*)' TNH3 exist in the external background file'
              stop
              ENDIF
             ELSEIF(nh3typ.eq.2) then
             cnh3ppb=(concg(i,j,ispecno3(4))+concg(i,j,ispecno3(7))+
     &        concgb(i,j,4))*f/17.
             else
             WRITE(*,*)' '
             WRITE(*,*)'FATAL ERROR - Review message written'
             WRITE(*,*)'in the list-file'
             WRITE(io1,*)' '
             WRITE(io1,*)'FATAL ERROR - if NH3 from source is used'
             WRITE(io1,*)' NH3TYP can be only 1 or 2'
             stop
             endif
            else
             IF(nh3typ.eq.3) then
              IF(lbnotnh3) then
              cnh3ppb=cnh3ppb1
              ELSE
              WRITE(*,*)' '
              WRITE(*,*)'FATAL ERROR - Review message written'
              WRITE(*,*)'in the list-file'
              WRITE(io1,*)' '
              WRITE(io1,*)'FATAL ERROR - can not use BCKTNH3 because'
              WRITE(io1,*)' TNH3 exist in the external background file'
              stop
              ENDIF
             ELSEIF(nh3typ.eq.4) then
             cnh3ppb=concgb(i,j,4)*f/17.
             else
             WRITE(*,*)' '
             WRITE(*,*)'FATAL ERROR - Review message written'
             WRITE(*,*)'in the list-file'
             WRITE(io1,*)' '
             WRITE(io1,*)'FATAL ERROR - if NH3 from source is not used'
             WRITE(io1,*)' NH3TYP can be only 3 or 4'
             stop
             endif
            endif
            IF(cnh3ppb.LT.0) then
            WRITE(*,*)' '
            WRITE(*,*)'FATAL ERROR - Review message written'
            WRITE(*,*)'in the list-file'
            WRITE(io1,*)' '
            WRITE(io1,*)'FATAL ERROR - There is no TNH3 provided'
            stop
            endif
c           if(LNH3) cnh3ppb=concg(i,j,ispecno3(4))*f/17.
            if(meqphase.EQ.2) then
c ---          ISORROPIA
               tnh3 = cnh3ppb/f
               rhfrac = rhum * 0.01    ! fractional relative humidity
               if(tno3.GT.tno3floor .AND. tnh3.GT.0.0)then
                  call ISODRIVER(tso4,tno3,tnh3,rhfrac,tempk,
     &                           pno3,gnh3c)
               elseif(l_tno3floor .AND. tnh3.gt.0.0) then
                  call ISODRIVER(tso4,tno3floor,tnh3,rhfrac,tempk,
     &                           pno3,gnh3c)
                  pno3=pno3*(tno3/tno3floor)
               else
                  pno3 = 0.
                  gnh3c = 0.
               endif
            else
c ---          CHEMEQ
c ---          Adjust total ammonia for that absorbed by SO4
               canh3=cnh3ppb-2.*cso4ppb
               if(canh3.GT.0.0)then
                  call CHEMEQ(canh3,ctno3ppb,rhum,tempk,cno3ppb,
     &                     chno3ppb,enh3,io6)
               else
                  cno3ppb=0.0
                  chno3ppb=ctno3ppb
                  enh3=0.0
               endif
            endif
c
c ---  NO3 RATIO
c---------------
c --- Total Nitrate from the source is sum of cno3ppb1 and chno3ppb1 saved above
            ctno3=(cno3ppb1*62./f)+(chno3ppb1*63./f)*r62_63
c --- Nitrate ALL is the new cno3ppb and chno3ppb computed just above
            if(meqphase.EQ.2) then
c ---          ISORROPIA
               chno3all=(tno3 - pno3)*63.
               cno3all=pno3*62.
            else
c ---          CHEMEQ
               chno3all=chno3ppb*63./f
               cno3all=cno3ppb*62./f
            endif
            ctno3all=cno3all+chno3all*r62_63
            if(ctno3all.GT.0.0) then
c ---          Repartition the nitrate
               cno3=ctno3*cno3all/ctno3all
               chno3=(ctno3-cno3)*r63_62
               chno3=AMAX1(chno3,0.0)
c ---          Update stored nitrate concentrations
               concg(i,j,ispecno3(2))=chno3
               concg(i,j,ispecno3(3))=cno3
            endif
         enddo
         enddo
c ---    Write information for last gridded receptor in DEBUG mode
         if(ldb)write(io1,2028)cno3,chno3,ctno3,
     &                         cno3all,chno3all,ctno3all
c
      endif
c
c --- Section for discrete receptors
c ----------------------------------
      if(LDISC) then
         do i=1,ndrec
c ---       Compute met grid cell index for this receptor
            rx=(xrec(i)-xorigkm)/delx
            ix=rx+1
            ix=MAX(ix,1)
            ix=MIN(ix,ielmet)
            ry=(yrec(i)-yorigkm)/dely
            iy=ry+1
            iy=MAX(iy,1)
            iy=MIN(iy,jelmet)
c ---       Extract met. variables (flag=2)
            if (metfm.eq.2) then
            call EXMET(3,ix,iy,dgrid,tempk,qsw,irh,rhoair,
     &              cldcov,coszen)
            ELSEIF (metfm.eq.0) then
            call EXMET(2,ix,iy,dgrid,tempk,qsw,irh,rhoair,
     &              cldcov,coszen)
            elseif (metfm.eq.1) then
            tempk=temp1d
            irh=irh1d
            rhoair=rho1d
            endif
            rhum=irh
c
c ---       Set conversion factor from g/m**3 to ppb
c ---       Note: constant 28.97e6 is based on rho in kg/m**3
            f=28.97e6/rhoair
c ---       Convert concentrations from g/m**3 to ppb
            cso4ppb=concd(i,ispecno3(1))*f/96.
            chno3ppb=concd(i,ispecno3(2))*f/63.
            cno3ppb=concd(i,ispecno3(3))*f/62.
c --- keep in memory in cno3ppb1 and chno3ppb1 - NO3 and HNO3 coming from the source
            cno3ppb1=cno3ppb
            chno3ppb1=chno3ppb
c --- Add to the source - background NO3, HNO3, SO4 and TNH3.
            cso4ppb=(concd(i,ispecno3(1))+concdb(i,1))*f/96.
            chno3ppb=(concd(i,ispecno3(2))+concdb(i,2))*f/63.
            cno3ppb=(concd(i,ispecno3(3))+concdb(i,3))*f/62.
c
            ctno3ppb=cno3ppb+chno3ppb
c ---       Convert concentrations from g/m**3 to moles/m3
            tso4 = cso4ppb/f
            tso4 = MAX(tso4,1.E-12)
            tno3 = ctno3ppb/f
c
c --- Get TNH3 from different sources.
            if(LNH3) then
             IF(nh3typ.eq.1) then
              IF(lbnotnh3) then
              cnh3ppb=cnh3ppb1+concd(i,ispecno3(4))*f/17.+
     &        concd(i,ispecno3(7))*f/17.
              ELSE
              WRITE(*,*)' '
              WRITE(*,*)'FATAL ERROR - Review message written'
              WRITE(*,*)'in the list-file'
              WRITE(io1,*)' '
              WRITE(io1,*)'FATAL ERROR - can not use BCKTNH3 because'
              WRITE(io1,*)' TNH3 exist in the external background file'
              stop
              ENDIF
             ELSEIF(nh3typ.eq.2) then
             cnh3ppb=(concd(i,ispecno3(4))+concd(i,ispecno3(7))+
     &        concdb(i,4))*f/17.
             else
             WRITE(*,*)' '
             WRITE(*,*)'FATAL ERROR - Review message written'
             WRITE(*,*)'in the list-file'
             WRITE(io1,*)' '
             WRITE(io1,*)'FATAL ERROR - if NH3 from source is used'
             WRITE(io1,*)' NH3TYP can be only 1 or 2'
             stop
             endif
            else
             IF(nh3typ.eq.3) then
              IF(lbnotnh3) then
              cnh3ppb=cnh3ppb1
              ELSE
              WRITE(*,*)' '
              WRITE(*,*)'FATAL ERROR - Review message written'
              WRITE(*,*)'in the list-file'
              WRITE(io1,*)' '
              WRITE(io1,*)'FATAL ERROR - can not use BCKTNH3 because'
              WRITE(io1,*)' TNH3 exist in the external background file'
              stop
              ENDIF
             ELSEIF(nh3typ.eq.4) then
             cnh3ppb=concdb(i,4)*f/17.
             else
             WRITE(*,*)' '
             WRITE(*,*)'FATAL ERROR - Review message written'
             WRITE(*,*)'in the list-file'
             WRITE(io1,*)' '
             WRITE(io1,*)'FATAL ERROR - if NH3 from source is not used'
             WRITE(io1,*)' NH3TYP can be only 3 or 4'
             stop
             endif
            endif
            IF(cnh3ppb.LT.0) then
            WRITE(*,*)' '
            WRITE(*,*)'FATAL ERROR - Review message written'
            WRITE(*,*)'in the list-file'
            WRITE(io1,*)' '
            WRITE(io1,*)'FATAL ERROR - There is no TNH3 provided'
            stop
            endif
c           cnh3ppb2=concd(i,ispecno3(4))*f/17.
c           cnh3ppb3=concdb(i,4)*f/17.
c            WRITE(*,*)'NH3used bckgM bckg conc',cnh3ppb,cnh3ppb1,
c     &      cnh3ppb3,cnh3ppb2
            if(meqphase.EQ.2) then
c ---          ISORROPIA
               tnh3 = cnh3ppb/f
               rhfrac = rhum * 0.01    ! fractional relative humidity
               if(tno3.GT.tno3floor .AND. tnh3.GT.0.0)then
                  call ISODRIVER(tso4,tno3,tnh3,rhfrac,tempk,
     &                           pno3,gnh3c)
               elseif(l_tno3floor .AND. tnh3.gt.0.0) then
                  call ISODRIVER(tso4,tno3floor,tnh3,rhfrac,tempk,
     &                           pno3,gnh3c)
                  pno3=pno3*(tno3/tno3floor)
               else
                  pno3 = 0.
                  gnh3c = 0.
               endif
            else
c ---          CHEMEQ
c ---          Adjust total ammonia for that absorbed by SO4
               canh3=cnh3ppb-2.*cso4ppb
               if(canh3.GT.0.0)then
                  call CHEMEQ(canh3,ctno3ppb,rhum,tempk,cno3ppb,
     &                     chno3ppb,enh3,io6)
               else
                  cno3ppb=0.0
                  chno3ppb=ctno3ppb
                  enh3=0.0
               endif
            endif
c
c ---  NO3 RATIO
c---------------
c --- Total Nitrate from the source is sum of cno3ppb1 and chno3ppb1 saved above
            ctno3=(cno3ppb1*62./f)+(chno3ppb1*63./f)*r62_63
c --- Nitrate ALL is the new cno3ppb and chno3ppb computed just above
            if(meqphase.EQ.2) then
c ---          ISORROPIA
               chno3all=(tno3 - pno3)*63.
               cno3all=pno3*62.
            else
c ---          CHEMEQ
               chno3all=chno3ppb*63./f
               cno3all=cno3ppb*62./f
            endif
            ctno3all=cno3all+chno3all*r62_63
            if(ctno3all.GT.0.0) then
c ---          Repartition the nitrate
               cno3=ctno3*cno3all/ctno3all
               chno3=(ctno3-cno3)*r63_62
               chno3=AMAX1(chno3,0.0)
c ---          Update stored nitrate concentrations
               concd(i,ispecno3(2))=chno3
               concd(i,ispecno3(3))=cno3
            endif
         enddo
c ---    Write information for last gridded receptor in DEBUG mode
         if(ldb)write(io1,2028)cno3,chno3,ctno3,
     &                         cno3all,chno3all,ctno3all
      endif
c
c --- Section for complex terrain receptors
c -----------------------------------------
      if(LCTSG) then
         do i=1,nctrec
c ---       Compute met grid cell index for this receptor
            rx=(xctr(i)-xorigkm)/delx
            ix=rx+1
            ix=MAX(ix,1)
            ix=MIN(ix,ielmet)
            ry=(yctr(i)-yorigkm)/dely
            iy=ry+1
            iy=MAX(iy,1)
            iy=MIN(iy,jelmet)
c ---       Extract met. variables (flag=2)
            if (metfm.eq.2) then
            call EXMET(3,ix,iy,dgrid,tempk,qsw,irh,rhoair,
     &              cldcov,coszen)
            ELSEIF (metfm.eq.0) then
            call EXMET(2,ix,iy,dgrid,tempk,qsw,irh,rhoair,
     &              cldcov,coszen)
            elseif (metfm.eq.1) then
            tempk=temp1d
            irh=irh1d
            rhoair=rho1d
            endif
            rhum=irh
c ---       Set conversion factor from g/m**3 to ppb
c ---       Note: constant 28.97e6 is based on rho in kg/m**3
            f=28.97e6/rhoair
c ---       Convert concentrations from g/m**3 to ppb
            cso4ppb=conct(i,ispecno3(1))*f/96.
            chno3ppb=conct(i,ispecno3(2))*f/63.
            cno3ppb=conct(i,ispecno3(3))*f/62.
c --- keep in memory in cno3ppb1 and chno3ppb1 - NO3 and HNO3 coming from the source
            cno3ppb1=cno3ppb
            chno3ppb1=chno3ppb
c --- Add to the source - background NO3, HNO3, SO4 and TNH3.
            cso4ppb=(conct(i,ispecno3(1))+conctb(i,1))*f/96.
            chno3ppb=(conct(i,ispecno3(2))+conctb(i,2))*f/63.
            cno3ppb=(conct(i,ispecno3(3))+conctb(i,3))*f/62.
c
            ctno3ppb=cno3ppb+chno3ppb
c ---       Convert concentrations from g/m**3 to moles/m3
            tso4 = cso4ppb/f
            tso4 = MAX(tso4,1.E-12)
            tno3 = ctno3ppb/f
c
c --- Get TNH3 from different sources.
             if(LNH3) then
             IF(nh3typ.eq.1) then
              IF(lbnotnh3) then
              cnh3ppb=cnh3ppb1+conct(i,ispecno3(4))*f/17.+
     &        conct(i,ispecno3(7))*f/17.
              ELSE
              WRITE(*,*)' '
              WRITE(*,*)'FATAL ERROR - Review message written'
              WRITE(*,*)'in the list-file'
              WRITE(io1,*)' '
              WRITE(io1,*)'FATAL ERROR - can not use BCKTNH3 because'
              WRITE(io1,*)' TNH3 exist in the external background file'
              stop
              ENDIF
             ELSEIF(nh3typ.eq.2) then
             cnh3ppb=(conct(i,ispecno3(4))+conct(i,ispecno3(7))+
     &        conctb(i,4))*f/17.
             else
             WRITE(*,*)' '
             WRITE(*,*)'FATAL ERROR - Review message written'
             WRITE(*,*)'in the list-file'
             WRITE(io1,*)' '
             WRITE(io1,*)'FATAL ERROR - if NH3 from source is used'
             WRITE(io1,*)' NH3TYP can be only 1 or 2'
             stop
             endif
            else
             IF(nh3typ.eq.3) then
              IF(lbnotnh3) then
              cnh3ppb=cnh3ppb1
              ELSE
              WRITE(*,*)' '
              WRITE(*,*)'FATAL ERROR - Review message written'
              WRITE(*,*)'in the list-file'
              WRITE(io1,*)' '
              WRITE(io1,*)'FATAL ERROR - can not use BCKTNH3 because'
              WRITE(io1,*)' TNH3 exist in the external background file'
              stop
              ENDIF
             ELSEIF(nh3typ.eq.4) then
             cnh3ppb=conctb(i,4)*f/17.
             else
             WRITE(*,*)' '
             WRITE(*,*)'FATAL ERROR - Review message written'
             WRITE(*,*)'in the list-file'
             WRITE(io1,*)' '
             WRITE(io1,*)'FATAL ERROR - if NH3 from source is not used'
             WRITE(io1,*)' NH3TYP can be only 3 or 4'
             stop
             endif
            endif
            IF(cnh3ppb.LT.0) then
            WRITE(*,*)' '
            WRITE(*,*)'FATAL ERROR - Review message written'
            WRITE(*,*)'in the list-file'
            WRITE(io1,*)' '
            WRITE(io1,*)'FATAL ERROR - There is no TNH3 provided'
            stop
            endif
            if(meqphase.EQ.2) then
c ---          ISORROPIA
               tnh3 = cnh3ppb/f
               rhfrac = rhum * 0.01    ! fractional relative humidity
               if(tno3.GT.tno3floor .AND. tnh3.GT.0.0)then
                  call ISODRIVER(tso4,tno3,tnh3,rhfrac,tempk,
     &                           pno3,gnh3c)
               elseif(l_tno3floor .AND. tnh3.gt.0.0) then
                  call ISODRIVER(tso4,tno3floor,tnh3,rhfrac,tempk,
     &                           pno3,gnh3c)
                  pno3=pno3*(tno3/tno3floor)
               else
                  pno3 = 0.
                  gnh3c = 0.
               endif
            else
c ---          CHEMEQ
c ---          Adjust total ammonia for that absorbed by SO4
               canh3=cnh3ppb-2.*cso4ppb
               if(canh3.GT.0.0)then
                  call CHEMEQ(canh3,ctno3ppb,rhum,tempk,cno3ppb,
     &                     chno3ppb,enh3,io6)
               else
                  cno3ppb=0.0
                  chno3ppb=ctno3ppb
                  enh3=0.0
               endif
            endif
c
c ---  NO3 RATIO
c---------------
c --- Total Nitrate from the source is sum of cno3ppb1 and chno3ppb1 saved above
            ctno3=(cno3ppb1*62./f)+(chno3ppb1*63./f)*r62_63
c --- Nitrate ALL is the new cno3ppb and chno3ppb computed just above
            if(meqphase.EQ.2) then
c ---          ISORROPIA
               chno3all=(tno3 - pno3)*63.
               cno3all=pno3*62.
            else
c ---          CHEMEQ
               chno3all=chno3ppb*63./f
               cno3all=cno3ppb*62./f
            endif
            ctno3all=cno3all+chno3all*r62_63
            if(ctno3all.GT.0.0) then
c ---          Repartition the nitrate
               cno3=ctno3*cno3all/ctno3all
               chno3=(ctno3-cno3)*r63_62
               chno3=AMAX1(chno3,0.0)
c ---          Update stored nitrate concentrations
               conct(i,ispecno3(2))=chno3
               conct(i,ispecno3(3))=cno3
            endif
         enddo
c ---    Write information for last gridded receptor in DEBUG mode
         if(ldb)write(io1,2028)cno3,chno3,ctno3,
     &                         cno3all,chno3all,ctno3all
      endif
c
2028  format(1x, 'NO3RATIO Results for last receptor:',/5x,
     1      'NO3=',e10.4,' HNO3=',e10.4,' TNO3=',e10.4,
     2      ' NO3(all)=',e10.4,' HNO3(all)=',e10.4,' TNO3(all)=',e10.4)
c
c ---    Read the next set if results for all sources were not found
         if(.not.LTOTAL) goto 5
c
      return

c --- EOF encountered
999   ieof=1
      return

      end
c-----------------------------------------------------------------------
      subroutine filvers(io,ab,fname,ftype,dataver)
c-----------------------------------------------------------------------
c --- POSTUTIL   Version: 1.641    Level: 060329                 FILVERS
c
c --- Currently used for CALMET files, other types require modification
c
c --- CALPUFF    Version: 6.11     Level: 060309                 FILVERS
c                D. Strimaitis   Earth Tech, Inc.
c
c --- PURPOSE:  Opens file of input data for the application and sets
c               the file version format based on information from one
c               or more header records.  (File is closed on exit)
c
c --- UPDATE
c --- V6.1-V6.11   060309  (DGS): filename c*70 to c*132
c                                 remove LTYPE2 declaration (not used)
c
c --- INPUTS:
c            io - integer      - file unit number
c            ab - character*3  - file form (ASC or BIN)
c         fname - character*132- full filename
c         ftype - character*12 - type of file (e.g., 'CALMET')
c
c
c --- OUTPUT:
c       dataver - character*16 - file version identifier (e.g., '2.0')
c
c --- FILVERS called by: SETUP
c --- FILVERS calls:     none
c----------------------------------------------------------------------
c
      parameter(mxftype=8)

      integer iquote(4)
      character*132 aline
      character*80 title(3)
      character*132 fname
      character*64 adoc
      character*16 anames(2,mxftype),blank16
      character*16 dataset,dataver
      character*12 ftype
      character*12 ver0
      character*8 vermet,levmet
      character*8 versn,level
      character*3 ab
      character*1 aquote
      logical lascii

c --- Set fixed characters
      data aquote/''''/
      data blank16/'                '/

c --- Set file type names
      data anames/'CALMET          ','CALMET.DAT      ',
     &            'ISCMET          ','ISCMET.DAT      ',
     &            'PLMMET          ','PLMMET.DAT      ',
     &            'PROFILE         ','PROFILE.DAT     ',
     &            'SURFACE         ','SURFACE.DAT     ',
     &            'RELHUM2D        ','VISB.DAT        ',
     &            'TEMPK2D         ','TK2D.DAT        ',
     &            'RHOAIR2D        ','RHO2D.DAT       '/

c --- Initialize dataset version
      dataver=blank16

c --- Identify file type
      itype=0
      do k=1,mxftype
         if(ftype.EQ.anames(1,k)) itype=k
      enddo
      if(itype.EQ.0) then
         write(*,*)'FATAL Error in FILVERS - unknown file type: ',
     &              ftype
         write(*,*)'Expected one of the following: '
         do k=1,mxftype
            write(*,*) anames(1,k)
         enddo
         stop
      endif

c --- Open file
      if(ab.EQ.'BIN') then
         lascii=.FALSE.
         open(io,file=fname,status='old',form='unformatted',err=999)
      elseif(ab.EQ.'ASC') then
         lascii=.TRUE.
         open(io,file=fname,status='old',err=999)
      else
         write(*,*)'FATAL Error in FILVERS - unknown file format: ',ab
         write(*,*)'Expected BIN or ASC'
         write(*,*)'File Type input: ',ftype
         stop
      endif

      if(LASCII) then

c ---    Read first record into string
         read(io,'(a132)') aline
c ---    Assume type 2 file first
         if(aline(1:16).EQ.anames(2,itype)) then
            dataver=blank16
            j=0
            do i=17,32
               if(aline(i:i).NE.' ') then
                  j=j+1
                  dataver(j:j)=aline(i:i)
               endif
            enddo
         else
c ---       Process file formats by file type

            if(itype.EQ.2) then
c ---          ISC met file
c ---          ------------
               if(aline(1:3) .EQ. 'BEG') then
c ---             CALPUFF MOD-6.0 ISC format conventions
                  dataver(1:3)='1.6'
               else
c ---             Standard ISC format conventions
                  dataver(1:3)='1.0'
               endif

            elseif(itype.EQ.3) then
c ---          PLM met file
c ---          ------------
c ---          Standard PLM format conventions
               dataver(1:3)='1.0'

            elseif(itype.EQ.4) then
c ---          CTDM or AERMET PROFILE met file
c ---          -------------------------------
               if(aline(1:3) .EQ. 'BEG') then
c ---             CALPUFF MOD-6.0 PROFILE format conventions
                  dataver(1:3)='1.6'
               else
c ---             Standard PROFILE format conventions
                  dataver(1:3)='1.0'
               endif

            elseif(itype.EQ.5) then
c ---          CTDM or AERMET SURFACE met file
c ---          -------------------------------
               if(aline(1:3) .EQ. 'BEG') then
c ---             CALPUFF MOD-6.0 SURFACE format conventions
                  dataver(1:3)='1.6'
               else
c ---             Standard SURFACE format conventions
                  dataver(1:3)='1.0'
               endif

            endif

         endif

      else
c ---    BINARY files

         if(itype.EQ.1) then
c ---       CALMET file
c ---       -----------
            read(io) dataset,dataver,adoc
            if(dataset.NE.anames(2,itype)) then
c ---          Older dataset 

               REWIND(io)
c ---          record #1 - run title -- 60 words
               read(io)title
c ---          record #2 - run control parameters
               read(io)vermet,levmet

               read(levmet(1:6),'(i6)') ilevmet
               read(vermet(1:6),'(f6.0)') rvermet
               if(ilevmet.GE.980304 .OR. rvermet.GT.5.1) then
                  dataver='1.5'
               else
                  dataver='1.0'
               endif
            endif

         else
c ---       CALPUFF output files
c ---       --------------------
            read(io) dataset,dataver,adoc
            if(dataset.NE.anames(2,itype)) then
c ---          Older dataset 

               REWIND(io)
c ---          record #1 - run title -- 60 words
               read(io)title
c ---          record #2 - run control parameters
               read(io)versn,level

               read(level(1:6),'(i6)') ilev
               read(versn(1:6),'(f6.0)') rver
               if(rver.GE.5.) then
                  dataver='1.5'
               else
                  dataver='1.0'
               endif
            endif

         endif

      endif

c --- Close file
      close(io)

      return

999   write(*,*)
      write(*,*)'FATAL Error in FILVERS - bad filename:'
      write(*,*)fname
      stop

      end
c----------------------------------------------------------------------
      subroutine fin
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 090511                    FIN
c ---            D. Strimaitis
c
c --- PURPOSE:  Run termination routine
c
c --- INPUTS:
c
c       Common/QA/rcpu,rtime,rdate
c       Parameters: IO1, IO6
c
c --- OUTPUT:  none
c
c --- FIN called by:  (main)
c --- FIN calls:      JULDAY, DELTT, DATETM, FMT_DATE
c----------------------------------------------------------------------
c --- Include parameters
      INCLUDE 'params.utl'
      INCLUDE 'qa.utl'

      character*8 rtime2
      character*10 rdate2
      character*12 rdate12
c
c --- Get system date & time at end of run
      call DATETM(rdate2,rtime2,rcpu)
c
c --- Compute runtime
      read(rtime(1:2),10)ihr1
      read(rtime(4:5),10)imin1
      read(rtime(7:8),10)isec1
10    format(i2)
      t1=ihr1*3600.+imin1*60.+isec1

      read(rtime2(1:2),10)ihr2
      read(rtime2(4:5),10)imin2
      read(rtime2(7:8),10)isec2
      t2=ihr2*3600.+imin2*60.+isec2

      if(rdate.eq.rdate2)then
         delt=t2-t1
      else
         read(rdate(1:2),10)imo1
         read(rdate(4:5),10)iday1
         read(rdate(7:10),'(i4)')iyr1
         call JULDAY(io1,iyr1,imo1,iday1,ijul1)

         read(rdate2(1:2),10)imo2
         read(rdate2(4:5),10)iday2
         read(rdate2(7:10),'(i4)')iyr2
         call JULDAY(io1,iyr2,imo2,iday2,ijul2)

c ---    Compute no. hours from beg. of first hour of run to
c ---    ending hour of ending day of the run
         call DELTT(iyr1,ijul1,ihr1,iyr2,ijul2,ihr2,idelhr)

c ---    Adjust for minutes and seconds
         delt=idelhr*3600.-imin1*60.-isec1+imin2*60.+isec2
      endif

c --- On the PC, the runtime and CPU time are the same
c --- (DATETM provides RCPU = 0.0 on the PC)
      if(rcpu.eq.0.0)rcpu=delt

c --- Report current date
      rdate12=rdate2(1:10)//'  '
      call FMT_DATE(io1,'MM-DD-YYYY','DD-MMM-YYYY',rdate12)
      write(io1,1402)rtime2,rdate12,NINT(delt),NINT(rcpu)
      write(io6,1402)rtime2,rdate12,NINT(delt),NINT(rcpu)
1402  format(//2x,'End of run -- Clock time: ',a8/
     1         2x,'                    Date: ',a12//
     2         2x,'      Elapsed clock time: ',i12,' (seconds)'//
     3         2x,'                CPU time: ',i12,' (seconds)')

c
      return
      end

c----------------------------------------------------------------------
      subroutine isodriver(so4,no3,nh3,nrh,ntempk,pno3c,gnh3c)
c----------------------------------------------------------------------
c
c --- POSTUTIL   Version: 1.641    Level: 101025             ISODRIVER
c                P. Karamchandani, AER
c
c --- PURPOSE:  Driver routine to calculate gas/particle equilibrium
c               using ISORROPIA aerosol equilibrium module
c               This version uses double precision
c
c --- UPDATES:
c --- V1.63-V1.64  101025 (DGS): Revise ISOROPIA arguments for V2.1
c
c --- INPUTS:
c            so4    - real - Total sulfate concentration (mole/m3)
c            no3    - real - Total nitrate concentration (mole/m3)
c            nh3    - real - Total ammonia concentration (mole/m3)
c            nrh    - real - Fractional relative humidity
c            ntempk - real - Temperature (K)
c
c --- OUTPUT:
c            pno3c  - real - Particle-phase equilibrium NO3
c                            concentration (mole/m3)
c            gnh3c  - real - Gas-phase equilibrium NH3
c                            concentration (mole/m3)
c
c --- ISODRIVER called by:  PARTITION, ALM1STEP
c --- ISODRIVER calls:      ISOROPIA
c----------------------------------------------------------------------

C***********************************************************************
C                                                                      * 
C  REVISION HISTORY:                                                   *
C     Version 1.0 developed November 2006 by PK, AER, Inc. for         *
C     CALPUFF for API Contract No. 2006-102376                         *
C                                                                      *
C***********************************************************************
C
C...........  INCLUDES

      INCLUDE 'isrpia.inc'

C...........  ARGUMENTS and their descriptions

      REAL SO4             ! Total sulfate (moles/m3)
      REAL NO3             ! Total nitrate (moles/m3)
      REAL NH3             ! Total ammonia (moles/m3)
      REAL NRH             ! Relative humidity as fraction
      REAL NTEMPK          ! Temperature in Kelvin

      REAL PNO3C           ! Particle-phase NO3 concentration (mole/m**3)
      REAL GNH3C           ! Gas-phase NH3 concentration (mole/m**3)

! --- Locals
      REAL GNO3C           ! Gas-phase concentration of HNO3 in mole/m**3 air

      INTEGER NCTRL, NOTHER
cV1.7 PARAMETER(NCTRL = 2,NOTHER = 6)
cV2.1 PARAMETER(NCTRL = 2,NOTHER = 9)
      PARAMETER(NCTRL = 2,NOTHER = 9)

! --- Gas-phase concentration array in moles/m**3 air
      REAL*8  GAS(NGASAQ)

! --- Aqueous-phase concentration array in moles/m**3 air
      REAL*8  AERLIQ(NIONS+NGASAQ+2)

! --- Solid-phase concentration array in moles/m**3 air
      REAL*8  AERSLD(NSLDS)

! --- Flag for different types of problems solved and
! --- different state of aerosols (deliquescent or metastable)
      REAL*8  CNTRL(NCTRL)

! Solution information array (see ISOCOM.f for details)
      REAL*8  OTHER(NOTHER)

! Total species concentrations in moles/m**3 air
      REAL*8  WI(NCOMP)

! Temperature and humidity
      REAL*8  TEMPI, RHI
C***********************************************************************
C  begin body of subroutine 
C
C *** Assign input total concentrations to WI

      WI(1) = 0.    ! Na concentration
      WI(2) = so4
      WI(3) = nh3
      WI(4) = no3
      WI(5) = 0.    ! Cl concentration
cV2.1 Additional 3 are not used here
      WI(6) = 0.    ! total calcium concentration
      WI(7) = 0.    ! total potassium concentration
      WI(8) = 0.    ! total magnesium concentration

      RHI   = NRH
      TEMPI = NTEMPK
C
C *** CALL ISORROPIA 
C
      CNTRL(1) = 0.     ! 0 = FORWARD PROBLEM,  1 = REVERSE PROBLEM
      CNTRL(2) = 0.     ! 0 = SOLID + LIQUID AEROSOL,  1 = METASTABLE
C
      CALL ISOROPIA (WI,  RHI,  TEMPI,   CNTRL,  
     &               W,   GAS,  AERLIQ,  AERSLD,  SCASE,  OTHER)
C
C *** SAVE RESULTS

! Gas-phase HNO3 (moles/m3)
      GNO3C  = GAS(2)

! Particle-phase NO3 (moles/m3)
      PNO3C  = MAX(0.,NO3 - GNO3C)

! Gas-phase NH3 (moles/m3)
      GNH3C  = GAS(1)
C
      RETURN
      END

c----------------------------------------------------------------------
c --- BRING IN ISORROPIA ROUTINES
      include 'isorropia.for'
c --- BRING IN CALPUFF SYSTEM UTILITY SUBROUTINES
      include 'calutils.for'
c----------------------------------------------------------------------
